/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2007

   Read ethernet MAC address

   Build: make a_mac
*/

#include <WyStr.h>
#include <WySockFile.h>

#include <net/if.h>
#include <sys/ioctl.h>
#include <netdb.h>       // for ::getprotobyname

// [Syn] Convert argument ch to hexidecimal string
//
static WyStr to_hex(unsigned char ch)
{
 const size_t MinLen=2;
 WyStr str( Wy::wrd(ch,16) );
 if(str.size()<MinLen) {   // prepend '0' if str.size()<MinLen
   WyRet r=str.insert(0,MinLen-str.size(),'0');
   if(r!=Ok) {
     WY_THROW(r);
   }
 }
 return str;
};

const WyCSeg syntax=
   "Usage: a_mac interface\n"
   " e.g. ./a_mac eth0\n";

int main(int argc, char* argv[])
try {

 if(argc<2) {
   Wy::cerr << syntax;
   return(-1);
 }

 WyRet r;
 WySockFile sock;
 struct protoent *proto;
 struct ifreq ifr;
 const char* const interface= argv[1];

 proto= ::getprotobyname("tcp");
 if((r=sock.reset(PF_INET,SOCK_STREAM,proto->p_proto))!=Ok) {
   WY_THROW(r);
 }
 if(::strlen(interface)>=sizeof(ifr.ifr_name)) {
   WY_THROW( WyRet(Wym_ENAMETOOLONG) );
 }
 ::strcpy(ifr.ifr_name,interface);
 if(::ioctl(sock.fh().fd(),SIOCGIFHWADDR,&ifr)==-1) {
   WY_THROW( WyRet(errno) );
 }

 Wy::cout << to_hex( (unsigned char)ifr.ifr_hwaddr.sa_data[0] );
 for(size_t i=1; i<6; ++i) {
   Wy::cout << ':' << to_hex( (unsigned char)ifr.ifr_hwaddr.sa_data[i] );
 }
 Wy::cout << '\n';

 return(0);
}
catch(const WyRet& e) {
 Wy::cerr << "main() caught: " << Wy::wrd(e) << '\n';
 return(e->repcode());
}
catch(...) {
 Wy::cerr << "main() caught: Unknown type\n";
 throw;
};
