/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2006

  Read password from standard input (as tty)

  Build: make a_readpass
*/

#include "WyStr.h"
#include "WyTermios.h"
#include "WyTerminal.h"

int main()
try {
 WyRet r;

 // Define PwdTerm and construct object pterm to modify the device Wy::cin handles (assume tty)
 //
 // PwdTerm is a WyTerminal that stores tty attribute at construction, restores
 // the tty attribute at object destruction (caused by return/throw, or cancellation)
 //
 struct PwdTerm : WyTerminal {
     WyTermios ots;             // origianl termios

     PwdTerm( const WyByteFlow& tdev)
       : WyTerminal(tdev) {
         WyRet r=WyTerminal::getattr(ots);
         if(r!=Ok) {
           WY_THROW(r);
         }
       };
    ~PwdTerm() {
         WyTerminal::setattr(ots,TCSANOW);
       };
 } pterm( Wy::cin );

 // Declare WyTermios object ts.
 // Set ts from pterm, modify it suitable for inputting password
 WyTermios ts=pterm.ots;

 ts.lflag_and(~ECHO);      // turn off echo input
 ts.lflag_or(ECHONL);      // turn on echo NL character

 // Write the new attribute ts through pterm
 if((r=pterm.setattr(ts,TCSAFLUSH))!=Ok) {
   WY_THROW(r);
 }

 // Read back the tty attribute into ts, check the setting is in effect
 if((r=pterm.getattr(ts))!=Ok) {
   WY_THROW(r);
 }
 if(ts.lflag()&ECHO) {
   Wy::cerr << "cin tty failed to turn off ECHO";
   return(-1);
 }

 // Read password into pstr
 pterm << "Enter password: ";
 if((r=pterm.drain())!=Ok) {
   WY_THROW(r);
 }
 WyStr pstr;
 pstr._reserve(256);
 pterm >> pstr;

 // Print pstr to standard output and end
 Wy::cout << "Read password :" << pstr;

 // Note: Writting to pterm (the input tty) might also be fine
 //pterm << "Read password :" << pstr;
 return(0);
}
catch(const WyRet& e) {
 Wy::cerr << Wy::wrd(e) << "\n";
 return(-1);
}
catch(...) {
 Wy::cerr << "unknown throw type\n";
 throw;
};
