/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2007

 Echo serial line

 Build: make a_serialecho
*/
#include "WyTermios.h"
#include "WyTerminal.h"
#include "WyStr.h"

// Note: Setting works for 3-wire serial line
//
class SerDev : public WyTerminal {
    WyTermios  _otio;        // Original termios setting
  public:
    WY_THROW_REPLY;          // Use Reply
    SerDev(const char* devfile, speed_t baudrate);
    ~SerDev();
    WyRet reset() { return WyTerminal::reset(); };  // for hiding other reset overloads
};

SerDev::SerDev(const char* devfile, speed_t baudrate)
try : WyTerminal(devfile, O_RDWR|O_NOCTTY) 
{
 const cc_t SerVMin=32;
 WyRet r;
 WyTermios  ntio;

 // Get original termios
 if((r=WyTerminal::getattr(_otio))!=Ok) {
   WY_THROW( Reply(r) );
 }

 // Setup ntio (new termios)
 //
 // modify c_cflag,c_lflag,c_iflag,c_oflag
 // (non-canon mode, noecho, no flow control, no parity checking, 1 stop bit...)
 //
 ntio=_otio;
 ntio.cflag_and(~(CSIZE|CSTOPB|HUPCL|PARENB|CRTSCTS));

 ntio.lflag_and(~(ICANON|ECHO|ECHOE|ECHOK|ECHONL|ISIG|TOSTOP|IEXTEN));
 ntio.lflag_or(NOFLSH);

 ntio.iflag_and(~(BRKINT|IGNBRK| IGNPAR| PARMRK| INPCK|
                 ISTRIP|INLCR|IGNCR|ICRNL| IXON|IXOFF
              ));
 ntio.oflag_and(~OPOST);

 ntio.cflag_or(CLOCAL|CREAD|CS8);   // no modem, read enable,8 bits
 ntio.cc_VMIN(SerVMin);
 ntio.cc_VTIME(1);
 ntio.setispeed(baudrate);
 ntio.setospeed(baudrate);

 // Write ntio to *this device
 //
 if((r=WyTerminal::setattr(ntio,TCSADRAIN))!=Ok) {
   WY_THROW( Reply(r) );
 }
 WyTermios tt;
 if((r=WyTerminal::getattr(tt))!=Ok) {
   WY_THROW( Reply(r) );
 }
 if(tt!=ntio) {
   WY_THROW( Reply(Wym_EIO) );
 }
 if((r=WyTerminal::drain())!=Ok) {
   WY_THROW( Reply(r) );
 }
}
catch(const WyTerminal::Reply& e) {
  WY_THROW( Reply(e) );
}
catch(const SerDev::Reply& e) {
  WyTerminal::setattr(_otio,TCSADRAIN);
  WY_THROW( Reply(e) );
}
catch(const WyRet& e) {
  WyTerminal::setattr(_otio,TCSADRAIN);
  WY_THROW( WyRet(e) );
}
catch(...) {
  WyTerminal::setattr(_otio,TCSADRAIN);
  throw;
};

SerDev::~SerDev()
{
 WyTerminal::setattr(_otio,TCSADRAIN);
};

static const char* syntax=
 "[Syntax]$./a_serialecho devfile\n"
 " e.g. $a_serialecho /dev/ttyS0\n";


int main(int argc, char* argv[])
try {
 if(argc!=2) {
   Wy::cerr << syntax;
   return(-1);
 }
 WyRet r;
 SerDev ser(argv[1],B9600);
 Wy::cout << "Echo serial line: " << argv[1] 
          << ", speed=9600, no parity, 8 data-bits, 1 stop-bit\n";
 WyStr rbuf;
 for(;;) {
   ser >> rbuf;               // Read data from serial device
   Wy::cout << rbuf << '\n';  // Display received data
   ser << rbuf;               // Write back to the serial device
 }
}
catch(const WyRet& e) {
 Wy::cerr << "main caught: " << Wy::wrd(e) << '\n';
 return e->repcode();
}
catch(...) {
 Wy::cerr << "main caught(...)\n";
 throw;
};
