/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2005

 TCP client for time service

 Build: a_tcptime_cli
*/
#include "WyNum.h"            // for converting numbers (built-in types)
#include "WySockFile.h"       // for socket
#include "WySockAddrINet4.h"  // for IPv4 socket address
#include "WyDate.h"           // for date

const time_t UNIX_EPOCH= 2208988800UL;

static const char* syntax=
 "[Syntax]$./a_tcptime_cli ip port\n"
 " ip  = server ip in dotted-decimal notation\n"
 " port= port number\n";

int main(int argc, char* argv[])
try {
 WyRet r;
 uint16_t port;
 const char* SvrIP;

 // Convert argv[1] to URL and argv[2] to port number
 if(argc==3) {
   SvrIP=argv[1];
   if((r=Wy::_strnum(port,argv[2]))!=Ok) {
     WY_THROW(r);
   }
 } else {
   if(argc!=1) {
     Wy::cout << syntax;
     return(-1);
   }
   port=37;            // timeserver
   SvrIP="127.0.0.1";  // local machine
 }

 // Setup socket and send message (unspecified) by the server
 const WySockAddrINet4 SvrAddr(WySockAddrINet4::IPAddr(SvrIP),port);
 WySockFile sock(PF_INET,SOCK_STREAM,0);
 const char TrigMsg[]="Request for time\n";
 size_t tlen;
 if((r=sock.connect(SvrAddr))!=Ok) {
   WY_THROW(r);
 }
 sock << TrigMsg;

 // Read time_t
 time_t tm;
 if((r=Wy::full_read(sock,&tm,sizeof(tm),tlen))!=Ok) {
   WY_THROW(r);
 }
 tm=ntohl(tm)-UNIX_EPOCH;

 // Convert the time_t as in the local time zone and print it (may not be right!)
 const WyDate date(Wy::local_tzname(), WyTimeSpec(tm,0));
 
 Wy::cout << "Time from "
          << "ip=" << Wy::wrd(SvrAddr.ip())
          << " port=" << SvrAddr.port() << '\n'
          << Wy::wrd(date) << "\n\n";
 return(0);
}
catch(const WyRet& e) {
 if(e!=Ok) {
   Wy::cerr << Wy::wrd(e) << '\n';
 }
 return e->repcode();
}
catch(...) {
 Wy::cerr << "main caught(...)\n";
 throw;
};
