/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2005

 UDP server for time service

 Build: make a_udptime_svr
*/
#include "WyNum.h"            // for converting numbers (built-in types)
#include "WySockFile.h"       // for socket
#include "WySockAddrINet4.h"  // for IPv4 socket address
#include "WyDate.h"           // for date

const time_t UNIX_EPOCH= 2208988800UL;

static const char* syntax=
 "[Syntax]$./a_udptime_svr port\n";

int main(int argc, char* argv[])
try {
 if(argc!=2) {
   Wy::cout << syntax;
   return(-1);
 }
 WyRet r;
 uint16_t port;

 // Convert argv[1] as the port number and save into port
 if((r=Wy::_strnum(port,argv[1]))!=Ok) {
   WY_THROW(r);
 }

 // Setup server socket
 WySockFile svr_sock(PF_INET,SOCK_DGRAM,0);
 const WySockAddrINet4 SvrAddr( WySockAddrINet4::IPAddr(INADDR_ANY),port);

 if((r=svr_sock.bind(SvrAddr))!=Ok) {
   WY_THROW(r);
 }

 // Endless loop: respond each incoming message by writing current time back
 for(;;) {
   WySockAddrINet4 taddr;
   char rbuf[256];
   size_t rlen;

   if((r=svr_sock.recvfrom(rbuf,sizeof(rbuf)-1,0,&taddr,rlen))!=Ok) {
     WY_THROW(r);
   }
   rbuf[rlen]=0;  // make it zero-terminate string

   Wy::cout << "Connection from " << Wy::wrd(taddr.ip())
            << " (" << taddr.port() << "): "
            << rbuf << '\n';

   // Get current time into time_t and write it back
   const WyTimeSpec tm_now( Wy::now() );
   time_t tm(tm_now.second()+UNIX_EPOCH);
   tm=htonl(tm);
   if((r=svr_sock.sendto(&tm,sizeof(tm),0,&taddr,rlen))!=Ok) {
     Wy::cerr << Wy::wrd(r) << '\n';
   }
   if(rlen!=sizeof(tm)) {
     Wy::cerr << "sendto bytes is not " << sizeof(tm) << "\n";
   }
 }
 WY_ASSERT("UNREACHABLE");
 return(0);
}
catch(const WyRet& e) {
 if(e!=Ok) {
   Wy::cerr << Wy::wrd(e) << '\n';
 }
 return e->repcode();
}
catch(...) {
 Wy::cerr << "unknown throw type\n";
 throw;
};
