/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2005

  Simulate shell command who

  Build: make a_who
*/

#include "WyRegFile.h"  // for reading UTMP_FILE
#include "WyDate.h"     // for date and time
#include <utmp.h>       // for struct utmp ubuf

// Print text of ubuf
WyRet dump_info(struct utmp& ubuf)
try {
 if(ubuf.ut_type!=USER_PROCESS) {
   return(Ok);
 }
 WyStr str;
 str=ubuf.ut_user;
 str.resize(8,' '); str+=' ';  
 str+=ubuf.ut_line;
 str.resize(14,' '); str+=' ';
 str+=ubuf.ut_host;
 str.resize(20,' '); str+=' ';

 WyTimeSpec utm(ubuf.ut_tv.tv_sec,ubuf.ut_tv.tv_usec*1000);
 str+=Wy::wrd( WyDate(Wy::local_tzname(),utm) );
 
 Wy::cout << str << '\n';
 return(Ok);
}
catch(const WyStr::Reply& e) {
 WY_RETURN(e);
}
catch(const WyByteFlow::Reply& e) {
 WY_RETURN(e);
}
catch(const WyRet& e) {
 throw;
}
catch(...) {
 WY_TERMINATE("unknown throw type");
};

int main()
try {
 WyRet r;
 size_t n_rd;
 struct utmp ubuf;
 WyRegFile ufile(UTMP_FILE,O_RDONLY);

 for(;;) {
   if((r=Wy::full_read(ufile,&ubuf,sizeof(ubuf),n_rd))!=Ok) {
     WY_THROW(r);
   }
   if(n_rd!=sizeof(ubuf)) {
     if(n_rd==0) {
       break;
     }
     WY_THROW( WyRet(Wym_EIO) );
   }
   if((r=dump_info(ubuf))!=Ok) {
     throw(r);
   }
 };
 return(0);
}
catch(const WyRet& e) {
 if(e!=Ok) {
   Wy::cerr << Wy::wrd(e) << '\n';
 }
 return e->repcode();
}
catch(...) {
 Wy::cerr << "main caught(...)\n";
 throw;
};
