/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

    Demonstrate usage of WyDate

    Build: make demo_date
*/
#include "WyByteFlow.h"   // for Wy::cout
#include "WyDate.h"       // for WyDate
#include "WyStr.h"

// Get the time zone string (once)
//
static const WyStr& tz_string()
{
static const WyStr tzstr(Wy::local_tzname());
 return tzstr;
}

// function for demonstration
//
static WyRet f()
try {
 WyRet r;

 // Present date in the time zone tz_string
 //
 WyDate tm(tz_string(),Wy::now());  

 // Print tm
 Wy::cout << "It is now " << Wy::wrd(tm) << '\n';
 Wy::cout << "23 hours latter: " << Wy::wrd(tm+WyTimeSpec(23L*60*60,0)) << '\n';

 // Set tm to made date and write to standard output
 //
 if((r=tm.set_date(2003,9,9,0,0,0))!=Ok) {
   WY_RETURN(r);
 }
 Wy::cout << "Made time: 2003-9-9. Read back= " << Wy::wrd(tm) << '\n';
 
 return(Ok);
}
catch(const WyDate::Reply& e) {
 WY_RETURN(e);
}
catch(const WyRet& e) {
 WY_THROW( WyRet(e) );
};

int main()
try {

 WyRet r=f();
 if(r!=Ok) {
   WY_THROW(r);
 }

 // Print the result r
 //
 Wy::cout << Wy::wrd(r) << '\n';
 return(0);
}
catch(const WyRet& e) {
 if(e!=Ok) {
   Wy::cerr << Wy::wrd(e) << '\n';
 }
 return e->repcode();
}
catch(...) {
 Wy::cout << "main caught(...)\n";
 throw;
};
