/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2004

 Demonstrate use of Wy::_scanum by implementing a simple calculaor

 Build: 1. Select the type of NumType (default WyTimeSpec)
        2. make demo_num
*/
#include "WyByteFlow.h"  // for Wy::cout
#include "WyNum.h"       // for Wy::_scanum(..)
#include "WyStr.h"
#include "WyTimeSpec.h"  // WyTimeSpec is also a number for demonstratation

// Select type of NumType for this calculator

//typedef double NumType;
//typedef long long NumType;
typedef WyTimeSpec NumType;

static const char titmsg[]=
   "\nSimple expr calculator, Cntrl-C to exit.\n"
   "  Usage> num(+-*/)num  (e.g. '243-0', No blanks)\n"
   "  num= number (C number literal)\n\n";

static void print_bad_expr(const WyStr& str)
{
 Wy::cout << "Invalid expr: " << str; 
};

static void read_and_calculate_expr()
{
 WyRet r;
 WyStr rbuf;

 // Print prompt '>' and read input into rbuf
 Wy::cout << ">";
 Wy::cin >> rbuf;

 // Check the string in rbuf is exactly the format <ddd>(+-*/)<ddd>
 if(rbuf.size()<4) {
   print_bad_expr(rbuf);     // not enough chararacters to parse
   return; 
 } else if(rbuf.back()!='\n') {
   print_bad_expr(rbuf);     // not expected string format
   return;                   
 } else {};

 NumType lft;        // left side number of expr
 NumType rit;        // right side number of expr
 char opch;          // ( +| -| *| /)
 WyCSeg nstr( rbuf.cseg(0,rbuf.size()-1) );

 // Convert and get the first number into lft
 r=Wy::_scanum(lft,nstr,0);
 if(r!=Ok) {
   print_bad_expr(rbuf);   // no number/expr identified
   return;
 }
 if(nstr.size()<2) {
   print_bad_expr(rbuf);   // not enough left to parse
   return;
 }

 // Get the operator into opch
 opch=nstr[0];
 nstr.move_begin(1);

 // Convert and get the second number into rit
 r=Wy::_scanum(rit,nstr,0);
 if(r!=Ok) {
   print_bad_expr(rbuf);
   return;
 }
 if(nstr.size()!=0) {
   print_bad_expr(rbuf);   // string not fully recognized
   return;
 }
 
 // Calulate: res=lft op rit
 NumType res(lft);
 switch(opch) {
   case '+': res+=rit; break;
   case '-': res-=rit; break;
   case '*': res*=rit; break;
   case '/': res/=rit; break;   // on my machine, divide 0 produces garbage
   default:
        print_bad_expr(rbuf);
        return;
 };

 Wy::cout << ' ' << rbuf;       // one LF in rbuf

 // Print the result in radix 10,16. If NumType is integer, also in radix 2 and 8.
 Wy::cout << " =(dec) " << Wy::wrd(res) << '\n';
 if(std::numeric_limits<NumType>::is_integer) {
   Wy::cout << " =(bin) " << Wy::wrd(res,2) << '\n';
   Wy::cout << " =(oct) " << Wy::wrd(res,8) << '\n';
 }
 Wy::cout << " =(hex) " << Wy::wrd(res,16) << '\n';
};

int main(int,char**)
try {
 Wy::cout << titmsg;
 Wy::cout << (std::numeric_limits<NumType>::is_integer?
              "Compiled for integer type\n" : "Compiled for float type\n");
 for(;;) {
   read_and_calculate_expr();
 }
 return(0);
}
catch(const WyRet& e) {
 if(e!=Ok) {
   Wy::cerr << Wy::wrd(e) << '\n';
 }
 return e->repcode();
}
catch(...) {
 Wy::cerr << "main() caught(...)\n";
 throw;
};
