/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2007

   Demonstrate basic usage of Wy_RepInfo

   Build: make demo_repinfo
*/
#include "WyStr.h"       // for Wy::wrd
#include "WyByteFlow.h"  // for Wy::cout,cerr and operator<<

// Typical use, Step 1: (Re)Implement Wy_RepInfo for the user error message class.
//
class ErrMsgInfo : public Wy_RepInfo {
  public:
    ErrMsgInfo(int ec, const char* cstr) throw() 
      : Wy_RepInfo(ec,cstr) {};

  // This member must be reimplemented
  // Reimplementation has no action for simplicity. So this class has minimal
  // function.
  //
    void setmsg(const char*, const void*, size_t) throw() {};
};

// Typical use, Step 2: Declare static ErrMsgInfo objects
//
static ErrMsgInfo em1(1001,"User defined reply E001");
static ErrMsgInfo em2(1002,"User defined reply E002");
static ErrMsgInfo em3(1003,"User defined reply E003");

// Typical use, Step 3: Declare const WyReply objects
//
const WyReply E001(em1);   // user reply
const WyReply E002(em2);
const WyReply E003(em3);

static void dump_reply(const WyCSeg& name, const WyReply& rep)
{
 Wy::cout << name << ":\n";
 Wy::cout << "   repcode()= " << Wy::wrd(rep->repcode()) << '\n';
 Wy::cout << "    repstr()= \"" << rep->repstr() << "\"\n";
};

int main()
try {

 dump_reply("E001", E001);
 dump_reply("E002", E002);
 dump_reply("E003", E003);

 return(0);
}
catch(const WyRet& e) {
 Wy::cerr << Wy::wrd(e) << '\n';
 return(-1);
}
catch(...) {
 Wy::cerr << "main() caught(...)\n";
 throw;
};
