/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   Demonstrate usage of Wy_Thread 

   Build: make demo_thread
*/
#include "Wy_Thread.h"
#include "WyTimeSpec.h"     // for Wy::sleep(..)
#include "WyByteFlow.h"     // for Wy::cout

// Class Thread inherits Wy_Thread, is a simple thread object for demo.
//
class Thread : public Wy_Thread {
    const WyTimeSpec dtime;  // Delay time

  public:
    Thread() : dtime(0.7) /*0.7 seconds*/ {};
    ~Thread() throw() {
       tmain_close();     // Stop possibly active thread, then destruction begins.
    };

  protected:
    WyRet tmain() {   // Reimplement the default tmain()
      for(int i=0; i<10; ++i) { // Loop 10 times: sleep and print 'Z'
        Wy::sleep(dtime);
        Wy::cerr << 'Z';
      }
      return(Ok);
    };
};

int main()
try {
  WyRet r;
  Thread thrd;             // Declare a Thread (defined above) object 

  // Begin a new thread to run thrd.tmain()
  if((r=thrd.begin())!=Ok) {
    WY_THROW(r);
  }

  // Wait until thread state is not Active (the loop exausted)
  if((r=thrd.wait_not(Wy_Thread::Active))!=Ok) {
    WY_THROW(r);                                 
  }
  return(0);
}
catch(const WyRet& e) {
  Wy::cerr << Wy::wrd(e) << '\n';
  throw;
}
catch(...) {
  Wy::cerr << "Unidentified thrown object\n";
  throw;
};
