/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   Documentation is in file wybyteflow.3wy
*/
#ifndef WYBYTEFLOW_H__
#define WYBYTEFLOW_H__
#define WYBYTEFLOW_VERSION 33

#include "WySysFile.h"
#include "WyCSeg.h"

class WyByteFlow : public WySysFile {
    static Wy__FtBits cftbits() WY__NOTHROW__; // Wy__FtBits for this class

    // Hidden
    const WyByteFlow & operator =(const WyByteFlow &);
    bool operator==(const WyByteFlow&) const;
    bool operator!=(const WyByteFlow&) const;
    
  public:
    static const char class_name[];
    WY_THROW_REPLY;

    explicit  // no implicit conversion
    WyByteFlow(const Wy__FtBits cif=cftbits());
    explicit  // no implicit conversion
    WyByteFlow(WyFileHandle fh, const Wy__FtBits cif=cftbits());
    WyByteFlow(const WySysFile& sysf, const Wy__FtBits cif=cftbits());

    // [Added/forward]
    WyByteFlow(const char* pathname,int f, const Wy__FtBits cif=cftbits());
    WyByteFlow(const WyStr& pathname,int f, const Wy__FtBits cif=cftbits());

    // Note: If no this copy constructor, WyByteFlow(WyByteFlow&) throws
    //       WySysFile::Reply (fixed by provides one, so may guess that
    //       compiler generates by its own)
    //
    WyByteFlow(const WyByteFlow& sysf, const Wy__FtBits cif=cftbits());

    // [Move Constructor]
    //
    // Note: sysf must be whole object except in initialization list
    //
    WyByteFlow(WyByteFlow& sysf, Wy::ByMove_t) WY__TSPC();

    //virtual ~WyByteFlow();
 
    // [Derived]
    //bool is_default() const WY__TSPC();
    //WyFileHandle fh() const WY__TSPC();
    //mode_t ftype() const WY__TSPC();
    //WyRet stat(WyFileStat& filestat) const;

    // [Override/forward]
    virtual WyRet reset()
            { return WySysFile::reset(); };
    virtual WyRet reset(WyFileHandle h)
            { return WySysFile::reset(h); };
    virtual WyRet reset(const WySysFile& sysf)
            { return WySysFile::reset(sysf); };
    virtual WyRet reset(const char* pathname, int flags)
            { return WySysFile::reset(pathname,flags); };
    virtual WyRet reset(const WyStr& pathname, int flags)
            { return WySysFile::reset(pathname,flags); };

    // [Derived]
    //virtual WyRet _swap(WySysFile& bf) WY__TSPC();

    // [Override]
    virtual WyByteFlow* _alloc(WyRet&) const;

    // [Added]
    virtual WyRet read(void*, size_t, size_t&);
    virtual WyRet read(WyStr&, size_t, size_t&);

    virtual WyRet write(const void*, size_t, size_t&);
    virtual WyRet write(WyCSeg, size_t&);
    virtual WyRet write(const WyStr&, size_t&);

    WyRet _fsync();
    WyRet _fdatasync();
    WyRet _tcdrain();
    bool  _isatty() const;

    bool wy_is_std_fd() const WY__TSPC();
};

namespace Wy {
 extern WyByteFlow& cerr;
 extern WyByteFlow& cin;
 extern WyByteFlow& cout;

 WyRet unlink(const WyStr&);  // SRC: wy_kfunc.cpp
 WyRet unlink(const char*);

 WyRet access(const WyStr&,int);
 WyRet access(const char*,int);

 WyRet chdir(const WyStr& path);
 WyRet chdir(const char* path);

 WyRet getcwd(WyStr& path);

 WyRet chown(const WyStr& path,uid_t owner,gid_t group);
 WyRet chown(const char* path,uid_t owner,gid_t group);

 WyRet chmod(const WyStr& path,mode_t mode);
 WyRet chmod(const char* path,mode_t mode);

 WyRet link(const WyStr& oldpath,const WyStr& newpath);
 WyRet link(const char* oldpath,const char* newpath);
 WyRet link(const char* oldpath,const WyStr& newpath);
 WyRet link(const WyStr& oldpath,const char* newpath);

 WyRet gethostname(WyStr& buf);
};

#include "wy_iouty.h"

#endif
