/* Copyright is licensed by GNU LGPL.                         by I.J.Wang 2004
*/
#define WYLIB_SOURCE

#include "WyCSeg.h"

static const WyCSeg::value_type wy__default_seg[]=
      { WyCSeg::value_type(0), WyCSeg::value_type(1) };

const WyCSeg::value_type* WyCSeg::DefaultAddr=wy__default_seg;

bool WyCSeg::find(size_type& res, size_type idx, const WyCSeg& cs) const WY__TSPC()
{
 size_type tsize(_pz-_pa );
 if(idx>=tsize) {
   return false;
 }
 tsize-=idx;
 const size_type csize( cs.size() );
 if((csize>tsize)||(csize==0)) {
   return false;
 }
 tsize-=csize;
 const value_type* ptr( _pa+idx );
 const value_type* cbgn( cs.begin() );
 for( ; ; --tsize,++ptr) {
   if(std::memcmp(ptr,cbgn,csize)==0) {
     res=ptr-_pa;
     return(true);
   }
   if(tsize==0) {
     return(false);
   }
 }
};

bool WyCSeg::rfind(size_type& res,size_type idx,const WyCSeg& cs) const WY__TSPC()
{
 const size_type csize( cs.size() );
 if(csize<=0) {
   return(false); // note: memcmp indicates "" is found
 }
 size_type fidx= _pz-_pa;
 if(fidx<csize) {
   return(false);
 }
 fidx-=csize;
 if(idx<=fidx) {
   if(idx==0) {
     return(false);
   }
   fidx=idx-1;
 }
 const char* const cs_bgn( cs.begin() );
 for(;;) {
   if(std::memcmp(_pa+fidx,cs_bgn,csize)==0) {
     res=fidx;
     return(true);
   }
   if(fidx--==0) {
     return(false);
   }
 };
};

bool WyCSeg::findc(size_type& res,size_type idx,value_type ch) const WY__TSPC()
{
 const size_type msize(_pz-_pa);
 if(idx>=msize) {
   return(false);
 }
 const value_type * const ptr(  static_cast<const value_type*>(
         std::memchr(_pa+idx,(unsigned char)ch,msize)) );
 if(ptr==NULL) {
   return(false);
 }
 res=ptr-_pa;
 return(true);
};

bool WyCSeg::findc_not(size_type& res,size_type idx,value_type ch) const WY__TSPC()
{
 if(idx>=size_type(_pz-_pa)) {
   return false;
 }
 for(const value_type *ptr(_pa+idx) ; ptr<_pz; ++ptr) {
   if(*ptr!=ch) {
     res=ptr-_pa;
     return(true);
   }
 }
 return(false);
};

bool WyCSeg::findc_not(size_type& res,size_type idx,const WyCSeg& cs) const WY__TSPC()
{
 if(idx>=size_type(_pz-_pa)) {
   return false;
 }
 const value_type *cs_begin(cs.begin());
 const size_type   cs_size(cs.size());
 for(const value_type *ptr(_pa+idx) ; ptr<_pz; ++ptr) {
   if(std::memchr(cs_begin,*(unsigned char*)ptr,cs_size)==NULL) {
     res=ptr-_pa;
     return(true);
   }
 }
 return(false);
};

bool WyCSeg::findc(size_type& res,size_type idx,const WyCSeg& cs) const WY__TSPC()
{
 if(idx>=size_type(_pz-_pa)) {
   return false;
 }
 const value_type *cs_begin(cs.begin());
 const size_type   cs_size(cs.size());
 for(const value_type *ptr(_pa+idx) ; ptr<_pz; ++ptr) {
   if(std::memchr(cs_begin,*(unsigned char*)ptr,cs_size)!=NULL) {
     res=ptr-_pa;
     return(true);
   }
 }
 return(false);
};

bool WyCSeg::rfindc(size_type& res,size_type idx,value_type ch) const WY__TSPC()
{
 const size_type msize( _pz-_pa );
 if(msize==0) {
   return(false);
 }
 const value_type * const ptr( static_cast<const value_type*> (
                memrchr(_pa,(unsigned char)ch, (idx>=msize)? msize: idx)) );
 if(ptr==NULL) {
   return(false);
 }
 res=ptr-_pa;
 return(true);
};

bool WyCSeg::rfindc_not(size_type& res,size_type idx,value_type ch) const WY__TSPC()
{
 const value_type* ptr= (idx>=size_type(_pz-_pa))?  (_pz-1): (_pa+idx)-1;
 for(; ptr>=_pa; --ptr) {
   if(*ptr!=ch) {
     res=ptr-_pa;
     return(true);
   }
 }
 return(false);
};

bool WyCSeg::rfindc_not(size_type& res,size_type idx, const WyCSeg& cs) const WY__TSPC()
{
 const value_type* ptr= (idx>=size_type(_pz-_pa))?  (_pz-1): (_pa+idx)-1;

 const value_type *cs_begin( cs.begin() );
 const size_type  cs_size( cs.size() );
 for(; ptr>=_pa; --ptr) {
   if(std::memchr(cs_begin,*(const unsigned char*)ptr,cs_size)==NULL) {
     res=ptr-_pa;
     return(true);
   }
 }
 return(false);
};

bool WyCSeg::rfindc(size_type& res,size_type idx, const WyCSeg& cs) const WY__TSPC()
{
 const value_type* ptr= (idx>=size_type(_pz-_pa))?  (_pz-1): (_pa+idx)-1;

 const value_type *cs_begin( cs.begin() );
 const size_type  cs_size( cs.size() );
 for(; ptr>=_pa; --ptr) {
   if(std::memchr(cs_begin,*(const unsigned char*)ptr,cs_size)!=NULL) {
     res=ptr-_pa;
     return(true);
   }
 }
 return(false);
};

