/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   Documentation in file wychrfile.3wy

   Note: Many system character files in /dev are seekable, e.g mem,dsp1,vcs,usb,
         full,null,urandom,zero,..
         App. is thought to use ::lseek(chrf.fh().fd(), .....)
*/
#ifndef WYCHRFILE_H__
#define WYCHRFILE_H__
#define WYCHRFILE_VERSION 33

#include "WyByteFlow.h"

class WyRegFile;
class WyChrFile;
class WyTerminal;
class WyFifoFile;
class WySockFile;

class WyChrFile : public WyByteFlow {
    static Wy__FtBits cftbits() WY__NOTHROW__; // Wy__FtBits for this class

    // Hidden
    const WyChrFile & operator =(const WyChrFile &);
    bool operator==(const WyChrFile&) const;
    bool operator!=(const WyChrFile&) const;
    mode_t ftype() const WY__TSPC();

    // Hidden
    WyRet _fsync();
    WyRet _fdatasync();

    // Hidden (for compile time rule-out)
    WyChrFile(const WyRegFile&);
    //WyChrFile(const WyChrFile&);
    //WyChrFile(const WyTerminal&);
    WyChrFile(const WyFifoFile&);
    WyChrFile(const WySockFile&);
    WyRet reset(const WyRegFile&);
    //WyRet reset(const WyChrFile&);
    //WyRet reset(const WyTerminal&);
    WyRet reset(const WyFifoFile&);
    WyRet reset(const WySockFile&);
  public:
    static const char class_name[];
    WY_THROW_REPLY;

    WyChrFile();
    explicit  // no implicit conversion
    WyChrFile(WyFileHandle fh);
    WyChrFile(const WySysFile& chrf);
    WyChrFile(const char* pathname,int f);
    WyChrFile(const WyStr& pathname,int f);

    WyChrFile(const WyChrFile& chrf);

    // [Move Constructor]
    //
    // Note: chrf must be whole object except in initialization list
    //
    WyChrFile(WyChrFile& chrf, Wy::ByMove_t) WY__TSPC();

    //virtual ~WyChrFile();

    // [Derived]
    //bool is_default() const WY__TSPC();
    //WyFileHandle fh() const WY__TSPC();
    //WyRet stat(WyFileStat& filestat) const;

    // [Override]
    virtual WyRet reset()
         { return WyByteFlow::reset(); };
    virtual WyRet reset(WyFileHandle fh)
         { return WyByteFlow::reset(fh); };
    virtual WyRet reset(const WySysFile& chrf)
         { return WyByteFlow::reset(chrf); };
    virtual WyRet reset(const char* pathname,int f)
         { return WyByteFlow::reset(pathname,f); };
    virtual WyRet reset(const WyStr& pathname,int f)
         { return WyByteFlow::reset(pathname,f); };

    // [Derived]
    //virtual WyRet _swap(WySysFile& chrf) WY__TSPC();

    //[Override]
    virtual WyChrFile* _alloc(WyRet& r) const;

    // [Derived]
    //virtual WyRet read(void *, size_t, size_t&);
    //virtual WyRet read(WyStr&, size_t, size_t&);

    //virtual WyRet write(const void *, size_t, size_t&);
    //virtual WyRet write(WyCSeg, size_t&);
    //virtual WyRet write(const WyStr&, size_t&);

    //WyRet _tcdrain();
    //bool  _isatty() const;
};

#endif
