/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003
*/

#define WYLIB_SOURCE
#include "WyCond.h"

const char WyCond::class_name[]="WyCond";

WyCond::WyCond() WY__TSPC(Reply)
#ifndef NDEBUG
 : _sig(WY_NONE_ID)
#endif
{
 const int v=::pthread_cond_init(&_cond,NULL);
 if(v!=0) {
   //
   // EBUSY/EINVAL should not occur and not explicitly documented
   //
   WY_THROW( Reply(v) );
 }
#ifndef NDEBUG
 _sig=WYCOND_ID;
#endif
};

WyCond::~WyCond() WY__TSPC()
{
#ifndef NDEBUG
 if(_sig!=WYCOND_ID) {
   WY_TERMINATE("");
 }
 _sig=WY_NONE_ID;
#endif

 do {
   const int v( ::pthread_cond_destroy(&_cond) );
   if(v==0) {
     return;      // succeed
   }
   if(v==EBUSY) {
     // Try to wakeup threads waiting on _cond to have them chance
     // to leave (with bad object signature)
     //
     ::pthread_cond_broadcast(&_cond);
     Wy__Base::yield();
     continue;
   }

   // others, EINVAL should not occur. break to terminate
   break;
 } while(true);
 WY_TERMINATE("");
};

WyRet WyCond::signal() WY__TSPC()
{
#ifndef NDEBUG
 if(_sig!=WYCOND_ID) {
   WY_TERMINATE("");
 }
#endif
 const int v=::pthread_cond_signal(&_cond);
 if(v!=0) {
   WY_RETURN(v);
 }
 return(Ok);
};

WyRet WyCond::broadcast() WY__TSPC()
{
#ifndef NDEBUG
 if(_sig!=WYCOND_ID) {
   WY_TERMINATE("");
 }
#endif
 const int v=::pthread_cond_broadcast(&_cond);
 if(v!=0) {
   WY_RETURN(v);
 }
 return(Ok);
};

WyRet WyCond::wait(WyLock &mtx)
{
#ifndef NDEBUG
 if(_sig!=WYCOND_ID) {
   WY_THROW( WyRet() );
 }
#endif
 if(mtx._pmtx==NULL) {
   WY_THROW( WyRet(Wym_EINVAL) );   // WyLock is empty
 }
 const int retv( ::pthread_cond_wait(&_cond,&((mtx._pmtx)->_mtx)) );

 // Linux manpage says never return errcode, other source says it may
 if(retv!=0) {
   WY_RETURN(retv);
 }
 return(Ok);
};

WyRet WyCond::wait(WyLock &mtx, WyTimeSpec abstime)
{
#ifndef NDEBUG
 if(_sig!=WYCOND_ID) {
   WY_TERMINATE("");
 }
#endif
 if(mtx._pmtx==NULL) {
   WY_THROW( WyRet(Wym_EINVAL) );   // WyLock is empty
 }
 
 const int v( ::pthread_cond_timedwait(&_cond,&((mtx._pmtx)->_mtx),
                                       &abstime.wy_timespec()) );
 WY_RETURN(v);
};
