/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   WyCond is for a thread to suspend execution, waiting to resume
   (wrapper of ::pthread_cond_t)
*/

#ifndef WYCOND_H__
#define WYCOND_H__

#define WYCOND_VERSION 33

#include "WyMutex.h"
#include "WyTimeSpec.h"

#ifndef NDEBUG
#include <signal.h>   // for sig_atomic_t
#endif

class WyCond {
  public:
    static const char class_name[];
    WY_THROW_REPLY;

    WyCond() WY__TSPC(Reply);

    ~WyCond() WY__TSPC();

    WyRet signal() WY__TSPC();

    WyRet broadcast() WY__TSPC();

    WyRet wait(WyLock &mtx);

    WyRet wait(WyLock &mtx, WyTimeSpec abstime);

  private:
   #ifndef NDEBUG
    volatile sig_atomic_t _sig;                // class signature
   #endif
    ::pthread_cond_t  _cond;

    // Hidden
    WyCond(const WyCond&);
    const WyCond& operator =(const WyCond&);
    bool operator ==(const WyCond&);
};

#endif
