/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   documentation is in file wyfifofile.3wy
*/

#ifndef WYFIFOFILE_H__
#define WYFIFOFILE_H__
#define WYFIFOFILE_VERSION 33

#include "WyByteFlow.h"

class WyRegFile;
class WyChrFile;
class WyTerminal;
class WyFifoFile;
class WySockFile;

class WyFifoFile : public WyByteFlow {
    static Wy__FtBits cftbits() WY__NOTHROW__;

    // Hidden
    const WyFifoFile & operator =(const WyFifoFile &);
    bool operator==(const WyFifoFile&) const;
    bool operator!=(const WyFifoFile&) const;
    mode_t ftype() const WY__TSPC();

    // Hidden
    WyRet _fsync();
    WyRet _fdatasync();
    WyRet _tcdrain();
    bool _isatty() const;

    // Hidden (for compile time rule-out)
    WyFifoFile(const WyRegFile&);
    WyFifoFile(const WyChrFile&);
    WyFifoFile(const WyTerminal&);
    //WyFifoFile(const WyFifoFile&);
    WyFifoFile(const WySockFile&);
    WyRet reset(const WyRegFile&);
    WyRet reset(const WyChrFile&);
    WyRet reset(const WyTerminal&);
    //WyRet reset(const WyFifoFile&);
    WyRet reset(const WySockFile&);
  public:
    static const char class_name[];
    WY_THROW_REPLY;

    WyFifoFile();
    explicit  // no implicit conversion
    WyFifoFile(WyFileHandle fh);
    WyFifoFile(const WySysFile& ff);
    WyFifoFile(const char* pathname,int f);
    WyFifoFile(const WyStr& pathname,int f);

    WyFifoFile(const WyFifoFile &ff);

    // [Move Constructor]
    //
    // Note: ff must be whole object except in initialization list
    //
    WyFifoFile(WyFifoFile &ff, Wy::ByMove_t) WY__TSPC();

    //virtual ~WyFifoFile();

    // [Derived]
    // bool is_default() const WY__TSPC();
    // WyFileHandle fh() const WY__TSPC();
    // WyRet stat(WyFileStat& filestat) const;

    // [Override]
    virtual WyRet reset()
         { return WyByteFlow::reset(); };
    virtual WyRet reset(WyFileHandle fh)
         { return WyByteFlow::reset(fh); };
    virtual WyRet reset(const WySysFile& chrf)
         { return WyByteFlow::reset(chrf); };
    virtual WyRet reset(const char* pathname,int f)
         { return WyByteFlow::reset(pathname,f); };
    virtual WyRet reset(const WyStr& pathname,int f)
         { return WyByteFlow::reset(pathname,f); };

    // [Derived]
    //virtual WyRet _swap(WySysFile& f) WY__TSPC();

    //[Override]
    virtual WyFifoFile* _alloc(WyRet&) const;

    // [Derived]
    // virtual WyRet read(void *, size_t, size_t&)
    // virtual WyRet read(WyStr&, size_t, size_t&)

    // [Derived]
    // virtual WyRet write(const void *,size_t, size_t&)
    // virtual WyRet write(WyCSeg, size_t&);
    // virtual WyRet write(const WyStr&,size_t&);

    static WyRet mkpipe(WyFifoFile& rd,WyFifoFile& wr);
    static WyRet create(const char* pathname,mode_t m);
    static WyRet create(const WyStr& pathname,mode_t m);
};

#endif
