/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003
*/

#ifndef WYFILEHANDLE_H__
#define WYFILEHANDLE_H__
#define WYFILEHANDLE_VERSION 33

#include "WyRet.h"
#include <sys/types.h>   // for mode_t, gid_t, uid_t,..

class WyFileStat;
/*
   WyFileHandle is a class for file descriptor (file handle)
*/
class WyFileHandle {
  public:
    static const char class_name[];

    WyFileHandle() WY__TSPC()
              : _fd(WY__DEFAULT_FD) {};

    WyFileHandle(const WyFileHandle &src) WY__TSPC()
              { _fd=src._fd; };

    // Note: This member takes any value of fd
    //
    explicit
    WyFileHandle(Wy__FdType fd) WY__TSPC()
             : _fd(fd) {};

    WyFileHandle(WyFileHandle& src, Wy::ByMove_t) WY__TSPC()
              { _fd=src._fd; };  // self copy ok

    bool is_default() const WY__TSPC()
              { return _fd==WY__DEFAULT_FD; };

    Wy__FdType fd() const WY__TSPC()
        { return _fd; };

    WyRet stat(WyFileStat& filestat) const;

    void reset() WY__TSPC()
              { _fd=WY__DEFAULT_FD; };

    void reset(const WyFileHandle& src) WY__TSPC()
        { _fd=src._fd; };

    void reset(Wy__FdType fd) WY__TSPC()
        { _fd=fd; };

    const WyFileHandle& operator =(const WyFileHandle& src) WY__TSPC()
        { _fd=src._fd; return(*this); };

    const WyFileHandle& operator =(Wy__FdType fd) WY__TSPC()
        { _fd=fd; return(*this); };

    bool operator==(const WyFileHandle& rhs) const WY__TSPC()
        { return _fd==rhs._fd; };
    bool operator!=(const WyFileHandle& rhs) const WY__TSPC()
        { return _fd!=rhs._fd; };

  private:
    Wy__FdType _fd;

    class Assertion;
};

#endif
