/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

*/

#define WYLIB_SOURCE
#include "WyFileStat.h"
#include "WyStr.h"
#include "wy__cfunc.h"
#include <cstring>      // for memset
#include <unistd.h>

const char WyFileStat::class_name[]="WyFileStat";

WyFileStat::WyFileStat() WY__TSPC()
{
 std::memset(&_stt,0,sizeof(_stt));
};

WyFileStat::WyFileStat(const WyFileStat& src) WY__TSPC()
{
 _stt=src._stt;
};

WyFileStat::WyFileStat(WyFileStat &src, Wy::ByMove_t) WY__TSPC()
{
 _stt=src._stt;
};

WyFileStat::WyFileStat(const struct stat& fs) WY__TSPC()
{
 // self-copy ok
 std::memcpy(&_stt,&fs,sizeof(_stt));
};

bool WyFileStat::is_default() const WY__TSPC()
{
 static const WyFileStat DObj;
 return std::memcmp(&_stt,&DObj,sizeof(_stt))==0;
};

void WyFileStat::reset() WY__TSPC()
{
 std::memset(&_stt,0,sizeof(_stt));
};

void WyFileStat::reset(const WyFileStat& src) WY__TSPC()
{
 _stt=src._stt;
};

void WyFileStat::swap(WyFileStat& src) WY__TSPC()
{
 Wy__Base::valswp(_stt,src._stt); 
};

const WyFileStat& WyFileStat::operator=(const WyFileStat& src) WY__TSPC()
{
 _stt=src._stt;
 return *this;
};

mode_t WyFileStat::sf_mode() const WY__TSPC()
{
 return _stt.st_mode;
};

ino_t WyFileStat::sf_ino() const WY__TSPC()
{
 return _stt.st_ino;
};

dev_t WyFileStat::sf_dev() const WY__TSPC()
{
 return _stt.st_dev;
};

nlink_t WyFileStat::sf_nlink() const WY__TSPC()
{
 return _stt.st_nlink;
};

uid_t WyFileStat::sf_uid() const WY__TSPC()
{
 return _stt.st_uid;
};

gid_t WyFileStat::sf_gid() const WY__TSPC()
{
 return _stt.st_gid;
};

dev_t WyFileStat::sf_rdev() const WY__TSPC()
{
 return _stt.st_rdev;
};

off_t WyFileStat::sf_size() const WY__TSPC()
{
 return _stt.st_size;
};

blksize_t WyFileStat::sf_blksize() const WY__TSPC()
{
 return _stt.st_blksize;
};

blkcnt_t WyFileStat::sf_blocks() const WY__TSPC()
{
 return _stt.st_blocks;
};

std::time_t WyFileStat::sf_atime() const WY__TSPC()
{
 return _stt.st_atime;
};

std::time_t WyFileStat::sf_ctime() const WY__TSPC()
{
 return _stt.st_ctime;
};

std::time_t WyFileStat::sf_mtime() const WY__TSPC()
{
 return _stt.st_mtime;
};

bool WyFileStat::is_dir() const WY__TSPC()
{
 return S_ISDIR(_stt.st_mode);
};

bool WyFileStat::is_chr() const WY__TSPC()
{
 return S_ISCHR(_stt.st_mode);
};

bool WyFileStat::is_blk() const WY__TSPC()
{
 return S_ISBLK(_stt.st_mode); 
};

bool WyFileStat::is_reg() const WY__TSPC()
{
 return S_ISREG(_stt.st_mode);
};

bool WyFileStat::is_fifo() const WY__TSPC()
{
 return S_ISFIFO(_stt.st_mode);
};

bool WyFileStat::is_lnk() const WY__TSPC()
{
 return S_ISLNK(_stt.st_mode);
};

bool WyFileStat::is_sock() const WY__TSPC()
{
 return S_ISSOCK(_stt.st_mode);
};

bool WyFileStat::operator ==(const WyFileStat& rhs) const WY__TSPC()
{
 return std::memcmp(&_stt,&rhs._stt,sizeof(_stt))==0;
};

bool WyFileStat::operator !=(const WyFileStat& rhs) const WY__TSPC()
{
 return std::memcmp(&_stt,&rhs._stt,sizeof(_stt))!=0;
};

struct stat& WyFileStat::wy_stat_ref() WY__NOTHROW__
{
 return _stt;
};

//----------------------

WyRet Wy::stat(const char* filename, WyFileStat& stt)
{
 if(wyc_stat(filename, &stt.wy_stat_ref())==-1) {
   WY_RETURN(errno);
 }
 return(Ok);
};

WyRet Wy::stat(const WyStr& filename, WyFileStat& stt)
{
 if(wyc_stat(filename.c_str(), &stt.wy_stat_ref())!=0) {
   WY_RETURN(errno);
 }
 return(Ok);
};

WyRet Wy::lstat(const char* filename, WyFileStat& stt)
{
 if(::lstat(filename, &stt.wy_stat_ref())!=0) {
   WY_RETURN(errno);
 }
 return(Ok);
};

WyRet Wy::lstat(const WyStr& filename, WyFileStat& stt)
{
 if(::lstat(filename.c_str(), &stt.wy_stat_ref())!=0) {
   WY_RETURN(errno);
 }
 return(Ok);
};
