/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003
*/

#ifndef WYFILESTAT_H__
#define WYFILESTAT_H__
#define WYFILESTAT_VERSION 33

#include "WyRet.h"
#include <ctime>         // for time_t
#include <sys/types.h>   // for mode_t, gid_t, uid_t,..
#include <sys/stat.h>

class WyStr;
/*
   WyFileStat is the class for "C" struct stat
*/
class WyFileStat {
  public:
    static const char class_name[];
    WY_THROW_REPLY;

    WyFileStat() WY__TSPC();
    WyFileStat(const WyFileStat &src) WY__TSPC();
    WyFileStat(WyFileStat &src, Wy::ByMove_t) WY__TSPC();

    // This member is not documented
    WyFileStat(const struct stat&) WY__TSPC();

    bool is_default() const WY__TSPC();

    void reset() WY__TSPC();
    void reset(const WyFileStat& src) WY__TSPC();

    void swap(WyFileStat& src) WY__TSPC();

    const WyFileStat& operator =(const WyFileStat &rhs) WY__TSPC();

    mode_t sf_mode() const WY__TSPC();
    ino_t sf_ino() const WY__TSPC();
    dev_t sf_dev() const WY__TSPC();
    nlink_t sf_nlink() const WY__TSPC();
    uid_t sf_uid() const WY__TSPC();
    gid_t sf_gid() const WY__TSPC();
    dev_t sf_rdev() const WY__TSPC();
    off_t sf_size() const WY__TSPC();
    blksize_t sf_blksize() const WY__TSPC();
    blkcnt_t sf_blocks() const WY__TSPC();
    std::time_t sf_atime() const WY__TSPC();
    std::time_t sf_ctime() const WY__TSPC();
    std::time_t sf_mtime() const WY__TSPC();

    bool is_dir() const WY__TSPC();
    bool is_chr() const WY__TSPC();
    bool is_blk() const WY__TSPC();
    bool is_reg() const WY__TSPC();
    bool is_fifo() const WY__TSPC();
    bool is_lnk() const WY__TSPC();
    bool is_sock() const WY__TSPC();

    bool operator ==(const WyFileStat& rhs) const WY__TSPC();
    bool operator !=(const WyFileStat& rhs) const WY__TSPC();

    // [Internal] Get the struct stat reference in *this
    //
    struct stat& wy_stat_ref() WY__NOTHROW__;

  private:
    struct stat _stt;
};

namespace Wy {
 WyRet stat(const char*,WyFileStat& stt);
 WyRet stat(const WyStr&,WyFileStat& stt);
 WyRet lstat(const char*,WyFileStat& stt);
 WyRet lstat(const WyStr&,WyFileStat& stt);
};

#endif
