/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

 v0.32: code update according to the mapage on Fedora 6
*/

#define WYLIB_SOURCE
#include "WyMutex.h"
#include <climits> // for INT_MAX

const char WyMutex::class_name[]="WyMutex";

WyMutex::WyMutex() WY__TSPC(Reply)
#ifndef NDEBUG
 : _sig(WY_NONE_ID)
#endif
{
 const int v=::pthread_mutex_init(&_mtx,NULL);
 if(v!=0) {
   WY_THROW( Reply(v) );
 };
 #ifndef NDEBUG
 _sig=WYMUTEX_ID;
 #endif
};

WyMutex::WyMutex(Normal_t) WY__TSPC(Reply)
#ifndef NDEBUG
 : _sig(WY_NONE_ID)
#endif
{
 // RAII wrapper of ::pthread_mutexattr_t to initialize mutex
 //
 struct Wy__MutexAttr {
   ::pthread_mutexattr_t _mtxattr;
   inline Wy__MutexAttr() {
     if(::pthread_mutexattr_init(&_mtxattr)!=0) {
       WY_TERMINATE(""); // man. always return 0
     }
     if(::pthread_mutexattr_settype(&_mtxattr,PTHREAD_MUTEX_NORMAL)!=0) {
       ::pthread_mutexattr_destroy(&_mtxattr);
       WY_TERMINATE(""); // man. says only EINVAL
     }
   };
   inline ~Wy__MutexAttr() { ::pthread_mutexattr_destroy(&_mtxattr); };
 } static const MutexAttr_Normal;

 const int v=::pthread_mutex_init(&_mtx,&MutexAttr_Normal._mtxattr);
 if(v!=0) {
   WY_THROW( Reply(v) );
 };

 #ifndef NDEBUG
 _sig=WYMUTEX_ID;
 #endif
};

WyMutex::WyMutex(ErrorCheck_t) WY__TSPC(Reply)
#ifndef NDEBUG
 : _sig(WY_NONE_ID)
#endif
{
 // RAII wrapper of ::pthread_mutexattr_t to initialize mutex
 //
 struct Wy__MutexAttr {
   ::pthread_mutexattr_t _mtxattr;
   inline Wy__MutexAttr() {
     if(::pthread_mutexattr_init(&_mtxattr)!=0) {
       WY_TERMINATE(""); // man. always return 0
     }
     if(::pthread_mutexattr_settype(&_mtxattr,PTHREAD_MUTEX_ERRORCHECK)!=0) {
       ::pthread_mutexattr_destroy(&_mtxattr);
       WY_TERMINATE(""); // man. says only EINVAL
     }
   };
   inline ~Wy__MutexAttr() { ::pthread_mutexattr_destroy(&_mtxattr); };
 } static const MutexAttr_ErrorCheck;

 const int v=::pthread_mutex_init(&_mtx,&MutexAttr_ErrorCheck._mtxattr);
 if(v!=0) {
   WY_THROW( Reply(v) );
 };
 #ifndef NDEBUG
 _sig=WYMUTEX_ID;
 #endif
};

WyMutex::WyMutex(Recursive_t) WY__TSPC(Reply)
#ifndef NDEBUG
 : _sig(WY_NONE_ID)
#endif
{
 // RAII wrapper of ::pthread_mutexattr_t to initialize mutex
 //
 struct Wy__MutexAttr {
   ::pthread_mutexattr_t _mtxattr;
   inline Wy__MutexAttr() {
     if(::pthread_mutexattr_init(&_mtxattr)!=0) {
       WY_TERMINATE(""); // man. always return 0
     }
     if(::pthread_mutexattr_settype(&_mtxattr,PTHREAD_MUTEX_RECURSIVE)!=0) {
       ::pthread_mutexattr_destroy(&_mtxattr);
       WY_TERMINATE(""); // man. says only EINVAL
     }
   };
   inline ~Wy__MutexAttr() { ::pthread_mutexattr_destroy(&_mtxattr); };
 } static const MutexAttr_Recursive;

 const int v=::pthread_mutex_init(&_mtx,&MutexAttr_Recursive._mtxattr);
 if(v!=0) {
   WY_THROW( Reply(v) );
 };
 #ifndef NDEBUG
 _sig=WYMUTEX_ID;
 #endif
};

WyMutex::~WyMutex() WY__TSPC()
{
 #ifndef NDEBUG
 if(_sig!=WYMUTEX_ID) {
   WY_TERMINATE("");
 }

 // Note: Further access of the object would cause stack unwind or probably SIGSEGV
 //       signal
 //
 _sig=WY_NONE_ID;
 #endif

 for(int i=0; i<INT_MAX; ++i) {  // better have a long limit than an endless loop
                                 // this assumes deadlock is never what wanted
   const int v=::pthread_mutex_destroy(&_mtx);
   if(v==0) {
     return;      // succeed
   }
   //
   // man. indicates may fail of EBUSY
   //
   if(v==EBUSY) {
     Wy__Base::yield(); // yield should be ok
     continue;
   }

   // others(EINVAL) are not expected to occur
   break;
 }
 WY_TERMINATE("");
};

WyRet WyMutex::lock() WY__TSPC()
{
 #ifndef NDEBUG
 if(_sig!=WYMUTEX_ID) {
   WY_TERMINATE("");
 }
 #endif
 const int v=::pthread_mutex_lock(&_mtx);
 if(v!=0) {
   WY_RETURN(v);
 }
 return(Ok);
};

WyRet WyMutex::trylock() WY__TSPC()
{
 #ifndef NDEBUG
 if(_sig!=WYMUTEX_ID) {
   WY_TERMINATE("");
 }
 #endif
 const int v=::pthread_mutex_trylock(&_mtx);
 if(v!=0) {
   WY_RETURN(v);
 }
 return(Ok);
};

WyRet WyMutex::unlock() WY__TSPC()
{
 #ifndef NDEBUG
 if(_sig!=WYMUTEX_ID) {
   WY_TERMINATE("");
 }
 #endif

 const int v=::pthread_mutex_unlock(&_mtx);
 if(v!=0) {
   WY_RETURN(v);
 }
 return(Ok);
};

void WyMutex::lock(WyMutex &mtx) WY__TSPC(Reply)
{
 #ifndef NDEBUG
 if(mtx._sig!=WYMUTEX_ID) {
   WY_TERMINATE("");
 }
 #endif
 const int v=::pthread_mutex_lock(&mtx._mtx);
 if(v!=0) {
   WY_THROW( Reply(v) );
 }
};

void WyMutex::trylock(WyMutex &mtx) WY__TSPC(Reply)
{
 #ifndef NDEBUG
 if(mtx._sig!=WYMUTEX_ID) {
   WY_TERMINATE("");
 }
 #endif
 const int v=::pthread_mutex_trylock(&mtx._mtx);
 if(v!=0) {
   WY_THROW( Reply(v) );
 }
};

void WyMutex::unlock(WyMutex &mtx) WY__TSPC(Reply)
{
 #ifndef NDEBUG
 if(mtx._sig!=WYMUTEX_ID) {
   WY_TERMINATE("");
 }
 #endif
 const int v=::pthread_mutex_unlock(&mtx._mtx);
 if(v!=0) {
   WY_THROW( Reply(v) );
 }
};

//------------------------------------------------
const char WyLock::class_name[]="WyLock";

WyLock::WyLock(WyMutex &mtx) WY__TSPC(Reply)
 :_pmtx(NULL)
{
 WyRet r( mtx.lock() );
 if(r!=Ok) {
   WY_THROW( Reply(r) );
 }
 _pmtx=&mtx;
};

WyLock::~WyLock() WY__TSPC()
{
#ifndef NDEBUG
 if(_pmtx==NULL) {
   WY_TERMINATE("");
 }
#endif
 if(_pmtx->unlock()!=Ok) {

   // none of the manpage listed EPERM/EINVAL should occur
   //
   WY_TERMINATE("");
 }
#ifndef NDEBUG
 _pmtx=NULL; 
#endif
};

WyRet WyLock::xlock(WyMutex& mtx)
{
#ifndef NDEBUG
 if(_pmtx==NULL) {
   WY_THROW( WyRet(Wym_EFAULT) );
 }
#endif
 WyRet r( mtx.lock() );
 if(r!=Ok) {
   WY_RETURN(r);
 }
 if((r=_pmtx->unlock())!=Ok) {

   // Linux manpage listed errno's EINVAL/EPERM indicate, they are 'internal'
   // errors for this function.
   //
   mtx.unlock();
   WY_THROW(r);
 }
 _pmtx=&mtx;
 return(Ok);
};
