/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   WyMutex provides serializing access of codes between threads.
*/
#ifndef WYMUTEX_H__
#define WYMUTEX_H__
#define WYMUTEX_VERSION 33

#include "WyRet.h"
#include <pthread.h>

#ifndef NDEBUG
#include <signal.h>   // for sig_atomic_t
#endif

class WyMutex {

    // Theses struct's are for ctor signature, inaccessible to app.
    //
    struct Normal_t {};
    struct ErrorCheck_t {};
    struct Recursive_t {};
  public:
    static const Normal_t Normal;
    static const ErrorCheck_t ErrorCheck;
    static const Recursive_t Recursive;

    static const char class_name[];
    WY_THROW_REPLY;  // declaraion of Reply

    WyMutex() WY__TSPC(Reply);
    explicit WyMutex(Normal_t) WY__TSPC(Reply);
    explicit WyMutex(ErrorCheck_t) WY__TSPC(Reply);
    explicit WyMutex(Recursive_t) WY__TSPC(Reply);

    ~WyMutex() WY__TSPC();

    WyRet lock() WY__TSPC();

    WyRet trylock() WY__TSPC();

    WyRet unlock() WY__TSPC();

    static void lock(WyMutex &mtx) WY__TSPC(Reply);
    static void trylock(WyMutex &mtx) WY__TSPC(Reply);
    static void unlock(WyMutex &mtx) WY__TSPC(Reply);

  private:
   #ifndef NDEBUG
    volatile sig_atomic_t _sig; // For checking object is valid or not
   #endif
    ::pthread_mutex_t _mtx; 

    friend class WyCond;    // WyCond needs to access _mtx directly

    // Hidden members
    WyMutex(const WyMutex&);
    const WyMutex& operator =(const WyMutex&);
    bool operator==(const WyMutex&) const;
};

class WyLock {
  public:
    static const char class_name[];
    WY_THROW_REPLY;  // declaraion of Reply

    explicit
    WyLock(WyMutex& mtx) WY__TSPC(Reply);

    ~WyLock() WY__TSPC();

    WyRet xlock(WyMutex& mtx);

  private:
    WyMutex *_pmtx;
    friend class WyCond;      // WyCond needs to access _pmtx directly

    // Hidden members
    WyLock();
    WyLock(const WyLock&);
    const WyLock &operator=(const WyLock&);
    bool operator==(const WyLock&) const;
    void* operator new(size_t);
};

#endif
