/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2004

   Basci conversions between ascii-string and integral type
*/
#ifndef WYNUM_H__
#define WYNUM_H__
#define WYNUM_VERSION 33

#include "WyCSeg.h"
#include <limits>

namespace Wy {

 // Radix range of conversion (string to num or vise versa)
 const Wy::WyRadix Wy_MinRadix=2;
 const Wy::WyRadix Wy_MaxRadix=36;

 inline int _charnum(char ch) WY__TSPC()
 {
  if(ch<='9') {
    return(ch-'0');
  } else if(ch>='a') {
    return(ch-('a'-(char)10));
  } else if(ch>='A') {
    return(ch-('A'-(char)10));
  } else {
    return -1;
  }
 };

// SRC: wy_strnum_char.cpp
WyRet _strnum(char& , const WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(char& , const WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(char& , const WyCSeg&) WY__TSPC();
WyRet _strnum(char& , WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(char& , WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(char& , WyCSeg&) WY__TSPC();

WyRet _strnum(signed char& , const WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed char& , const WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed char& , const WyCSeg&) WY__TSPC();
WyRet _strnum(signed char& , WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed char& , WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed char& , WyCSeg&) WY__TSPC();

WyRet _strnum(unsigned char& , const WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned char& , const WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned char& , const WyCSeg&) WY__TSPC();
WyRet _strnum(unsigned char& , WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned char& , WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned char& , WyCSeg&) WY__TSPC();

// SRC: wy_strnum_short.cpp
WyRet _strnum(signed short int& , const WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed short int& , const WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed short int& , const WyCSeg&) WY__TSPC();
WyRet _strnum(signed short int& , WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed short int& , WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed short int& , WyCSeg&) WY__TSPC();

WyRet _strnum(unsigned short int& , const WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned short int& , const WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned short int& , const WyCSeg&) WY__TSPC();
WyRet _strnum(unsigned short int& , WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned short int& , WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned short int& , WyCSeg&) WY__TSPC();

// SRC: wy_strnum_int.cpp
WyRet _strnum(signed int& , const WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed int& , const WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed int& , const WyCSeg&) WY__TSPC();
WyRet _strnum(signed int& , WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed int& , WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed int& , WyCSeg&) WY__TSPC();

WyRet _strnum(unsigned int& , const WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned int& , const WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned int& , const WyCSeg&) WY__TSPC();
WyRet _strnum(unsigned int& , WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned int& , WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned int& , WyCSeg&) WY__TSPC();

// SRC: wy_strnum_long.cpp
WyRet _strnum(signed long int& , const WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed long int& , const WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed long int& , const WyCSeg&) WY__TSPC();
WyRet _strnum(signed long int& , WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed long int& , WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed long int& , WyCSeg&) WY__TSPC();

WyRet _strnum(unsigned long int& , const WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned long int& , const WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned long int& , const WyCSeg&) WY__TSPC();
WyRet _strnum(unsigned long int& , WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned long int& , WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned long int& , WyCSeg&) WY__TSPC();

// SRC: wy_strnum_llong.cpp 
WyRet _strnum(signed long long int& , const WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed long long int& , const WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed long long int& , const WyCSeg&) WY__TSPC();
WyRet _strnum(signed long long int& , WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed long long int& , WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(signed long long int& , WyCSeg&) WY__TSPC();

WyRet _strnum(unsigned long long int& , const WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned long long int& , const WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned long long int& , const WyCSeg&) WY__TSPC();
WyRet _strnum(unsigned long long int& , WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned long long int& , WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(unsigned long long int& , WyCSeg&) WY__TSPC();

// SRC: wy_strnum_float.cpp
WyRet _strnum(float& , const WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(float& , const WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(float& , const WyCSeg&) WY__TSPC();

WyRet _strnum(float& , WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(float& , WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(float& , WyCSeg&) WY__TSPC();

// SRC: wy_strnum_double.cpp
WyRet _strnum(double& , const WyCSeg&, Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(double& , const WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(double& , const WyCSeg&) WY__TSPC();

WyRet _strnum(double& , WyCSeg&, Wy::WyRadix& ) WY__TSPC(); 
WyRet _strnum(double& , WyCSeg&, const Wy::WyRadix& ) WY__TSPC();
WyRet _strnum(double& , WyCSeg&) WY__TSPC();

// [Syn] Range checked multiplication
//       The result of n1*n2 is stored into res
//
// [Ret] Ok
//       Wym_ERANGE
//
template <typename NumT>
WyRet wy_mul(NumT& res, NumT n1, NumT n2)
{
 if(std::numeric_limits<NumT>::is_exact) {
   const NumT tmp(n1*n2);
   if(n1!=0) {
     if((tmp/n1)!=n2) {
       WY_RETURN(Wym_ERANGE);
     }
   } else if(n2!=0) {
     if((tmp/n2)!=n1) {
       WY_RETURN(Wym_ERANGE);
     }
   } else {};
   res=tmp;
 } else {
   WY_TERMINATE("");   // in case
 }
 return(Ok);
};

// [Syn] Range checked addition
//       The result of n1+n2 is stored into res
//
// [Ret] Ok
//       Wym_ERANGE
//
template <typename NumT>
WyRet wy_add(NumT& res, NumT n1, NumT n2)
{
 if(std::numeric_limits<NumT>::is_signed) {
   const NumT tmp(n1+n2);
   if(n1<=NumT(0)) {
     if(tmp>n2) {
       WY_RETURN(Wym_ERANGE);
     }
   } else {
     if(tmp<=n2) {
       WY_RETURN(Wym_ERANGE);
     }
   }
   res=tmp;
 } else {
   const NumT tmp(n1+n2);
   if(tmp<n2) {
     WY_RETURN(Wym_ERANGE);
   }
   res=tmp;
 }
 return(Ok);
};

// SRC: wy_scanum_char.cpp
WyRet _scanum(char&, WyCSeg&, const Wy::WyRadix&) WY__TSPC();
WyRet _scanum(signed char&, WyCSeg&, const Wy::WyRadix&) WY__TSPC();
WyRet _scanum(unsigned char&, WyCSeg&, const Wy::WyRadix&) WY__TSPC();
WyRet _scanum(char&, WyCSeg&, Wy::WyRadix&) WY__TSPC();
WyRet _scanum(signed char&, WyCSeg&, Wy::WyRadix&) WY__TSPC();
WyRet _scanum(unsigned char&, WyCSeg&, Wy::WyRadix&) WY__TSPC();
WyRet _scanum(char&, WyCSeg&) WY__TSPC();
WyRet _scanum(signed char&, WyCSeg&) WY__TSPC();
WyRet _scanum(unsigned char&, WyCSeg&) WY__TSPC();

 // SRC: wy_scanum_short.cpp
WyRet _scanum(signed short int&, WyCSeg&, const Wy::WyRadix&) WY__TSPC();
WyRet _scanum(unsigned short int&, WyCSeg&, const Wy::WyRadix&) WY__TSPC();
WyRet _scanum(signed short int&, WyCSeg&, Wy::WyRadix&) WY__TSPC();
WyRet _scanum(unsigned short int&, WyCSeg&, Wy::WyRadix&) WY__TSPC();
WyRet _scanum(signed short int&, WyCSeg&) WY__TSPC();
WyRet _scanum(unsigned short int&, WyCSeg&) WY__TSPC();

// SRC: wy_scanum_int.cpp
WyRet _scanum(signed int&, WyCSeg&, const Wy::WyRadix&) WY__TSPC();
WyRet _scanum(unsigned int&, WyCSeg&, const Wy::WyRadix&) WY__TSPC();
WyRet _scanum(signed int&, WyCSeg&, Wy::WyRadix&) WY__TSPC();
WyRet _scanum(unsigned int&, WyCSeg&, Wy::WyRadix&) WY__TSPC();
WyRet _scanum(signed int&, WyCSeg&) WY__TSPC();
WyRet _scanum(unsigned int&, WyCSeg&) WY__TSPC();

// SRC: wy_scanum_long.cpp
WyRet _scanum(signed long int&, WyCSeg&, const Wy::WyRadix&) WY__TSPC();
WyRet _scanum(unsigned long int&, WyCSeg&, const Wy::WyRadix&) WY__TSPC();
WyRet _scanum(signed long int&, WyCSeg&, Wy::WyRadix&) WY__TSPC();
WyRet _scanum(unsigned long int&, WyCSeg&, Wy::WyRadix&) WY__TSPC();
WyRet _scanum(signed long int&, WyCSeg&) WY__TSPC();
WyRet _scanum(unsigned long int&, WyCSeg&) WY__TSPC();

// SRC: wy_scanum_llong.cpp
WyRet _scanum(unsigned long long int&, WyCSeg&, const Wy::WyRadix&) WY__TSPC();
WyRet _scanum(signed long long int&, WyCSeg&, const Wy::WyRadix&) WY__TSPC();
WyRet _scanum(unsigned long long int&, WyCSeg&, Wy::WyRadix&) WY__TSPC();
WyRet _scanum(signed long long int&, WyCSeg&, Wy::WyRadix&) WY__TSPC();
WyRet _scanum(signed long long int&, WyCSeg&) WY__TSPC();
WyRet _scanum(unsigned long long int&, WyCSeg&) WY__TSPC();

WyRet _scanum(float&, WyCSeg&, const Wy::WyRadix&) WY__TSPC();  // SRC: wy_scanum_float.cpp
WyRet _scanum(float&, WyCSeg&, Wy::WyRadix&) WY__TSPC();
WyRet _scanum(float&, WyCSeg&) WY__TSPC();
WyRet _scanum(double&, WyCSeg&, const Wy::WyRadix&) WY__TSPC(); // SRC: wy_scanum_double.cpp
WyRet _scanum(double&, WyCSeg&, Wy::WyRadix&) WY__TSPC();
WyRet _scanum(double&, WyCSeg&) WY__TSPC();

unsigned char _sqrt(unsigned char n) WY__TSPC();  // SRC: wy_sqrt_int.cpp
unsigned short _sqrt(unsigned short n) WY__TSPC();
unsigned int _sqrt(unsigned int n) WY__TSPC();
unsigned long _sqrt(unsigned long n) WY__TSPC();
unsigned long long _sqrt(unsigned long long n) WY__TSPC();

size_t _ntoa(char* buf, size_t blen, char num, Wy::WyRadix radix) WY__TSPC();      // SRC: wy_ntoa_char.cpp
size_t _ntoa_r(char* buf, size_t blen, char num, Wy::WyRadix radix) WY__TSPC();
size_t _ntoa(char* buf, size_t blen, signed char num, Wy::WyRadix radix) WY__TSPC();
size_t _ntoa_r(char* buf, size_t blen, signed char num, Wy::WyRadix radix) WY__TSPC();
size_t _ntoa(char* buf, size_t blen, unsigned char num, Wy::WyRadix radix) WY__TSPC();
size_t _ntoa_r(char* buf, size_t blen, unsigned char num, Wy::WyRadix radix) WY__TSPC();

size_t _ntoa(char* buf, size_t blen, signed short num, Wy::WyRadix radix) WY__TSPC(); // SRC: wy_ntoa_short.cpp
size_t _ntoa_r(char* buf, size_t blen, signed short num, Wy::WyRadix radix) WY__TSPC();
size_t _ntoa(char* buf, size_t blen, unsigned short num, Wy::WyRadix radix) WY__TSPC();
size_t _ntoa_r(char* buf, size_t blen, unsigned short num, Wy::WyRadix radix) WY__TSPC();

size_t _ntoa(char* buf, size_t blen, signed int num, Wy::WyRadix radix) WY__TSPC(); // SRC: wy_ntoa_int.cpp
size_t _ntoa_r(char* buf, size_t blen, signed int num, Wy::WyRadix radix) WY__TSPC();
size_t _ntoa(char* buf, size_t blen, unsigned int num, Wy::WyRadix radix) WY__TSPC();
size_t _ntoa_r(char* buf, size_t blen, unsigned int num, Wy::WyRadix radix) WY__TSPC();

size_t _ntoa(char* buf, size_t blen, signed long num, Wy::WyRadix radix) WY__TSPC(); // SRC: wy_ntoa_long.cpp
size_t _ntoa_r(char* buf, size_t blen, signed long num, Wy::WyRadix radix) WY__TSPC();
size_t _ntoa(char* buf, size_t blen, unsigned long num, Wy::WyRadix radix) WY__TSPC();
size_t _ntoa_r(char* buf, size_t blen, unsigned long num, Wy::WyRadix radix) WY__TSPC();

size_t _ntoa(char* buf, size_t blen, signed long long num, Wy::WyRadix radix) WY__TSPC();// SRC: wy_ntoa_llong.cpp
size_t _ntoa_r(char* buf, size_t blen, signed long long num, Wy::WyRadix radix) WY__TSPC();
size_t _ntoa(char* buf, size_t blen, unsigned long long num, Wy::WyRadix radix) WY__TSPC();
size_t _ntoa_r(char* buf, size_t blen, unsigned long long num, Wy::WyRadix radix) WY__TSPC();

size_t _ntoa(char*, size_t, float, Wy::WyRadix, size_t) WY__TSPC(); //SRC: wy_ntoa_double.cpp
size_t _ntoa_r(char*, size_t, float, Wy::WyRadix, size_t) WY__TSPC();

size_t _ntoa(char*, size_t, double, Wy::WyRadix, size_t) WY__TSPC(); //SRC: wy_ntoa_double.cpp
size_t _ntoa_r(char*, size_t, double, Wy::WyRadix, size_t) WY__TSPC();

// [Internal] For implement, see wy__itoa_lr(..) in wy__itoa.hh
//
size_t wy_itoa_l(char* buf, size_t blen, char num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_l(char* buf, size_t blen, signed char num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_l(char* buf, size_t blen, unsigned char num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_l(char* buf, size_t blen, signed short num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_l(char* buf, size_t blen, unsigned short num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_l(char* buf, size_t blen, signed int num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_l(char* buf, size_t blen, unsigned int num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_l(char* buf, size_t blen, signed long num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_l(char* buf, size_t blen, unsigned long num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_l(char* buf, size_t blen, signed long long num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_l(char* buf, size_t blen, unsigned long long num, Wy::WyRadix radix) WY__TSPC();

size_t wy_itoa_r(char* buf, size_t blen, char num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_r(char* buf, size_t blen, signed char num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_r(char* buf, size_t blen, unsigned char num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_r(char* buf, size_t blen, signed short num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_r(char* buf, size_t blen, unsigned short num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_r(char* buf, size_t blen, signed int num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_r(char* buf, size_t blen, unsigned int num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_r(char* buf, size_t blen, signed long num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_r(char* buf, size_t blen, unsigned long num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_r(char* buf, size_t blen, signed long long num, Wy::WyRadix radix) WY__TSPC();
size_t wy_itoa_r(char* buf, size_t blen, unsigned long long num, Wy::WyRadix radix) WY__TSPC();

// [Internal] Get the number of digits required for radix Base representation
//            (sign is ignored, 0 is one digit)
//
template <typename NumType>
size_t wy_ndigits(NumType n, Wy::WyRadix radix)
{
 WY_ASSERT(std::numeric_limits<NumType>::is_integer);
 WY_ASSERT(radix>=2);
 size_t rv=0;
 do { ++rv; } while(n/=static_cast<NumType>(radix)); // cast to NumType, othersise
                                                     // strange result (g++ 4.1.1)
 return(rv);
};

 inline double to_double(double v) WY__TSPC()
 { return v; };

};  // end namespace Wy

#endif  // end WYNUM_H__
