/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   Header only

*/

#ifndef WYONCE_H__
#define WYONCE_H__
#define WYONCE_VERSION 33

#include "WyRet.h"     // include libwy base definitions
#include <pthread.h>

/*
  WyOnce defines an object to call a given function once.
  WyOnce.h is header only.
*/
class WyOnce {
    static const ::pthread_once_t DefaultValue=PTHREAD_ONCE_INIT;
    ::pthread_once_t _ovar;

    WyOnce(const WyOnce&) WY__TSPC(); // not to use
    const WyOnce& operator=(const WyOnce&) WY__TSPC(); // not to use
    bool operator==(const WyOnce&) WY__TSPC(); // not to use

  public:
    
    // [Syn] Construct default object
    //
    inline WyOnce() WY__TSPC()
	  :_ovar(DefaultValue) {};
    
    // [Syn] Construct object and call the given function argument immedicately.
    //
    // [Throw]
    //
    explicit  // no implicit conversion
    inline WyOnce( void(*once_func)() )
	      :_ovar(DefaultValue)
              { ::pthread_once(&_ovar, once_func); };

    // [Syn] Is object the same as that of default constructed state.
    //
    // [Ret] true = object is the same as default constructed state.
    //       false= otherwise
    //
    inline bool is_default() const WY__TSPC()
              { return _ovar==DefaultValue; };

    // [Syn] Call the parameter function at most once.
    //       This function moves object away from the default state
    //
    // [Throw]
    //
    inline void once( void(*once_func)() )
              { ::pthread_once(&_ovar, once_func); };
};

#endif
