/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2007

  This file is header only.

*/
#ifndef WYPTR_HH__
#define WYPTR_HH__
#define WYPTR_VERSION 33

#include "wyreqs.h"
#include "wydefs.h"

// [Internal] Template WyPtr wraps pointers (providing common libwy members and is the
//            base of class template Wy_MFP?)
//
//       WyPtr is the template of which the parameter type T can be NULL by default
//
template <typename T>
class WyPtr {
  public:
    typedef T pointer_type;
    WyPtr() WY__NOTHROW__ : _fp(NULL) {};
    WyPtr(const WyPtr& src) WY__NOTHROW__ : _fp(src._fp) {};
    WyPtr(T mfptr) WY__NOTHROW__ : _fp(mfptr) {};
    WyPtr(WyPtr& src, Wy::ByMove_t) WY__NOTHROW__ : _fp(src._fp) {};

    operator T() const WY__NOTHROW__ { return _fp; };

    bool is_default() const WY__NOTHROW__ { return _fp==NULL; };
    void reset() WY__NOTHROW__ { _fp=NULL; };
    void reset(const WyPtr& rhs) WY__NOTHROW__
       { this->_fp=rhs._fp; };
    void reset(const T& rhs) WY__NOTHROW__
       { this->_fp=rhs; };

    const WyPtr& operator=(const WyPtr& rhs) WY__NOTHROW__
       { this->_fp=rhs._fp; return *this; };
    const WyPtr& operator=(const T& rhs) WY__NOTHROW__
       { this->_fp=rhs; return *this; };

    bool operator==(const WyPtr& rhs) const WY__NOTHROW__ 
       { return this->_fp==rhs._fp; };
    bool operator==(const T& rhs) const WY__NOTHROW__ 
       { return this->_fp==rhs; };
    bool operator!=(const WyPtr& rhs) const WY__NOTHROW__ 
       { return this->_fp!=rhs._fp; };
    bool operator!=(const T& rhs) const WY__NOTHROW__ 
       { return this->_fp!=rhs; };

    friend bool operator==(const T& lhs,const WyPtr<T>& rhs) WY__NOTHROW__
       { return lhs==rhs._fp; };
    friend bool operator!=(const T& lhs,const WyPtr<T>& rhs) WY__NOTHROW__
       { return lhs!=rhs._fp; };

  private:
    T _fp;
};

#endif  // end WYPTR_HH__
