/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

  WySockAddrLocal deals with the local address (AF_LOCAL)
*/

#ifndef WYSOCKADDRLOCAL_H__
#define WYSOCKADDRLOCAL_H__
#define WYSOCKADDRLOCAL_VERSION 33

#include "WyRet.h"
#include "Wy_SockAddr.h"
#include "WyStr.h"
#include <sys/un.h>

class WySockAddrLocal : public Wy_SockAddr {
  public:
    static const size_t MaxAddrLen= sizeof(((struct ::sockaddr_un*)0)->sun_path)-1;
    static const char class_name[];

    WY_THROW_REPLY;
    static int family() WY__TSPC()
              { return AF_LOCAL; };

    WySockAddrLocal() WY__TSPC();
    WySockAddrLocal(const WySockAddrLocal &src) WY__TSPC();

    explicit  // no implicit conversion
    WySockAddrLocal(const char* path);

    explicit  // no implicit conversion
    WySockAddrLocal(const WyStr& path) WY__TSPC(WyRet);

    explicit  // no implicit conversion
    WySockAddrLocal(const WyCSeg& path) WY__TSPC(WyRet);

    // [Move Constructor]
    //
    WySockAddrLocal(WySockAddrLocal &src, Wy::ByMove_t) WY__TSPC();

    bool is_default() const WY__TSPC();

    // [Rationale] Because the address can begin with char(0) and the last can
    //             also be char(0) (getsockname/getpeername), the interface let
    //             user interpret what is given in sun_path. However, the address
    //             addrname() returned ends with an uncounted char(0).
    //
    const char* addrname() const WY__TSPC();
    size_t addrsize() const WY__TSPC();

    // [Override]
    virtual WyRet notation(WyStr& str) const;

    // [Override]
    virtual void reset() WY__TSPC();

    // [Added]
    virtual void reset(const WySockAddrLocal& src) WY__TSPC();
    virtual WyRet reset(const char* path);
    virtual WyRet reset(const WyStr& path) WY__TSPC();
    virtual WyRet reset(const WyCSeg& path) WY__TSPC();

    // [Override]
    virtual WyRet _swap(Wy_SockAddr& laddr) WY__TSPC();
    virtual WySockAddrLocal* _alloc(WyRet&) const;

    // [Added]
    virtual const WySockAddrLocal& operator=(const WySockAddrLocal& rhs) WY__TSPC();
    virtual const WySockAddrLocal& operator=(const char* rhs);

    virtual const WySockAddrLocal& operator=(const WyStr& rhs) WY__TSPC(WyRet);

    virtual const WySockAddrLocal& operator=(const WyCSeg& rhs) WY__TSPC(WyRet);

    bool operator==(const WySockAddrLocal &rhs) const WY__TSPC();
    bool operator!=(const WySockAddrLocal &rhs) const WY__TSPC();
    bool operator==(const char* rhs) const;
    bool operator!=(const char* rhs) const;
    bool operator==(const WyStr& rhs) const WY__TSPC(WyRet);
    bool operator!=(const WyStr& rhs) const WY__TSPC(WyRet);
    bool operator==(const WyCSeg& rhs) const WY__TSPC(WyRet);
    bool operator!=(const WyCSeg& rhs) const WY__TSPC(WyRet);
  private:
    // [Override]
    virtual struct sockaddr& c_sockaddr() WY__TSPC()
              { return (struct sockaddr&)(_saddr); };
    virtual const struct sockaddr& c_sockaddr() const WY__TSPC()
              { return (const struct sockaddr&)(_saddr); };
    virtual size_t c_sockaddr_size() const WY__TSPC()
              { return sizeof(_saddr); };
    virtual  socklen_t& c_addrlen() WY__TSPC()
              { return _slen; };
    virtual const socklen_t& c_addrlen() const WY__TSPC()
              { return _slen; };

    // [Internal] Initialize object to contain the arguments indicated
    //            address (zero terminated string)
    //
    // [Ret] Ok
    //       Wym_EFAULT       null pointer
    //       Wym_ENAMETOOLONG path too long
    //
    WyRet _init_sockaddr(const char* pdata);

    // [Internal] Initialize object to contain the arguments indicated
    //            address
    //
    // [Ret] Ok
    //       Wym_EFAULT       null pointer
    //       Wym_ENAMETOOLONG path too long
    //
    WyRet _init_sockaddr(const char* pdata, size_t plen) WY__NOTHROW__;

  private:
    class Wy__Assert_Check__;

    socklen_t _slen;         // effective length of _saddr (>=SockLenNoData)
    struct ::sockaddr_un _saddr;
    char   _padding;         // might be overwritten
};

inline bool operator==(const char* lhs,const WySockAddrLocal& rhs)
              { return rhs.operator==(lhs); };
inline bool operator!=(const char* lhs,const WySockAddrLocal& rhs)
              { return rhs.operator!=(lhs); };

inline bool operator==(const WyStr& lhs,const WySockAddrLocal& rhs) WY__TSPC(WyRet)
              { return rhs.operator==(lhs); };
inline bool operator!=(const WyStr& lhs,const WySockAddrLocal& rhs) WY__TSPC(WyRet)
              { return rhs.operator!=(lhs); };
inline bool operator==(const WyCSeg& lhs,const WySockAddrLocal& rhs) WY__TSPC(WyRet)
              { return rhs.operator==(lhs); };
inline bool operator!=(const WyCSeg& lhs,const WySockAddrLocal& rhs) WY__TSPC(WyRet)
              { return rhs.operator!=(lhs); };

namespace Wy {
 WyStr wrd(const WySockAddrLocal& addr);
};

#endif
