/* Copyright is licensed by GNU LGPL.                         by I.J.Wang 2004
*/
#define WYLIB_SOURCE
#include "WyStr.h"
#include <cstdlib>
#include <climits> // for SSIZE_MAX

const WyStr::ElemType WyStr::class_name[]="WyStr";

// [Syn] Get the memory size enough for storing dlen elements
//
// Note: Removed 'inline' because other modules can't see this
//
const WyStr::SizeType WyStr::_mem_dlen(SizeType dlen) WY__NOTHROW__
{ return ((~BufGranBitsMask)&(dlen+sizeof(EOS))) +DefaultBufSize; };

// [Syn] Get the capacity that the memory block of size msize can be
//
// Note: msize must be >= 1
//
inline const WyStr::SizeType WyStr::_cap_mem(SizeType msize) WY__NOTHROW__
{ return msize-sizeof(EOS); };

// [Syn] _cap_mem( _mem_dlen(dlen) )
//
const WyStr::SizeType WyStr::_cap_dlen(SizeType dlen) WY__NOTHROW__
{ return ((~BufGranBitsMask)&(dlen+sizeof(EOS))) +DefaultBufSize -sizeof(EOS); };

// [Syn] Allocate nbytes of memory
//
WyStr::ElemType* WyStr::_mem_alloc(SizeType nbytes) WY__NOTHROW__
{ return reinterpret_cast<ElemType*>( std::malloc(nbytes) ); };

// [Syn] Free the memory block allocated by _mem_alloc(SizeType)
//
// Note: Removed 'inline' because other modules can't see this
//
void WyStr::_mem_free(void* p) WY__NOTHROW__
{ std::free(p); };

//--------------------------------------------------------------------

WyStr::SizeType WyStr::max_capacity() WY__TSPC()
{ return SSIZE_MAX/2-1; };

WyStr::SizeType WyStr::min_capacity() WY__TSPC()
{
 static const SizeType v=_cap_dlen(0);
 return v;
};

WyStr::WyStr() WY__TSPC(Reply)
 : _m_dcap(_cap_dlen(0)),_m_dlen(0),_m_buffer( _mem_alloc(_mem_dlen(0)) )
{
 if(_m_buffer==NULL) {
   WY_THROW( Reply(Wym_ENOMEM) );
 }
 _m_buffer[0]=EOS;
};

WyStr::WyStr(const WyStr& src) WY__TSPC(Reply)
 : _m_dcap(_cap_dlen(src._m_dlen)),_m_dlen(src._m_dlen),
   _m_buffer( _mem_alloc( _mem_dlen(src._m_dlen)) )
{
 if(this->_m_buffer==NULL) {
   _mem_free(this->_m_buffer);
   WY_THROW( Reply(Wym_ENOMEM) );
 }
 std::memcpy(this->_m_buffer,src._m_buffer,_m_dlen);
 this->_m_buffer[this->_m_dlen]=EOS;
};

WyStr::WyStr(const WyCSeg& src) WY__TSPC(Reply)
{
 const SizeType csize( src.size() );
 if(csize>WyStr::max_capacity()) {
   WY_THROW( Reply(Wym_EFBIG) );
 }
 this->_m_dcap= _cap_dlen(csize);
 this->_m_dlen=csize;
 this->_m_buffer= _mem_alloc( _mem_dlen(csize) );
 if(_m_buffer==NULL) {
   WY_THROW( Reply(Wym_ENOMEM) );
 }
 std::memcpy(this->_m_buffer,src.begin(),csize);
 this->_m_buffer[csize]=EOS;
};

WyStr::WyStr(SizeType cnum, ElemType ch) WY__TSPC(Reply)
{
 if(cnum>WyStr::max_capacity()) {
   WY_THROW( Reply(Wym_EFBIG) );
 }
 _m_buffer= _mem_alloc( _mem_dlen(cnum) );
 if(this->_m_buffer==NULL) {
   WY_THROW( Reply(Wym_ENOMEM) );
 }
 this->_m_dcap= _cap_dlen(cnum);
 this->_m_dlen= cnum;
 std::memset(this->_m_buffer,ch,cnum);
 this->_m_buffer[cnum]=EOS;
};

WyStr::WyStr(WyStr& str, Wy::ByMove_t) WY__TSPC()
 : _m_dcap(str._m_dcap),_m_dlen(str._m_dlen),_m_buffer(str._m_buffer)
{
 str._m_buffer=NULL;   // for safety and debug.
};

WyStr::WyStr(const WyStr& s1, const WyStr& s2) WY__TSPC(Reply)
{
 const SizeType tlen=s1._m_dlen+s2._m_dlen;
 if(tlen>WyStr::max_capacity()) {
   WY_THROW( Reply(Wym_EFBIG) );
 }
 this->_m_buffer= _mem_alloc( _mem_dlen(tlen) );
 if(this->_m_buffer==NULL) {
   WY_THROW( Reply(Wym_ENOMEM) );
 }
 std::memcpy(this->_m_buffer,s1._m_buffer,s1._m_dlen);
 std::memcpy(this->_m_buffer+s1._m_dlen,s2._m_buffer,s2._m_dlen);
 this->_m_buffer[tlen]=EOS;
 this->_m_dcap=_cap_dlen(tlen);
 this->_m_dlen=tlen;
};

WyStr::WyStr(const WyStr& s1, const WyCSeg& s2) WY__TSPC(Reply)
{
 const SizeType tlen=s1._m_dlen+s2.size();
 if(tlen>WyStr::max_capacity()) {
   WY_THROW( Reply(Wym_EFBIG) );
 }
 this->_m_buffer= _mem_alloc( _mem_dlen(tlen) );
 if(this->_m_buffer==NULL) {
   WY_THROW( Reply(Wym_ENOMEM) );
 }
 std::memcpy(this->_m_buffer,s1._m_buffer,s1._m_dlen);
 std::memcpy(this->_m_buffer+s1._m_dlen,s2.begin(),s2.size());
 this->_m_buffer[tlen]=EOS;
 this->_m_dcap=_cap_dlen(tlen);
 this->_m_dlen=tlen;
};

WyStr::WyStr(const WyCSeg& s1, const WyStr& s2) WY__TSPC(Reply)
{
 const SizeType tlen=s1.size()+s2._m_dlen;
 if(tlen>WyStr::max_capacity()) {
   WY_THROW( Reply(Wym_EFBIG) );
 }
 this->_m_buffer= _mem_alloc( _mem_dlen(tlen) );
 if(this->_m_buffer==NULL) {
   WY_THROW( Reply(Wym_ENOMEM) );
 }
 std::memcpy(this->_m_buffer,s1.begin(),s1.size());
 std::memcpy(this->_m_buffer+s1.size(),s2._m_buffer,s2._m_dlen);
 this->_m_buffer[tlen]=EOS;
 this->_m_dcap=_cap_dlen(tlen);
 this->_m_dlen=tlen;
};

WyStr::WyStr(const WyStr& s1, ElemType ch) WY__TSPC(Reply)
{
 const SizeType tlen=s1._m_dlen+sizeof(ch);
 if(tlen>WyStr::max_capacity()) {
   WY_THROW( Reply(Wym_EFBIG) );
 }
 this->_m_buffer= _mem_alloc( _mem_dlen(tlen) );
 if(this->_m_buffer==NULL) {
   WY_THROW( Reply(Wym_ENOMEM) );
 }
 std::memcpy(this->_m_buffer,s1._m_buffer,s1._m_dlen);
 this->_m_buffer[s1._m_dlen]=ch;
 this->_m_buffer[tlen]=EOS;
 this->_m_dcap=_cap_dlen(tlen);
 this->_m_dlen=tlen;
};

WyStr::WyStr(ElemType ch, const WyStr& s1) WY__TSPC(Reply)
{
 const SizeType tlen=s1._m_dlen+sizeof(ch);
 if(tlen>WyStr::max_capacity()) {
   WY_THROW( Reply(Wym_EFBIG) );
 }
 this->_m_buffer= _mem_alloc( _mem_dlen(tlen) );
 if(this->_m_buffer==NULL) {
   WY_THROW( Reply(Wym_ENOMEM) );
 }
 this->_m_buffer[0]=ch;
 std::memcpy(this->_m_buffer+sizeof(ch),s1._m_buffer,s1._m_dlen);
 this->_m_buffer[tlen]=EOS;
 this->_m_dcap=_cap_dlen(tlen);
 this->_m_dlen=tlen;
};

WyStr::~WyStr() WY__TSPC()
{
 _mem_free(this->_m_buffer);
 this->_m_buffer=NULL;
};

bool WyStr::is_default() const WY__TSPC()
{
 return this->_m_dlen==0;
};

void WyStr::reset() WY__TSPC()
{
 this->_m_dlen=0;
 this->_m_buffer[0]=EOS;
};

WyRet WyStr::reset(const WyStr& src) WY__TSPC()
{
 if(src._m_buffer==this->_m_buffer) {
   return(Ok);        // self assignment
 }

 const SizeType csize= src._m_dlen;
 if(this->_m_dcap>=csize) {
   std::memcpy(this->_m_buffer, src._m_buffer, csize);
   this->_m_dlen=csize;
   this->_m_buffer[csize]=EOS;
   return(Ok);
 }

 ElemType* dptr= _mem_alloc( _mem_dlen(csize) );
 if(dptr==NULL) {
   WY_RETURN( Wym_ENOMEM );
 }
 _mem_free(this->_m_buffer);
 this->_m_buffer=dptr;
 std::memcpy(dptr,src._m_buffer,csize);
 this->_m_dlen=csize;
 this->_m_dcap=_cap_dlen(csize);
 this->_m_buffer[csize]=EOS;
 return(Ok);
};

WyRet WyStr::reset(const WyCSeg& src) WY__TSPC()
{
 if(src.is_overlap(this->cseg())) {
   WY_RETURN( Wym_ELOOP );
 }

 const SizeType csize= src.size();
 if(this->_m_dcap>=csize) {
   std::memcpy(this->_m_buffer, src.begin(), csize);
   this->_m_dlen=csize;
   this->_m_buffer[csize]=EOS;
   return(Ok);
 }
 if(csize>this->WyStr::max_capacity()) {
   WY_RETURN( Wym_EFBIG );
 }
 ElemType* dptr= _mem_alloc( _mem_dlen(csize) );
 if(dptr==NULL) {
   WY_RETURN( Wym_ENOMEM );
 }
 _mem_free(this->_m_buffer);
 this->_m_buffer=dptr;
 std::memcpy(dptr,src.begin(),csize);
 this->_m_dlen=csize;
 this->_m_dcap=_cap_dlen(csize);
 this->_m_buffer[csize]=EOS;
 return(Ok);
};

WyRet WyStr::reset(SizeType cnum, ElemType ch) WY__TSPC()
{
 if(this->_m_dcap>=cnum) {
   std::memset(this->_m_buffer, ch, cnum);
   this->_m_dlen=cnum;
   this->_m_buffer[cnum]=EOS;
   return(Ok);
 }
 if(cnum>this->WyStr::max_capacity()) {
   WY_RETURN( Wym_EFBIG );
 }
 ElemType* dptr= _mem_alloc( _mem_dlen(cnum) );
 if(dptr==NULL) {
   WY_RETURN( Wym_ENOMEM );
 }
 _mem_free(this->_m_buffer);
 this->_m_buffer=dptr;
 std::memset(dptr, ch, cnum);
 this->_m_dlen=cnum;
 this->_m_dcap=_cap_dlen(cnum);
 this->_m_buffer[cnum]=EOS;
 return(Ok);
};

WyStr::SizeType WyStr::size() const WY__TSPC()
{
 return this->_m_dlen;
};

const WyStr::ElemType& WyStr::operator[](SizeType pos) const WY__TSPC(Reply)
{
 if(pos>=this->_m_dlen) {
   WY_THROW( Reply(Wym_EINVAL) );
 }
 return this->_m_buffer[pos];
};

WyStr::ElemType& WyStr::operator[](SizeType pos) WY__TSPC(Reply)
{
 if(pos>=this->_m_dlen) {
   WY_THROW( Reply(Wym_EINVAL) );
 }
 return this->_m_buffer[pos];
};

const WyStr::ElemType& WyStr::at(SizeType pos) const WY__TSPC(Reply)
{
 if(pos>=this->_m_dlen) {
   WY_THROW( Reply(Wym_EINVAL) );
 }
 return this->_m_buffer[pos];
};

WyStr::ElemType& WyStr::at(SizeType pos) WY__TSPC(Reply)
{
 if(pos>=this->_m_dlen) {
   WY_THROW( Reply(Wym_EINVAL) );
 }
 return this->_m_buffer[pos];
};

const WyStr::ElemType* WyStr::data() const WY__TSPC()
{
 return this->_m_buffer;
};

const WyStr::ElemType* WyStr::c_str() const WY__TSPC()
{
 this->_m_buffer[this->_m_dlen]=EOS;
 return this->_m_buffer;
};

const WyStr::ElemType* WyStr::begin() const WY__TSPC()
{
 return this->_m_buffer;
};

WyStr::ElemType* WyStr::begin() WY__TSPC()
{
 return this->_m_buffer;
};

const WyStr::ElemType* WyStr::end() const WY__TSPC()
{
 return this->_m_buffer+this->_m_dlen;
};

const WyStr::ElemType& WyStr::front() const WY__TSPC(Reply)
{
 if(this->_m_dlen<=0) {
   WY_THROW( Reply(Wym_ENOENT) );
 }
 return this->_m_buffer[0];
};

const WyStr::ElemType& WyStr::back() const WY__TSPC(Reply)
{
 if(this->_m_dlen<=0) {
   WY_THROW( Reply(Wym_ENOENT) );
 }
 return this->_m_buffer[this->_m_dlen-1];
};

WyCSeg WyStr::cseg() const WY__TSPC()
{
 return WyCSeg(this->_m_buffer, this->_m_dlen);
};

WyCSeg WyStr::cseg(SizeType idx) const WY__TSPC(Reply)
{
 if(idx>this->_m_dlen) {
   WY_THROW( Reply(Wym_EINVAL) );
 }
 return WyCSeg(this->_m_buffer+idx, this->_m_dlen-idx);
};

WyCSeg WyStr::cseg(SizeType idx, SizeType num) const WY__TSPC(Reply)
{
 if(idx>this->_m_dlen) {
   WY_THROW( Reply(Wym_EINVAL) );
 }
 const SizeType rs= this->_m_dlen-idx;
 return WyCSeg(this->_m_buffer+idx, num<rs? num:rs);
};

const WyStr& WyStr::operator=(const WyStr& src) WY__TSPC(Reply)
{
 if(src._m_buffer==this->_m_buffer) {
   return(*this);         // self assignment
 }

 const SizeType csize= src._m_dlen;
 if(this->_m_dcap>=csize) {
   std::memcpy(this->_m_buffer, src._m_buffer, csize);
   this->_m_dlen=csize;
   this->_m_buffer[csize]=EOS;
   return(*this);
 }

 ElemType* dptr= _mem_alloc( _mem_dlen(csize) );
 if(dptr==NULL) {
   WY_THROW( Reply(Wym_ENOMEM) );
 }
 _mem_free(this->_m_buffer);
 this->_m_buffer=dptr;
 std::memcpy(dptr,src._m_buffer,csize);
 this->_m_dlen=csize;
 this->_m_dcap=_cap_dlen(csize);
 this->_m_buffer[csize]=EOS;
 return(*this);
};

const WyStr& WyStr::operator=(const WyCSeg& src) WY__TSPC(Reply)
{
 if(src.is_overlap(this->cseg())) {
   WY_THROW( Reply(Wym_ELOOP) );
 }

 const SizeType csize= src.size();
 if(this->_m_dcap>=csize) {
   std::memcpy(this->_m_buffer, src.begin(), csize);
   this->_m_dlen=csize;
   this->_m_buffer[csize]=EOS;
   return(*this);
 }
 if(csize>this->WyStr::max_capacity()) {
   WY_THROW( Reply(Wym_EFBIG) );
 }
 ElemType* dptr= _mem_alloc( _mem_dlen(csize) );
 if(dptr==NULL) {
   WY_THROW( Reply(Wym_ENOMEM) );
 }
 _mem_free(this->_m_buffer);
 this->_m_buffer=dptr;
 std::memcpy(dptr,src.begin(),csize);
 this->_m_dlen=csize;
 this->_m_dcap=_cap_dlen(csize);
 this->_m_buffer[csize]=EOS;
 return(*this);
};

void WyStr::swap(WyStr& src) WY__TSPC()
{
 SizeType vtmp= src._m_dcap;
 src._m_dcap=this->_m_dcap;
 this->_m_dcap=vtmp;

 vtmp=src._m_dlen;
 src._m_dlen=this->_m_dlen;
 this->_m_dlen=vtmp;

 ElemType* const ptr=src._m_buffer;
 src._m_buffer=this->_m_buffer;
 this->_m_buffer=ptr;
};

WyRet WyStr::append(ElemType ch) WY__TSPC()
{
 const SizeType rs= this->_m_dlen+1;
 if(rs>this->_m_dcap) {
   if(rs>this->WyStr::max_capacity()) {
     WY_RETURN( Wym_EFBIG );
   }
   ElemType* dptr= _mem_alloc( _mem_dlen(rs) );
   if(dptr==NULL) {
     WY_RETURN( Wym_ENOMEM );
   }
   std::memcpy(dptr,this->_m_buffer,this->_m_dlen);
   _mem_free(this->_m_buffer);
   this->_m_buffer=dptr;
   this->_m_dcap=_cap_dlen(rs);
 }
 this->_m_buffer[this->_m_dlen]=ch;
 ++this->_m_dlen;
 this->_m_buffer[this->_m_dlen]=EOS;
 return(Ok);
};

WyRet WyStr::append(const WyStr& src) WY__TSPC()
{
 // self append ok, no ELOOP
 const SizeType rs= this->_m_dlen+src._m_dlen;
 if(rs<this->_m_dlen) {
   WY_RETURN( Wym_EFBIG );  // overflow
 }
 if(rs<=this->_m_dcap) {
   std::memcpy(this->_m_buffer+this->_m_dlen,src._m_buffer,src._m_dlen);
   this->_m_buffer[rs]=EOS;
   this->_m_dlen=rs;
   return(Ok);
 }
 if(rs>this->WyStr::max_capacity()) {
   WY_RETURN( Wym_EFBIG );
 }
 ElemType* dptr= _mem_alloc( _mem_dlen(rs) );
 if(dptr==NULL) {
   WY_RETURN( Wym_ENOMEM );
 }
 std::memcpy(dptr,this->_m_buffer,this->_m_dlen);
 std::memcpy(dptr+this->_m_dlen,src._m_buffer,src._m_dlen);
 _mem_free(this->_m_buffer);
 this->_m_buffer=dptr;
 this->_m_dcap=_cap_dlen(rs);
 this->_m_buffer[rs]=EOS;
 this->_m_dlen=rs;
 return(Ok);
};

WyRet WyStr::append(const WyCSeg& src) WY__TSPC()
{
 if(src.is_overlap( WyCSeg(this->_m_buffer,this->_m_dcap) )) {
   WY_RETURN( Wym_ELOOP );
 }
 const SizeType csize= src.size();
 const SizeType rs= this->_m_dlen+csize;
 if(rs<this->_m_dlen) {
   WY_RETURN( Wym_EFBIG );  // overflow
 }
 if(rs<=this->_m_dcap) {
   ElemType *dptr= this->_m_buffer+this->_m_dlen;
   std::memcpy(dptr,src.begin(),csize);
   dptr[csize]=EOS;
   this->_m_dlen=rs;
   return(Ok);
 }
 if(rs>this->WyStr::max_capacity()) {
   WY_RETURN( Wym_EFBIG );
 }
 ElemType* dptr= _mem_alloc( _mem_dlen(rs) );
 if(dptr==NULL) {
   WY_RETURN( Wym_ENOMEM );
 }
 std::memcpy(dptr,this->_m_buffer,this->_m_dlen);
 _mem_free(this->_m_buffer);
 this->_m_buffer=dptr;
 dptr+=this->_m_dlen;
 std::memcpy(dptr,src.begin(),csize);
 dptr[csize]=EOS;
 this->_m_dcap=_cap_dlen(rs);
 this->_m_dlen=rs;
 return(Ok);
};

WyRet WyStr::append(SizeType cnum, ElemType ch) WY__TSPC()
{
 const SizeType rs= this->_m_dlen+cnum;
 if(rs<this->_m_dlen) {
   WY_RETURN( Wym_EFBIG );  // overflow
 }
 if(rs<=this->_m_dcap) {
   ElemType *dptr= this->_m_buffer+this->_m_dlen;
   std::memset(dptr, ch, cnum);
   dptr[cnum]=EOS;
   this->_m_dlen=rs;
   return(Ok);
 }
 if(rs>this->WyStr::max_capacity()) {
   WY_RETURN( Wym_EFBIG );
 }
 ElemType* dptr= _mem_alloc( _mem_dlen(rs) );
 if(dptr==NULL) {
   WY_RETURN( Wym_ENOMEM );
 }
 std::memcpy(dptr,this->_m_buffer,this->_m_dlen);
 _mem_free(this->_m_buffer);
 this->_m_buffer=dptr;
 dptr+=this->_m_dlen;
 std::memset(dptr, ch, cnum);
 dptr[cnum]=EOS;
 this->_m_dcap=_cap_dlen(rs);
 this->_m_dlen=rs;
 return(Ok);
};

const WyStr& WyStr::operator +=(ElemType ch) WY__TSPC(Reply)
{
 const WyRet r( this->append(ch) );
 if(r!=Ok) {
   WY_THROW( Reply(r) );
 }
 return(*this);
};

const WyStr& WyStr::operator +=(const WyStr& src) WY__TSPC(Reply)
{
 const WyRet r( this->append(src) );
 if(r!=Ok) {
   WY_THROW( Reply(r) );
 }
 return(*this);
};

const WyStr& WyStr::operator +=(const WyCSeg& src) WY__TSPC(Reply)
{
 const WyRet r( this->append(src) );
 if(r!=Ok) {
   WY_THROW( Reply(r) );
 }
 return(*this);
};

WyRet WyStr::erase(SizeType idx) WY__TSPC()
{
 if(idx>this->_m_dlen) {
   WY_RETURN( Wym_EINVAL );
 }
 this->_m_dlen=idx;
 this->_m_buffer[idx]=EOS;
 return(Ok);
};

WyRet WyStr::erase(SizeType idx, SizeType len) WY__TSPC()
{
 if(idx>this->_m_dlen) {
   WY_RETURN( Wym_EINVAL );
 }
 const SizeType zidx=idx+len;
 if((zidx>=this->_m_dlen)||(len>WyStr::max_capacity())) {
   this->_m_dlen=idx;
   this->_m_buffer[idx]=EOS;
   return(Ok);
 }
 std::memmove(this->_m_buffer+idx, this->_m_buffer+zidx, this->_m_dlen-zidx);
 this->_m_dlen-=len;
 this->_m_buffer[this->_m_dlen]=EOS;
 return(Ok);
};

WyRet WyStr::resize(SizeType nsize, ElemType ch) WY__TSPC()
{
 if(nsize<=this->_m_dcap) {
   if(nsize>this->_m_dlen) {
     std::memset(this->_m_buffer+this->_m_dlen, ch, nsize-this->_m_dlen);
   }
   this->_m_dlen=nsize;
   this->_m_buffer[nsize]=EOS;
   return(Ok);
 }
 if(nsize>WyStr::max_capacity()) {
   WY_RETURN( Wym_EFBIG );
 }
 ElemType* dptr= _mem_alloc( _mem_dlen(nsize) );
 if(dptr==NULL) {
   WY_RETURN( Wym_ENOMEM );
 }
 std::memcpy(dptr,this->_m_buffer,this->_m_dlen);
 _mem_free(this->_m_buffer);
 std::memset(dptr+this->_m_dlen,ch,nsize-this->_m_dlen);
 this->_m_buffer=dptr;
 this->_m_dlen=nsize;
 this->_m_dcap=_cap_dlen(nsize);
 this->_m_buffer[nsize]=EOS;
 return(Ok);
};

WyRet WyStr::resize(SizeType nsize) WY__TSPC()
{
 return this->resize(nsize,char(0));
};

bool WyStr::find(SizeType& res, SizeType idx, const WyStr& str) const WY__TSPC()
{
 return this->cseg().find(res,idx,str.cseg());
};

bool WyStr::findc(SizeType& res, SizeType idx, const WyStr& str) const WY__TSPC()
{
 return this->cseg().findc(res,idx,str.cseg());
};

bool WyStr::findc_not(SizeType& res, SizeType idx, const WyStr& str) const WY__TSPC()
{
 return this->cseg().findc_not(res,idx,str.cseg());
};

bool WyStr::find(SizeType& res, SizeType idx, const WyCSeg& cs) const WY__TSPC()
{
 return this->cseg().find(res,idx,cs);
};

/*
bool WyStr::find(SizeType& res, const WyCSeg& cs) const WY__TSPC()
{
 return this->cseg().find(res,SizeType(0),cs);
};

bool WyStr::findc(SizeType& res, ElemType ch) const WY__TSPC()
{
 const ElemType* const ptr(  static_cast<const ElemType*>(
         std::memchr(this->_m_buffer,(unsigned char)ch,this->_m_dlen)) );
 if(ptr==NULL) {
   return(false);
 }
 res=ptr-this->_m_buffer;
 return(true);
};*/

bool WyStr::findc(SizeType& res, SizeType idx, ElemType ch) const WY__TSPC()
{
 return this->cseg().findc(res,idx,ch);
};

bool WyStr::findc(SizeType& res, SizeType idx, const WyCSeg& cs) const WY__TSPC()
{
 return this->cseg().findc(res,idx,cs);
};

bool WyStr::findc_not(SizeType& res, SizeType idx, ElemType ch) const WY__TSPC()
{
 return this->cseg().findc_not(res,idx,ch);
};

bool WyStr::findc_not(SizeType& res, SizeType idx, const WyCSeg& cs) const WY__TSPC()
{
 return this->cseg().findc_not(res,idx,cs);
};

bool WyStr::rfind(SizeType& res, SizeType idx, const WyStr& str) const WY__TSPC()
{
 return this->cseg().rfind(res,idx,str.cseg());
};

bool WyStr::rfindc(SizeType& res, SizeType idx, const WyStr& str) const WY__TSPC()
{
 return this->cseg().rfindc(res,idx,str.cseg());
};

bool WyStr::rfindc_not(SizeType& res, SizeType idx, const WyStr& str) const WY__TSPC()
{
 return this->cseg().rfindc_not(res,idx,str.cseg());
};

bool WyStr::rfind(SizeType& res, SizeType idx, const WyCSeg& cs) const WY__TSPC()
{
 return this->cseg().rfind(res,idx,cs);
};

bool WyStr::rfindc(SizeType& res, SizeType idx, const WyCSeg& cs) const WY__TSPC()
{
 return this->cseg().rfindc(res,idx,cs);
};

bool WyStr::rfindc(SizeType& res, SizeType idx, ElemType ch) const WY__TSPC()
{
 return this->cseg().rfindc(res,idx,ch);
};

bool WyStr::rfindc_not(SizeType& res, SizeType idx, ElemType ch) const WY__TSPC()
{
 return this->cseg().rfindc_not(res,idx,ch);
};

bool WyStr::rfindc_not(SizeType& res, SizeType idx, const WyCSeg& cs) const WY__TSPC()
{
 return this->cseg().rfindc_not(res,idx,cs);
};

bool WyStr::operator==(const WyStr& rhs) const WY__TSPC()
{
 return this->cseg()==rhs.cseg();
};

bool WyStr::operator!=(const WyStr& rhs) const WY__TSPC()
{
 return this->cseg()!=rhs.cseg();
};

bool WyStr::operator==(const WyCSeg& rhs) const WY__TSPC()
{
 return this->cseg()==rhs;
};

bool WyStr::operator!=(const WyCSeg& rhs) const WY__TSPC()
{
 return this->cseg()!=rhs;
};

bool WyStr::operator==(const ElemType* cstr) const
{
 return this->cseg()==cstr;
};

bool WyStr::operator!=(const ElemType* cstr) const
{
 return this->cseg()!=cstr;
};

WyStr::SizeType WyStr::_capacity() const WY__TSPC()
{
 return this->_m_dcap;
};

WyRet WyStr::_reserve(SizeType nsize) WY__TSPC()
{
 if(nsize<=this->_m_dcap) {
   return(Ok);
 }
 if(nsize>WyStr::max_capacity()) {
   WY_RETURN( Wym_EFBIG );
 }
 ElemType* dptr= _mem_alloc( _mem_dlen(nsize) );
 if(dptr==NULL) {
   WY_RETURN( Wym_ENOMEM );
 }
 std::memcpy(dptr,this->_m_buffer,this->_m_dlen);
 _mem_free(this->_m_buffer);
 this->_m_buffer=dptr;
 this->_m_dcap=_cap_dlen(nsize);
 this->_m_buffer[this->_m_dlen]=EOS;
 return(Ok);
};

void WyStr::_setsize(SizeType nsize) WY__TSPC()
{
 if(nsize>this->_m_dcap) {
   this->_m_buffer[this->_m_dlen=this->_m_dcap]=EOS;
   return;
 }
 this->_m_dlen=nsize;
 this->_m_buffer[nsize]=EOS;
};

WyStr WyStr::operator+(const WyStr& rhs) const WY__TSPC(Reply)
{
 return WyStr(*this,rhs);
};

WyStr WyStr::operator+(const WyCSeg& rhs) const WY__TSPC(Reply)
{
 return WyStr(*this,rhs);
};

WyStr WyStr::operator+(ElemType ch) const WY__TSPC(Reply)
{
 return WyStr(*this,ch);
};

WyStr operator+(const WyCSeg& s1, const WyStr& s2) WY__TSPC(WyStr::Reply)
{
 return WyStr(s1,s2);
};

WyStr operator+(WyStr::ElemType ch, const WyStr& str) WY__TSPC(WyStr::Reply)
{
 return WyStr(ch,str);
};

//------------------------------------------------------------------------------
// [Syn] Make string representation of the given argument file_name/line_num.
//
// [Ret] WyStr, string representation of the arguments
//              in the form: file_name '(' line_num ')'
//
typedef int wy__line_num_type;
static WyStr mkstr_sli(const char *file_name,wy__line_num_type line_num) WY__TSPC(WyStr::Reply)
try {
 WyStr nstr;

 if(file_name!=NULL) {
#ifndef WY_SHORT_SLI    // personal use for my application
   nstr=file_name;
#else
  // Sample the first MaxFlen characters of file_name or until the dot 
  // character as the source file name information.
  //
  const int MaxFlen=8;  
  int flen;
  for(flen=0; flen<=MaxFlen; ++flen) {
     const char ch=file_name[flen];
     if(((ch==0)||(ch=='.'))||(flen==MaxFlen)) {
       nstr.reset(WyCSeg(file_name,flen));
       break;
     }
  }
#endif
 };

#ifndef WY_SHORT_SLI
 nstr+='(';
#endif
 
 // Convert line_num number
 // 
 char dbuf[2+3*sizeof(wy__line_num_type)];
 std::div_t dt;
 int n;

 if(line_num<0) {
   dt.quot=-line_num;
   nstr+='-';
 } else {
   dt.quot=line_num;
 }
  
 n=0;
 do {
   dt=std::div(dt.quot,10);
   dbuf[n++]=(char)(dt.rem);
 } while(dt.quot>0);

 for(--n; n>=0; --n) {
   const char ch=dbuf[n];
   nstr+= (ch>9)?  char(ch+'7'):char(ch+'0');
 }
#ifndef WY_SHORT_SLI
 nstr+=')';
#endif
 return(nstr);
}
catch(const WyStr::Reply&) {
 throw;
};

//------------------------------------------------------------
WyStr Wy::wrd(const WyReply& em) WY__TSPC(WyStr::Reply)
{
 WyRet r;
 WyStr str;
 const char* mtid;  // dummy
 for(;;) {
   if((r=str._reserve(em->getmsg(mtid,NULL,0)+1))!=Ok) {  // one more byte for detecting
                                                          // for sufficient buffer size
     WY_THROW( WyStr::Reply(r) );
   }
   const size_t mlen=em->getmsg(mtid,str.begin(),str._capacity());
   if(mlen<str._capacity()) {
     str._setsize(mlen);
     break;
   }
 }
 return(str);
};

WyStr Wy::wrd(const Wy_SrcLoc& s) WY__TSPC(WyStr::Reply)
try {
 return mkstr_sli(s.src_file(),s.src_line());
}
catch(const WyStr::Reply& e) {
 if(e==Wym_ENOMEM) {
   throw;
 }
 WY_TERMINATE("");
};

WyStr Wy::wrd(const WyRet& arg) WY__TSPC(WyStr::Reply)
{
#ifndef WYLIB_CONFIG_NORSLI
 WyRet r;
 WyStr str;
 const char* mtid;  // dummy
 for(;;) {
   if((r=str._reserve(arg->getmsg(mtid,NULL,0)+1))!=Ok) {  // one more byte for detecting
     WY_THROW( WyStr::Reply(r) );
   }
   const size_t mlen=arg->getmsg(mtid,str.begin(),str._capacity());
   if(mlen<str._capacity()) {
     str._setsize(mlen);
     break;
   }
 }
 if(arg.sli_ptr()!=NULL) {
   str+=", ";
   str+=Wy::wrd(*arg.sli_ptr());
 }
 return(str);
#else
 return Wy::wrd( static_cast<WyReply>(arg) );
#endif
};

