/* Copyright is licensed by GNU LGPL.                        (I.J. Wang, 2004)
*/

#ifndef WYSTR_H__
#define WYSTR_H__
#define WYSTR_VERSION 33

#include "WyRet.h"
#include "WyCSeg.h"

class WyStr {
    typedef size_t SizeType;
    typedef char ElemType;
    static const ElemType EOS=0;
    static const SizeType BufGranBits=8; // .at leat 2
                                         // .considering number conversions
                                         //  at least 7 (or 8)
    static const SizeType DefaultBufSize=1<<BufGranBits;
    static const SizeType BufGranBitsMask=DefaultBufSize-1;

    SizeType  _m_dcap;      // capacity of _m_buffer
    SizeType  _m_dlen;      // data length in _m_buffer
    ElemType* _m_buffer;

    static const SizeType _mem_dlen(SizeType) WY__NOTHROW__;
    static const SizeType _cap_mem(SizeType) WY__NOTHROW__;
    static const SizeType _cap_dlen(SizeType) WY__NOTHROW__;
    static ElemType* _mem_alloc(SizeType) WY__NOTHROW__;
    static void _mem_free(void* p) WY__NOTHROW__;

    // Hidden
    WyStr(int);
    WyStr(SizeType,int);
    void reset(SizeType,int);
    WyRet append(int);
    WyRet append(SizeType,int);
    const WyStr& operator+=(int);

  public:
    typedef SizeType size_type;
    typedef ElemType value_type;
    WY_THROW_REPLY;

    static const ElemType class_name[];
    static SizeType max_capacity() WY__TSPC();
    static SizeType min_capacity() WY__TSPC();

    WyStr() WY__TSPC(Reply);
    WyStr(const WyStr&) WY__TSPC(Reply);
    WyStr(const WyCSeg&) WY__TSPC(Reply);
    WyStr(SizeType cnum, ElemType ch) WY__TSPC(Reply);
    WyStr(WyStr& str, Wy::ByMove_t) WY__TSPC();
    ~WyStr() WY__TSPC();

    bool is_default() const WY__TSPC();
    void reset() WY__TSPC();
    WyRet reset(const WyStr&) WY__TSPC();
    WyRet reset(const WyCSeg& src) WY__TSPC();
    WyRet reset(SizeType cnum, ElemType ch) WY__TSPC();
    SizeType size() const WY__TSPC();
    const ElemType& operator[](SizeType pos) const WY__TSPC(Reply);
    ElemType& operator[](SizeType pos) WY__TSPC(Reply);
    const ElemType& at(SizeType pos) const WY__TSPC(Reply);
    ElemType& at(SizeType pos) WY__TSPC(Reply);
    const ElemType* data() const WY__TSPC();
    const ElemType* c_str() const WY__TSPC();
    const ElemType* begin() const WY__TSPC();
    ElemType* begin() WY__TSPC();
    const ElemType* end() const WY__TSPC();
    const ElemType& front() const WY__TSPC(Reply);
    const ElemType& back() const WY__TSPC(Reply);
    WyCSeg cseg() const WY__TSPC();
    WyCSeg cseg(SizeType) const WY__TSPC(Reply);
    WyCSeg cseg(SizeType,SizeType) const WY__TSPC(Reply);
    const WyStr& operator=(const WyStr&) WY__TSPC(Reply);
    const WyStr& operator=(const WyCSeg&) WY__TSPC(Reply);
    void swap(WyStr&) WY__TSPC();
    WyRet append(ElemType) WY__TSPC();
    WyRet append(const WyStr&) WY__TSPC();
    WyRet append(const WyCSeg&) WY__TSPC();
    WyRet append(SizeType,ElemType) WY__TSPC();
    const WyStr& operator +=(ElemType) WY__TSPC(Reply);
    const WyStr& operator +=(const WyStr&) WY__TSPC(Reply);
    const WyStr& operator +=(const WyCSeg&) WY__TSPC(Reply);

    // SRC: WyStr_insert.cpp
    WyRet insert(SizeType,const WyStr&) WY__TSPC();
    WyRet insert(SizeType,const WyCSeg&) WY__TSPC();
    WyRet insert(SizeType,SizeType,ElemType) WY__TSPC();

    WyRet erase(SizeType) WY__TSPC();
    WyRet erase(SizeType,SizeType) WY__TSPC();
    WyRet resize(SizeType) WY__TSPC();
    WyRet resize(SizeType,ElemType) WY__TSPC();

    // SRC: WyStr_replace.cpp
    WyRet replace(SizeType, SizeType, const WyStr&) WY__TSPC();
    WyRet replace(SizeType, SizeType, const WyCSeg&) WY__TSPC();
    WyRet replace(SizeType, SizeType, SizeType, ElemType) WY__TSPC();

    bool find(SizeType&,SizeType,const WyStr&) const WY__TSPC();
    bool find(SizeType&,SizeType,const WyCSeg&) const WY__TSPC();
    bool findc(SizeType&,SizeType,const WyStr&) const WY__TSPC();
    bool findc(SizeType&,SizeType,const WyCSeg&) const WY__TSPC();
    bool findc(SizeType&,SizeType,ElemType) const WY__TSPC();
    bool findc_not(SizeType&,SizeType,const WyStr&) const WY__TSPC();
    bool findc_not(SizeType&,SizeType,const WyCSeg&) const WY__TSPC();
    bool findc_not(SizeType&,SizeType,ElemType) const WY__TSPC();

    bool rfind(SizeType&,SizeType,const WyStr&) const WY__TSPC();
    bool rfind(SizeType&,SizeType,const WyCSeg&) const WY__TSPC();
    bool rfindc(SizeType&,SizeType,const WyStr&) const WY__TSPC();
    bool rfindc(SizeType&,SizeType,const WyCSeg&) const WY__TSPC();
    bool rfindc(SizeType&,SizeType,ElemType ch) const WY__TSPC();
    bool rfindc_not(SizeType&,SizeType,const WyStr&) const WY__TSPC();
    bool rfindc_not(SizeType&,SizeType,const WyCSeg&) const WY__TSPC();
    bool rfindc_not(SizeType&,SizeType,ElemType ch) const WY__TSPC();

    bool operator==(const WyStr&) const WY__TSPC();
    bool operator!=(const WyStr&) const WY__TSPC();
    bool operator==(const WyCSeg&) const WY__TSPC();
    bool operator!=(const WyCSeg&) const WY__TSPC();
    bool operator==(const ElemType*) const;
    bool operator!=(const ElemType*) const;

    SizeType _capacity() const WY__TSPC();
    WyRet _reserve(SizeType nsize) WY__TSPC();
    void _setsize(SizeType nsize) WY__TSPC();

    WyStr operator+(const WyStr&) const WY__TSPC(Reply);
    WyStr operator+(const WyCSeg&) const WY__TSPC(Reply);
    WyStr operator+(ElemType ch) const WY__TSPC(Reply);

    friend WyStr operator+(const WyCSeg&,const WyStr&) WY__TSPC(WyStr::Reply);
    friend WyStr operator+(ElemType,const WyStr&) WY__TSPC(WyStr::Reply);

private:
    WyStr(const WyStr&, const WyStr&) WY__TSPC(Reply);
    WyStr(const WyStr&, const WyCSeg&) WY__TSPC(Reply);
    WyStr(const WyCSeg&, const WyStr&) WY__TSPC(Reply);
    WyStr(const WyStr&, ElemType ch) WY__TSPC(Reply);
    WyStr(ElemType ch, const WyStr&) WY__TSPC(Reply);
};

 WyStr operator+(const WyCSeg&, const WyStr&) WY__TSPC(WyStr::Reply);
 WyStr operator+(WyStr::ElemType,const WyStr&) WY__TSPC(WyStr::Reply);

 inline bool operator==(const WyCSeg& s1, const WyStr& s2) WY__TSPC()
      { return s2.operator==(s1); };
 inline bool operator!=(const WyCSeg& s1, const WyStr& s2) WY__TSPC()
      { return s2.operator!=(s1); };
 inline bool operator==(const char* s1, const WyStr& s2)
      { return s2.operator==(s1); };
 inline bool operator!=(const char* s1, const WyStr& s2)
      { return s2.operator!=(s1); };

namespace Wy {
 WyStr wrd(const WyReply&) WY__TSPC(WyStr::Reply);
 WyStr wrd(const Wy_SrcLoc&) WY__TSPC(WyStr::Reply);
 WyStr wrd(const WyRet&) WY__TSPC(WyStr::Reply);
 inline WyStr wrd(const WyCSeg& s) WY__TSPC(WyStr::Reply)
   { return WyStr(s); };

 inline int _strcmp(const WyStr& s1,const WyStr& s2) WY__TSPC()
       { return Wy::_strcmp(s1.cseg(),s2.cseg()); };
 inline int _strcmp(const WyStr& str,const WyCSeg& cs) WY__TSPC()
       { return Wy::_strcmp(str.cseg(),cs); };
 inline int _strcmp(const WyCSeg& cs,const WyStr& str) WY__TSPC()
       { return Wy::_strcmp(cs,str.cseg()); };
 inline int _strcmp(const char* cstr, const WyStr& str)
       { return Wy::_strcmp(WyCSeg(cstr),str.cseg()); };
 inline int _strcmp(const WyStr& str,const char* cstr)
       { return Wy::_strcmp(str.cseg(), WyCSeg(cstr)); };

 // SRC: WyStr_mis.cpp
 WyStr& _toupper(WyStr& str) WY__TSPC();
 WyStr& _tolower(WyStr& str) WY__TSPC();
 
 // SRC: wy_ntoa_char.cpp
 WyRet _mkstr(WyStr& str, char num, Wy::WyRadix radix) WY__TSPC();
 WyRet _mkstr(WyStr& str, signed char num, Wy::WyRadix radix) WY__TSPC();
 WyRet _mkstr(WyStr& str, unsigned char num, Wy::WyRadix radix) WY__TSPC();
 WyStr wrd(char num, Wy::WyRadix radix=10);
 WyStr wrd(signed char num, Wy::WyRadix radix=10);
 WyStr wrd(unsigned char num, Wy::WyRadix radix=10);

 // SRC: wy_ntoa_short.cpp
 WyRet _mkstr(WyStr& str, signed short num, Wy::WyRadix radix) WY__TSPC();
 WyRet _mkstr(WyStr& str, unsigned short num, Wy::WyRadix radix) WY__TSPC();
 WyStr wrd(signed short num, Wy::WyRadix radix=10);
 WyStr wrd(unsigned short num, Wy::WyRadix radix=10);

 // SRC: wy_ntoa_int.cpp
 WyRet _mkstr(WyStr& str, signed int num, Wy::WyRadix radix) WY__TSPC();
 WyRet _mkstr(WyStr& str, unsigned int num, Wy::WyRadix radix) WY__TSPC();
 WyStr wrd(signed int num, Wy::WyRadix radix=10);
 WyStr wrd(unsigned int num, Wy::WyRadix radix=10);

 // SRC: wy_ntoa_long.cpp
 WyRet _mkstr(WyStr& str, signed long num, Wy::WyRadix radix) WY__TSPC();
 WyRet _mkstr(WyStr& str, unsigned long num, Wy::WyRadix radix) WY__TSPC();
 WyStr wrd(signed long num, Wy::WyRadix radix=10);
 WyStr wrd(unsigned long num, Wy::WyRadix radix=10);

 // SRC: wy_ntoa_llong.cpp
 WyRet _mkstr(WyStr& str, signed long long num, Wy::WyRadix radix) WY__TSPC();
 WyRet _mkstr(WyStr& str, unsigned long long num, Wy::WyRadix radix) WY__TSPC();
 WyStr wrd(signed long long num, Wy::WyRadix radix=10);
 WyStr wrd(unsigned long long num, Wy::WyRadix radix=10);

 // SRC: wy_ntoa_double.cpp
 WyRet _mkstr(WyStr&, double, Wy::WyRadix, size_t) WY__TSPC();
 WyRet _mkstr(WyStr&, float, Wy::WyRadix, size_t) WY__TSPC();
 WyStr wrd(double, Wy::WyRadix=10, size_t=6);
 WyStr wrd(float, Wy::WyRadix=10, size_t=6);

}; // End of namespace Wy

#endif
