/* Copyright is licensed by GNU LGPL.                         by I.J.Wang 2004
*/
#define WYLIB_SOURCE
#include "WyStr.h"
#include <cstdlib>

WyRet WyStr::insert(SizeType idx, const WyStr& str) WY__TSPC()
{
 if(idx>this->_m_dlen) {
   WY_RETURN( Wym_EINVAL );
 }
 const SizeType rs= this->_m_dlen+str._m_dlen;
 if(rs<this->_m_dlen) {
   WY_RETURN( Wym_EFBIG );  // overflow
 }
 if(this->_m_buffer==str._m_buffer) {
   WY_RETURN( Wym_ELOOP );
 }
 const SizeType rlen= this->_m_dlen-idx;
 if(rs<this->_m_dcap) {
   ElemType *dptr= this->_m_buffer+idx;
   std::memmove(dptr+str._m_dlen, dptr, rlen);
   std::memcpy(dptr,str._m_buffer,str._m_dlen);
   this->_m_buffer[rs]=EOS;
   this->_m_dlen=rs;
   return(Ok);
 }
 if(rs>this->WyStr::max_capacity()) {
   WY_RETURN( Wym_EFBIG );
 }
 ElemType* dptr= _mem_alloc( _mem_dlen(rs) );
 if(dptr==NULL) {
   WY_RETURN( Wym_ENOMEM );
 }
 ElemType* const dptr2=dptr;
 std::memcpy(dptr,this->_m_buffer,idx);       // copy the first segment
 dptr+=idx;
 std::memcpy(dptr,str._m_buffer,str._m_dlen); // copy str
 dptr+=str._m_dlen;
 std::memcpy(dptr,_m_buffer+idx,rlen);        // copy the rest segment
 _mem_free(this->_m_buffer);

 this->_m_buffer=dptr2;
 dptr2[rs]=EOS;
 this->_m_dcap=_cap_dlen(rs);
 this->_m_dlen=rs;
 return(Ok);
};

WyRet WyStr::insert(SizeType idx, const WyCSeg& cs) WY__TSPC()
{
 if(this->cseg().is_overlap(cs)) {
   WY_RETURN( Wym_ELOOP );
 }
 if(idx>this->_m_dlen) {
   WY_RETURN( Wym_EINVAL );
 }
 const SizeType csize= cs.size();
 const SizeType rs= this->_m_dlen+csize;
 if(rs<this->_m_dlen) {
   WY_RETURN( Wym_EFBIG );  // overflow
 }
 const SizeType rlen= this->_m_dlen-idx;
 if(rs<this->_m_dcap) {
   ElemType *dptr= this->_m_buffer+idx;
   std::memmove(dptr+csize, dptr, rlen);
   std::memcpy(dptr,cs.begin(),csize);
   this->_m_buffer[rs]=EOS;
   this->_m_dlen=rs;
   return(Ok);
 }
 if(rs>this->WyStr::max_capacity()) {
   WY_RETURN( Wym_EFBIG );
 }
 ElemType* dptr= _mem_alloc( _mem_dlen(rs) );
 if(dptr==NULL) {
   WY_RETURN( Wym_ENOMEM );
 }
 ElemType* const dptr2=dptr;
 std::memcpy(dptr,this->_m_buffer,idx);  // copy the first segment
 dptr+=idx;
 std::memcpy(dptr,cs.begin(),csize);     // copy str
 dptr+=csize;
 std::memcpy(dptr,_m_buffer+idx,rlen);   // copy the rest segment
 _mem_free(this->_m_buffer);

 this->_m_buffer=dptr2;
 dptr2[rs]=EOS;
 this->_m_dcap=_cap_dlen(rs);
 this->_m_dlen=rs;
 return(Ok);
};

WyRet WyStr::insert(SizeType idx, SizeType cnum, ElemType ch) WY__TSPC()
{
 if(idx>this->_m_dlen) {
   WY_RETURN( Wym_EINVAL );
 }
 const SizeType rs= this->_m_dlen+cnum;
 if(rs<this->_m_dlen) {
   WY_RETURN( Wym_EFBIG );  // overflow
 }
 const SizeType rlen= this->_m_dlen-idx;
 if(rs<this->_m_dcap) {
   ElemType *dptr= this->_m_buffer+idx;
   std::memmove(dptr+cnum, dptr, rlen);
   std::memset(dptr,ch,cnum);
   this->_m_buffer[rs]=EOS;
   this->_m_dlen=rs;
   return(Ok);
 }
 if(rs>this->WyStr::max_capacity()) {
   WY_RETURN( Wym_EFBIG );
 }
 ElemType* dptr= _mem_alloc( _mem_dlen(rs) );
 if(dptr==NULL) {
   WY_RETURN( Wym_ENOMEM );
 }
 ElemType* const dptr2=dptr;
 std::memcpy(dptr,this->_m_buffer,idx);  // copy the first segment
 dptr+=idx;
 std::memset(dptr,ch,cnum);              // set string
 dptr+=cnum;
 std::memcpy(dptr,_m_buffer+idx,rlen);   // copy the rest segment
 _mem_free(this->_m_buffer);

 this->_m_buffer=dptr2;
 dptr2[rs]=EOS;
 this->_m_dcap=_cap_dlen(rs);
 this->_m_dlen=rs;
 return(Ok);
};

