/* Copyright is licensed by GNU LGPL.                         by I.J.Wang 2004
*/
#define WYLIB_SOURCE
#include "WyStr.h"
#include <cstdlib>

WyRet WyStr::replace(SizeType idx, SizeType num, const WyStr& str) WY__TSPC()
{
 if(idx>this->_m_dlen) {
   WY_RETURN( Wym_EINVAL );
 }
 if(num>WyStr::max_capacity()) {
   WY_RETURN( Wym_EFBIG );
 }
 if(this->_m_buffer==str._m_buffer) {
   WY_RETURN( Wym_ELOOP );
 }
 SizeType idx2=idx+num;
 if(idx2>this->_m_dlen) {
   idx2=this->_m_dlen;
 }
 const SizeType rlen=this->_m_dlen-idx2;
 const SizeType rs=(idx+rlen)+str._m_dlen;
 if(rs<=this->_m_dcap) {
   ElemType* const dptr= this->_m_buffer+idx;
   std::memmove(dptr+str._m_dlen, this->_m_buffer+idx2, rlen);
   std::memcpy(dptr, str._m_buffer, str._m_dlen);
   this->_m_dlen=rs;
   this->_m_buffer[rs]=EOS;
   return(Ok);
 }
 if(rs>WyStr::max_capacity()) {
   WY_RETURN( Wym_EFBIG );
 }
 ElemType* dptr= _mem_alloc( _mem_dlen(rs) );
 if(dptr==NULL) {
   WY_RETURN( Wym_ENOMEM );
 }
 ElemType* const dptr2=dptr;
 std::memcpy(dptr,this->_m_buffer,idx);       // copy the first segment
 dptr+=idx;
 std::memcpy(dptr,str._m_buffer,str._m_dlen); // copy str
 dptr+=str._m_dlen;
 std::memcpy(dptr,_m_buffer+idx2, rlen);      // copy the rest segment
 _mem_free(this->_m_buffer);

 this->_m_buffer=dptr2;
 this->_m_buffer[rs]=EOS;
 this->_m_dcap=_cap_dlen(rs);
 this->_m_dlen=rs;
 return(Ok);
};

WyRet WyStr::replace(SizeType idx, SizeType num, const WyCSeg& cs) WY__TSPC()
{
 if(idx>this->_m_dlen) {
   WY_RETURN( Wym_EINVAL );
 }
 SizeType idx2=idx+num;
 if((idx2>this->_m_dlen)||(idx2<idx)) {
   idx2=this->_m_dlen;
 }
 const SizeType csize= cs.size();
 if(csize>WyStr::max_capacity()) {
   WY_RETURN( Wym_EFBIG );
 }
 if(this->cseg().is_overlap(cs)) {
   WY_RETURN( Wym_ELOOP );
 }
 const SizeType rlen=this->_m_dlen-idx2;
 const SizeType rs=(idx+rlen)+csize;
 if(rs<=this->_m_dcap) {
   ElemType* const dptr= this->_m_buffer+idx;
   std::memmove(dptr+csize, this->_m_buffer+idx2, rlen);
   std::memcpy(dptr, cs.begin(), csize);
   this->_m_dlen=rs;
   this->_m_buffer[rs]=EOS;
   return(Ok);
 }
 if(rs>WyStr::max_capacity()) {
   WY_RETURN( Wym_EFBIG );
 }
 ElemType* dptr= _mem_alloc( _mem_dlen(rs) );
 if(dptr==NULL) {
   WY_RETURN( Wym_ENOMEM );
 }
 ElemType* const dptr2=dptr;
 std::memcpy(dptr,this->_m_buffer,idx);   // copy the first segment
 dptr+=idx;
 std::memcpy(dptr, cs.begin(), csize);    // copy str
 dptr+=csize;
 std::memcpy(dptr,_m_buffer+idx2, rlen);  // copy the rest segment
 _mem_free(this->_m_buffer);

 this->_m_buffer=dptr2;
 this->_m_buffer[rs]=EOS;
 this->_m_dcap=_cap_dlen(rs);
 this->_m_dlen=rs;
 return(Ok);
};

WyRet WyStr::replace(SizeType idx, SizeType num, SizeType cnum, ElemType ch) WY__TSPC()
{
 if(idx>this->_m_dlen) {
   WY_RETURN( Wym_EINVAL );
 }
 SizeType idx2=idx+num;
 if((idx2>this->_m_dlen)||(idx2<idx)) {
   idx2=this->_m_dlen;
 }
 if(cnum>WyStr::max_capacity()) {
   WY_RETURN( Wym_EFBIG );
 }
 const SizeType rlen=this->_m_dlen-idx2;
 const SizeType rs=(idx+rlen)+cnum;
 if(rs<=this->_m_dcap) {
   ElemType* const dptr= this->_m_buffer+idx;
   std::memmove(dptr+cnum, this->_m_buffer+idx2, rlen);
   std::memset(dptr, ch, cnum);
   this->_m_dlen=rs;
   this->_m_buffer[rs]=EOS;
   return(Ok);
 }
 if(rs>WyStr::max_capacity()) {
   WY_RETURN( Wym_EFBIG );
 }
 ElemType* dptr= _mem_alloc( _mem_dlen(rs) );
 if(dptr==NULL) {
   WY_RETURN( Wym_ENOMEM );
 }
 ElemType* const dptr2=dptr;
 std::memcpy(dptr,this->_m_buffer,idx);   // copy the first segment
 dptr+=idx;
 std::memset(dptr, ch, cnum);    // copy str
 dptr+=cnum;
 std::memcpy(dptr,_m_buffer+idx2, rlen);  // copy the rest segment
 _mem_free(this->_m_buffer);

 this->_m_buffer=dptr2;
 this->_m_buffer[rs]=EOS;
 this->_m_dcap=_cap_dlen(rs);
 this->_m_dlen=rs;
 return(Ok);
};

