/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003
*/
#define WYLIB_SOURCE
#include "WyTermios.h"
#include <unistd.h>
#include <cstring>

const char WyTermios::class_name[]="WyTermios";

WyTermios::WyTermios(WyTermios::DumyFuncSig) WY__NOTHROW__
{
 // Default value of members
 static const speed_t  WYDFL__ISPEED=B0;
 static const speed_t  WYDFL__OSPEED=B0;
 static const cc_t     WYDFL__CC_T=0;

 _tio.c_iflag=0;
 _tio.c_oflag=0;
 _tio.c_cflag=0;
 _tio.c_lflag=0;

 std::memset(_tio.c_cc,WYDFL__CC_T,NCCS);

 // cfsetispeed/cfsetospeed can store bits in iflag/oflag members, so set lastly.
 //
 ::cfsetispeed(&_tio,WYDFL__ISPEED);
 ::cfsetospeed(&_tio,WYDFL__OSPEED);
};

const struct ::termios& WyTermios::_default_object() WY__NOTHROW__
{
 static const WyTermios Dfl_Obj(( WyTermios::DumyFuncSig() ));
 return Dfl_Obj._tio;
};

WyTermios::WyTermios() WY__TSPC()
 : _tio( WyTermios::_default_object() ) 
{
};

WyTermios::WyTermios(const WyTermios& src) WY__TSPC()
{
 _tio=src._tio;
};

WyTermios::WyTermios(WyTermios& src, Wy::ByMove_t) WY__TSPC()
{
 _tio=src._tio;
};

bool WyTermios::is_default() const WY__TSPC()
{
 // WyTermios uses bit image of struct termios copy for the 'default object'
 return(std::memcmp(&_tio,&WyTermios::_default_object(),sizeof(_tio))==0);
}

void WyTermios::reset() WY__TSPC()
{
 // WyTermios uses bit image of struct termios copy for the 'default object'
 std::memcpy(&this->_tio,&WyTermios::_default_object(),sizeof(this->_tio));
};

void WyTermios::reset(const WyTermios& src) WY__TSPC()
{
 _tio=src._tio;
};

void WyTermios::swap(WyTermios& src) WY__TSPC()
{
 Wy__Base::memswp(&_tio,&src._tio,sizeof(_tio));
};

const WyTermios& WyTermios::operator =(const WyTermios &src) WY__TSPC()
{
 _tio=src._tio;
 return(*this);
};

speed_t WyTermios::getispeed() const WY__TSPC()
{
 return ::cfgetispeed(&_tio);
};

speed_t WyTermios::getospeed() const WY__TSPC()
{
 return ::cfgetospeed(&_tio);
};

/*
 These two members do not convert the return value(v0.32) from the considerations:
   1. The common doc. just says return zero on success, -1 on error.
      a) POSIX Programmer's Guide, DONALD LEWINE, p159 says (and from The Open Group):
         'the standard does not require any error checking', while the 0/-1 return value
         is specified.
      b) Linux manpage(FC6) does not indicate any specific error.
   2. Check code in chk_termios.cpp, t.setispeed(speed_t(-1)) has been returning
      with errno=EINVAL (RH9.0,FC3,FC6)
 But returning the written value for user to verify the result. Such implement should
 be more portable.
*/
speed_t WyTermios::setispeed(speed_t speed) WY__TSPC()
{
 ::cfsetispeed(&_tio,speed);
 return ::cfgetispeed(&_tio);
};

speed_t WyTermios::setospeed(speed_t speed) WY__TSPC()
{
 ::cfsetospeed(&_tio,speed);
 return ::cfgetospeed(&_tio);
};

tcflag_t WyTermios::iflag() const WY__TSPC()
{
 return _tio.c_iflag;
};

void WyTermios::iflag_set(tcflag_t v) WY__TSPC()
{
 _tio.c_iflag=v;
};

void WyTermios::iflag_or(tcflag_t v) WY__TSPC()
{
 _tio.c_iflag|=v;
};

void WyTermios::iflag_and(tcflag_t v) WY__TSPC()
{
 _tio.c_iflag&=v;
};

tcflag_t WyTermios::oflag() const WY__TSPC()
{
 return _tio.c_oflag;
};

void WyTermios::oflag_set(tcflag_t v) WY__TSPC()
{ 
 _tio.c_oflag=v;
};

void WyTermios::oflag_or(tcflag_t v) WY__TSPC()
{ 
 _tio.c_oflag|=v;
};

void WyTermios::oflag_and(tcflag_t v) WY__TSPC()
{ 
 _tio.c_oflag&=v;
};

tcflag_t WyTermios::cflag() const WY__TSPC()
{
 return _tio.c_cflag;
};

void WyTermios::cflag_set(tcflag_t v) WY__TSPC()
{
 _tio.c_cflag=v;
};

void WyTermios::cflag_or(tcflag_t v) WY__TSPC()
{ 
 _tio.c_cflag|=v;
};

void WyTermios::cflag_and(tcflag_t v) WY__TSPC()
{ 
 _tio.c_cflag&=v;
};

tcflag_t WyTermios::lflag() const WY__TSPC()
{
 return _tio.c_lflag;
};

void WyTermios::lflag_set(tcflag_t v) WY__TSPC()
{
 _tio.c_lflag=v;
};

void WyTermios::lflag_or(tcflag_t v) WY__TSPC()
{
 _tio.c_lflag|=v;
};

void WyTermios::lflag_and(tcflag_t v) WY__TSPC()
{
 _tio.c_lflag&=v;
};

cc_t WyTermios::cc_VSTOP() const WY__TSPC()
{
 return _tio.c_cc[VSTOP];
};

void WyTermios::cc_VSTOP(cc_t v) WY__TSPC()
{
 _tio.c_cc[VSTOP]=v;
};

cc_t WyTermios::cc_VSTART() const WY__TSPC()
{
  return _tio.c_cc[VSTART];
};

void WyTermios::cc_VSTART(cc_t v) WY__TSPC()
{ 
 _tio.c_cc[VSTART]=v;
};

cc_t WyTermios::cc_VSUSP() const WY__TSPC()
{ 
 return _tio.c_cc[VSUSP];
};

void WyTermios::cc_VSUSP(cc_t v) WY__TSPC()
{
 _tio.c_cc[VSUSP]=v;
};

cc_t WyTermios::cc_VQUIT() const WY__TSPC()
{ 
 return _tio.c_cc[VQUIT]; 
};

void WyTermios::cc_VQUIT(cc_t v) WY__TSPC()
{
 _tio.c_cc[VQUIT]=v;
};

cc_t WyTermios::cc_VKILL() const WY__TSPC()
{
 return _tio.c_cc[VKILL];
};

void WyTermios::cc_VKILL(cc_t v) WY__TSPC()
{
 _tio.c_cc[VKILL]=v; 
};

cc_t WyTermios::cc_VERASE() const WY__TSPC()
{
 return _tio.c_cc[VERASE]; 
};

void WyTermios::cc_VERASE(cc_t v) WY__TSPC()
{
 _tio.c_cc[VERASE]=v; 
};

cc_t WyTermios::cc_VEOL() const WY__TSPC()
{
 return _tio.c_cc[VEOL]; 
};

void WyTermios::cc_VEOL(cc_t v) WY__TSPC()
{
 _tio.c_cc[VEOL]=v; 
};

cc_t WyTermios::cc_VEOF() const WY__TSPC()
{
 return _tio.c_cc[VEOF]; 
};

void WyTermios::cc_VEOF(cc_t v) WY__TSPC()
{
 _tio.c_cc[VEOF]=v; 
};

cc_t WyTermios::cc_VINTR() const WY__TSPC()
{
 return _tio.c_cc[VINTR]; 
};

void WyTermios::cc_VINTR(cc_t v) WY__TSPC()
{
 _tio.c_cc[VINTR]=v;
};

cc_t WyTermios::cc_VTIME() const WY__TSPC()
{
 return _tio.c_cc[VTIME]; 
};

void WyTermios::cc_VTIME(cc_t v) WY__TSPC()
{
 _tio.c_cc[VTIME]=v; 
};

cc_t WyTermios::cc_VMIN() const WY__TSPC()
{
 return _tio.c_cc[VMIN]; 
};

void WyTermios::cc_VMIN(cc_t v) WY__TSPC()
{
 _tio.c_cc[VMIN]=v;
};

bool WyTermios::operator ==(const WyTermios& rhs) const WY__TSPC()
{
 return(std::memcmp(&this->_tio,&rhs._tio,sizeof(this->_tio))==0);
};

struct ::termios& WyTermios::wy_tio_ref() WY__NOTHROW__
{
 return _tio;
};

const struct ::termios& WyTermios::wy_tio_ref() const WY__NOTHROW__
{
 return _tio;
};

