/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003
*/
#ifndef WYTERMIOS_H__
#define WYTERMIOS_H__
#define WYTERMIOS_VERSION 33

#include "WyRet.h"
#include <termios.h>

/*
  WyTermios defines the control attribute of WyTerminal (struct termios)
*/
class WyTermios {
  public:
    static const char class_name[];
    WY_THROW_REPLY;

    WyTermios() WY__TSPC();
    WyTermios(const WyTermios& tio) WY__TSPC();
    WyTermios(WyTermios& tio, Wy::ByMove_t) WY__TSPC();

    bool is_default() const WY__TSPC();

    void reset() WY__TSPC();
    void reset(const WyTermios& tio) WY__TSPC();

    void swap(WyTermios& src) WY__TSPC();
    const WyTermios& operator =(const WyTermios &src) WY__TSPC();

    speed_t getispeed() const WY__TSPC();
    speed_t getospeed() const WY__TSPC();

    speed_t setispeed(speed_t speed) WY__TSPC();
    speed_t setospeed(speed_t speed) WY__TSPC();
    
    // 1.There are many symbols for cflag_t and cc_t array index, much work is required
    //   to convert them
    // 2.Mapping with C program source (easy conversion, understanding, maintainence)
    //
    tcflag_t iflag() const WY__TSPC();
    void iflag_set(tcflag_t v) WY__TSPC();
    void iflag_or(tcflag_t v) WY__TSPC();
    void iflag_and(tcflag_t v) WY__TSPC();

    tcflag_t oflag() const WY__TSPC();
    void oflag_set(tcflag_t v) WY__TSPC();
    void oflag_or(tcflag_t v) WY__TSPC();
    void oflag_and(tcflag_t v) WY__TSPC();

    tcflag_t cflag() const WY__TSPC();
    void cflag_set(tcflag_t v) WY__TSPC();
    void cflag_or(tcflag_t v) WY__TSPC();
    void cflag_and(tcflag_t v) WY__TSPC();

    tcflag_t lflag() const WY__TSPC();
    void lflag_set(tcflag_t v) WY__TSPC();
    void lflag_or(tcflag_t v) WY__TSPC();
    void lflag_and(tcflag_t v) WY__TSPC();

    cc_t cc_VSTOP() const WY__TSPC();
    void cc_VSTOP(cc_t v) WY__TSPC();

    cc_t cc_VSTART() const WY__TSPC();
    void cc_VSTART(cc_t v) WY__TSPC();

    cc_t cc_VSUSP() const WY__TSPC();
    void cc_VSUSP(cc_t v) WY__TSPC();

    cc_t cc_VQUIT() const WY__TSPC();
    void cc_VQUIT(cc_t v) WY__TSPC();

    cc_t cc_VKILL() const WY__TSPC();
    void cc_VKILL(cc_t v) WY__TSPC();

    cc_t cc_VERASE() const WY__TSPC();
    void cc_VERASE(cc_t v) WY__TSPC();

    cc_t cc_VEOL() const WY__TSPC();
    void cc_VEOL(cc_t v) WY__TSPC();

    cc_t cc_VEOF() const WY__TSPC();
    void cc_VEOF(cc_t v) WY__TSPC();

    cc_t cc_VINTR() const WY__TSPC();
    void cc_VINTR(cc_t v) WY__TSPC();

    cc_t cc_VTIME() const WY__TSPC();
    void cc_VTIME(cc_t v) WY__TSPC();

    cc_t cc_VMIN() const WY__TSPC();
    void cc_VMIN(cc_t v) WY__TSPC();

    bool operator ==(const WyTermios& rhs) const WY__TSPC();
    bool operator !=(const WyTermios& rhs) const WY__TSPC()
         { return !(this->operator==(rhs)); };

    // [Internal] Get the reference of the ::termios of this class
    //
    struct ::termios& wy_tio_ref() WY__NOTHROW__;
    const struct ::termios& wy_tio_ref() const WY__NOTHROW__;

  private:
    struct ::termios _tio;

    // The following are for creating a single copy of 'default object' for
    // the definition of 'default object', and as early as possible, for easier maintenance
    //
    class DumyFuncSig {};
    WyTermios(DumyFuncSig) WY__NOTHROW__;
    static const struct ::termios& _default_object() WY__NOTHROW__;
};

#endif
