/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003
*/

#ifndef WYTIMESPEC_H__
#define WYTIMESPEC_H__
#define WYTIMESPEC_VERSION 33

#include "WyStr.h"
#include <ctime>    // for struct timespec
#include <limits>

// WyTimeSpec is the class for time interval.
//
class WyTimeSpec {
  public:
    typedef std::time_t SecondType;      // type of timespec.tv_sec
    typedef signed long int NanoSecType; // type of timespec.tv_nsec

    static const NanoSecType Wy__Giga=WY__CONST_GIGA;  // 1x10^9 (internal)
    static const int Wy__GigaDigit10=9;                // constant (internal)

    static const char class_name[];
    static const WyTimeSpec min() WY__TSPC()
           { return WyTimeSpec( std::numeric_limits<SecondType>::min(),
                                NanoSecType(-Wy__Giga+1)); };
    static const WyTimeSpec max() WY__TSPC()
           { return WyTimeSpec( std::numeric_limits<SecondType>::max(),
                                NanoSecType(Wy__Giga-1)); };
    WY_THROW_REPLY;

    WyTimeSpec() WY__TSPC();
    WyTimeSpec(const WyTimeSpec& c) WY__TSPC();
    WyTimeSpec(SecondType sec, NanoSecType nano) WY__TSPC(Reply);
    WyTimeSpec(SecondType sec) WY__TSPC();
    WyTimeSpec(double fvalue) WY__TSPC(Reply);

    // note: This member is not documented
    WyTimeSpec(long double fvalue) WY__TSPC(Reply);

    explicit  // no implicit conversion
    WyTimeSpec(const WyCSeg& cseg) WY__TSPC(Reply);

    WyTimeSpec(WyTimeSpec& c, Wy::ByMove_t) WY__TSPC();

    bool is_default() const WY__TSPC();
    SecondType second() const WY__TSPC();

    // Note: Let tm be an object of WyTimeSpec. The following equation holds.
    //       tm == WyTimeSpec(tm.secs(),tm.nano())
    //
    NanoSecType nano() const WY__TSPC();

    void reset() WY__TSPC();
    void reset(const WyTimeSpec &t) WY__TSPC();
    WyRet reset(SecondType sec, NanoSecType nano) WY__TSPC();
    void reset(SecondType sec) WY__TSPC();
    WyRet reset(const WyCSeg& cseg) WY__TSPC();
    WyRet reset(double) WY__TSPC();

    void swap(WyTimeSpec &tm) WY__TSPC();
    WyTimeSpec& operator =(const WyTimeSpec &rhs) WY__TSPC();
    WyTimeSpec& operator =(const WyCSeg& cseg) WY__TSPC(Reply);
    WyTimeSpec& operator =(double rhs) WY__TSPC(Reply);
    WyTimeSpec& operator =(SecondType sec) WY__TSPC();

    const WyTimeSpec operator -() const WY__TSPC(Reply);

    WyTimeSpec& operator ++() WY__TSPC(Reply);
    WyTimeSpec& operator --() WY__TSPC(Reply);
    const WyTimeSpec operator ++(int) WY__TSPC(Reply);
    const WyTimeSpec operator --(int) WY__TSPC(Reply);

    WyTimeSpec& operator +=(const WyTimeSpec &rhs) WY__TSPC(Reply);
    WyTimeSpec& operator +=(double rhs) WY__TSPC(Reply);
    WyTimeSpec& operator +=(SecondType rhs) WY__TSPC(Reply);
    WyTimeSpec& operator -=(const WyTimeSpec &rhs) WY__TSPC(Reply);
    WyTimeSpec& operator -=(double rhs) WY__TSPC(Reply);
    WyTimeSpec& operator -=(SecondType rhs) WY__TSPC(Reply);
    WyTimeSpec& operator *=(const WyTimeSpec &rhs) WY__TSPC(Reply);
    WyTimeSpec& operator *=(double rhs) WY__TSPC(Reply);
    WyTimeSpec& operator *=(SecondType rhs) WY__TSPC(Reply);
    WyTimeSpec& operator /=(const WyTimeSpec &rhs) WY__TSPC(Reply);
    WyTimeSpec& operator /=(double rhs) WY__TSPC(Reply);
    WyTimeSpec& operator /=(SecondType rhs) WY__TSPC(Reply);

    WyTimeSpec operator +(const WyTimeSpec &rhs) const WY__TSPC(Reply);
    WyTimeSpec operator +(double rhs) const WY__TSPC(Reply);
    WyTimeSpec operator +(SecondType rhs) const WY__TSPC(Reply);
    WyTimeSpec operator -(const WyTimeSpec &rhs) const WY__TSPC(Reply);
    WyTimeSpec operator -(double rhs) const WY__TSPC(Reply);
    WyTimeSpec operator -(SecondType rhs) const WY__TSPC(Reply);
    WyTimeSpec operator *(const WyTimeSpec &dem) const WY__TSPC(Reply);
    WyTimeSpec operator *(double dem) const WY__TSPC(Reply);
    WyTimeSpec operator *(SecondType dem) const WY__TSPC(Reply);
    WyTimeSpec operator /(const WyTimeSpec &dem) const WY__TSPC(Reply);
    WyTimeSpec operator /(double dem) const WY__TSPC(Reply);
    WyTimeSpec operator /(SecondType dem) const WY__TSPC(Reply);

    // [Syn] Comparisons
    //
    bool operator ==(const WyTimeSpec &rhs) const WY__TSPC();
    bool operator ==(double rhs) const WY__TSPC();
    bool operator ==(SecondType rhs) const WY__TSPC();
    bool operator !=(const WyTimeSpec &rhs) const WY__TSPC();
    bool operator !=(double rhs) const WY__TSPC();
    bool operator !=(SecondType rhs) const WY__TSPC();
    bool operator >(const WyTimeSpec &rhs) const WY__TSPC();
    bool operator >(double rhs) const WY__TSPC();
    bool operator >(SecondType rhs) const WY__TSPC();
    bool operator >=(const WyTimeSpec &rhs) const WY__TSPC();
    bool operator >=(double rhs) const WY__TSPC();
    bool operator >=(SecondType rhs) const WY__TSPC();
    bool operator <(const WyTimeSpec &rhs) const WY__TSPC();
    bool operator <(double rhs) const WY__TSPC();
    bool operator <(SecondType rhs) const WY__TSPC();
    bool operator <=(const WyTimeSpec &rhs) const WY__TSPC();
    bool operator <=(double rhs) const WY__TSPC();
    bool operator <=(SecondType rhs) const WY__TSPC();

    WyRet add(SecondType sec, NanoSecType nsec) WY__TSPC();
    WyRet add(const WyTimeSpec& tm) WY__TSPC();
    WyRet add(SecondType n) WY__TSPC();

    WyRet sub(SecondType sec, NanoSecType nsec) WY__TSPC();
    WyRet sub(const WyTimeSpec& tm) WY__TSPC();
    WyRet sub(SecondType n) WY__TSPC();

    WyRet mul(SecondType sec, NanoSecType nsec) WY__TSPC();
    WyRet mul(const WyTimeSpec& n) WY__TSPC();
    WyRet mul(SecondType n) WY__TSPC();

    WyRet div(SecondType sec, NanoSecType nsec) WY__TSPC();
    WyRet div(const WyTimeSpec& n) WY__TSPC();
    WyRet div(SecondType n) WY__TSPC();

    // [Internal]
    //
    const ::timespec& wy_timespec() const
              { return _tspc; };
    ::timespec& wy_timespec()
              { return _tspc; };
  private:
    class _Assertion;

    // [Syn] Normalize sec/nsec
    //       sec/nsec is normalized iff
    //
    //       1. (sec>0) &&(nsec>=0)&&(nsec<Giga)
    //       2. (sec<0) &&(nsec<=0)&&(nsec>-Giga)
    //       3. (sec==0)&&(nsec>-Giga)&&(nsec<Giga)
    // 
    // [Ret] Ok    
    //       Wym_ERANGE sec/nsec normalization failure
    // 
    // [Property]
    //       nsec>0 --> sec>=0
    //       nsec<0 --> sec<=0
    //       T= sec + nsec/Giga
    //
    static WyRet _normalize(long& sec, long& nsec) WY__TSPC();
    static WyRet _normalize(long long int& sec, long long int& nsec) WY__TSPC();
    static inline long long _mul(const WyTimeSpec& lhs, const WyTimeSpec& rhs) WY__NOTHROW__;
    static inline long long _mul(const WyTimeSpec& lhs, SecondType rhs) WY__NOTHROW__;

    inline WyRet _init(const char* buf,size_t blen) WY__TSPC();

    struct ::timespec _tspc;

    // disabled
    WyTimeSpec& operator =(int);

    friend WyTimeSpec operator+(double lhs, const WyTimeSpec& rhs) WY__TSPC(WyTimeSpec::Reply);
    friend WyTimeSpec operator+(SecondType lhs, const WyTimeSpec& rhs) WY__TSPC(WyTimeSpec::Reply);
    friend WyTimeSpec operator-(double lhs, const WyTimeSpec& rhs) WY__TSPC(WyTimeSpec::Reply);
    friend WyTimeSpec operator-(SecondType lhs, const WyTimeSpec& rhs) WY__TSPC(WyTimeSpec::Reply);
    friend WyTimeSpec operator*(double lhs, const WyTimeSpec& rhs) WY__TSPC(WyTimeSpec::Reply);
    friend WyTimeSpec operator*(SecondType lhs, const WyTimeSpec& rhs) WY__TSPC(WyTimeSpec::Reply);
    friend WyTimeSpec operator/(double lhs, const WyTimeSpec& rhs) WY__TSPC(WyTimeSpec::Reply);
    friend WyTimeSpec operator/(SecondType lhs, const WyTimeSpec& rhs) WY__TSPC(WyTimeSpec::Reply);

    friend bool operator ==(double lhs, const WyTimeSpec& rhs) WY__TSPC()
                { return rhs==lhs; };
    friend bool operator !=(double lhs, const WyTimeSpec& rhs) WY__TSPC()
                { return rhs!=lhs; };
    friend bool operator >(double lhs, const WyTimeSpec& rhs) WY__TSPC()
                { return rhs<lhs; };
    friend bool operator >=(double lhs, const WyTimeSpec& rhs) WY__TSPC()
                { return rhs<=lhs; };
    friend bool operator <(double lhs, const WyTimeSpec& rhs) WY__TSPC()
                { return rhs>lhs; };
    friend bool operator <=(double lhs, const WyTimeSpec& rhs) WY__TSPC() 
                { return rhs>=lhs; };
    friend bool operator ==(SecondType lhs, const WyTimeSpec& rhs) WY__TSPC()
                { return rhs==lhs; };
    friend bool operator !=(SecondType lhs, const WyTimeSpec& rhs) WY__TSPC()
                { return rhs!=lhs; };
    friend bool operator >(SecondType lhs, const WyTimeSpec& rhs) WY__TSPC()
                { return rhs<lhs; };
    friend bool operator >=(SecondType lhs, const WyTimeSpec& rhs) WY__TSPC()
                { return rhs<=lhs; };
    friend bool operator <(SecondType lhs, const WyTimeSpec& rhs) WY__TSPC()
                { return rhs>lhs; };
    friend bool operator <=(SecondType lhs, const WyTimeSpec& rhs) WY__TSPC()
                { return rhs>=lhs; };
};

namespace std {

template<>
class numeric_limits<WyTimeSpec> {
  public:
    static const bool is_specialized=false;
    static const bool is_signed=true;
    static const bool is_integer=false;
    static const int  digits= std::numeric_limits<WyTimeSpec::SecondType>::digits+29;
    static const int  digits10= std::numeric_limits<WyTimeSpec::SecondType>::digits10+9;
    static const int  max_exponent10=18;
    static const int  min_exponent10=-18;
    static const bool is_exact=false;
    static const int  radix= std::numeric_limits<WyTimeSpec::SecondType>::radix;
    static WyTimeSpec min() WY__TSPC() { return WyTimeSpec::min(); };
    static WyTimeSpec max() WY__TSPC() { return WyTimeSpec::max(); };
};

}; // End of std

namespace Wy {

 WyTimeSpec abs(const WyTimeSpec&) WY__TSPC(WyTimeSpec::Reply);
 // not implemented
 WyTimeSpec _sqrt(const WyTimeSpec&) WY__TSPC(WyTimeSpec::Reply);

 // SRC: WyTimeSpec_sleep.cpp
 WyRet sleep(WyTimeSpec tsp);
 WyRet sleep(WyTimeSpec req, WyTimeSpec& rem);
 WyRet sleep_till(WyTimeSpec wtm);

 // SRC: WyTimeSpec_systime.cpp
 WyTimeSpec now();
 WyRet set_systime(const WyTimeSpec& t);

 // SRC: wy_strnum_timespec.cpp
 WyRet _strnum(WyTimeSpec& res, const WyCSeg& cseg, const Wy::WyRadix& radix) WY__TSPC();
 WyRet _strnum(WyTimeSpec& res, const WyCSeg& cseg, Wy::WyRadix& radix) WY__TSPC();
 WyRet _strnum(WyTimeSpec& res, const WyCSeg& cseg) WY__TSPC();
 WyRet _strnum(WyTimeSpec& res, WyCSeg& cseg, const Wy::WyRadix& radix) WY__TSPC();
 WyRet _strnum(WyTimeSpec& res, WyCSeg& cseg, Wy::WyRadix& radix) WY__TSPC();
 WyRet _strnum(WyTimeSpec& res, WyCSeg& cseg) WY__TSPC(); 

 // SRC: wy_scanum_timespec.cpp
 WyRet _scanum(WyTimeSpec&, WyCSeg&, const Wy::WyRadix&) WY__TSPC();
 WyRet _scanum(WyTimeSpec&, WyCSeg&, Wy::WyRadix&) WY__TSPC();
 WyRet _scanum(WyTimeSpec&, WyCSeg&) WY__TSPC();

 inline double to_double(const WyTimeSpec& src) WY__TSPC()
 { return static_cast<double>(src.second())+
          static_cast<double>(src.nano()*1e-9); };

 // SRC: wy_ntoa_timespec.cpp
 size_t _ntoa(char* buf, size_t blen, const WyTimeSpec& num, Wy::WyRadix radix, size_t frdig) WY__TSPC();
 WyRet _mkstr(WyStr& str, WyTimeSpec num, Wy::WyRadix radix, size_t frdig) WY__TSPC();
 WyStr wrd(const WyTimeSpec&, Wy::WyRadix=10, size_t=9);

}; // end namespace Wy

class WyByteFlow;
// SRC: wy_iouty_timespec.cpp
WyByteFlow& operator <<(WyByteFlow& bf, WyTimeSpec num);

#endif
