/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003
*/

#define WYLIB_SOURCE
#include "WyTimeSpec.h"
#include <limits>
#include <ctime>
#include <cmath>              // for modf
#include <cstdlib>

WyTimeSpec::WyTimeSpec(double fvalue) WY__TSPC(Reply)
{
 if(std::isnan(fvalue)) {
   WY_THROW( Reply(Wym_ERANGE) );
 }
 if(std::isinf(fvalue)) {
   WY_THROW( Reply(Wym_ERANGE) );
 }
 double it,fr=std::modf(fvalue,&it);
 if(it> static_cast<double>( std::numeric_limits<SecondType>::max())) {
   if((it<=static_cast<double>( std::numeric_limits<SecondType>::max())+1.0)&&(fr<0.1)) { 
     _tspc.tv_sec=WyTimeSpec::max()._tspc.tv_sec;
     _tspc.tv_nsec=WyTimeSpec::max()._tspc.tv_nsec;
   } else {
     WY_THROW( Reply(Wym_ERANGE) );
   }
 } else if(it< static_cast<double>( std::numeric_limits<SecondType>::min() )) {
   if((it>=static_cast<double>( std::numeric_limits<SecondType>::min())-1.0)&&(fr>-0.1)) { 
     _tspc.tv_sec=WyTimeSpec::min()._tspc.tv_sec;
     _tspc.tv_nsec=WyTimeSpec::min()._tspc.tv_nsec;
   } else {
     WY_THROW( Reply(Wym_ERANGE) );
   }
 }
 _tspc.tv_sec =static_cast<SecondType>(it);
 _tspc.tv_nsec=static_cast<NanoSecType>(fr*WY__CONST_GIGA);
};

WyTimeSpec::WyTimeSpec(long double fvalue) WY__TSPC(Reply)
{
 if(std::isnan(fvalue)) {
   WY_THROW( Reply(Wym_ERANGE) );
 }
 if(std::isinf(fvalue)) {
   WY_THROW( Reply(Wym_ERANGE) );
 }
 long double it,fr=std::modf(fvalue,&it);
 if((it> static_cast<long double>( std::numeric_limits<SecondType>::max() ))||
    (it< static_cast<long double>( std::numeric_limits<SecondType>::min() ))) {
   WY_THROW( Reply(Wym_ERANGE) );
 }
 _tspc.tv_sec =static_cast<SecondType>(it);
 _tspc.tv_nsec=static_cast<NanoSecType>(fr*WY__CONST_GIGA);
};

WyRet WyTimeSpec::reset(double fvalue) WY__TSPC()
{
 if(std::isnan(fvalue)) {
   WY_RETURN( Wym_ERANGE );
 }
 if(std::isinf(fvalue)) {
   WY_RETURN( Wym_ERANGE );
 }
 double it,fr=std::modf(fvalue,&it);
 if(it> static_cast<double>( std::numeric_limits<SecondType>::max())) {
   if((it<=static_cast<double>( std::numeric_limits<SecondType>::max())+1.0)&&(fr<0.1)) { 
     _tspc.tv_sec=WyTimeSpec::max()._tspc.tv_sec;
     _tspc.tv_nsec=WyTimeSpec::max()._tspc.tv_nsec;
   } else {
     WY_RETURN( Wym_ERANGE );
   }
 } else if(it< static_cast<double>( std::numeric_limits<SecondType>::min() )) {
   if((it>=static_cast<double>( std::numeric_limits<SecondType>::min())-1.0)&&(fr>-0.1)) { 
     _tspc.tv_sec=WyTimeSpec::min()._tspc.tv_sec;
     _tspc.tv_nsec=WyTimeSpec::min()._tspc.tv_nsec;
   } else {
     WY_RETURN( Wym_ERANGE );
   }
 }
 _tspc.tv_sec =static_cast<SecondType>(it);
 _tspc.tv_nsec=static_cast<NanoSecType>(fr*WY__CONST_GIGA);
 return(Ok);
};

WyTimeSpec& WyTimeSpec::operator =(double fvalue) WY__TSPC(Reply)
{
 if(std::isnan(fvalue)) {
   WY_THROW( Reply(Wym_ERANGE) );
 }
 if(std::isinf(fvalue)) {
   WY_THROW( Reply(Wym_ERANGE) );
 }
 double it,fr=std::modf(fvalue,&it);
 if(it> static_cast<double>( std::numeric_limits<SecondType>::max())) {
   if((it<=static_cast<double>( std::numeric_limits<SecondType>::max())+1.0)&&(fr<0.1)) { 
     _tspc.tv_sec=WyTimeSpec::max()._tspc.tv_sec;
     _tspc.tv_nsec=WyTimeSpec::max()._tspc.tv_nsec;
   } else {
     WY_THROW( Reply(Wym_ERANGE) );
   }
 } else if(it< static_cast<double>( std::numeric_limits<SecondType>::min() )) {
   if((it>=static_cast<double>( std::numeric_limits<SecondType>::min())-1.0)&&(fr>-0.1)) { 
     _tspc.tv_sec=WyTimeSpec::min()._tspc.tv_sec;
     _tspc.tv_nsec=WyTimeSpec::min()._tspc.tv_nsec;
   } else {
     WY_THROW( Reply(Wym_ERANGE) );
   }
 }
 _tspc.tv_sec =static_cast<SecondType>(it);
 _tspc.tv_nsec=static_cast<NanoSecType>(fr*WY__CONST_GIGA);
 return(*this);
};

WyTimeSpec & WyTimeSpec::operator +=(double rhs) WY__TSPC(Reply)
{
 return this->operator +=( WyTimeSpec(rhs) );
};

WyTimeSpec & WyTimeSpec::operator -=(double rhs) WY__TSPC(Reply)
{
 return this->operator -=( WyTimeSpec(rhs) );
};

WyTimeSpec& WyTimeSpec::operator *=(double rhs) WY__TSPC(Reply)
{
 return this->operator *=( WyTimeSpec(rhs) );
};

WyTimeSpec& WyTimeSpec::operator /=(double rhs) WY__TSPC(Reply)
{
 return this->operator /=( WyTimeSpec(rhs) );
};

WyTimeSpec WyTimeSpec::operator +(double rhs) const WY__TSPC(Reply)
{
 return *this+WyTimeSpec(rhs);
};

WyTimeSpec WyTimeSpec::operator -(double rhs) const WY__TSPC(Reply)
{
 return *this-WyTimeSpec(rhs);
};

WyTimeSpec WyTimeSpec::operator *(double rhs) const WY__TSPC(Reply)
{
 WyTimeSpec tmp(*this);
 return tmp*= WyTimeSpec(rhs);
};

WyTimeSpec WyTimeSpec::operator /(double rhs) const WY__TSPC(Reply)
{
 WyTimeSpec tmp(*this);
 return tmp/= WyTimeSpec(rhs);
};

WyTimeSpec operator+(double lhs, const WyTimeSpec& rhs) WY__TSPC(WyTimeSpec::Reply)
{
 return WyTimeSpec(lhs)+rhs;
};

WyTimeSpec operator-(double lhs, const WyTimeSpec& rhs) WY__TSPC(WyTimeSpec::Reply)
{
 return WyTimeSpec(lhs)-rhs;
};

WyTimeSpec operator*(double lhs, const WyTimeSpec& rhs) WY__TSPC(WyTimeSpec::Reply)
{
 return WyTimeSpec(lhs)*rhs;
};

WyTimeSpec operator/(double lhs, const WyTimeSpec& rhs) WY__TSPC(WyTimeSpec::Reply)
{
 return WyTimeSpec(lhs)/rhs;
};

// note: If following the basic rule: ctor(lhs) first than compare, throw may occur or
//       try/catch might be needed. Impl just takes the easy way.
//
bool WyTimeSpec::operator ==(double rhs) const WY__TSPC()
{ return Wy::to_double(*this)==rhs; };

bool WyTimeSpec::operator !=(double rhs) const WY__TSPC()
{ return Wy::to_double(*this)!=rhs; };

bool WyTimeSpec::operator >(double rhs) const WY__TSPC()
{ return Wy::to_double(*this)>rhs; };

bool WyTimeSpec::operator >=(double rhs) const WY__TSPC()
{ return Wy::to_double(*this)>=rhs; };

bool WyTimeSpec::operator <(double rhs) const WY__TSPC()
{ return Wy::to_double(*this)<rhs; };

bool WyTimeSpec::operator <=(double rhs) const WY__TSPC()
{ return Wy::to_double(*this)<=rhs; };
