/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003
*/

#define WYLIB_SOURCE
#include "WyTimeSpec.h"
#include "Wy_AtDestroy.hh"
#include <limits>
#include <ctime>
#include <cstdlib>
#include <new>

WyRet Wy::sleep(WyTimeSpec tsp)
{
 WY__CANCEL_POINT;
 const int v( ::nanosleep(&tsp.wy_timespec(),NULL) );
 WY__CANCEL_POINT;
 if(v!=0) {
   WY_RETURN(errno);
 }
 return(Ok);
};

WyRet Wy::sleep(WyTimeSpec req, WyTimeSpec& rem)
{
 WY__CANCEL_POINT;
 const int v( ::nanosleep(&req.wy_timespec(),&rem.wy_timespec()) );
 WY__CANCEL_POINT;
 if(v!=0) {
   WY_RETURN(errno);
 }
 return(Ok);
};

// [Interna] Dedicated for sleep_till
//           Unlock mtx and destroy mtx/cond
//
void static wy__sleep_till_cleanup(::pthread_cond_t& cond, ::pthread_mutex_t& mtx)
{
 ::pthread_mutex_unlock(&mtx);
 ::pthread_mutex_destroy(&mtx);
 ::pthread_cond_destroy(&cond);
};

WyRet Wy::sleep_till(WyTimeSpec wtm)
{
 ::pthread_cond_t cond;
 ::pthread_mutex_t mtx;

 ::pthread_mutex_init(&mtx,NULL);   // manual: always ok
 const int v=::pthread_mutex_lock(&mtx);
 if(v!=0) {
   ::pthread_mutex_destroy(&mtx);
   WY_THROW( WyRet(v) );  // internal error
 }
 ::pthread_cond_init(&cond,NULL);   // manual: always ok
 Wy_AtDestroy2<void,::pthread_cond_t&,::pthread_mutex_t&> 
   rrid(wy__sleep_till_cleanup,cond,mtx);

 WY_RETURN( ::pthread_cond_timedwait(&cond,&mtx,&wtm.wy_timespec()) );
};
