/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003
*/

#define WYLIB_SOURCE
#include "WyTimeSpec.h"
#include "WyMutex.h"
#include <limits>
#include <ctime>
#include <cstdlib>
#include <new>

//#undef _POSIX_TIMERS        // for check

#ifndef _POSIX_TIMERS
#include <sys/time.h>

// Mutex dedicated for sycronizing system time access (gettimeofday)
//
// 1. This has an initialization bug
// 2. May be removed (rarely used till now)
//
static WyMutex wy__cmtx;
#endif

WyTimeSpec Wy::now()
try {
#ifdef _POSIX_TIMERS
 struct timespec tm_now;
 if(::clock_gettime(CLOCK_REALTIME,&tm_now)!=0) {
   // man. only EINVAL which should not happen here.
   WY_THROW( WyRet(errno) );
 }

 //
 // note: WyTimeSpec ctor is assumed not to throw in this condition
 //
 return WyTimeSpec(tm_now.tv_sec,tm_now.tv_nsec);
#else
 struct ::timeval tm_now;
 WyRet r;
 if((r=wy__cmtx.lock())!=Ok) {
   WY_THROW(r);
 }
 //
 // guess this function is not thread-safe, so a mutex supplied.
 //
 if(::gettimeofday(&tm_now,0)!=0) {
   //
   // man indicates all errors are associated with time zone, which is not 
   // used int time(), so it is not supposed to happen here.
   //
   const int v=errno;
   wy__cmtx.unlock();    // not check the error, for result is the same.
   WY_THROW( WyRet(v) );
 }
 if((r=wy__cmtx.unlock())!=Ok) {
   WY_THROW(r);
 }

 //
 // note: WyTimeSpec ctor is assumed not to throw in this condition
 //
 return WyTimeSpec(tm_now.tv_sec,tm_now.tv_usec*1000);
#endif
}
catch(const WyTimeSpec::Reply& e) {
 WY_THROW( WyRet(e) );
}
catch(const WyRet& e) {
 WY_THROW( WyRet(e) );
};

WyRet Wy::set_systime(const WyTimeSpec& tm)
{
#ifdef _POSIX_TIMERS
 if(::clock_settime(CLOCK_REALTIME,&tm.wy_timespec())!=0) {
   const int v=errno;
   switch(v) {
     case EPERM: WY_RETURN(Wym_EPERM);
     case EINVAL: WY_RETURN(Wym_ERANGE); // guess this reply
     case EFAULT: WY_THROW(WyRet(Wym_EFAULT)); // should not happen
     default:    WY_RETURN( WyRet(v) );  
   };
 }
 return(Ok);
#else
 WyRet r;
 struct timeval tm_now;

 if((r=wy__cmtx.lock())!=Ok) {
   WY_THROW(r);
 }
 tm_now.tv_sec=wytm.second();
 tm_now.tv_usec=static_cast<long>(wytm.nano()/1000);

 // guess this function is not thread-safe, so a mutex supplied
 const int en=(::settimeofday(&tm_now,0)==0)? 0:errno;
 if((r=wy__cmtx.unlock())!=Ok) {
   WY_THROW(r);
 }
 switch(en) {
   case 0: return(Ok);
   case EPERM: WY_RETURN(Wym_EPERM);
   case EINVAL: // FALL_THROUGH
   case EFAULT: WY_THROW(WyRet(Wym_EFAULT));  // should not happen
   default:    WY_RETURN( WyRet(en) ); 
 };
#endif
};

