/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   This file should conform to the documentation in file wy_dirfile.3wy
*/

#ifndef WY_DIRFILE_H__
#define WY_DIRFILE_H__
#define WY_DIRFILE_VERSION 33

#include "WyFileHandle.h"

class WyStr;
class WyDirEnt;
class Wy__DirHandle;

class Wy_DirFile {
    // Hidden
    const Wy_DirFile & operator =(const Wy_DirFile &);
    bool operator==(const Wy_DirFile&) const;
    bool operator!=(const Wy_DirFile&) const;

    Wy__DirHandle *_hptr;
   
  public:
    static const char class_name[];
    WY_THROW_REPLY;  // declaraion of Reply

    Wy_DirFile();
    Wy_DirFile(const Wy_DirFile &src);
    Wy_DirFile(const char* dirname);
    Wy_DirFile(const WyStr& dirname);

    // [Move Constructor]
    //
    // Note: src must be whole object except in initialization list
    //
    Wy_DirFile(Wy_DirFile &src, Wy::ByMove_t) WY__TSPC();

    virtual ~Wy_DirFile();

    bool is_default() const WY__TSPC();
    WyFileHandle fh() const WY__TSPC();
    WyRet stat(WyFileStat& filestat) const;

    virtual WyRet reset();
    virtual WyRet reset(const Wy_DirFile& src);
    virtual WyRet reset(const char* dirname);
    virtual WyRet reset(const WyStr& dirname);
    virtual WyRet _swap(Wy_DirFile& f) WY__TSPC();
    virtual Wy_DirFile* _alloc(WyRet&) const;

    virtual WyRet _readdir(WyDirEnt& dirent);

    virtual WyRet _rewinddir();

    static WyRet create(const char* pathname,mode_t mode);
    static WyRet create(const WyStr& pathname,mode_t mode);

    static WyRet unlink(const char* pathname);
    static WyRet unlink(const WyStr& pathname);

    // [Internal] Get the reference count
    //
    // [Ret] reference count of this object
    //       zero if object is default
    //
    size_t wy_ref_count() const;

};

#endif
