/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   Base protocol class of socket address, inherited by WySockAddrLocal, WySockAddrINet4
   WySockAddrINet6
*/

#ifndef WY_SOCKADDR_H__
#define WY_SOCKADDR_H__
#define WY_SOCKADDR_VERSION 33

#include "WyRet.h"
#include <sys/types.h>
#include <sys/socket.h>

class WyStr;

class Wy_SockAddr {
    friend class WySockFile;
  public:
    virtual ~Wy_SockAddr() {};

    int family() const WY__TSPC()
         { return this->c_sockaddr().sa_family; };

    // note: This member is primarily for the intended Wy::wrd(const Wy_SockAddr&)
    //       to work. Sth about this function is not yet explored.
    //
    virtual WyRet notation(WyStr& str) const=0;

  protected:

    // Note: sockaddr.sa_family is setup by the derived class and should
    //       be constant throughout object life time.
    //
    virtual void reset() WY__TSPC()=0;

    virtual WyRet _swap(Wy_SockAddr& saddr) WY__TSPC()=0;

    virtual Wy_SockAddr* _alloc(WyRet& r) const=0;

    // Note: Derived class should always setup sockaddr.sa_family data member.
    //       Codes in WySockFile depends on this information to identify and 
    //       r/w the structure.
    //
    //  struct ::sockaddr& c_sockaddr()
    //  size_t c_sockaddr_size() const
    //  socklen_t& c_addrlen()
    //
    // Note: These membesrs may be called in constructor of this object
    //       Function behavior can not depend on *this object state
    //
    // [Ret] reference of struct sockaddr
    //
    virtual struct ::sockaddr& c_sockaddr() WY__TSPC()=0;
    virtual const struct ::sockaddr& c_sockaddr() const WY__TSPC()=0;

    // [Syn] Get the size (maximal, static) of the structure that c_sockaddr() returns.
    //       This value is normally initially written to *c_addrlen before calling 
    //       accept/getsockname/getpeername
    //
    // [Ret] The structure size c_sockaddr() pointed to
    //
    virtual size_t c_sockaddr_size() const WY__TSPC()=0;

    // [Syn] Get/Set the socklen_t object, which is for the effective length of the
    //       address structure pointed by c_sockaddr (<=c_sockaddr_size)
    // 
    // [Ret] Reference of the variable that specifies the used number of bytes
    //       c_sockaddr() references
    //
    virtual  socklen_t& c_addrlen() WY__TSPC()=0;
    virtual const socklen_t& c_addrlen() const WY__TSPC()=0;
};

#endif
