/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   Build: make chk_cseg
*/

#include "WyRet.h"
#include "WyCSeg.h"
#include "WyStr.h"
#include <iostream>

#if WYCSEG_VERSION!=33
#error Test code is for WYCSEG_VERSION 33
#endif

static void verify_equ(const WyCSeg& s1,const WyCSeg& s2)
{
 if(s1.begin()==NULL) {
   WY_THROW( WyRet() );
 }
 if(s1.end()==NULL) {
   WY_THROW( WyRet() );
 }
 if(s1.size()!=s2.size()) {
   WY_THROW( WyRet() );
 }
 if(s1!=s2) {
   WY_THROW( WyRet() );
 }
 if(s1==s2) {
 } else {
   WY_THROW( WyRet() );
 }
 if(s1.size()!=0) {
   if(std::memcmp(s1.begin(),s2.begin(),s1.size())!=0) {
     WY_THROW( WyRet() );
   }
 }
};

#define CHK_EQU(s1,s2) try { verify_equ(s1,s2); } catch(...) { WY_THROW( WyRet()); };

static void ck_cseg(void)
{
 WyRet r;

 {
  WyCSeg s0;
  if(s0.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(s0.size()!=0) {
    WY_THROW( WyRet() );
  }
  if(s0.begin()==NULL) {
    WY_THROW( WyRet() );
  }
  if(s0.end()==NULL) {
    WY_THROW( WyRet() );
  }
  if(*s0.begin()!=0) {
    WY_THROW( WyRet() );  // feature test
  }

  const char cstr1[]="123";
  WyCSeg s1(cstr1),s2(s1),s3="xyz";
  CHK_EQU(s1,s2);
  if(s1.size()!=3) {
    WY_THROW( WyRet() );
  }
  if((s1[0]!='1')||(s1[1]!='2')||(s1[2]!='3')) {
    WY_THROW( WyRet() );
  }
  if((s3[0]!='x')||(s3[1]!='y')||(s3[2]!='z')) {
    WY_THROW( WyRet() );
  }
  WyCSeg s4="",s5(s3);
  CHK_EQU(s0,s4);
  CHK_EQU(s3,s5);

  // chk operator==(const WyCSeg&)
  if(s0==s4) {
  } else {
    WY_THROW( WyRet() );
  }
  if(s0!=s4) {
    WY_THROW( WyRet() );
  } else {
  }

  if(s0=="") {
  } else {
    WY_THROW( WyRet() );
  }
  if(s0!="") {
    WY_THROW( WyRet() );
  } else {
  }

  if(""==s0) {
  } else {
    WY_THROW( WyRet() );
  }
  if(""!=s0) {
    WY_THROW( WyRet() );
  } else {
  }

  if(s1==s3) {
    WY_THROW( WyRet() );
  }
  if(s1!=s3) {
  } else {
    WY_THROW( WyRet() );
  }
  if(s1==s3.begin()) {
    WY_THROW( WyRet() );
  }
  if(s1!=s3.begin()) {
  } else {
    WY_THROW( WyRet() );
  }
  if(s1.begin()==s3) {
    WY_THROW( WyRet() );
  }
  if(s1.begin()!=s3) {
  } else {
    WY_THROW( WyRet() );
  }

  if(s3==s5) {
  } else {
    WY_THROW( WyRet() );
  }
  if(s3!=s5) {
    WY_THROW( WyRet() );
  }
  if(s3==s5.begin()) {
  } else {
    WY_THROW( WyRet() );
  }
  if(s3!=s5.begin()) {
    WY_THROW( WyRet() );
  }
  if(s3.begin()==s5) {
  } else {
    WY_THROW( WyRet() );
  }
  if(s3.begin()!=s5) {
    WY_THROW( WyRet() );
  }

  s0.reset(cstr1,size_t(0));
  s1.reset(cstr1+1,size_t(0));
  CHK_EQU(s0,s1);
 }

 {
  WyCSeg s1("abc",4),s2(s1);
  CHK_EQU(s1,s2);
  if(s1.size()!=4) {
    WY_THROW( WyRet() );
  }
  if((s1[0]!='a')||(s1[1]!='b')||(s1[2]!='c')||(s1[3]!=0)) {
    WY_THROW( WyRet() );
  }
 }

 // WyCSeg(const char*, const char*) 
 {
  const char* tmp="helo";
  WyCSeg s1(tmp,tmp+4);
  if(s1.size()!=4) {
    WY_THROW( WyRet() );
  }
  if((s1[0]!='h')||(s1[1]!='e')||(s1[2]!='l')||(s1[3]!='o')) {
    WY_THROW( WyRet() );
  }

  WyCSeg s2("","");
  if(s2.size()!=0) {
    WY_THROW( WyRet() );
  }
 }

 // reset()/reset(const WyCSeg&)
 {
  WyCSeg s1("x",1),s2("a");
  if(s1.size()!=1) {
    WY_THROW( WyRet() );
  }
  s2.reset(s1);
  CHK_EQU(s1,s2);

  if(s1.is_default()) {
    WY_THROW( WyRet() );
  }
  s1.reset();
  if(s1.is_default()==false) {
    WY_THROW( WyRet() );
  }
 }

 // reset(const char*)
 {
  WyCSeg s1("x");
  CHK_EQU(s1,s1);
  if((r=s1.reset("123"))!=Ok) {
    WY_THROW(r);
  } 
  if((s1[0]!='1')||(s1[1]!='2')||(s1[2]!='3')) {
    WY_THROW( WyRet() );
  }
 }

 // reset(const char*,size_t)
 {
  WyCSeg s1("x");
  if((r=s1.reset("ijk",3))!=Ok) {
    WY_THROW(r);
  } 
  if(s1.size()!=3) {
    WY_THROW( WyRet() );
  } 
  if((s1[0]!='i')||(s1[1]!='j')||(s1[2]!='k')) {
    WY_THROW( WyRet() );
  }
 }

 // reset(const char*, const char*) 
 {
  const char* tmp="helo";
  WyCSeg s1(tmp,size_t(0)),s2;
  // note: s1("",0) may or may be be default, may depend on compiler 
  //       optimization level
  //
  if(s1.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(s2.is_default()==false) {
    WY_THROW( WyRet() );
  }
  
  if((r=s1.reset(tmp,tmp+4))!=Ok) {
    WY_THROW( WyRet() );
  }
  if(s1.size()!=4) {
    WY_THROW( WyRet() );
  }
  if((s1[0]!='h')||(s1[1]!='e')||(s1[2]!='l')||(s1[3]!='o')) {
    WY_THROW( WyRet() );
  }
  if((r=s1.reset(tmp,tmp))!=Ok) {
    WY_THROW( WyRet() );
  }
  if(s1.size()!=0) {
    WY_THROW( WyRet() );
  }
  s1.reset();
  if(s1.is_default()==false) {
    WY_THROW( WyRet() );
  }
 }

 // chk front()/back()
 {
  WyCSeg s="asdf";
  if(s.front()!='a') {
    WY_THROW( WyRet() );
  }
  if(s.back()!='f') {
    WY_THROW( WyRet() );
  }

  s.reset();
  try {
    s.front();
    WY_THROW( WyRet() );
  }
  catch(const WyCSeg::Reply& e) {
    if(e!=Wym_ENOENT) {
      WY_THROW( WyRet() );
    }
  };
  try {
    s.back();
    WY_THROW( WyRet() );
  }
  catch(const WyCSeg::Reply& e) {
    if(e!=Wym_ENOENT) {
      WY_THROW( WyRet() );
    }
  };
 }

 // chk move_begin()
 {
  const char cstr[]="234";
  WyCSeg s(cstr);
  s.move_begin(1);
  if(s.size()!=2) {
    WY_THROW( WyRet() );
  }
  if(s.front()!='3') {
    WY_THROW( WyRet() );
  }
  if(*s.move_begin(1)!='4') {
    WY_THROW( WyRet() );
  }
  if(s.size()!=1) {
    WY_THROW( WyRet() );
  }
  if(s.front()!='4') {
    WY_THROW( WyRet() );
  }
  s.move_begin(1);
  if(s.size()!=0) {
    WY_THROW( WyRet() );
  }
  s.move_begin(-3);
  if(s.size()!=3) {
    WY_THROW( WyRet() );
  }
  s.move_begin(s.size());
  if(s.size()!=0) {
    WY_THROW( WyRet() );
  }

  s=cstr;
  if(s!=cstr) {
    WY_THROW( WyRet() );
  }
  try {
    s.move_begin(s.size()+1);
    WY_THROW( WyRet() );
  }
  catch(const WyCSeg::Reply& e) {
    if(e!=Wym_ERANGE) {
      WY_THROW( WyRet(e) );
    }
  };
 }

 // chk move_end()
 {
  const char cstr[]="234";
  WyCSeg s(cstr);
  s.move_end(-1);
  if(s.size()!=2) {
    WY_THROW( WyRet() );
  }
  if(s.back()!='3') {
    WY_THROW( WyRet() );
  }
  if(*s.move_end(-1)!='3') {
    WY_THROW( WyRet() );
  }
  if(s.size()!=1) {
    WY_THROW( WyRet() );
  }
  if(s.back()!='2') {
    WY_THROW( WyRet() );
  }
  s.move_end(-1);
  if(s.size()!=0) {
    WY_THROW( WyRet() );
  }
  s.move_end(2);
  if(s.size()!=2) {
    WY_THROW( WyRet() );
  }
  s.move_end(-s.size());
  if(s.size()!=0) {
    WY_THROW( WyRet() );
  }

  s=cstr;
  if(s!=cstr) {
    WY_THROW( WyRet() );
  }
  try {
    s.move_end(-1-s.size());
    WY_THROW( WyRet() );
  }
  catch(const WyCSeg::Reply& e) {
    if(e!=Wym_ERANGE) {
      WY_THROW( WyRet(e) );
    }
  };
 }

 // swap(..)
 {
  const char *cstr1="ab";
  const char *cstr2="fgg";
  WyCSeg s1(cstr1);
  WyCSeg s2(cstr2);
  if(s1.begin()!=cstr1) {
    WY_THROW( WyRet() );
  }
  if(s1.size()!=::strlen(cstr1)) {
    WY_THROW( WyRet() );
  }
  if(s2.begin()!=cstr2) {
    WY_THROW( WyRet() );
  }
  if(s2.size()!=::strlen(cstr2)) {
    WY_THROW( WyRet() );
  }
  s1.swap(s2);
  if(s1.begin()!=cstr2) {
    WY_THROW( WyRet() );
  }
  if(s1.size()!=::strlen(cstr2)) {
    WY_THROW( WyRet() );
  }
  if(s2.begin()!=cstr1) {
    WY_THROW( WyRet() );
  }
  if(s2.size()!=::strlen(cstr1)) {
    WY_THROW( WyRet() );
  }
 }

 // reset(const char*)
 {
  WyCSeg s1("x");
  if((r=s1.reset("B"))!=Ok) {
    WY_THROW(r);
  }
  if(s1.size()!=1) {
    WY_THROW( WyRet() );
  }
  if(s1[0]!='B') {
    WY_THROW( WyRet() );
  }
 }

 // reset(const WyCSeg&)
 {
  WyCSeg s1("x"),s2("aa");
  s1.reset(s2);
  CHK_EQU(s1,s2);
  if(s1.size()!=2) {
    WY_THROW( WyRet() );
  }
 }

 // operator=(const WyCSeg&)
 {
  WyCSeg s1("x"),s2("aa");
  if(s1.size()!=1) {
    WY_THROW( WyRet() );
  }
  if(&(s1=s2)!=&s1) {
    WY_THROW( WyRet() );
  }
  CHK_EQU(s1,s2);
 }

 // operator=(const char*)
 {
  WyCSeg s1("xx");
  const char abc[]="abc";
  if(s1.size()!=2) {
    WY_THROW( WyRet() );
  }
  if(&(s1=abc)!=&s1) {
    WY_THROW( WyRet() );
  }
  if(s1.size()!=3) {
    WY_THROW( WyRet() );
  }
  if(s1!=abc) {
    WY_THROW( WyRet() );
  }
  
  try {
    s1=NULL;
    WY_THROW( WyRet() );
  }
  catch(const WyCSeg::Reply& e) {
    if(e!=Wym_EFAULT) {
      WY_THROW( WyRet() );
    }
  };
 }

 // chk operator [](size_t)
 {
  WyCSeg s="1234567";
  if((s[0]!='1')||(s[1]!='2')) {
    WY_THROW( WyRet() );
  }
  
  try {
    s[s.size()];
    WY_THROW( WyRet() );
  }
  catch(const WyCSeg::Reply& e) {
    if(e!=Wym_EINVAL) {
      WY_THROW( WyRet() );
    }
  };
 }

 // chk cseg(size_t,size_t)
 {
  WyCSeg s="1234567";
  if(Wy::_strcmp(s.cseg(0,3),"123")!=0) {
    WY_THROW( WyRet() );
  }
  if(Wy::_strcmp(s.cseg(0,3),"12")<0) {
    WY_THROW( WyRet() );
  }
  if(Wy::_strcmp(s.cseg(0,3),"1234")>0) {
    WY_THROW( WyRet() );
  }
  if(Wy::_strcmp(s.cseg(1,2),"23")!=0) {
    WY_THROW( WyRet() );
  }
  if(Wy::_strcmp(s.cseg(5,6),"67")!=0) {
    WY_THROW( WyRet() );
  }
  if(s.cseg(7,6).size()!=0) {
    WY_THROW( WyRet() );
  }

  if(Wy::_strcmp(s.cseg(5),"67")!=0) {
    WY_THROW( WyRet() );
  }
  if(s.cseg(s.size()).size()!=0) {
    WY_THROW( WyRet() );
  }
  if(Wy::_strcmp(s.cseg(s.size()),"")!=0) {
    WY_THROW( WyRet() );
  }
 }

 // is_overlap(const WyCSeg&)
 {
  WyCSeg s1("x"),s2("aa");
  if(s1.is_overlap(s2)) {
    WY_THROW( WyRet() );
  }
 }

 // end()
 {
  WyCSeg s1("3456",3);
  if(s1.size()!=3) {
    WY_THROW( WyRet() );
  }
  if(s1.end()!=s1.begin()+s1.size()) { 
    WY_THROW( WyRet() );
  }
 }

 // chk average coding for const char*
 {
  WyCSeg c1="abtt";
  WyCSeg c2("hdlm");
  if(c1.begin()==c2.begin()) {  // for supressing unused warning
    WY_THROW( WyRet() );
  }
 }

 // chk move ctor
 {
  char buf[2*sizeof(WyCSeg)];
  ::memset(buf,0,sizeof(buf));
  WyCSeg *p=reinterpret_cast<WyCSeg*>(buf);
  new(p) WyCSeg("bn",2);
  new(p+1) WyCSeg(*p,Wy::ByMove);
  if(Wy::_strcmp(*(p+1),WyCSeg("bn",2))!=0) {
    (p+1)->~WyCSeg();
    WY_THROW( WyRet() );
  }
  (p+1)->~WyCSeg();
 }

 // chk wy_set_begin
 {
  char tbuf[]={'a','s','d'};
  WyCSeg tmp(tbuf,sizeof(tbuf));
  if(tmp[0]!='a') {
    WY_THROW( WyRet() );
  }
  tmp.wy_set_begin(tmp.begin()+1);
  if(tmp.size()!=2) {
    WY_THROW( WyRet() );
  }
  if(tmp[0]!='s') {
    WY_THROW( WyRet() );
  }
 }
};

static void ck_cseg_find(void)
{
 // find(size_t&,size_t,const WyCSeg&)
 {
  const WyCSeg ss("\xffx1111231237890\xff");
  size_t fidx;
  fidx=100;
  if(ss.find(fidx,2,WyCSeg())==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=100) {
    WY_THROW( WyRet() );
  }
  if(ss.find(fidx,2,WyCSeg("123"))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=5) {
    WY_THROW( WyRet() );
  }
  if(ss.find(fidx,2,WyCSeg("123"))==false) {
    WY_THROW( WyRet() );
  }
  if(ss.find(fidx,0,WyCSeg("0\xff"))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=14) {
    WY_THROW( WyRet() );
  }
  if(ss.find(fidx,0,WyCSeg("0\xff"))==false) {
    WY_THROW( WyRet() );
  }
  fidx=100;
  if(ss.find(fidx,100,WyCSeg("\xff"))==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=100) {
    WY_THROW( WyRet() );
  }
  if(ss.find(fidx,0,ss)==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=0) {
    WY_THROW( WyRet() );
  }
  if(WyStr(ss).cseg(0,1).find(fidx,0,ss)==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=0) {
    WY_THROW( WyRet() );
  }
  if(ss.find(fidx,0,ss.cseg(1))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=1) {
    WY_THROW( WyRet() );
  }

  const WyCSeg ss2;
  fidx=100;
  if(ss2.find(fidx,0,WyCSeg())==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=100) {
    WY_THROW( WyRet() );
  }
 }

 // rfind(size_tr&,size_t,const WyCSeg&)
 {
  const WyCSeg ss("\xffx1111231237890\xff");
  size_t fidx;
  fidx=100;
  if(ss.rfind(fidx,2,WyCSeg())==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=100) {
    WY_THROW( WyRet() );
  }
  if(ss.rfind(fidx,200,WyCSeg())==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=100) {
    WY_THROW( WyRet() );
  }
  if(ss.rfind(fidx,200,WyCSeg("123"))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=8) {
    WY_THROW( WyRet() );
  }
  if(ss.rfind(fidx,8,WyCSeg("123"))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=5) {
    WY_THROW( WyRet() );
  }
  if(ss.rfind(fidx,5,WyCSeg("123"))==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=5) {
    WY_THROW( WyRet() );
  }
  if(ss.rfind(fidx,ss.size(),WyCSeg("0\xff"))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=14) {
    WY_THROW( WyRet() );
  }
  fidx=100;
  if(ss.rfind(fidx,ss.size()-2,WyCSeg("0\xff"))==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=100) {
    WY_THROW( WyRet() );
  }
  if(ss.rfind(fidx,0,WyCSeg("0\xff"))==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=100) {
    WY_THROW( WyRet() );
  }
  if(ss.rfind(fidx,10,WyCSeg("\xff"))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=0) {
    WY_THROW( WyRet() );
  }
  if(ss.rfind(fidx,0,ss)==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=0) {
    WY_THROW( WyRet() );
  }
  if(ss.rfind(fidx,1,ss)==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=0) {
    WY_THROW( WyRet() );
  }
  if(ss.rfind(fidx,2,ss.cseg(1))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=1) {
    WY_THROW( WyRet() );
  }

  const WyCSeg ss2;
  fidx=100;
  if(ss2.rfind(fidx,0,WyCSeg())==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=100) {
    WY_THROW( WyRet() );
  }
 }

 // findc(size_t&,size_t,char)
 {
  const WyCSeg ss("\xffx1111231237890\xff");
  size_t fidx;
  fidx=100;
  if(ss.findc(fidx,100,'1')==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=100) {
    WY_THROW( WyRet() );
  }
  if(ss.findc(fidx,0,'1')==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=2) {
    WY_THROW( WyRet() );
  }
  if(ss.findc(fidx,0,'A')==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=2) {
    WY_THROW( WyRet() );
  }
  if(ss.findc(fidx,1,'\xff')==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=ss.size()-1) {
    WY_THROW( WyRet() );
  }
 }

 // findc(size_t&,size_t,const WyCSeg&)
 {
  const WyCSeg ss("\xffx1111231237890\xff");
  size_t fidx;
  fidx=100;
  if(ss.findc(fidx,0,WyCSeg("92"))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=6) {
    WY_THROW( WyRet() );
  }
  if(ss.findc(fidx,7,WyCSeg("92"))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=9) {
    WY_THROW( WyRet() );
  }
  if(ss.findc(fidx,0,WyCSeg("_+=-*"))==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=9) {
    WY_THROW( WyRet() );
  }
  if(ss.findc(fidx,1,WyCSeg("_+=-*\xff"))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=ss.size()-1) {
    WY_THROW( WyRet() );
  }
  fidx=100;
  if(ss.findc(fidx,ss.size(),ss)==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=100) {
    WY_THROW( WyRet() );
  }
 }

 // findc_not(size_t&,size_t,char)
 {
  const WyCSeg ss("\xffx1111231237890\xff");
  size_t fidx;
  fidx=100;
  if(ss.findc_not(fidx,100,'1')==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=100) {
    WY_THROW( WyRet() );
  }
  if(ss.findc_not(fidx,2,'1')==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=6) {
    WY_THROW( WyRet() );
  }
 }

 // findc_not(size_t&,size_t,const WyCSeg&)
 {
  const WyCSeg ss("\xffx1111231237890\xff");
  size_t fidx;
  fidx=100;
  if(ss.findc_not(fidx,1,WyCSeg())==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=1) {
    WY_THROW( WyRet() );
  }
  if(ss.findc_not(fidx,0,ss)==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=1) {
    WY_THROW( WyRet() );
  }
  if(ss.findc_not(fidx,0,WyCSeg("92"))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=0) {
    WY_THROW( WyRet() );
  }
  if(ss.findc_not(fidx,0,WyCSeg("19x2\xff"))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=7) {
    WY_THROW( WyRet() );
  }
  if(ss.findc_not(fidx,8,WyCSeg("19x2\xff"))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=10) {
    WY_THROW( WyRet() );
  }
 }

 // rfindc(size_t&,size_t,char)
 {
  const WyCSeg ss("\xffx1111231237890\xff");
  size_t fidx;
  fidx=100;
  if(ss.rfindc(fidx,100,'x')==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=1) {
    WY_THROW( WyRet() );
  }
  if(ss.rfindc(fidx,1,'x')==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=1) {
    WY_THROW( WyRet() );
  }
  if(ss.rfindc(fidx,10,'z')==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=1) {
    WY_THROW( WyRet() );
  }
  if(ss.rfindc(fidx,10,'1')==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=8) {
    WY_THROW( WyRet() );
  }
  if(WyCSeg().rfindc(fidx,0,'a')==true) {
    WY_THROW( WyRet() );
  }
 }

 // rfindc(size_t&,size_t,const WyCSeg&)
 {
  const WyCSeg ss("\xffx1111231237890\xff");
  size_t fidx;
  fidx=100;
  if(WyCSeg().rfindc(fidx,100,ss)==true) {
    WY_THROW( WyRet() );
  }
  if(ss.rfindc(fidx,100,WyCSeg("321"))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=10) {
    WY_THROW( WyRet() );
  }
  if(ss.rfindc(fidx,10,WyCSeg("321"))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=9) {
    WY_THROW( WyRet() );
  }
  if(ss.rfindc(fidx,9,WyCSeg("321"))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=8) {
    WY_THROW( WyRet() );
  }
  if(ss.rfindc(fidx,0,ss)==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=8) {
    WY_THROW( WyRet() );
  }
  if(ss.rfindc(fidx,9,WyCSeg("abc\xff"))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=0) {
    WY_THROW( WyRet() );
  }
  if(ss.rfindc(fidx,ss.size(),WyCSeg())==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=0) {
    WY_THROW( WyRet() );
  }
  if(ss.rfindc(fidx,9,WyCSeg("_+-)("))==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=0) {
    WY_THROW( WyRet() );
  }
 }

 // rfindc_not(size_t&,size_t,char)
 {
  const WyCSeg ss("\xffx1111231237890\xff");
  size_t fidx;
  fidx=100;
  if(ss.rfindc_not(fidx,100,'x')==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=ss.size()-1) {
    WY_THROW( WyRet() );
  }
  if(ss.rfindc_not(fidx,fidx,'x')==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=ss.size()-2) {
    WY_THROW( WyRet() );
  }
  if(WyCSeg().rfindc_not(fidx,100,'x')==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=ss.size()-2) {
    WY_THROW( WyRet() );
  }
  if(ss.rfindc_not(fidx,100,'\xff')==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=ss.size()-2) {
    WY_THROW( WyRet() );
  }
  if(ss.rfindc_not(fidx,5,'1')==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=1) {
    WY_THROW( WyRet() );
  }
  if(ss.rfindc_not(fidx,fidx,'1')==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=0) {
    WY_THROW( WyRet() );
  }
 }

 // rfindc_not(size_t&,size_t,const WyCSeg&)
 {
  const WyCSeg ss("\xffx1111231237890\xff");
  size_t fidx;
  fidx=100;
  if(WyCSeg().rfindc_not(fidx,100,ss)==true) {
    WY_THROW( WyRet() );
  } 
  if(ss.rfindc_not(fidx,100,WyCSeg("03\xff"))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=13) {
    WY_THROW( WyRet() );
  }
  if(ss.rfindc_not(fidx,13,WyCSeg("03\xff"))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=12) {
    WY_THROW( WyRet() );
  }
  if(ss.rfindc_not(fidx,13,WyCSeg("789123"))==false) {
    WY_THROW( WyRet() );
  }
  if(fidx!=1) {
    WY_THROW( WyRet() );
  }
  if(ss.rfindc_not(fidx,1,WyCSeg("x\xff"))==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=1) {
    WY_THROW( WyRet() );
  }
  if(ss.rfindc_not(fidx,0,WyCSeg())==true) {
    WY_THROW( WyRet() );
  }
  if(fidx!=1) {
    WY_THROW( WyRet() );
  }
  for(size_t t=1; t<=ss.size(); ++t) { // all character in ss is not in WyCSeg()
    if(ss.rfindc_not(fidx,t,WyCSeg())==false) {
      WY_THROW( WyRet() );
    }
    if(fidx!=t-1) {
      WY_THROW( WyRet() );
    }
  }
 }
};

static void ck_cseg_error(void)
{
  WyRet r;

  // WyCseg(const char*)
  try {
    WyCSeg s1((const char*)0);
    s1.size();
    WY_THROW( WyRet() );
  }
  catch(const WyCSeg::Reply& e) {
    if(e!=Wym_EFAULT) {
      throw;
    }
    // FALL_THROUGH
  };

  // WyCseg(const char*,size_t)
  try {
    WyCSeg s1((const char*)0,1);
    s1.size();
    WY_THROW( WyRet() );
  }
  catch(const WyCSeg::Reply& e) {
    if(e!=Wym_EFAULT) {
      throw;
    }
    // FALL_THROUGH
  };

  // reset
  {
   WyCSeg s1("");
   if(s1.size()!=0) {
     WY_THROW( WyRet() );
   }
   if((r=s1.reset((const char*)0))!=Wym_EFAULT) {
     WY_THROW(r);
   }
   if((r=s1.reset((const char*)0,size_t(1)))!=Wym_EFAULT) {
     WY_THROW(r);
   }
   if((r=s1.reset((const char*)"",(size_t)0))!=Ok) {
     WY_THROW(r);
   }
   if(s1.is_default()==false) {
     WY_THROW( WyRet() );
   }

   WyCSeg s2("",size_t(0));
   if(s2.is_default()==false) {
     WY_THROW( WyRet() );
   }
  }

  // reset(const char*)
  {
    WyCSeg s1("");
    if((r=s1.reset((const char*)0))!=Wym_EFAULT) {
      WY_THROW(r);
    }
  }

  // chk cseg(size_t,size_t)
  try {
    WyCSeg s1("rrr");
    s1.cseg(4,1).size();
    WY_THROW( WyRet() );
  }
  catch(const WyCSeg::Reply& e) {
    if(e!=Wym_EINVAL) {
      throw;
    }
    // FALL_THROUGH
  };

  // chk cseg(size_t)
  try {
    WyCSeg s1("rrr");
    s1.cseg(4).size();
    WY_THROW( WyRet() );
  }
  catch(const WyCSeg::Reply& e) {
    if(e!=Wym_EINVAL) {
      throw;
    }
    // FALL_THROUGH
  };
};

static void ck_cseg_aux(void)
{
  // chk strcmp
  {
   WyCSeg s1("helo"), s2("heloX"),s3(s1);
  CHK_EQU(s1,s3);
   if(Wy::_strcmp(s1,s3)!=0) {
     WY_THROW( WyRet() );
   }
   if(Wy::_strcmp(s1,s1)!=0) {
     WY_THROW( WyRet() );
   }
   if(Wy::_strcmp(s1,s2)>=0) {
     WY_THROW( WyRet() );
   }
   if(Wy::_strcmp(s2,s1)<=0) {
     WY_THROW( WyRet() );
   }

   if(Wy::_strcmp(s1.begin(),s3.begin())!=0) {
     WY_THROW( WyRet() );
   }
   if(Wy::_strcmp(s1.begin(),s1.begin())!=0) {
     WY_THROW( WyRet() );
   }
   if(Wy::_strcmp(s1.begin(),s2.begin())>=0) {
     WY_THROW( WyRet() );
   }
   if(Wy::_strcmp(s2.begin(),s1.begin())<=0) {
     WY_THROW( WyRet() );
   }

   {
     try {
       WyCSeg s1("");
       Wy::_strcmp(s1,(const char*)0);
       WY_THROW( WyRet() );
     }
     catch(const WyCSeg::Reply& e) {
       if(e!=Wym_EFAULT) {
         WY_THROW( WyRet(e) );
       }
       // FALL_THROUGH
     };
   }

   {
     try {
       WyCSeg s1("");
       Wy::_strcmp((const char*)0,s1);
       WY_THROW( WyRet() );
     }
     catch(const WyCSeg::Reply& e) {
       if(e!=Wym_EFAULT) {
         WY_THROW( WyRet(e) );
       }
       // FALL_THROUGH
     };
   }

  }

  // chk _strcpy
  {
   WyCSeg s1("helo");
   char buf[6]={0,0,0,0,0,0};
   Wy::_strcpy(buf+1,s1);
   if(std::memcmp(buf+1,s1.begin(),s1.size())!=0) {
     WY_THROW( WyRet() );
   }
   if((buf[0]!=0)||(buf[sizeof(buf)-1]!=0)) {
     WY_THROW( WyRet() );
   }

  }

  // chk _strmove
  {
   char buf[]="0123456789aaabbb";
   Wy::_strmove(buf+1,buf+3,6);
   if(std::memcmp(buf,"0345678789aaabbb",16)!=0) {
     WY_THROW( WyRet() );
   }

   char buf2[]="0123456789aaabbb";
   Wy::_strmove(buf2+4,buf2,6);
   if(std::memcmp(buf2,"0123012345aaabbb",16)!=0) {
     WY_THROW( WyRet() );
   }
  }

  // chk _strchr
  {
   WyCSeg s("aacddfgg");
   const char *p;

   p=Wy::_strchr(s,'a');
   if(p==NULL) {
     WY_THROW( WyRet() );
   }
   if(*p!='a') {
     WY_THROW( WyRet() );
   }
   if(p!=&s[0]) {
     WY_THROW( WyRet() );
   }

   p=Wy::_strchr(s,'d');
   if(p==NULL) {
     WY_THROW( WyRet() );
   }
   if(p!=&s[3]) {
     WY_THROW( WyRet() );
   }
   if(*p!='d') {
     WY_THROW( WyRet() );
   }

   p=Wy::_strchr(s,'g');
   if(p==NULL) {
     WY_THROW( WyRet() );
   }
   if(p!=&s[6]) {
     WY_THROW( WyRet() );
   }
   if(*p!='g') {
     WY_THROW( WyRet() );
   }

   p=Wy::_strchr(s,'x');
   if(p!=NULL) {
     WY_THROW( WyRet() );
   }
   p=Wy::_strchr(WyCSeg(),'x');
   if(p!=NULL) {
     WY_THROW( WyRet() );
   }
  }

  // chk _strrchr
  {
   WyCSeg s("aacddfgg");
   const char *p;

   p=Wy::_strrchr(s,'a');
   if(p==NULL) {
     WY_THROW( WyRet() );
   }
   if(*p!='a') {
     WY_THROW( WyRet() );
   }
   if(p!=&s[1]) {
     WY_THROW( WyRet() );
   }

   p=Wy::_strrchr(s,'d');
   if(p==NULL) {
     WY_THROW( WyRet() );
   }
   if(p!=&s[4]) {
     WY_THROW( WyRet() );
   }
   if(*p!='d') {
     WY_THROW( WyRet() );
   }

   p=Wy::_strrchr(s,'g');
   if(p==NULL) {
     WY_THROW( WyRet() );
   }
   if(p!=&s[7]) {
     WY_THROW( WyRet() );
   }
   if(*p!='g') {
     WY_THROW( WyRet() );
   }

   p=Wy::_strrchr(s,'x');
   if(p!=NULL) {
     WY_THROW( WyRet() );
   }
   p=Wy::_strrchr(WyCSeg(),'x');
   if(p!=NULL) {
     WY_THROW( WyRet() );
   }
  }

  // chk _strstr
  {
   WyCSeg s("rrstrstt");
   const char *p;
   
   p=Wy::_strstr(s,"rst");
   if(p==NULL) {
     WY_THROW( WyRet() );
   }
   if(p!=&s[1]) {
     WY_THROW( WyRet() );
   }

   p=Wy::_strstr(s,"rstt");
   if(p==NULL) {
     WY_THROW( WyRet() );
   }
   if(p!=&s[4]) {
     WY_THROW( WyRet() );
   }

   p=Wy::_strstr(s,"");
   if(p==NULL) {
     WY_THROW( WyRet() );
   }
   if(p!=&s[0]) {
     WY_THROW( WyRet() );   // feature probably
   }

   p=Wy::_strstr(s,"rss");
   if(p!=NULL) {
     WY_THROW( WyRet() );
   }
   p=Wy::_strstr(WyCSeg(),"rss");
   if(p!=NULL) {
     WY_THROW( WyRet() );
   }
  }
};

static const WyStr chdr(
                  "+---------------+\n"
                  "| main() caught:|\n"
                  "+---------------+\n");
int main(void) throw()
try {
 std::cout << "Checking WyCSeg.h ... ";
 ck_cseg();
 ck_cseg_aux();
 ck_cseg_error();
 ck_cseg_find();
 std::cout << "Checked Ok\n";
 return(0);
}
catch(const WyRet& e) {
 std::cerr << chdr.c_str() << Wy::wrd(e).c_str() << std::endl;
 return(-1);
}
catch(const std::exception& e) {
 std::cerr << chdr.c_str() << "std::exception" << std::endl;
 return(-1);
}
catch(...) {
 std::cerr << chdr.c_str() << "unknown unwind" << std::endl;
 return(-1);
};
