/* Copyright is licensed under GNU LGPL.                 by IJ. Wang 2007

  Build: make chk_mf
*/
#include "WyStr.h"
#include "Wy_MF.hh"
#include <iostream>

struct S1 {
 S1() : m_cnt(0) {};
 mutable int m_cnt;
 void f0() { ++m_cnt; };
 void f0c() const { ++m_cnt; };
 int if0() { return ++m_cnt; };
 int if0c() const { return ++m_cnt; };
 void f1(int v) { m_cnt=v; };
 void f1c(int v) const { m_cnt=v; };
 int if1(int v) { return m_cnt=v; };
 int if1c(int v) const { return m_cnt=v; };
};

struct S_vbase {
 virtual ~S_vbase() {};
};

struct S2 : public S_vbase {  // inherits a virtual class
 S2() : m_cnt(0) {};
 mutable int m_cnt;
 void f0() { ++m_cnt; };
 void f0c() const { ++m_cnt; };
 int if0() { return ++m_cnt; };
 int if0c() const { return ++m_cnt; };
 void f1(int v) { m_cnt=v; };
 void f1c(int v) const { m_cnt=v; };
 int if1(int v) { return m_cnt=v; };
 int if1c(int v) const { return m_cnt=v; };
};

template<typename FuncClass, typename ResType, typename SlotBase>
static void ck_mf0()
{
 typedef ResType(FuncClass::*MFType)();
 const MFType RefPtr=&FuncClass::if0;
 typedef Wy_MF0<SlotBase,ResType> ClassMF;

 FuncClass s;

 // Wy_MF0(), reset()
 {
  ClassMF tmp,tmp1;
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(tmp==tmp1) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW( WyRet() );
  }

  tmp.reset();
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW( WyRet() );
  }
  try {
    tmp();
    WY_THROW( WyRet() );
  }
  catch(const typename ClassMF::Reply& e) {
    if(e!=Wym_EFAULT) {
      WY_THROW( WyRet(e) );
    }
  };
 }
 
 // Wy_MF0(obj,mem_func)/(const Wy_MF0&)
 {
  s.m_cnt=441;
  ClassMF tmp(s,RefPtr);
  if(tmp.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp()!=442) {
    WY_THROW( WyRet() );
  }
  if(s.m_cnt!=442) {
    WY_THROW( WyRet() );
  }

  const ClassMF tmp2(tmp);
  if(tmp2==tmp) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp2!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp2()!=443) {
    WY_THROW( WyRet() );
  }
  if(s.m_cnt!=443) {
    WY_THROW( WyRet() );
  }

  try {
    ClassMF tmp(s,(MFType)NULL);
    WY_THROW( WyRet() );
  }
  catch(const typename ClassMF::Reply& e) {
    if(e!=Wym_EFAULT) {
      WY_THROW( WyRet(e) );
    }
  };

 }

 // move ctor
 {
  ClassMF arr[2];
  if(arr[0].is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(arr[1].is_default()==false) {
    WY_THROW( WyRet() );
  }
  new(&arr[0]) ClassMF(arr[0],Wy::ByMove);
  if(arr[0].is_default()==false) {
    WY_THROW( WyRet() );
  }

  ClassMF tmp(s,RefPtr);
  arr[1]=tmp;
  if(arr[1].is_default()) {
    WY_THROW( WyRet() );
  }
  new(&arr[0]) ClassMF(arr[1],Wy::ByMove);
  if(arr[0]!=tmp) {
    WY_THROW( WyRet() );
  }
 }

 // reset(const Wy_FM0&)
 {
  const ClassMF mf0, mf1(s,RefPtr);
  ClassMF tmp;
  if(mf0.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(mf1.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }

  tmp.reset(mf1);
  if(tmp!=mf1) {
    WY_THROW( WyRet() );
  }
  tmp.reset(mf0);
  if(tmp!=mf0) {
    WY_THROW( WyRet() );
  }
 }

 // reset(FuncClass&, MFType)
 {
  ClassMF tmp;
  if(tmp.is_default()) {} else {
    WY_THROW( WyRet() );
  }
  WyRet r;
  if((r=tmp.reset(s,RefPtr))!=Ok) {
    WY_THROW( WyRet() );
  }
  if(tmp!= ClassMF(s,RefPtr)) {
    WY_THROW( WyRet() );
  }
  if((r=tmp.reset(s,(MFType)NULL))!=Wym_EFAULT) {
    WY_THROW(r);
  }
 }

 // operator =(const Wy_FM0&)
 {
  const ClassMF mf0, mf1(s,RefPtr);
  ClassMF tmp;
  if(mf0.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(mf1.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }

  if((tmp=mf1)!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=mf1) {
    WY_THROW( WyRet() );
  }
  if((tmp=mf0)!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=mf0) {
    WY_THROW( WyRet() );
  }
 }
};

template<typename FuncClass, typename ResType, typename SlotBase>
static void ck_mfv0()
{
 typedef ResType(FuncClass::*MFType)();
 const MFType RefPtr=&FuncClass::f0;
 typedef Wy_MF0<SlotBase,ResType> ClassMF;
 FuncClass s;

 // Wy_MF0(), reset()
 {
  ClassMF tmp,tmp1;
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(tmp==tmp1) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW( WyRet() );
  }

  tmp.reset();
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW( WyRet() );
  }

  try {
    tmp();
    WY_THROW( WyRet() );
  }
  catch(const typename ClassMF::Reply& e) {
    if(e!=Wym_EFAULT) {
      WY_THROW( WyRet(e) );
    }
  };
 }
 
 // Wy_MF0(obj,mem_func)/(const Wy_MF0&)
 {
  s.m_cnt=441;
  ClassMF tmp(s,RefPtr);
  if(tmp.is_default()) {
    WY_THROW( WyRet() );
  }

  tmp();
  if(s.m_cnt!=442) {
    WY_THROW( WyRet() );
  }

  const ClassMF tmp2(tmp);
  if(tmp2==tmp) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp2!=tmp) {
    WY_THROW( WyRet() );
  }

  tmp2();
  if(s.m_cnt!=443) {
    WY_THROW( WyRet() );
  }

  try {
    ClassMF tmp(s,(MFType)NULL);
    WY_THROW( WyRet() );
  }
  catch(const typename ClassMF::Reply& e) {
    if(e!=Wym_EFAULT) {
      WY_THROW( WyRet(e) );
    }
  };

 }

 // move ctor
 {
  ClassMF arr[2];
  if(arr[0].is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(arr[1].is_default()==false) {
    WY_THROW( WyRet() );
  }
  new(&arr[0]) ClassMF(arr[0],Wy::ByMove);
  if(arr[0].is_default()==false) {
    WY_THROW( WyRet() );
  }

  ClassMF tmp(s,RefPtr);
  arr[1]=tmp;
  if(arr[1].is_default()) {
    WY_THROW( WyRet() );
  }
  new(&arr[0]) ClassMF(arr[1],Wy::ByMove);
  if(arr[0]!=tmp) {
    WY_THROW( WyRet() );
  }
 }

 // reset(const Wy_FM0&)
 {
  const ClassMF mf0, mf1(s,RefPtr);
  ClassMF tmp;
  if(mf0.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(mf1.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }

  tmp.reset(mf1);
  if(tmp!=mf1) {
    WY_THROW( WyRet() );
  }
  tmp.reset(mf0);
  if(tmp!=mf0) {
    WY_THROW( WyRet() );
  }
 }

 // reset(FuncClass&, MFType)
 {
  ClassMF tmp;
  if(tmp.is_default()) {} else {
    WY_THROW( WyRet() );
  }
  WyRet r;
  if((r=tmp.reset(s,RefPtr))!=Ok) {
    WY_THROW( WyRet() );
  }
  if(tmp!= ClassMF(s,RefPtr)) {
    WY_THROW( WyRet() );
  }
  if((r=tmp.reset(s,(MFType)NULL))!=Wym_EFAULT) {
    WY_THROW(r);
  }
 }

 // operator =(const Wy_FM0&)
 {
  const ClassMF mf0, mf1(s,RefPtr);
  ClassMF tmp;
  if(mf0.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(mf1.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }

  if((tmp=mf1)!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=mf1) {
    WY_THROW( WyRet() );
  }
  if((tmp=mf0)!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=mf0) {
    WY_THROW( WyRet() );
  }
 }
};

template<typename FuncClass, typename ResType, typename SlotBase>
static void ck_mf0_const()
{
 typedef ResType(FuncClass::*MFType)()const;
 const MFType RefPtr=&FuncClass::if0c;
 typedef Wy_MF0_const<SlotBase,ResType> ClassMF;
 const FuncClass s;

 // Wy_MF0_const(), reset()
 {
  ClassMF tmp,tmp1;
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(tmp==tmp1) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW( WyRet() );
  }

  tmp.reset();
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW( WyRet() );
  }

  try {
    tmp();
    WY_THROW( WyRet() );
  }
  catch(const typename ClassMF::Reply& e) {
    if(e!=Wym_EFAULT) {
      WY_THROW( WyRet(e) );
    }
  };
 }
 
 // Wy_MF0_const(obj,mem_func)/(const Wy_MF0_const&)
 {
  s.m_cnt=441;
  ClassMF tmp(s,RefPtr);
  if(tmp.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp()!=442) {
    WY_THROW( WyRet() );
  }
  if(s.m_cnt!=442) {
    WY_THROW( WyRet() );
  }

  const ClassMF tmp2(tmp);
  if(tmp2==tmp) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp2!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp2()!=443) {
    WY_THROW( WyRet() );
  }
  if(s.m_cnt!=443) {
    WY_THROW( WyRet() );
  }

  try {
    ClassMF tmp(s,(MFType)NULL);
    WY_THROW( WyRet() );
  }
  catch(const typename ClassMF::Reply& e) {
    if(e!=Wym_EFAULT) {
      WY_THROW( WyRet(e) );
    }
  };

 }

 // move ctor
 {
  ClassMF arr[2];
  if(arr[0].is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(arr[1].is_default()==false) {
    WY_THROW( WyRet() );
  }
  new(&arr[0]) ClassMF(arr[0],Wy::ByMove);
  if(arr[0].is_default()==false) {
    WY_THROW( WyRet() );
  }

  ClassMF tmp(s,RefPtr);
  arr[1]=tmp;
  if(arr[1].is_default()) {
    WY_THROW( WyRet() );
  }
  new(&arr[0]) ClassMF(arr[1],Wy::ByMove);
  if(arr[0]!=tmp) {
    WY_THROW( WyRet() );
  }
 }

 // reset(const Wy_FM0&)
 {
  const ClassMF mf0, mf1(s,RefPtr);
  ClassMF tmp;
  if(mf0.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(mf1.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }

  tmp.reset(mf1);
  if(tmp!=mf1) {
    WY_THROW( WyRet() );
  }
  tmp.reset(mf0);
  if(tmp!=mf0) {
    WY_THROW( WyRet() );
  }
 }

 // reset(FuncClass&, MFType)
 {
  ClassMF tmp;
  if(tmp.is_default()) {} else {
    WY_THROW( WyRet() );
  }
  WyRet r;
  if((r=tmp.reset(s,RefPtr))!=Ok) {
    WY_THROW( WyRet() );
  }
  if(tmp!= ClassMF(s,RefPtr)) {
    WY_THROW( WyRet() );
  }
  if((r=tmp.reset(s,(MFType)NULL))!=Wym_EFAULT) {
    WY_THROW(r);
  }
 }

 // operator =(const Wy_FM0&)
 {
  const ClassMF mf0, mf1(s,RefPtr);
  ClassMF tmp;
  if(mf0.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(mf1.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }

  if((tmp=mf1)!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=mf1) {
    WY_THROW( WyRet() );
  }
  if((tmp=mf0)!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=mf0) {
    WY_THROW( WyRet() );
  }
 }
};

template<typename FuncClass, typename ResType, typename SlotBase>
static void ck_mfv0_const()
{
 typedef ResType(FuncClass::*MFType)()const;
 const MFType RefPtr=&FuncClass::f0c;
 typedef Wy_MF0_const<SlotBase,ResType> ClassMF;
 const FuncClass s;

 // Wy_MF0_const(), reset()
 {
  ClassMF tmp,tmp1;
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(tmp==tmp1) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW( WyRet() );
  }

  tmp.reset();
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW( WyRet() );
  }

  try {
    tmp();
    WY_THROW( WyRet() );
  }
  catch(const typename ClassMF::Reply& e) {
    if(e!=Wym_EFAULT) {
      WY_THROW( WyRet(e) );
    }
  };
 }
 
 // Wy_MF0_const(obj,mem_func)/(const Wy_MF0_const&)
 {
  s.m_cnt=441;
  ClassMF tmp(s,RefPtr);
  if(tmp.is_default()) {
    WY_THROW( WyRet() );
  }
  tmp();
  if(s.m_cnt!=442) {
    WY_THROW( WyRet() );
  }

  const ClassMF tmp2(tmp);
  if(tmp2==tmp) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp2!=tmp) {
    WY_THROW( WyRet() );
  }
  tmp2();
  if(s.m_cnt!=443) {
    WY_THROW( WyRet() );
  }

  try {
    ClassMF tmp(s,(MFType)NULL);
    WY_THROW( WyRet() );
  }
  catch(const typename ClassMF::Reply& e) {
    if(e!=Wym_EFAULT) {
      WY_THROW( WyRet(e) );
    }
  };

 }

 // move ctor
 {
  ClassMF arr[2];
  if(arr[0].is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(arr[1].is_default()==false) {
    WY_THROW( WyRet() );
  }
  new(&arr[0]) ClassMF(arr[0],Wy::ByMove);
  if(arr[0].is_default()==false) {
    WY_THROW( WyRet() );
  }

  ClassMF tmp(s,RefPtr);
  arr[1]=tmp;
  if(arr[1].is_default()) {
    WY_THROW( WyRet() );
  }
  new(&arr[0]) ClassMF(arr[1],Wy::ByMove);
  if(arr[0]!=tmp) {
    WY_THROW( WyRet() );
  }
 }

 // reset(const Wy_FM0&)
 {
  const ClassMF mf0, mf1(s,RefPtr);
  ClassMF tmp;
  if(mf0.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(mf1.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }

  tmp.reset(mf1);
  if(tmp!=mf1) {
    WY_THROW( WyRet() );
  }
  tmp.reset(mf0);
  if(tmp!=mf0) {
    WY_THROW( WyRet() );
  }
 }

 // reset(FuncClass&, MFType)
 {
  ClassMF tmp;
  if(tmp.is_default()) {} else {
    WY_THROW( WyRet() );
  }
  WyRet r;
  if((r=tmp.reset(s,RefPtr))!=Ok) {
    WY_THROW( WyRet() );
  }
  if(tmp!= ClassMF(s,RefPtr)) {
    WY_THROW( WyRet() );
  }
  if((r=tmp.reset(s,(MFType)NULL))!=Wym_EFAULT) {
    WY_THROW(r);
  }
 }

 // operator =(const Wy_FM0&)
 {
  const ClassMF mf0, mf1(s,RefPtr);
  ClassMF tmp;
  if(mf0.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(mf1.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }

  if((tmp=mf1)!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=mf1) {
    WY_THROW( WyRet() );
  }
  if((tmp=mf0)!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=mf0) {
    WY_THROW( WyRet() );
  }
 }
};

template<typename FuncClass, typename ResType, typename Arg1Type, typename SlotBase>
static void ck_mf1()
{
 typedef ResType(FuncClass::*MFType)(Arg1Type);
 const MFType RefPtr=&FuncClass::if1;
 typedef Wy_MF1<SlotBase,ResType,Arg1Type> ClassMF;

 FuncClass s;

 // Wy_MF1(), reset()
 {
  ClassMF tmp,tmp1;
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(tmp==tmp1) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW( WyRet() );
  }

  tmp.reset();
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW( WyRet() );
  }

  try {
    tmp(1);
    WY_THROW( WyRet() );
  }
  catch(const typename ClassMF::Reply& e) {
    if(e!=Wym_EFAULT) {
      WY_THROW( WyRet(e) );
    }
  };
 }
 
 // Wy_MF1(obj,mem_func)/(const Wy_MF1&)
 {
  s.m_cnt=441;
  ClassMF tmp(s,RefPtr);
  if(tmp.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp(55)!=55) {
    WY_THROW( WyRet() );
  }
  if(s.m_cnt!=55) {
    WY_THROW( WyRet() );
  }

  const ClassMF tmp2(tmp);
  if(tmp2==tmp) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp2!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp2(345)!=345) {
    WY_THROW( WyRet() );
  }
  if(s.m_cnt!=345) {
    WY_THROW( WyRet() );
  }

  try {
    ClassMF tmp(s,(MFType)NULL);
    WY_THROW( WyRet() );
  }
  catch(const typename ClassMF::Reply& e) {
    if(e!=Wym_EFAULT) {
      WY_THROW( WyRet(e) );
    }
  };

 }

 // move ctor
 {
  ClassMF arr[2];
  if(arr[0].is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(arr[1].is_default()==false) {
    WY_THROW( WyRet() );
  }
  new(&arr[0]) ClassMF(arr[0],Wy::ByMove);
  if(arr[0].is_default()==false) {
    WY_THROW( WyRet() );
  }

  ClassMF tmp(s,RefPtr);
  arr[1]=tmp;
  if(arr[1].is_default()) {
    WY_THROW( WyRet() );
  }
  new(&arr[0]) ClassMF(arr[1],Wy::ByMove);
  if(arr[0]!=tmp) {
    WY_THROW( WyRet() );
  }
 }

 // reset(const Wy_FM0&)
 {
  const ClassMF mf0, mf1(s,RefPtr);
  ClassMF tmp;
  if(mf0.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(mf1.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }

  tmp.reset(mf1);
  if(tmp!=mf1) {
    WY_THROW( WyRet() );
  }
  tmp.reset(mf0);
  if(tmp!=mf0) {
    WY_THROW( WyRet() );
  }
 }

 // reset(FuncClass&, MFType)
 {
  ClassMF tmp;
  if(tmp.is_default()) {} else {
    WY_THROW( WyRet() );
  }
  WyRet r;
  if((r=tmp.reset(s,RefPtr))!=Ok) {
    WY_THROW( WyRet() );
  }
  if(tmp!= ClassMF(s,RefPtr)) {
    WY_THROW( WyRet() );
  }
  if((r=tmp.reset(s,(MFType)NULL))!=Wym_EFAULT) {
    WY_THROW(r);
  }
 }

 // operator =(const Wy_FM0&)
 {
  const ClassMF mf0, mf1(s,RefPtr);
  ClassMF tmp;
  if(mf0.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(mf1.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }

  if((tmp=mf1)!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=mf1) {
    WY_THROW( WyRet() );
  }
  if((tmp=mf0)!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=mf0) {
    WY_THROW( WyRet() );
  }
 }
};

template<typename FuncClass, typename ResType, typename Arg1Type, typename SlotBase>
static void ck_mfv1()
{
 typedef ResType(FuncClass::*MFType)(Arg1Type);
 const MFType RefPtr=&FuncClass::f1;
 typedef Wy_MF1<SlotBase,ResType,Arg1Type> ClassMF;
 FuncClass s;

 // Wy_MF1(), reset()
 {
  ClassMF tmp,tmp1;
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(tmp==tmp1) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW( WyRet() );
  }

  tmp.reset();
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW( WyRet() );
  }

  try {
    tmp(1);
    WY_THROW( WyRet() );
  }
  catch(const typename ClassMF::Reply& e) {
    if(e!=Wym_EFAULT) {
      WY_THROW( WyRet(e) );
    }
  };
 }
 
 // Wy_MF1(obj,mem_func)/(const Wy_MF1&)
 {
  s.m_cnt=441;
  ClassMF tmp(s,RefPtr);
  if(tmp.is_default()) {
    WY_THROW( WyRet() );
  }
  tmp(56);
  if(s.m_cnt!=56) {
    WY_THROW( WyRet() );
  }

  const ClassMF tmp2(tmp);
  if(tmp2==tmp) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp2!=tmp) {
    WY_THROW( WyRet() );
  }
  tmp2(345);
  if(s.m_cnt!=345) {
    WY_THROW( WyRet() );
  }

  try {
    ClassMF tmp(s,(MFType)NULL);
    WY_THROW( WyRet() );
  }
  catch(const typename ClassMF::Reply& e) {
    if(e!=Wym_EFAULT) {
      WY_THROW( WyRet(e) );
    }
  };

 }

 // move ctor
 {
  ClassMF arr[2];
  if(arr[0].is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(arr[1].is_default()==false) {
    WY_THROW( WyRet() );
  }
  new(&arr[0]) ClassMF(arr[0],Wy::ByMove);
  if(arr[0].is_default()==false) {
    WY_THROW( WyRet() );
  }

  ClassMF tmp(s,RefPtr);
  arr[1]=tmp;
  if(arr[1].is_default()) {
    WY_THROW( WyRet() );
  }
  new(&arr[0]) ClassMF(arr[1],Wy::ByMove);
  if(arr[0]!=tmp) {
    WY_THROW( WyRet() );
  }
 }

 // reset(const Wy_FM0&)
 {
  const ClassMF mf0, mf1(s,RefPtr);
  ClassMF tmp;
  if(mf0.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(mf1.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }

  tmp.reset(mf1);
  if(tmp!=mf1) {
    WY_THROW( WyRet() );
  }
  tmp.reset(mf0);
  if(tmp!=mf0) {
    WY_THROW( WyRet() );
  }
 }

 // reset(FuncClass&, MFType)
 {
  ClassMF tmp;
  if(tmp.is_default()) {} else {
    WY_THROW( WyRet() );
  }
  WyRet r;
  if((r=tmp.reset(s,RefPtr))!=Ok) {
    WY_THROW( WyRet() );
  }
  if(tmp!= ClassMF(s,RefPtr)) {
    WY_THROW( WyRet() );
  }
  if((r=tmp.reset(s,(MFType)NULL))!=Wym_EFAULT) {
    WY_THROW(r);
  }
 }

 // operator =(const Wy_FM0&)
 {
  const ClassMF mf0, mf1(s,RefPtr);
  ClassMF tmp;
  if(mf0.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(mf1.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }

  if((tmp=mf1)!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=mf1) {
    WY_THROW( WyRet() );
  }
  if((tmp=mf0)!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=mf0) {
    WY_THROW( WyRet() );
  }
 }
};

template<typename FuncClass, typename ResType, typename Arg1Type, typename SlotBase>
static void ck_mf1_const()
{
 typedef ResType(FuncClass::*MFType)(Arg1Type)const;
 const MFType RefPtr=&FuncClass::if1c;
 typedef Wy_MF1_const<SlotBase,ResType,Arg1Type> ClassMF;
 const FuncClass s;

 // Wy_MF1_const(), reset()
 {
  ClassMF tmp,tmp1;
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(tmp==tmp1) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW( WyRet() );
  }

  tmp.reset();
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW( WyRet() );
  }

  try {
    tmp(1);
    WY_THROW( WyRet() );
  }
  catch(const typename ClassMF::Reply& e) {
    if(e!=Wym_EFAULT) {
      WY_THROW( WyRet(e) );
    }
  };
 }
 
 // Wy_MF1_const(obj,mem_func)/(const Wy_MF1_const&)
 {
  s.m_cnt=441;
  ClassMF tmp(s,RefPtr);
  if(tmp.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp(55)!=55) {
    WY_THROW( WyRet() );
  }
  if(s.m_cnt!=55) {
    WY_THROW( WyRet() );
  }

  const ClassMF tmp2(tmp);
  if(tmp2==tmp) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp2!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp2(345)!=345) {
    WY_THROW( WyRet() );
  }
  if(s.m_cnt!=345) {
    WY_THROW( WyRet() );
  }

  try {
    ClassMF tmp(s,(MFType)NULL);
    WY_THROW( WyRet() );
  }
  catch(const typename ClassMF::Reply& e) {
    if(e!=Wym_EFAULT) {
      WY_THROW( WyRet(e) );
    }
  };

 }

 // move ctor
 {
  ClassMF arr[2];
  if(arr[0].is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(arr[1].is_default()==false) {
    WY_THROW( WyRet() );
  }
  new(&arr[0]) ClassMF(arr[0],Wy::ByMove);
  if(arr[0].is_default()==false) {
    WY_THROW( WyRet() );
  }

  ClassMF tmp(s,RefPtr);
  arr[1]=tmp;
  if(arr[1].is_default()) {
    WY_THROW( WyRet() );
  }
  new(&arr[0]) ClassMF(arr[1],Wy::ByMove);
  if(arr[0]!=tmp) {
    WY_THROW( WyRet() );
  }
 }

 // reset(const Wy_MF1_const&)
 {
  const ClassMF mf0, mf1(s,RefPtr);
  ClassMF tmp;
  if(mf0.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(mf1.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }

  tmp.reset(mf1);
  if(tmp!=mf1) {
    WY_THROW( WyRet() );
  }
  tmp.reset(mf0);
  if(tmp!=mf0) {
    WY_THROW( WyRet() );
  }
 }

 // reset(FuncClass&, MFType)
 {
  ClassMF tmp;
  if(tmp.is_default()) {} else {
    WY_THROW( WyRet() );
  }
  WyRet r;
  if((r=tmp.reset(s,RefPtr))!=Ok) {
    WY_THROW( WyRet() );
  }
  if(tmp!= ClassMF(s,RefPtr)) {
    WY_THROW( WyRet() );
  }
  if((r=tmp.reset(s,(MFType)NULL))!=Wym_EFAULT) {
    WY_THROW(r);
  }
 }

 // operator =(const Wy_MF1_const&)
 {
  const ClassMF mf0, mf1(s,RefPtr);
  ClassMF tmp;
  if(mf0.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(mf1.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }

  if((tmp=mf1)!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=mf1) {
    WY_THROW( WyRet() );
  }
  if((tmp=mf0)!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=mf0) {
    WY_THROW( WyRet() );
  }
 }
};

template<typename FuncClass, typename ResType, typename Arg1Type, typename SlotBase>
static void ck_mfv1_const()
{
 typedef ResType(FuncClass::*MFType)(Arg1Type)const;
 const MFType RefPtr=&FuncClass::f1c;
 typedef Wy_MF1_const<SlotBase,ResType,Arg1Type> ClassMF;
 const FuncClass s;

 // Wy_MF1_const(), reset()
 {
  ClassMF tmp,tmp1;
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(tmp==tmp1) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW( WyRet() );
  }

  tmp.reset();
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW( WyRet() );
  }

  try {
    tmp(1);
    WY_THROW( WyRet() );
  }
  catch(const typename ClassMF::Reply& e) {
    if(e!=Wym_EFAULT) {
      WY_THROW( WyRet(e) );
    }
  };
 }
 
 // Wy_MF1_const(obj,mem_func)/(const Wy_MF1_const&)
 {
  s.m_cnt=441;
  ClassMF tmp(s,RefPtr);
  if(tmp.is_default()) {
    WY_THROW( WyRet() );
  }
  tmp(56);
  if(s.m_cnt!=56) {
    WY_THROW( WyRet() );
  }

  const ClassMF tmp2(tmp);
  if(tmp2==tmp) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp2!=tmp) {
    WY_THROW( WyRet() );
  }
  tmp2(345);
  if(s.m_cnt!=345) {
    WY_THROW( WyRet() );
  }

  try {
    ClassMF tmp(s,(MFType)NULL);
    WY_THROW( WyRet() );
  }
  catch(const typename ClassMF::Reply& e) {
    if(e!=Wym_EFAULT) {
      WY_THROW( WyRet(e) );
    }
  };

 }

 // move ctor
 {
  ClassMF arr[2];
  if(arr[0].is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(arr[1].is_default()==false) {
    WY_THROW( WyRet() );
  }
  new(&arr[0]) ClassMF(arr[0],Wy::ByMove);
  if(arr[0].is_default()==false) {
    WY_THROW( WyRet() );
  }

  ClassMF tmp(s,RefPtr);
  arr[1]=tmp;
  if(arr[1].is_default()) {
    WY_THROW( WyRet() );
  }
  new(&arr[0]) ClassMF(arr[1],Wy::ByMove);
  if(arr[0]!=tmp) {
    WY_THROW( WyRet() );
  }
 }

 // reset(const Wy_MF1_const&)
 {
  const ClassMF mf0, mf1(s,RefPtr);
  ClassMF tmp;
  if(mf0.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(mf1.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }

  tmp.reset(mf1);
  if(tmp!=mf1) {
    WY_THROW( WyRet() );
  }
  tmp.reset(mf0);
  if(tmp!=mf0) {
    WY_THROW( WyRet() );
  }
 }

 // reset(FuncClass&, MFType)
 {
  ClassMF tmp;
  if(tmp.is_default()) {} else {
    WY_THROW( WyRet() );
  }
  WyRet r;
  if((r=tmp.reset(s,RefPtr))!=Ok) {
    WY_THROW( WyRet() );
  }
  if(tmp!= ClassMF(s,RefPtr)) {
    WY_THROW( WyRet() );
  }
  if((r=tmp.reset(s,(MFType)NULL))!=Wym_EFAULT) {
    WY_THROW(r);
  }
 }

 // operator =(const Wy_MF1_const&)
 {
  const ClassMF mf0, mf1(s,RefPtr);
  ClassMF tmp;
  if(mf0.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(mf1.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }

  if((tmp=mf1)!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=mf1) {
    WY_THROW( WyRet() );
  }
  if((tmp=mf0)!=tmp) {
    WY_THROW( WyRet() );
  }
  if(tmp!=mf0) {
    WY_THROW( WyRet() );
  }
 }
};

int main(void)
try {
 std::cout << "Checking Wy_MF.hh ...\n";

 ck_mf0<S1,int,S1>();
 ck_mfv0<S1,void,S1>();
 ck_mf0_const<S1,int,S1>();
 ck_mfv0_const<S1,void,S1>();
 ck_mf1<S1,int,int,S1>();
 ck_mfv1<S1,void,int,S1>();
 ck_mf1_const<S1,int,int,S1>();
 ck_mfv1_const<S1,void,int,S1>();

 ck_mf0<S2,int,S2>();
 ck_mfv0<S2,void,S2>();
 ck_mf0_const<S2,int,S2>();
 ck_mfv0_const<S2,void,S2>();
 ck_mf1<S2,int,int,S2>();
 ck_mfv1<S2,void,int,S2>();
 ck_mf1_const<S2,int,int,S2>();
 ck_mfv1_const<S2,void,int,S2>();

 ck_mf0<S2,int,S_vbase>();
 ck_mfv0<S2,void,S_vbase>();
 ck_mf0_const<S2,int,S_vbase>();
 ck_mfv0_const<S2,void,S_vbase>();
 ck_mf1<S2,int,int,S_vbase>();
 ck_mfv1<S2,void,int,S_vbase>();
 ck_mf1_const<S2,int,int,S_vbase>();
 ck_mfv1_const<S2,void,int,S_vbase>();

 std::cout << "Checked Ok\n";
 return(0);
}
catch(const WyRet& e) {
 std::cerr << Wy::wrd(e).c_str() << "\n";
 return(e->repcode());
}
catch(...) {
 std::cerr << "unknown unwind\n";
 return(-1);
};
