/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   Note: No thread involved. This is basic check.

   Build: make chk_mutex
*/

#include "WyMutex.h"
#include <iostream>
#include <cassert>
#include <cstring>
#include "WyStr.h"

#if WYMUTEX_VERSION!=33
#error Test code is for WYMUTEX_VERSION 33
#endif

static void test_default_mutex(void)
{
 WyRet r;
 WyMutex mtx;

 if((r=mtx.lock())!=Ok) {
   WY_THROW(r);
 }
 if((r=mtx.trylock())!=Wym_EBUSY) {
   WyMutex::unlock(mtx);
   WY_THROW(r);
 }
 if((r=mtx.unlock())!=Ok) {
   WY_THROW(r);
 }
 if((r=mtx.trylock())!=Ok) {
   WY_THROW(r);
 }
 if((r=mtx.unlock())!=Ok) {
   WY_THROW(r);
 }

 // static functions
 WyMutex::lock(mtx);
 try {
   WyMutex::trylock(mtx);
   WY_THROW( WyRet() );
 }
 catch(const WyMutex::Reply& e) {
   if(e!=Wym_EBUSY) {
     WY_THROW( WyRet(e) );
   }
 };

 WyMutex::unlock(mtx);
 WyMutex::trylock(mtx);
 WyMutex::unlock(mtx);

 // chk mutex in the heap
 {
  WyMutex *pm= new WyMutex;
  if((r=pm->lock())!=Ok) {
    delete pm;
    WY_THROW(r);
  }
  if((r=pm->unlock())!=Ok) {
    delete pm;
    WY_THROW(r);
  }
  delete pm;
 }
};

static void test_normal_mutex(void)
{
 WyRet r;
 WyMutex mtx(WyMutex::Normal);

 if((r=mtx.lock())!=Ok) {
   WY_THROW(r);
 }
 if((r=mtx.trylock())!=Wym_EBUSY) {
   WyMutex::unlock(mtx);
   WY_THROW(r);
 }
 if((r=mtx.unlock())!=Ok) {
   WY_THROW(r);
 }
 if((r=mtx.trylock())!=Ok) {
   WY_THROW(r);
 }
 if((r=mtx.unlock())!=Ok) {
   WY_THROW(r);
 }

 // static functions
 WyMutex::lock(mtx);
 try {
   WyMutex::trylock(mtx);
   WY_THROW( WyRet() );
 }
 catch(const WyMutex::Reply& e) {
   if(e!=Wym_EBUSY) {
     WY_THROW( WyRet(e) );
   }
 };
 WyMutex::unlock(mtx);
 WyMutex::trylock(mtx);
 WyMutex::unlock(mtx);

 // chk mutex in the heap
 {
  WyMutex *pm= new WyMutex;
  if((r=pm->lock())!=Ok) {
    delete pm;
    WY_THROW(r);
  }
  if((r=pm->unlock())!=Ok) {
    delete pm;
    WY_THROW(r);
  }
  delete pm;
 }
};

static void test_recursive_mutex(void)
{
 WyRet r;
 WyMutex mtx(WyMutex::Recursive);

 if((r=mtx.lock())!=Ok) {
   WY_THROW(r);
 }
 if((r=mtx.trylock())!=Ok) {  // lock twice
   WyMutex::unlock(mtx);
   WY_THROW(r);
 }
 if((r=mtx.lock())!=Ok) {     // lock thrice
   WyMutex::unlock(mtx);
   WY_THROW(r);
 }
 if((r=mtx.unlock())!=Ok) {
   WY_THROW(r);
 }
 if((r=mtx.unlock())!=Ok) {
   WY_THROW(r);
 }
 if((r=mtx.unlock())!=Ok) {
   WY_THROW(r);
 }

 // Unlock unlocked-recursive-mutex
 //
 if((r=mtx.unlock())!=Wym_EPERM) {
   WY_THROW(r);
 }

 // static functions
 WyMutex::lock(mtx);
 WyMutex::trylock(mtx);
 WyMutex::lock(mtx);
 WyMutex::unlock(mtx);
 WyMutex::unlock(mtx);
 WyMutex::unlock(mtx);

 try {
   WyMutex::unlock(mtx);
   WY_THROW( WyRet() );
 }
 catch(const WyMutex::Reply& e) {
   if(e!=Wym_EPERM) {
     WY_THROW( WyRet(e) );
   }
 };
};

static void test_errcheck_mutex(void)
{
 WyRet r;
 WyMutex mtx(WyMutex::ErrorCheck);

 if((r=mtx.lock())!=Ok) {
   WY_THROW(r);
 }
 if((r=mtx.lock())!=Wym_EDEADLK) {
   WyMutex::unlock(mtx);
   WY_THROW(r);
 }
 r=mtx.trylock();
 std::cerr << "Note: trylock a locked mutex(ErrorCheck) returns ";
 if(r==Wym_EBUSY) {       // EBUSY before FC3, and is documented
   std::cerr << "EBUSY\n";
 } else if(r==Wym_EDEADLK) {  // EDEADLK since FC5, and not documented
   std::cerr << "EDEADLK\n";
   //std::cerr << "Continue or not <y/n>";
   //char ch;
   //std::cin >> ch;
   //if((ch!='y')&&(ch!='Y')) {
   //  WyMutex::unlock(mtx);
   //  WY_THROW(r);
   //}
 } else {
   WyMutex::unlock(mtx);
   WY_THROW(r);
 }

 if((r=mtx.unlock())!=Ok) {
   WY_THROW(r);
 }
 if((r=mtx.trylock())!=Ok) {
   WY_THROW(r);
 }
 if((r=mtx.unlock())!=Ok) {
   WY_THROW(r);
 }

 // static functions
 WyMutex::lock(mtx);
 WyMutex::unlock(mtx);
 WyMutex::trylock(mtx);
 WyMutex::unlock(mtx);

 WyMutex::lock(mtx);
 try {
   WyMutex::lock(mtx);
   WY_THROW( WyRet() );
 }
 catch(const WyMutex::Reply& e) {
   if(e!=Wym_EDEADLK) {
     WY_THROW( WyRet() );
   }
 };
 try {
   WyMutex::trylock(mtx);
   WY_THROW( WyRet() );
 }
 catch(const WyMutex::Reply& e) {
   if((e!=Wym_EBUSY)&&
      (e!=Wym_EDEADLK)) {      // on Fedora, EDEADLK is returned
     WY_THROW( WyRet() );
   }
 };
 WyMutex::unlock(mtx);
};

//
// Test WyLock
//
static void test_WyLock(void)
{
 WyRet r;
 WyMutex mtx; 
 WyMutex mtx2;

 {
  WyMutex mtx(WyMutex::ErrorCheck);
  if((r=mtx.lock())!=Ok) {
    WY_THROW(r);
  }
  try {
    WyLock aa(mtx);   // should throw Wym_EDEADLK
    WY_TERMINATE("");
  }
  catch(const WyLock::Reply& e) {
    WyMutex::unlock(mtx);
    if(e!=Wym_EDEADLK) {
      WY_THROW( WyRet(e) );
    }
  }
  catch(...) {
    WyMutex::unlock(mtx);
    throw;
  }
 }

 {
  // test destructor unlock
  WyLock aa(mtx);
  if((r=mtx.trylock())!=Wym_EBUSY) {
    WY_THROW(r);
  }
 }  // aa should unlock mtx leaving the scope
 if((r=mtx.trylock())!=Ok) {
   WY_THROW(r);
 }
 if((r=mtx.unlock())!=Ok) {
   WY_THROW(r);
 }

 { // Test WyLock::xlock(..)
  WyLock aa(mtx);
   //if((r=aa.xlock(mtx))!=Ok) {  // deadlock
   // WY_THROW(r);
   //}
  //if((r=aa.xlock(mtx))!=Wym_EDEADLK) {
  //  WY_THROW(r);    // xlock the same mutex
  //}
  if((r=mtx.trylock())!=Wym_EBUSY) {
    WY_THROW(r);    // mtx did not locked
  }
  if((r=aa.xlock(mtx2))!=Ok) {
    WY_THROW(r);    // transfer failed
  }
  if((r=mtx.trylock())!=Ok) {
    WY_THROW(r);    // mtx should have unlocked
  }
  if((r=mtx.unlock())!=Ok) {
    WY_THROW(r);
  }

  if((r=mtx2.trylock())!=Wym_EBUSY) {
    WY_THROW(r);   // WyLock::unlock failed
  }
 }

 // check mtx2 is unlocked
 //
 if((r=mtx2.trylock())!=Ok) {
   WY_THROW(r);    // mtx should have unlocked
 }
 if((r=mtx2.unlock())!=Ok) {
   WY_THROW(r);
 }

 //
 // Test WyLock in a series of construct/and destruct
 //
 {
   for(int i=0; i<100; i++) try {
     WyLock aa(mtx);

     // transit to lock mtx2
     if((r=aa.xlock(mtx2))!=Ok) {
       WY_THROW(r);
     }
     if((r=mtx2.trylock())!=Wym_EBUSY) {
       WY_THROW(r);
     }
     if((r=mtx.trylock())!=Ok) {
       WY_THROW(r);
     }
     if((r=mtx.unlock())!=Ok) {
       WY_THROW(r);
     }

     // transit to lock mtx
     if((r=aa.xlock(mtx))!=Ok) {
       WY_THROW(r);
     }
     if((r=mtx.trylock())!=Wym_EBUSY) {
       WY_THROW(r);
     }
     if((r=mtx2.trylock())!=Ok) {
       WY_THROW(r);
     }
     if((r=mtx2.unlock())!=Ok) {
       WY_THROW(r);
     }
   }
   catch(const WyRet& e) {
     std::cerr << Wy::wrd(e).c_str();
     throw;
   };
 }

 if((r=mtx.trylock())!=Ok) {
   WY_THROW(r);
 }
 if((r=mtx.unlock())!=Ok) {
   WY_THROW(r);
 }
};

static const WyStr chdr(
                  "+---------------+\n"
                  "| main() caught:|\n"
                  "+---------------+\n");
int main(void) throw()
try {
 std::cout << "Checking WyMutex class (" __FILE__ ") ...\n";
 if(WyStr(WyMutex::class_name)!="WyMutex") {
   WY_TERMINATE("");
 }
 if(WyStr(WyLock::class_name)!="WyLock") {
   WY_TERMINATE("");
 }
 std::cout << "sizeof(WyMutex)= " << sizeof(WyMutex) << '\n';
 std::cout << "sizeof(WyLock)= " << sizeof(WyLock) << '\n';
 test_normal_mutex();
 test_recursive_mutex();
 test_errcheck_mutex();
 test_default_mutex();
 test_WyLock();
 std::cout << "Checked Ok\n";
 return(0);
}
catch(const WyRet& e) {
 std::cerr << chdr.c_str() << Wy::wrd(e).c_str() << std::endl;
 return(-1);
}
catch(const std::exception& e) {
 std::cerr << chdr.c_str() << "std::exception" << std::endl;
 return(-1);
}
catch(...) {
 std::cerr << chdr.c_str() << "unknown unwind" << std::endl;
 return(-1);
};
