/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2007
*/

#include "WyStr.h"
#include "WyNum.h"
#include <cmath>
#include <iostream>
#include <string>
#include <cstdlib>
#include <cassert>
#include <stdio.h>

struct ChkSmp_D {
 typedef double NumType;
 const NumType i_num;
 const int     i_radix;
 const size_t  i_frdig;
 size_t        i_blen;
 const WyReply res_reply;
 const char*   res_str;

 void verify(void) const {
   try {
     const char ChkChr=255;
     char dbuf[256];    // long enough for known types

     // chk _ntoa(char*..)
     //
     assert( (i_blen>=0)&&(i_blen<sizeof(dbuf)) );
     std::memset(dbuf,ChkChr,sizeof(dbuf)); // for chk overwritten
     size_t rv=Wy::_ntoa(dbuf,i_blen,i_num,i_radix,i_frdig);
     if(dbuf[i_blen]!=ChkChr) {
       std::cerr << "dbuf overwritten: " << (int)dbuf[i_blen] << '\n';
       dbuf[sizeof(dbuf)-1]=0;
       std::cout << "dbuf[]=" << dbuf << '\n';
       WY_THROW( WyRet() );
     }
     if(rv>0) {
       if(rv>=sizeof(dbuf)-1) {
         std::cout << "Unexpected rv: " << rv << '\n';
         WY_THROW( WyRet() );
       }
       dbuf[rv]=0;
       if(res_reply!=Ok) {
         std::cout << "Unexpected rv: " << rv << '\n';
         std::cout << "dbuf[]=" << dbuf << '\n';
         WY_THROW( WyRet() );
       }
       if(WyCSeg(dbuf,rv)!=res_str) {
         std::cerr << "dbuf= " << dbuf << '\n';
         WY_THROW( WyRet() );
       }
     } else {
       if(dbuf[0]!=ChkChr) {
         std::cerr << "dbuf overwritten: " << (int)dbuf[0] << '\n';
         std::cout << "dbuf[]=" << dbuf << '\n';
         WY_THROW( WyRet() );
       }
       if(res_reply==Ok) {
         std::cout << "Unexpected rv: " << rv << '\n';
         WY_THROW( WyRet() );
       }
     }

     // chk _ntoa_r(char*, i_blen,.)
     //
     std::memset(dbuf,ChkChr,sizeof(dbuf)); // for chk overwritten
     rv=Wy::_ntoa_r(dbuf,i_blen,i_num,i_radix,i_frdig);
     if(dbuf[i_blen]!=ChkChr) {
       std::cerr << "dbuf overwritten: " << (int)dbuf[i_blen] << '\n';
       dbuf[sizeof(dbuf)-1]=0;
       std::cout << "dbuf[]=" << dbuf << '\n';
       WY_THROW( WyRet() );
     }
     if(rv>0) {
       if(rv>=sizeof(dbuf)-1) {
         std::cout << "Unexpected rv: " << rv << '\n';
         WY_THROW( WyRet() );
       }
       dbuf[i_blen]=0;
       if(res_reply!=Ok) {
         std::cout << "Unexpected rv: " << rv << '\n';
         std::cout << "dbuf[]=" << dbuf << '\n';
         WY_THROW( WyRet() );
       }
       if(WyCSeg(dbuf+i_blen-rv,rv)!=res_str) {
         std::cerr << "dbuf= " << dbuf << '\n';
         WY_THROW( WyRet() );
       }
     } else {
       if(dbuf[i_blen-1]!=ChkChr) {
         std::cerr << "dbuf overwritten: " << (int)dbuf[i_blen-1] << '\n';
         std::cout << "dbuf[]=" << dbuf << '\n';
         WY_THROW( WyRet() );
       }
       if(res_reply==Ok) {
         std::cout << "Unexpected rv: " << rv << '\n';
         WY_THROW( WyRet() );
       }
     }

   }
   catch(...) {
     std::cerr << "ChkSmp_D{ " <<  i_num << ", " << i_radix  << ", " << i_frdig
                << ", " << i_blen << ", " << Wy::wrd(res_reply).c_str() 
               << ", \"" << res_str << "\"}\n";
     throw;
   };
 }
};

static void ck_ntoa_double(void)
{
 typedef double NumType;
 const ChkSmp_D chk_tab[]={
   // Checking takes exact value, verify manually some error dumps are ok
   // num radix frdig blen reply rstr
   {   0,  1, 0, 128,Wym_EINVAL, ""},
   {   0,  8, 0, 128,Wym_EINVAL, ""},
   {   0, 10, 0, 128,Ok, "0"},
   {   0,  0, 0,   4,Ok, "0"},
   {   0,  0, 1, 128,Ok, "0.0"},
   {  99, 10, 0,   2,Ok, "99"},
   { -99, 10, 0,   3,Ok, "-99"},
   { -99, 10, 0,   2,Wym_EINVAL, "-99"},
   {   0,  0, 0,   3,Ok, "0"},
   { 0.1, 10, 1,   3,Ok, "0.1"},
   {123.456, 10, 3, 128, Ok, "123.456"},
   {-123.456, 10, 3, 128, Ok, "-123.456"}, 
   {123.456,  0,  0, 128, Ok, "1e+2"},
   { 5678e13, 10, 1,  19, Ok,  "56780000000000000.0"},
   {5.00004e2, 10, 2, 128, Ok, "500.00"},
   {5.00004e2, 10, 3, 128, Ok, "500.004"},
   {5.00004e2, 10, 4, 128, Ok, "500.0040"},
   {-5.00004e2, 10, 2, 128, Ok, "-500.00"},
   {-5.00004e2, 10, 3, 128, Ok, "-500.004"},
   {-5.00004e2, 10, 4, 128, Ok, "-500.0040"},
   { 5678e3, 10, 11,  19, Ok,  "5678000.00000000000"},
   { 5678e14, 10, 1,  20, Ok,  "567800000000000000.0"},
   { 5678e15, 10, 1, 128, Ok,  "5678000000000000000.0"},
   { 5678e22, 10, 1,  28, Ok,  "56780000000000002000000000.0"},
   { 5678e22, 10, 2,  28, Wym_EINVAL,""},
   { 5678e128, 10, 1, 133,Wym_EINVAL,  ""},
   {-9482e13, 10, 0, 128, Ok, "-94820000000000000"},
   {-9482e14, 10, 0, 128, Ok, "-948200000000000000"},
   {-9482e15, 10, 0, 128, Ok, "-9482000000000000000"},
   {-9482e22, 10, 0, 128, Ok, "-94820000000000007600000000"},
   { 5678e3,  0,  5,  19, Ok,  "5.67800e+6"},
   { -5678e-3,  0,  3,  19, Ok,  "-5.678"},
   { NumType( 9e99999), 10, 0, 3, Ok, "inf"},
   { NumType(-9e99999), 10, 0, 4, Ok, "-inf"},
   { NumType(std::sqrt(-1.0)), 10, 0, 3, Ok, "nan"},
   { NumType( 9e99999),  0, 0, 3, Ok, "inf"},
   { NumType(-9e99999),  0, 0, 4, Ok, "-inf"},
   { NumType(std::sqrt(-1.0)), 10, 0, 3, Ok, "nan"},
   { NumType( 9e99999), 10, 0, 2, Wym_EINVAL, ""},
   { NumType(-9e99999), 10, 0, 3, Wym_EINVAL, ""},
   { NumType(std::sqrt(-1.0)), 10, 0, 2, Wym_EINVAL, ""},

   {123.456, 0, 3, 128, Ok, "1.234e+2"},
   {0.000000599246961220, 10, 18, 128, Ok, "0.000000599246961220"},
   {-0.000000599246961220, 10, 18, 128, Ok, "-0.000000599246961220"},

   // radix 16
   {      0, 16, 0,   1,Ok, "0"}, 
   {  65536, 16, 0,   5,Ok, "10000"}, 
   { -65535, 16, 1,   7,Ok, "-ffff.0"}, 
   { 0xffp0, 16, 0,   2,Ok, "ff"},
   {-0xffp0, 16, 0,   3,Ok, "-ff"},
   {-0xffp0, 16, 0,   2,Wym_EINVAL, "-ff"},
   { 0x1.00fp0, 16, 3,   5,Ok, "1.00f"},
   {-0x1.00fp0, 16, 3,   6,Ok, "-1.00f"},
   { 0x11p120, 16, 0, 32,Ok, "11000000000000000000000000000000"},
   { 0x11p119, 16, 0, 31,Ok,  "8800000000000000000000000000000"},
   { 0x11p118, 16, 0, 31,Ok,  "4400000000000000000000000000000"},
   { 0x11p117, 16, 0, 31,Ok,  "2200000000000000000000000000000"},
   { 0x11p116, 16, 0, 31,Ok,  "1100000000000000000000000000000"},
   { 0x123.456p0, 16, 3,   7,Ok, "123.456"},
   {-0x123.456p0, 16, 3,   7,Wym_EINVAL, ""},
   {-0x123.456p0, 16, 3,   8,Ok, "-123.456"},
   {-0x123.456p4, 16, 3,   9,Ok, "-1234.560"},
   {-0x123.456p-4, 16, 3,   9,Ok, "-12.345"},
 };
 for(size_t i=0; i<sizeof(chk_tab)/sizeof(ChkSmp_D); ++i) {
   chk_tab[i].verify();
 }

 // chk <max>
 {
   NumType num1= std::numeric_limits<NumType>::max();
   NumType num2;

   char dbuf[std::numeric_limits<NumType>::max_exponent10+16];
   size_t v;

   // ::strtod returns inf for the max. in scientific notation. So removed this check

   v=Wy::_ntoa(dbuf, sizeof(dbuf), num1,10, 1);
   if(v==0) {
     WY_THROW( WyRet() );
   }
   dbuf[v]=0;

   num2=::strtod(dbuf,NULL);

   // Dump instances of larger erronous ratio
   {
     double ErrRat;
     if((num2<-1e-10)||(num2>1e-10)) {
       if(num2!=0) {
         ErrRat=(double)num1/num2;
       } else if(num1!=0) {
         ErrRat=(double)num2/num1;
       } else {
         ErrRat=1.0;
       }
       if((ErrRat>1.000001)||(ErrRat<0.999999)) {  // 1/million precision
         std::cout << dbuf << '\n';
         std::cout << num1 << '\n' << num2 << '\n';
       };
     } else {
       if((num1<-1e-10)||(num1>1e-10)) {
         WY_THROW( WyRet() );
       }
     }
   }
 }

 // chk <min>
 {
   NumType num1= std::numeric_limits<NumType>::min();
   NumType num2;

   char dbuf[-std::numeric_limits<NumType>::min_exponent10+16];
   size_t v;

   dbuf[ Wy::_ntoa(dbuf, sizeof(dbuf), num1, 0, 18) ]=0;
   if(dbuf[0]==0) {
     WY_THROW( WyRet() );
   }

   num2=::strtod(dbuf,NULL);

   // Dump instances of larger erronous ratio
   {
     double ErrRat;
     if((num2<-1e-10)||(num2>1e-10)) {
       if(num2!=0) {
         ErrRat=(double)num1/num2;
       } else if(num1!=0) {
         ErrRat=(double)num2/num1;
       } else {
         ErrRat=1.0;
       }
       if((ErrRat>1.000001)||(ErrRat<0.999999)) {  // 1/million precision
         std::cout << dbuf << '\n';
         std::cout << num1 << '\n' << num2 << '\n';
       };
     } else {
       if((num1<-1e-10)||(num1>1e-10)) {
         WY_THROW( WyRet() );
       }
     }
   }

   v=Wy::_ntoa(dbuf, sizeof(dbuf), num1,10, 1);
   if(v==0) {
     WY_THROW( WyRet() );
   }
   dbuf[v]=0;

   num2=::strtod(dbuf,NULL);

   // Dump instances of larger erronous ratio
   {
     double ErrRat;
     if((num2<-1e-10)||(num2>1e-10)) {
       if(num2!=0) {
         ErrRat=(double)num1/num2;
       } else if(num1!=0) {
         ErrRat=(double)num2/num1;
       } else {
         ErrRat=1.0;
       }
       if((ErrRat>1.000001)||(ErrRat<0.999999)) {  // 1/million precision
         std::cout << dbuf << '\n';
         std::cout << num1 << '\n' << num2 << '\n';
       };
     } else {
       if((num1<-1e-10)||(num1>1e-10)) {
         WY_THROW( WyRet() );
       }
     }
   }
 }

 for(int i=0; i<10000; ++i) {
   //  res= x*2^y  ,x in [-1.0,+1.0], y in [-RandExp, +RandExp]
   //
   const long long RandExp=127;
   const NumType num1= std::ldexp( ::rand()*(2.0/(double)RAND_MAX)-1.0
              , (::rand()*(long long)2*RandExp)/(long long)RAND_MAX-RandExp 
             );

   // Scientific notation (l)
   //
   char dbuf[128];
   size_t v=Wy::_ntoa(dbuf, sizeof(dbuf), num1, 0, 6);
   if(v==0) {
     WY_THROW( WyRet() );
   }
   dbuf[v]=0;

   NumType num2=::strtod(dbuf,NULL);

   // Dump instances of larger erronous ratio
   {
     double ErrRat;
     if(num2!=0) {
       ErrRat=(double)num1/num2;
     } else if(num1!=0) {
       ErrRat=(double)num2/num1;
     } else {
       ErrRat=1.0;
     }
     if((ErrRat>1.000001)||(ErrRat<0.999999)) {  // 1/million precision
       std::cout << dbuf << '\n';
       std::cout << num1 << '\n' << num2 << '\n';
     };
   }

   // Radix=10, float notation (l)
   //
   v=Wy::_ntoa(dbuf, sizeof(dbuf), num1,10, 18);
   if(v==0) {
     WY_THROW( WyRet() );
   }
   dbuf[v]=0;

   num2=::strtod(dbuf,NULL);

   // Dump instances of larger erronous ratio
   {
     double ErrRat;
     if((num2<-1e-10)||(num2>1e-10)) {
       if(num2!=0) {
         ErrRat=(double)num1/num2;
       } else if(num1!=0) {
         ErrRat=(double)num2/num1;
       } else {
         ErrRat=1.0;
       }
       if((ErrRat>1.000001)||(ErrRat<0.999999)) {  // 1/million precision
         std::cout << dbuf << '\n';
         std::cout << num1 << '\n' << num2 << '\n';
       };
     } else {
       if((num1<-1e-10)||(num1>1e-10)) {
         WY_THROW( WyRet() );
       }
     }
   }

   // Radix=16, Hexidecimal (l)
   //
   std::string nstr;
   v=Wy::_ntoa(dbuf, sizeof(dbuf), num1,16, 18);
   if(v==0) {
     WY_THROW( WyRet() );
   }
   dbuf[v]=0;

   if(dbuf[0]=='-') {
     nstr="-0x";
     nstr+=dbuf+1;
     nstr+="p0";
   } else {
     nstr="0x";
     nstr+=dbuf;
     nstr+="p0";
   }
   num2=::strtod(nstr.c_str(),NULL);

   // Dump instances of larger erronous ratio
   if((num1<-1e-10)||(num1>1e-10)) {
     double ErrRat;
     if(num2!=0) {
       ErrRat=(double)num1/num2;
     } else if(num1!=0) {
       ErrRat=(double)num2/num1;
     } else {
       ErrRat=1.0;
     }
     if((ErrRat>1.000001)||(ErrRat<0.999999)) {  // 1/million precision
       std::cout << std::hex << dbuf << '\n';
       std::cout << std::hex << num1 << '\n' << std::hex << num2 << '\n';
     };
   } else {
     if((num2<-1e-10)||(num2>1e-10)) {
       WY_THROW( WyRet() );
     } else {
     }
   }

   // Scientific notation (r)
   //
   const size_t DLen=sizeof(dbuf)-1;
   v=Wy::_ntoa_r(dbuf, DLen, num1, 0, 6);
   if(v==0) {
     WY_THROW( WyRet() );
   }
   dbuf[DLen]=0;

   num2=::strtod((dbuf+DLen)-v,NULL);

   // Dump instances of larger erronous ratio
   {
     double ErrRat;
     if(num2!=0) {
       ErrRat=(double)num1/num2;
     } else if(num1!=0) {
       ErrRat=(double)num2/num1;
     } else {
       ErrRat=1.0;
     }
     if((ErrRat>1.000001)||(ErrRat<0.999999)) {  // 1/million precision
       std::cout << dbuf << '\n';
       std::cout << num1 << '\n' << num2 << '\n';
     };
   }

   // Radix=10, float notation (r)
   //
   v=Wy::_ntoa_r(dbuf, DLen, num1,10, 18);
   if(v==0) {
     WY_THROW( WyRet() );
   }
   dbuf[DLen]=0;

   num2=::strtod((dbuf+DLen)-v,NULL);

   // Dump instances of larger erronous ratio
   {
     double ErrRat;
     if((num2<-1e-10)||(num2>1e-10)) {
       if(num2!=0) {
         ErrRat=(double)num1/num2;
       } else if(num1!=0) {
         ErrRat=(double)num2/num1;
       } else {
         ErrRat=1.0;
       }
       if((ErrRat>1.000001)||(ErrRat<0.999999)) {  // 1/million precision
         std::cout << dbuf << '\n';
         std::cout << num1 << '\n' << num2 << '\n';
       };
     } else {
       if((num1<-1e-10)||(num1>1e-10)) {
         WY_THROW( WyRet() );
       }
     }
   }

   // Radix=16, Hexidecimal (r)
   //
   nstr="";
   v=Wy::_ntoa_r(dbuf, DLen, num1,16, 18);
   if(v==0) {
     WY_THROW( WyRet() );
   }
   dbuf[DLen]=0;

   char* hptr= (dbuf+DLen)-v;   // head ptr in dbuf
   if(hptr[0]=='-') {
     nstr="-0x";
     nstr+=hptr+1;
     nstr+="p0";
   } else {
     nstr="0x";
     nstr+=hptr;
     nstr+="p0";
   }
   num2=::strtod(nstr.c_str(),NULL);

   // Dump instances of larger erronous ratio
   if((num1<-1e-10)||(num1>1e-10)) {
     double ErrRat;
     if(num2!=0) {
       ErrRat=(double)num1/num2;
     } else if(num1!=0) {
       ErrRat=(double)num2/num1;
     } else {
       ErrRat=1.0;
     }
     if((ErrRat>1.000001)||(ErrRat<0.999999)) {  // 1/million precision
       std::cout << std::hex << dbuf << '\n';
       std::cout << std::hex << num1 << '\n' << std::hex << num2 << '\n';
     };
   } else {
     if((num2<-1e-10)||(num2>1e-10)) {
       WY_THROW( WyRet() );
     } else {
     }
   }

 }
};

static void ck_ntoa_float(void)
{
 const WyCSeg ChkStr("234.567");
 char buf[256];
 size_t v;
 v=Wy::_ntoa(buf,sizeof(buf),(float)234.567,10,3);
 if(v!=7) {
   WY_THROW( WyRet() );
 }
 if(WyCSeg(buf,7)!=ChkStr) {
   WY_THROW( WyRet() );
 }
 v=Wy::_ntoa_r(buf,sizeof(buf),(float)234.567,10,3);
 if(v!=7) {
   WY_THROW( WyRet() );
 }
 if(WyCSeg(buf+sizeof(buf)-7,7)!=ChkStr) {
   WY_THROW( WyRet() );
 }
};

int main(void)
try {
 std::cout << "Checking _ntoa(..,double,..) ";
 ck_ntoa_double();
 std::cout << "Ok\n";

 std::cout << "Checking _ntoa(..,float,..) ";
 ck_ntoa_float();
 std::cout << "Ok\n";
}
catch(const WyRet& e) {
 std::cerr << Wy::wrd(e).c_str() << '\n';
 return(-1);
}
catch(...) {
 std::cerr << "unknown throw\n";
 throw;
};
