/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2007
*/

#include "WyStr.h"
#include "WyNum.h"
#include <iostream>
#include <sstream>
#include <string>
#include <cstdlib>
#include <cassert>

template <typename NumType>
struct ChkSmp_S {
 const NumType i_num;
 const int     i_radix;
 const WyReply res_reply;
 const char* res_str;

 void verify(void) const {
   WyRet r;
   WyStr str;
   try {
     if((r=Wy::_mkstr(str,i_num,i_radix))!=res_reply) {
       WY_THROW(r);
     }
     if(r==Ok) {
       if(str!=res_str) {
         std::cerr << "str= " << str.c_str() << '\n';
         WY_THROW( WyRet() );
       }
     } else {
     }

     // chk _ntoa(char*..)
     //
     char dbuf[std::numeric_limits<NumType>::digits+2];  // [-] digits + one-chkchar
     std::memset(dbuf,0,sizeof(dbuf));
     size_t n=Wy::_ntoa(dbuf,sizeof(dbuf),i_num,i_radix);
     if(dbuf[n]!=0) {
       std::cerr << "dbuf overwritten: " << (int)dbuf[n] << '\n';
       WY_THROW( WyRet() );
     }
     if(n>0) {
       if(res_reply!=Ok) {
         WY_THROW( WyRet() );
       }
       if(WyCSeg(dbuf,n)!=res_str) {
         std::cerr << "str= " << str.c_str() << '\n';
         WY_THROW( WyRet() );
       }
     } else {
       if(res_reply==Ok) {
         WY_THROW( WyRet() );
       }
     }

     // chk _ntoa_r(char*..)
     //
     dbuf[sizeof(dbuf)-1]=0;
     n=Wy::_ntoa_r(dbuf,sizeof(dbuf)-1,i_num,i_radix);
     if(dbuf[sizeof(dbuf)-1]!=0) {
       std::cerr << "dbuf overwritten: " << (int)dbuf[sizeof(dbuf)-1] << '\n';
       WY_THROW( WyRet() );
     }
     if(n>0) {
       if(res_reply!=Ok) {
         WY_THROW( WyRet() );
       }
       if(WyCSeg(dbuf+sizeof(dbuf)-1-n,n)!=res_str) {
         std::cerr << "str= " << str.c_str() << '\n';
         WY_THROW( WyRet() );
       }
     } else {
       if(res_reply==Ok) {
         WY_THROW( WyRet() );
       }
     }
   }
   catch(...) {
     std::cerr << "ChkSmp_S{" <<  (long long)i_num << ", " << i_radix  << ", \""
            << res_str << "\"}\n";
     throw;
   };
 }
};

template <typename NumType>
static void ck_signed(void)
{
 const ChkSmp_S<NumType> chk_tab[]={
   {   0,  0, Wym_EINVAL, ""},
   {   0,  1, Wym_EINVAL, ""},
   {   0, 37, Wym_EINVAL, ""},
   {   0, 10, Ok, "0"},
   {   1, 10, Ok, "1"},      {  -1, 10, Ok, "-1"},
   {123,  2, Ok, "1111011"}, {-123,  2, Ok, "-1111011"},
   {0x7b, 16, Ok, "7b"},     {-0x7b, 16, Ok, "-7b"},
   { 074,  8, Ok, "74"},     {-074,  8, Ok, "-74"},
   {  71, 36, Ok, "1z"},     {- 71, 36, Ok, "-1z"},
 };
 for(size_t i=0; i<sizeof(chk_tab)/sizeof(ChkSmp_S<NumType>); ++i) {
   chk_tab[i].verify();
 }

 std::ostringstream os;

 // chk max
 {
  os.str("");
  const NumType Num= std::numeric_limits<NumType>::max();
  os << std::dec << static_cast<signed long long>(Num);
  std::string strNum;
  strNum=os.str();
  ChkSmp_S<NumType> cksmp1={ Num, 10, Ok, strNum.c_str()};
  cksmp1.verify();

  os.str("");
  os << std::oct << static_cast<signed long long>(Num);
  strNum=os.str();
  ChkSmp_S<NumType> cksmp2={ Num, 8, Ok, strNum.c_str()};
  cksmp2.verify();

  os.str("");
  os << std::hex << static_cast<signed long long>(Num);
  strNum=os.str();
  ChkSmp_S<NumType> cksmp3={ Num, 16, Ok, strNum.c_str()};
  cksmp3.verify();
 }

 // chk min
 {
  os.str("");
  const NumType Num= std::numeric_limits<NumType>::min();
  os << std::dec << static_cast<signed long long>(Num);
  std::string strNum;
  strNum=os.str();
  ChkSmp_S<NumType> cksmp1={ Num, 10, Ok, strNum.c_str()};
  cksmp1.verify();

  // Different to std interpret negative oct/hex, so not checked
  //
  //os.str("");
  //os << std::oct << static_cast<signed long long>(Num);
  //strNum=os.str();
  //ChkSmp_S<NumType> cksmp2={ Num, 8, Ok, strNum.c_str()};
  //cksmp2.verify();
 }
};


template <typename NumType>
struct ChkSmp_U {
 const NumType i_num;
 const int     i_radix;
 const WyReply res_reply;
 const char* res_str;

 void verify(void) const {
   WyRet r;
   WyStr str;
   try {
     if((r=Wy::_mkstr(str,i_num,i_radix))!=res_reply) {
       WY_THROW(r);
     }
     if(r==Ok) {
       if(str!=res_str) {
         std::cerr << "str= " << str.c_str() << '\n';
         WY_THROW( WyRet() );
       }
     } else {
     }

     // chk _ntoa(char*..)
     //
     char dbuf[std::numeric_limits<NumType>::digits+1];  // digits + one-chkchar
     std::memset(dbuf,0,sizeof(dbuf));
     size_t n=Wy::_ntoa(dbuf,sizeof(dbuf),i_num,i_radix);
     if(dbuf[n]!=0) {
       std::cerr << "dbuf overwritten: " << (int)dbuf[n] << '\n';
       WY_THROW( WyRet() );
     }
     if(n>0) {
       if(res_reply!=Ok) {
         WY_THROW( WyRet() );
       }
       if(WyCSeg(dbuf,n)!=res_str) {
         std::cerr << "str= " << str.c_str() << '\n';
         WY_THROW( WyRet() );
       }
     } else {
       if(res_reply==Ok) {
         WY_THROW( WyRet() );
       }
     }

     // chk _ntoa_r(char*..)
     //
     dbuf[sizeof(dbuf)-1]=0;
     n=Wy::_ntoa_r(dbuf,sizeof(dbuf)-1,i_num,i_radix);
     if(dbuf[sizeof(dbuf)-1]!=0) {
       std::cerr << "dbuf overwritten: " << (int)dbuf[sizeof(dbuf)-1] << '\n';
       WY_THROW( WyRet() );
     }
     if(n>0) {
       if(res_reply!=Ok) {
         WY_THROW( WyRet() );
       }
       if(WyCSeg(dbuf+sizeof(dbuf)-1-n,n)!=res_str) {
         std::cerr << "str= " << str.c_str() << '\n';
         WY_THROW( WyRet() );
       }
     } else {
       if(res_reply==Ok) {
         WY_THROW( WyRet() );
       }
     }
   }
   catch(...) {
     std::cerr << "ChkSmp_U{" <<  (long long)i_num << ", " << i_radix  << ", \""
            << res_str << "\"}\n";
     throw;
   };
 }
};

template <typename NumType>
static void ck_unsigned(void)
{
 const ChkSmp_U<NumType> chk_tab[]={
   {   0,  0, Wym_EINVAL, ""},
   {   0,  1, Wym_EINVAL, ""},
   {   0, 37, Wym_EINVAL, ""},
   {   0, 10, Ok, "0"},
   {   1, 10, Ok, "1"},
   {123,  2, Ok, "1111011"},
   {0x7b, 16, Ok, "7b"},
   { 074,  8, Ok, "74"},
   {  71, 36, Ok, "1z"},
 };
 for(size_t i=0; i<sizeof(chk_tab)/sizeof(ChkSmp_U<NumType>); ++i) {
   chk_tab[i].verify();
 }

 std::ostringstream os;

 // chk max
 {
  os.str("");
  const NumType Num= std::numeric_limits<NumType>::max();
  os << std::dec << static_cast<unsigned long long>(Num);
  std::string strNum;
  strNum=os.str();
  ChkSmp_U<NumType> cksmp1={ Num, 10, Ok, strNum.c_str()};
  cksmp1.verify();

  os.str("");
  os << std::oct << static_cast<unsigned long long>(Num);
  strNum=os.str();
  ChkSmp_U<NumType> cksmp2={ Num, 8, Ok, strNum.c_str()};
  cksmp2.verify();

  os.str("");
  os << std::hex << static_cast<unsigned long long>(Num);
  strNum=os.str();
  ChkSmp_U<NumType> cksmp3={ Num, 16, Ok, strNum.c_str()};
  cksmp3.verify();
 }

 // chk min
 {
  os.str("");
  const NumType Num= std::numeric_limits<NumType>::min();
  os << std::dec << static_cast<unsigned long long>(Num);
  std::string strNum;
  strNum=os.str();
  ChkSmp_U<NumType> cksmp1={ Num, 10, Ok, strNum.c_str()};
  cksmp1.verify();

  // Different to std interpret negative oct/hex, so not checked
  //
  //os.str("");
  //os << std::oct << static_cast<signed long long>(Num);
  //strNum=os.str();
  //ChkSmp_U<NumType> cksmp2={ Num, 8, Ok, strNum.c_str()};
  //cksmp2.verify();
 }
};

int main(void)
try {
 std::cout << "Checking _ntoa(..) for integers ";
 ck_signed<char>();
 ck_signed<signed char>();
 ck_signed<signed short>();
 ck_signed<signed int>();
 ck_signed<signed long>();
 ck_signed<signed long long>();

 ck_unsigned<unsigned char>();
 ck_unsigned<unsigned short>();
 ck_unsigned<unsigned int>();
 ck_unsigned<unsigned long>();
 ck_unsigned<unsigned long long>();
 std::cout << "Ok\n";
}
catch(const WyRet& e) {
 std::cerr << Wy::wrd(e).c_str() << '\n';
 return(-1);
}
catch(...) {
 std::cerr << "unknown throw\n";
 throw;
};
