/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2007
*/

#include "WyStr.h"
#include "WyNum.h"
#include "WyTimeSpec.h"
#include <cmath>
#include <iostream>
#include <string>
#include <cstdlib>
#include <cassert>
#include <stdio.h>

struct ChkSmp_D {
 typedef WyTimeSpec NumType;
 const NumType i_num;
 const int     i_radix;
 const size_t  i_frdig;
 size_t        i_blen;
 const WyReply res_reply;
 const char*   res_str;

 void verify(void) const {
   try {
     const char ChkChr=255;
     char dbuf[256];    // long enough for known types

     // chk _ntoa(char*..)
     //
     assert( (i_blen>=0)&&(i_blen<sizeof(dbuf)) );
     std::memset(dbuf,ChkChr,sizeof(dbuf)); // for chk overwritten
     size_t rv=Wy::_ntoa(dbuf,i_blen,i_num,i_radix,i_frdig);
     if(dbuf[i_blen]!=ChkChr) {
       std::cerr << "dbuf overwritten: " << (int)dbuf[i_blen] << '\n';
       dbuf[sizeof(dbuf)-1]=0;
       std::cout << "dbuf[]=" << dbuf << '\n';
       WY_THROW( WyRet() );
     }
     if(rv>0) {
       if(rv>=sizeof(dbuf)-1) {
         std::cout << "Unexpected rv: " << rv << '\n';
         WY_THROW( WyRet() );
       }
       dbuf[rv]=0;
       if(res_reply!=Ok) {
         std::cout << "Unexpected rv: " << rv << '\n';
         std::cout << "dbuf[]=" << dbuf << '\n';
         WY_THROW( WyRet() );
       }
       if(WyCSeg(dbuf,rv)!=res_str) {
         std::cerr << "dbuf= " << dbuf << '\n';
         WY_THROW( WyRet() );
       }
     } else {
       if(dbuf[0]!=ChkChr) {
         std::cerr << "dbuf overwritten: " << (int)dbuf[0] << '\n';
         std::cout << "dbuf[]=" << dbuf << '\n';
         WY_THROW( WyRet() );
       }
       if(res_reply==Ok) {
         std::cout << "Unexpected rv: " << rv << '\n';
         WY_THROW( WyRet() );
       }
     }

   }
   catch(...) {
     std::cerr << "ChkSmp_D{ " <<  Wy::wrd(i_num).c_str() << ", " << i_radix  << ", " << i_frdig
                << ", " << i_blen << ", " << Wy::wrd(res_reply).c_str() 
               << ", \"" << res_str << "\"}\n";
     throw;
   };
 }
};

static void ck_ntoa_timespec(void)
{
 typedef WyTimeSpec NumType;
 const ChkSmp_D chk_tab[]={
   // Checking takes exact value, verify manually some error dumps are ok
   // num radix frdig blen reply rstr

   {WyTimeSpec(),  1, 0, 128,Wym_EINVAL, ""},
   {WyTimeSpec(),  8, 0,   0,Wym_EINVAL, ""}, 
   {WyTimeSpec(), 10, 0, 128,Ok, "0"},

   {WyTimeSpec(99,2000000001), 10, 9,  13,Ok, "101.000000001"},
   {WyTimeSpec(-99,-2000000001), 10, 9,14,Ok, "-101.000000001"},
   {WyTimeSpec(99,0), 10, 0,   2,Ok, "99"},
   {WyTimeSpec(-99,0), 10, 0,   3,Ok, "-99"},
   {WyTimeSpec(-99,0), 10, 0,   2,Wym_EINVAL, "-99"},
   {WyTimeSpec(7,7700), 10, 7, 17, Ok, "7.0000077"},
   {WyTimeSpec(0,1000), 10, 6,   8,Ok, "0.000001"}, 
   {WyTimeSpec(0x123,71044922), 16, 3,   7,Ok, "123.123"}, 
   {WyTimeSpec(0x123,999999999), 2, 30, 40,Ok, "100100011.111111111111111111111111111110"},
   {WyTimeSpec(18,999999999), 3, 19, 23, Ok, "200.2222222222222222221"},
   {WyTimeSpec(27,999999999), 4, 15, 30, Ok, "123.333333333333332"},
   {WyTimeSpec(38,999999999), 5, 13, 30, Ok, "123.4444444444442"},
   {WyTimeSpec(51,999999999), 6, 12, 30, Ok, "123.555555555552"},
   {WyTimeSpec(66,999999999), 7, 11, 30, Ok, "123.66666666664"},
   {WyTimeSpec(83,999999999), 8, 10, 30, Ok, "123.7777777776"},
   {WyTimeSpec(102,999999999), 9, 10, 30, Ok, "123.8888888885"},
   {WyTimeSpec(123,999999999), 10, 9, 30, Ok, "123.999999999"},
   {WyTimeSpec(146,999999999), 11, 9, 30, Ok, "123.aaaaaaaa8"},
   {WyTimeSpec(171,999999999), 12, 9, 30, Ok, "123.bbbbbbbb6"},
   {WyTimeSpec(198,999999999), 13, 9, 30, Ok, "123.cccccccc0"},
   {WyTimeSpec(227,999999999), 14, 8, 30, Ok, "123.dddddddb"},
   {WyTimeSpec(258,999999999), 15, 8, 30, Ok, "123.eeeeeeeb"},
   {WyTimeSpec(291,999999999), 16, 8, 30, Ok, "123.fffffffa"},
   {WyTimeSpec(326,999999999), 17, 8, 30, Ok, "123.ggggggg8"},
   {WyTimeSpec(363,999999999), 18, 8, 30, Ok, "123.hhhhhhh5"},
   {WyTimeSpec(402,999999999), 19, 8, 30, Ok, "123.iiiiiii2"},
   {WyTimeSpec(443,999999999), 20, 7, 30, Ok, "123.jjjjjji"},
   {WyTimeSpec(486,999999999), 21, 7, 30, Ok, "123.kkkkkkh"},
   {WyTimeSpec(531,999999999), 22, 7, 30, Ok, "123.lllllli"},
   {WyTimeSpec(578,999999999), 23, 7, 30, Ok, "123.mmmmmmi"},
   {WyTimeSpec(627,999999999), 24, 7, 30, Ok, "123.nnnnnnh"},
   {WyTimeSpec(678,999999999), 25, 7, 30, Ok, "123.ooooooi"},
   {WyTimeSpec(731,999999999), 26, 7, 30, Ok, "123.ppppppg"},
   {WyTimeSpec(786,999999999), 27, 7, 30, Ok, "123.qqqqqqg"},
   {WyTimeSpec(843,999999999), 28, 7, 30, Ok, "123.rrrrrrd"},
   {WyTimeSpec(902,999999999), 29, 7, 30, Ok, "123.ssssssb"},
   {WyTimeSpec(963,999999999), 30, 7, 30, Ok, "123.tttttt6"},
   {WyTimeSpec(1026,999999999), 31, 7,  11,Ok, "123.uuuuuu2"},
   {WyTimeSpec(1091,999999999), 32, 6, 30, Ok, "123.vvvvvu"},
   {WyTimeSpec(1158,999999999), 33, 6, 30, Ok, "123.wwwwwv"},
   {WyTimeSpec(1227,999999999), 34, 6, 30, Ok, "123.xxxxxv"},
   {WyTimeSpec(1298,999999999), 35, 6, 30, Ok, "123.yyyyyw"},
   {WyTimeSpec(1371,999999999), 36, 6, 30, Ok, "123.zzzzzw"},

   {WyTimeSpec(-0x123,-999999999), 2, 30, 41,Ok, "-100100011.111111111111111111111111111110"},
   {WyTimeSpec(-18,-999999999), 3, 19, 24, Ok, "-200.2222222222222222221"},
   {WyTimeSpec(-27,-999999999), 4, 15, 30, Ok, "-123.333333333333332"},
   {WyTimeSpec(-38,-999999999), 5, 13, 30, Ok, "-123.4444444444442"},
   {WyTimeSpec(-51,-999999999), 6, 12, 30, Ok, "-123.555555555552"},
   {WyTimeSpec(-66,-999999999), 7, 11, 30, Ok, "-123.66666666664"},
   {WyTimeSpec(-83,-999999999), 8, 10, 30, Ok, "-123.7777777776"},
   {WyTimeSpec(-102,-999999999), 9, 10, 30, Ok, "-123.8888888885"},
   {WyTimeSpec(-123,-999999999), 10, 9, 30, Ok, "-123.999999999"},
   {WyTimeSpec(-146,-999999999), 11, 9, 30, Ok, "-123.aaaaaaaa8"},
   {WyTimeSpec(-171,-999999999), 12, 9, 30, Ok, "-123.bbbbbbbb6"},
   {WyTimeSpec(-198,-999999999), 13, 9, 30, Ok, "-123.cccccccc0"},
   {WyTimeSpec(-227,-999999999), 14, 8, 30, Ok, "-123.dddddddb"},
   {WyTimeSpec(-258,-999999999), 15, 8, 30, Ok, "-123.eeeeeeeb"},
   {WyTimeSpec(-291,-999999999), 16, 8, 30, Ok, "-123.fffffffa"},
   {WyTimeSpec(-326,-999999999), 17, 8, 30, Ok, "-123.ggggggg8"},
   {WyTimeSpec(-363,-999999999), 18, 8, 30, Ok, "-123.hhhhhhh5"},
   {WyTimeSpec(-402,-999999999), 19, 8, 30, Ok, "-123.iiiiiii2"},
   {WyTimeSpec(-443,-999999999), 20, 7, 30, Ok, "-123.jjjjjji"},
   {WyTimeSpec(-486,-999999999), 21, 7, 30, Ok, "-123.kkkkkkh"},
   {WyTimeSpec(-531,-999999999), 22, 7, 30, Ok, "-123.lllllli"},
   {WyTimeSpec(-578,-999999999), 23, 7, 30, Ok, "-123.mmmmmmi"},
   {WyTimeSpec(-627,-999999999), 24, 7, 30, Ok, "-123.nnnnnnh"},
   {WyTimeSpec(-678,-999999999), 25, 7, 30, Ok, "-123.ooooooi"},
   {WyTimeSpec(-731,-999999999), 26, 7, 30, Ok, "-123.ppppppg"},
   {WyTimeSpec(-786,-999999999), 27, 7, 30, Ok, "-123.qqqqqqg"},
   {WyTimeSpec(-843,-999999999), 28, 7, 30, Ok, "-123.rrrrrrd"},
   {WyTimeSpec(-902,-999999999), 29, 7, 30, Ok, "-123.ssssssb"},
   {WyTimeSpec(-963,-999999999), 30, 7, 30, Ok, "-123.tttttt6"},
   {WyTimeSpec(-1026,-999999999), 31, 7,  12,Ok, "-123.uuuuuu2"},
   {WyTimeSpec(-1091,-999999999), 32, 6, 30, Ok, "-123.vvvvvu"},
   {WyTimeSpec(-1158,-999999999), 33, 6, 30, Ok, "-123.wwwwwv"},
   {WyTimeSpec(-1227,-999999999), 34, 6, 30, Ok, "-123.xxxxxv"},
   {WyTimeSpec(-1298,-999999999), 35, 6, 30, Ok, "-123.yyyyyw"},
   {WyTimeSpec(-1371,-999999999), 36, 6, 30, Ok, "-123.zzzzzw"},

   {WyTimeSpec(7,123456789), 2, 30, 34, Ok, "111.000111111001101011011101001101"},
   {WyTimeSpec(13,123456789), 3, 19, 23, Ok, "111.0100222222222222221"},
   {WyTimeSpec(21,123456789), 4, 15, 19, Ok, "111.013321223131031"},
   {WyTimeSpec(31,123456789), 5, 13, 17, Ok, "111.0302040012322"},
   {WyTimeSpec(43,123456789), 6, 12, 16, Ok, "111.042355555553"},
   {WyTimeSpec(57,123456789), 7, 11, 15, Ok, "111.06022636551"},
   {WyTimeSpec(73,123456789), 8, 10, 14, Ok, "111.0771533515"},
   {WyTimeSpec(91,123456789), 9, 10, 14, Ok, "111.1088888885"},
   {WyTimeSpec(111,123456789), 10, 9, 13, Ok, "111.123456789"}, 
   {WyTimeSpec(133,123456789), 11, 9, 13, Ok, "111.13a359261"},
   {WyTimeSpec(157,123456789), 12, 9, 13, Ok, "111.1593bbbb6"},
   {WyTimeSpec(183,123456789), 13, 9, 13, Ok, "111.17b308456"},
   {WyTimeSpec(211,123456789), 14, 8, 12, Ok, "111.1a2aa049"},
   {WyTimeSpec(241,123456789), 15, 8, 12, Ok, "111.1cb9eeeb"},
   {WyTimeSpec(273,123456789), 16, 8, 12, Ok, "111.1f9add37"},
   {WyTimeSpec(307,123456789), 17, 8, 12, Ok, "111.21b93gcg"},
   {WyTimeSpec(343,123456789), 18, 8, 12, Ok, "111.23hhhhh5"},
   {WyTimeSpec(381,123456789), 19, 8, 12, Ok, "111.26af047c"},
   {WyTimeSpec(421,123456789), 20, 7, 11, Ok, "111.297d1e9"},
   {WyTimeSpec(463,123456789), 21, 7, 11, Ok, "111.2c96kki"},
   {WyTimeSpec(507,123456789), 22, 7, 11, Ok, "111.2fgcaij"},
   {WyTimeSpec(553,123456789), 23, 7, 11, Ok, "111.2j7265b"},
   {WyTimeSpec(601,123456789), 24, 7, 11, Ok, "111.2n2fnni"},
   {WyTimeSpec(651,123456789), 25, 7, 11, Ok, "111.32407hf"},
   {WyTimeSpec(703,123456789), 26, 7, 11, Ok, "111.35bmkdk"},
   {WyTimeSpec(757,123456789), 27, 7, 11, Ok, "111.38qqqqf"},
   {WyTimeSpec(813,123456789), 28, 7, 11, Ok, "111.3cm3clg"},
   {WyTimeSpec(871,123456789), 29, 7, 11, Ok, "111.3gnsih6"},
   {WyTimeSpec(931,123456789), 30, 7, 11, Ok, "111.3l39tt5"},
   {WyTimeSpec(993,123456789), 31, 7, 11, Ok, "111.3pjrt1l"},
   {WyTimeSpec(1057,123456789), 32, 6, 10, Ok, "111.3uddqd"},
   {WyTimeSpec(1123,123456789), 33, 6, 10, Ok, "111.42elwv"},
   {WyTimeSpec(1191,123456789), 34, 6, 10, Ok, "111.46obpi"},
   {WyTimeSpec(1261,123456789), 35, 6, 10, Ok, "111.4b87c1"},
   {WyTimeSpec(1333,123456789), 36, 6, 10, Ok, "111.4fzzzx"},

   {WyTimeSpec(-7,-123456789), 2, 30, 35, Ok, "-111.000111111001101011011101001101"},
   {WyTimeSpec(-13,-123456789), 3, 19, 24, Ok, "-111.0100222222222222221"},
   {WyTimeSpec(-21,-123456789), 4, 15, 20, Ok, "-111.013321223131031"},
   {WyTimeSpec(-31,-123456789), 5, 13, 18, Ok, "-111.0302040012322"},
   {WyTimeSpec(-43,-123456789), 6, 12, 17, Ok, "-111.042355555553"},
   {WyTimeSpec(-57,-123456789), 7, 11, 16, Ok, "-111.06022636551"},
   {WyTimeSpec(-73,-123456789), 8, 10, 15, Ok, "-111.0771533515"},
   {WyTimeSpec(-91,-123456789), 9, 10, 15, Ok, "-111.1088888885"},
   {WyTimeSpec(-111,-123456789), 10, 9, 14, Ok, "-111.123456789"}, 
   {WyTimeSpec(-133,-123456789), 11, 9, 14, Ok, "-111.13a359261"},
   {WyTimeSpec(-157,-123456789), 12, 9, 14, Ok, "-111.1593bbbb6"},
   {WyTimeSpec(-183,-123456789), 13, 9, 14, Ok, "-111.17b308456"},
   {WyTimeSpec(-211,-123456789), 14, 8, 13, Ok, "-111.1a2aa049"},
   {WyTimeSpec(-241,-123456789), 15, 8, 13, Ok, "-111.1cb9eeeb"},
   {WyTimeSpec(-273,-123456789), 16, 8, 13, Ok, "-111.1f9add37"},
   {WyTimeSpec(-307,-123456789), 17, 8, 13, Ok, "-111.21b93gcg"},
   {WyTimeSpec(-343,-123456789), 18, 8, 13, Ok, "-111.23hhhhh5"},
   {WyTimeSpec(-381,-123456789), 19, 8, 13, Ok, "-111.26af047c"},
   {WyTimeSpec(-421,-123456789), 20, 7, 12, Ok, "-111.297d1e9"},
   {WyTimeSpec(-463,-123456789), 21, 7, 12, Ok, "-111.2c96kki"},
   {WyTimeSpec(-507,-123456789), 22, 7, 12, Ok, "-111.2fgcaij"},
   {WyTimeSpec(-553,-123456789), 23, 7, 12, Ok, "-111.2j7265b"},
   {WyTimeSpec(-601,-123456789), 24, 7, 12, Ok, "-111.2n2fnni"},
   {WyTimeSpec(-651,-123456789), 25, 7, 12, Ok, "-111.32407hf"},
   {WyTimeSpec(-703,-123456789), 26, 7, 12, Ok, "-111.35bmkdk"},
   {WyTimeSpec(-757,-123456789), 27, 7, 12, Ok, "-111.38qqqqf"},
   {WyTimeSpec(-813,-123456789), 28, 7, 12, Ok, "-111.3cm3clg"},
   {WyTimeSpec(-871,-123456789), 29, 7, 12, Ok, "-111.3gnsih6"},
   {WyTimeSpec(-931,-123456789), 30, 7, 12, Ok, "-111.3l39tt5"},
   {WyTimeSpec(-993,-123456789), 31, 7, 12, Ok, "-111.3pjrt1l"},
   {WyTimeSpec(-1057,-123456789), 32, 6, 11, Ok, "-111.3uddqd"},
   {WyTimeSpec(-1123,-123456789), 33, 6, 11, Ok, "-111.42elwv"},
   {WyTimeSpec(-1191,-123456789), 34, 6, 11, Ok, "-111.46obpi"},
   {WyTimeSpec(-1261,-123456789), 35, 6, 11, Ok, "-111.4b87c1"},
   {WyTimeSpec(-1333,-123456789), 36, 6, 11, Ok, "-111.4fzzzx"},

   {WyTimeSpec(654,321), 10,10, 13, Ok, "654.000000321"},
   {WyTimeSpec(-654,-321), 10,10, 14, Ok, "-654.000000321"},
   {WyTimeSpec(123,456000000), 10, 3, 128, Ok, "123.456"},
   {WyTimeSpec(-123,-456000000), 10, 3, 128, Ok, "-123.456"}, 

   {WyTimeSpec(),  0, 0,   0,Wym_EINVAL, "x"}, 
   {WyTimeSpec(),  0, 0,   1,Ok, "0"},
   {WyTimeSpec(),  0, 1, 128,Ok, "0"},
   {WyTimeSpec(12345,0),  0,  0, 4, Wym_EINVAL, ""},
   {WyTimeSpec(12345,0),  0,  1, 5, Wym_EINVAL, ""},
   {WyTimeSpec(123,4567),  0,  0, 128, Ok, "1e+2"},
   {WyTimeSpec(123,4567),  0,  8, 13, Ok, "1.23000004e+2"},
   {WyTimeSpec(-123,-4567),  0,  8, 14, Ok, "-1.23000004e+2"},
   {WyTimeSpec(0,123456789),  0,  2, 7, Ok, "1.23e-1"},
   {WyTimeSpec(-0,-123456789),  0,  2, 8, Ok, "-1.23e-1"},
   {WyTimeSpec(0,12345678),  0,  2, 7, Ok, "1.23e-2"},
   {WyTimeSpec(-0,-12345678),  0,  2, 8, Ok, "-1.23e-2"},
   {WyTimeSpec(1,234567890),  0,  2, 4, Ok, "1.23"},
   {WyTimeSpec(-1,-234567890),  0,  2, 5, Ok, "-1.23"},
   {WyTimeSpec(10,123456789),  0,  3, 8, Ok, "1.012e+1"},
   {WyTimeSpec(-10,-123456789),  0,  3, 9, Ok, "-1.012e+1"},
   {WyTimeSpec(-10,-123456789),  0,  3, 2, Wym_EINVAL, ""},
   {WyTimeSpec(-10,-123456789),  0,  3, 8, Wym_EINVAL, ""},
   {WyTimeSpec(10,123456789),  0,  0, 4, Ok, "1e+1"},
   {WyTimeSpec(-10,-123456789),  0,  0, 5, Ok, "-1e+1"},
   {WyTimeSpec(0,123),  0,  2, 6, Wym_EINVAL, "1.23e-7"},
   {WyTimeSpec(0,123),  0,  2, 7, Ok, "1.23e-7"},
   {WyTimeSpec(-0,-123),  0,  2, 8, Ok, "-1.23e-7"},
   {WyTimeSpec(-0,-123),  0,  2,18, Ok, "-1.23e-7"},
 };
 for(size_t i=0; i<sizeof(chk_tab)/sizeof(ChkSmp_D); ++i) {
   chk_tab[i].verify();
 }

 // chk <max>
 {
   NumType num1= std::numeric_limits<NumType>::max();
   NumType num2;

   char dbuf[std::numeric_limits<NumType>::max_exponent10+16];
   size_t v;

   // ::strtod returns inf for the max. in scientific notation. So removed this check

   v=Wy::_ntoa(dbuf, sizeof(dbuf), num1,10, 1);
   if(v==0) {
     WY_THROW( WyRet() );
   }
   dbuf[v]=0;

   num2=::strtod(dbuf,NULL);

   // Dump instances of larger erronous ratio
   {
     double ErrRat;
     if((num2<-1e-10)||(num2>1e-10)) {
       if(num2!=0.0) {
         ErrRat=Wy::to_double(num1/num2);
       } else if(num1!=0.0) {
         ErrRat=Wy::to_double(num2/num1);
       } else {
         ErrRat=1.0;
       }
       if((ErrRat>1.000001)||(ErrRat<0.999999)) {  // 1/million precision
         std::cout << dbuf << '\n';
         std::cout << Wy::wrd(num1).c_str() << '\n' << Wy::wrd(num2).c_str() << '\n';
       };
     } else {
       if((num1<-1e-10)||(num1>1e-10)) {
         WY_THROW( WyRet() );
       }
     }
   }
 }

 // chk <min>
 {
   NumType num1= std::numeric_limits<NumType>::min();
   NumType num2;

   char dbuf[-std::numeric_limits<NumType>::min_exponent10+16];
   size_t v;

   dbuf[ Wy::_ntoa(dbuf, sizeof(dbuf), num1, 0, 18) ]=0;
   if(dbuf[0]==0) {
     WY_THROW( WyRet() );
   }

   num2=::strtod(dbuf,NULL);

   // Dump instances of larger erronous ratio
   {
     double ErrRat;
     if((num2<-1e-10)||(num2>1e-10)) {
       if(num2!=0.0) {
         ErrRat=Wy::to_double(num1/num2);
       } else if(num1!=0.0) {
         ErrRat=Wy::to_double(num2/num1);
       } else {
         ErrRat=1.0;
       }
       if((ErrRat>1.000001)||(ErrRat<0.999999)) {  // 1/million precision
         std::cout << dbuf << '\n';
         std::cout << Wy::wrd(num1).c_str() << '\n' << Wy::wrd(num2).c_str() << '\n';
       };
     } else {
       if((num1<-1e-10)||(num1>1e-10)) {
         WY_THROW( WyRet() );
       }
     }
   }

   v=Wy::_ntoa(dbuf, sizeof(dbuf), num1,10, 1);
   if(v==0) {
     WY_THROW( WyRet() );
   }
   dbuf[v]=0;

   num2=::strtod(dbuf,NULL);

   // Dump instances of larger erronous ratio
   {
     double ErrRat;
     if((num2<-1e-10)||(num2>1e-10)) {
       if(num2!=0.0) {
         ErrRat=Wy::to_double(num1/num2);
       } else if(num1!=0.0) {
         ErrRat=Wy::to_double(num2/num1);
       } else {
         ErrRat=1.0;
       }
       if((ErrRat>1.000001)||(ErrRat<0.999999)) {  // 1/million precision
         std::cout << dbuf << '\n';
         std::cout << Wy::wrd(num1).c_str() << '\n' << Wy::wrd(num2).c_str() << '\n';
       };
     } else {
       if((num1<-1e-10)||(num1>1e-10)) {
         WY_THROW( WyRet() );
       }
     }
   }
 }

 for(int i=0; i<10000; ++i) {
   //  res= x*2^y  ,x in [-1.0,+1.0], y in [-RandExp, +RandExp]
   //
   const long long RandExp=4;
   const NumType num1= std::ldexp( ::rand()*(2.0/(double)RAND_MAX)-1.0
              , (::rand()*(long long)2*RandExp)/(long long)RAND_MAX-RandExp 
             );

   // Scientific notation (l)
   //
   char dbuf[128];
   size_t v=Wy::_ntoa(dbuf, sizeof(dbuf), num1, 0, 6);
   if(v==0) {
     WY_THROW( WyRet() );
   }
   dbuf[v]=0;

   NumType num2=::strtod(dbuf,NULL);

   // Dump instances of larger erronous ratio
   {
     if((num2<-1e-3)||(num2>1e-3)) {
       double ErrRat;
       if(num2!=0.0) {
         ErrRat=Wy::to_double(num1/num2);
       } else if(num1!=0.0) {
         ErrRat=Wy::to_double(num2/num1);
       } else {
         ErrRat=1.0;
       }
       if((ErrRat>1.000001)||(ErrRat<0.999999)) {  // 1/million precision
         std::cout << dbuf << '\n';
         std::cout << Wy::wrd(num1).c_str() << '\n' << Wy::wrd(num2).c_str() << '\n';
       };
     } else {
       if((num1<-1e-3)||(num1>1e-3)) {
         WY_THROW( WyRet() );
       }
     }
   }

   // Radix=10, float notation (l)
   //
   v=Wy::_ntoa(dbuf, sizeof(dbuf), num1,10, 18);
   if(v==0) {
     WY_THROW( WyRet() );
   }
   dbuf[v]=0;

   num2=::strtod(dbuf,NULL);

   // Dump instances of larger erronous ratio
   {
     double ErrRat;
     if((num2<-1e-3)||(num2>1e-3)) {
       if(num2!=0.0) {
         ErrRat=Wy::to_double(num1/num2);
       } else if(num1!=0.0) {
         ErrRat=Wy::to_double(num2/num1);
       } else {
         ErrRat=1.0;
       }
       if((ErrRat>1.000001)||(ErrRat<0.999999)) {  // 1/million precision
         std::cout << dbuf << '\n';
         std::cout << Wy::wrd(num1).c_str() << '\n' << Wy::wrd(num2).c_str() << '\n';
       };
     } else {
       if((num1<-1e-3)||(num1>1e-3)) {
         WY_THROW( WyRet() );
       }
     }
   }

   // Radix=16, Hexidecimal (l)
   //
   std::string nstr;
   v=Wy::_ntoa(dbuf, sizeof(dbuf), num1,16, 18);
   if(v==0) {
     WY_THROW( WyRet() );
   }
   dbuf[v]=0;

   if(dbuf[0]=='-') {
     nstr="-0x";
     nstr+=dbuf+1;
     nstr+="p0";
   } else {
     nstr="0x";
     nstr+=dbuf;
     nstr+="p0";
   }
   num2=::strtod(nstr.c_str(),NULL);

   // Dump instances of larger erronous ratio
   if((num1<-1e-3)||(num1>1e-3)) {
     double ErrRat;
     if(num2!=0.0) {
       ErrRat=Wy::to_double(num1/num2);
     } else if(num1!=0.0) {
       ErrRat=Wy::to_double(num2/num1);
     } else {
       ErrRat=1.0;
     }
     if((ErrRat>1.000001)||(ErrRat<0.999999)) {  // 1/million precision
       std::cout << std::hex << dbuf << '\n';
       std::cout << std::hex << Wy::wrd(num1).c_str() << '\n' << std::hex << Wy::wrd(num2).c_str() << '\n';
     };
   } else {
     if((num2<-1e-3)||(num2>1e-3)) {
       WY_THROW( WyRet() );
     } else {
     }
   }

 }
};

int main(void)
try {
 std::cout << "Checking _ntoa(..,WyTimeSpec,..) ";
 ck_ntoa_timespec();
 std::cout << "Ok\n";
}
catch(const WyRet& e) {
 std::cerr << Wy::wrd(e).c_str() << '\n';
 return(-1);
}
catch(...) {
 std::cerr << "unknown throw\n";
 throw;
};
