/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2005

   Build: make chk_poly

   For the moment checking ctor(const WyByteFlow&) only 
   (since implement calls reset(..))
*/

#include "WyRegFile.h"
#include "WyChrFile.h"
#include "WyFifoFile.h"
#include "WySockFile.h"
#include "WyTerminal.h"
#include "WyFileStat.h"
#include <iostream>
#include "chk_data.h"

// Assert sysf is default
//
static void assert_default(const WySysFile& sysf)
{
 WyRet r;

 if(sysf.is_default()==false) {
   WY_THROW( WyRet() );
 }
 if(sysf.fh().is_default()==false) {
   WY_THROW( WyRet() );
 }

 // chk fh()
 if(sysf.fh()!=WyFileHandle()) {
   WY_THROW( WyRet() );
 }

 // stat
 {
   WyFileStat stt;
   if((r=sysf.stat(stt))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if(stt.is_default()==false) {
     WY_THROW( WyRet() );
   }
 }

 if(sysf.wy_fd()!=WY__DEFAULT_FD) {
   WY_THROW( WyRet() );
 }
 if(sysf.ftype()!=0) {
   WY_THROW( WyRet() );
 }
};

// Assert f1,f2 refers to the same systerm file
//
static void assert_same_ftype(const WySysFile& f1,const WySysFile& f2) 
{
 if(f1.ftype()!=f2.ftype()) {
   WY_THROW( WyRet() );
 }
};

template<typename Dev1, typename Dev2>
static void assert_same_sysf(const Dev1& f1,const Dev2& f2)
{
 assert_same_ftype(f1,f2);
 if(f1.is_default()&&f2.is_default()) {
   return;
 }
 WyRet r;
 WyFileStat stt1,stt2;
 if((r=f1.stat(stt1))!=Ok) {
   WY_THROW( WyRet() );
 }
 if((r=f2.stat(stt2))!=Ok) {
   WY_THROW( WyRet() );
 }
 if(stt1.sf_ino()!=stt2.sf_ino()) {
   WY_THROW( WyRet() );
 }
 if(stt1.sf_dev()!=stt2.sf_dev()) {
   WY_THROW( WyRet() );
 }
};

#define CHK_DEFAULT(p)\
    try { assert_default(p); } catch(...) { WY_THROW( WyRet()); }
#define CHK_SAME_SYSF(p1,p2)\
    try { assert_same_sysf(p1,p2); } catch(...) { WY_THROW( WyRet()); }


const WyStr TFifoName("DeleteMe.fifo");

class Chk_Poly {
    const bool _init_ok;

    const WyRegFile RegF_0;
    const WyRegFile RegF_1;
    const WyChrFile ChrF_0;
    const WyChrFile ChrF_1;
    const WyFifoFile Fifo_0;
    const WyFifoFile Fifo_1;
    const WySockFile SktF_0;
    const WySockFile SktF_1;
    const WyTerminal TermF_0;
    const WyTerminal TermF_1;
    bool _init(void) {
           Wy::unlink(TFifoName);
           WyRet r;
           if((r=WyFifoFile::create(TFifoName,S_IRUSR|S_IWUSR))!=Ok) {
             WY_THROW(r);
           }
           return(true);
         };
  public:
    Chk_Poly() 
      try :
        _init_ok(_init()),
        RegF_0(),
        RegF_1(__FILE__,O_RDONLY),
        ChrF_0(),
        ChrF_1("/dev/zero",O_RDONLY),
        Fifo_0(),
        Fifo_1(TFifoName,O_RDWR),
        SktF_0(),
        SktF_1(PF_LOCAL,SOCK_DGRAM),
        TermF_0(),
        TermF_1("/dev/tty",O_RDONLY)
    {
        CHK_DEFAULT(RegF_0);
        CHK_DEFAULT(ChrF_0);
        CHK_DEFAULT(Fifo_0);
        CHK_DEFAULT(SktF_0);
        CHK_DEFAULT(TermF_0);
    }
    catch(const WyRet& e) {
      std::cerr << Wy::wrd(e).c_str() << '\n';
    }
    catch(...) {
      Wy::unlink(TFifoName);
      throw;
    };

    ~Chk_Poly() {
      Wy::unlink(TFifoName);
    };

    void ck_chrfile(void);
    void ck_regfile(void);
    void ck_sockfile(void);
    void ck_fifofile(void);
    void ck_terminal(void);
    void ck_byteflow(void);

} static pchk_obj;

/*
 {
  WyChrFile tmp(RegF_0);
  WyChrFile tmp(RegF_1);
  WyChrFile tmp(ChrF_0);
  WyChrFile tmp(ChrF_1);
  WyChrFile tmp(Fifo_0);
  WyChrFile tmp(Fifo_1);
  WyChrFile tmp(SktF_0);
  WyChrFile tmp(SktF_1);
  WyChrFile tmp(TermF_0);
  WyChrFile tmp(TermF_1);
 }
*/
void Chk_Poly::ck_chrfile(void)
{
 WyRet r;

 {
   {
     try {
      WyChrFile tmp(static_cast<const WySysFile&>(RegF_0));
      WY_THROW( WyRet() );
     }
     catch(const WyChrFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
  
   {
     try {
      WyChrFile tmp(static_cast<const WySysFile&>(RegF_1));
      WY_THROW( WyRet() );
     }
     catch(const WyChrFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
 }

 {
   {
      WyChrFile tmp(ChrF_0);
      CHK_DEFAULT(tmp);
   }
  
   {
      WyChrFile tmp(ChrF_1);
      CHK_SAME_SYSF(tmp,ChrF_1);
   }
  
 }

 {
   {
     try {
      WyChrFile tmp(static_cast<const WyByteFlow&>(Fifo_0));
      WY_THROW( WyRet() );
     }
     catch(const WyChrFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }

   {
     try {
      WyChrFile tmp(static_cast<const WyByteFlow&>(Fifo_1));
      WY_THROW( WyRet() );
     }
     catch(const WyChrFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
 }

 {
   {
     try {
      WyChrFile tmp(static_cast<const WyByteFlow&>(SktF_0));
      WY_THROW( WyRet() );
     }
     catch(const WyChrFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }

   {
     try {
      WyChrFile tmp(static_cast<const WyByteFlow&>(SktF_1));
      WY_THROW( WyRet() );
     }
     catch(const WyChrFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
 }

 {
   {
      WyChrFile tmp(TermF_0);
      CHK_DEFAULT(tmp);
   }

   {
      WyChrFile tmp(TermF_1);
      CHK_SAME_SYSF(tmp,TermF_1);
   }
 }

 {
   WyChrFile tmp;
   if((r=tmp.reset(static_cast<const WyByteFlow&>(RegF_0)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if((r=tmp.reset(static_cast<const WyByteFlow&>(RegF_1)))!=Wym_EBADF) {
     WY_THROW(r);
   }

   if((r=tmp.reset(ChrF_0))!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(ChrF_1))!=Ok) {
     WY_THROW(r);
   }
   CHK_SAME_SYSF(tmp,ChrF_1);

   if((r=tmp.reset(static_cast<const WyByteFlow&>(Fifo_0)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if((r=tmp.reset(static_cast<const WyByteFlow&>(Fifo_1)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   
   if((r=tmp.reset(static_cast<const WyByteFlow&>(SktF_0)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if((r=tmp.reset(static_cast<const WyByteFlow&>(SktF_1)))!=Wym_EBADF) {
     WY_THROW(r);
   }

   if((r=tmp.reset(TermF_0))!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(TermF_1))!=Ok) {
     WY_THROW(r);
   }
   CHK_SAME_SYSF(tmp,TermF_1);
 }
};

void Chk_Poly::ck_regfile(void)
{
 WyRet r;

 {
   {
      WyRegFile tmp(RegF_0);
      CHK_DEFAULT(tmp);
   }
  
   {
      WyRegFile tmp(RegF_1);
      CHK_SAME_SYSF(tmp,RegF_1);
   }
 }

 {
   {
     try {
      WyRegFile tmp(static_cast<const WyByteFlow&>(ChrF_0));
      WY_THROW( WyRet() );
     }
     catch(const WyRegFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
  
   {
     try {
      WyRegFile tmp(static_cast<const WyByteFlow&>(ChrF_1));
      WY_THROW( WyRet() );
     }
     catch(const WyRegFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
  
 }

 {
   {
     try {
      WyRegFile tmp(static_cast<const WyByteFlow&>(Fifo_0));
      WY_THROW( WyRet() );
     }
     catch(const WyRegFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }

   {
     try {
      WyRegFile tmp(static_cast<const WyByteFlow&>(Fifo_1));
      WY_THROW( WyRet() );
     }
     catch(const WyRegFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
 }

 {
   {
     try {
      WyRegFile tmp(static_cast<const WyByteFlow&>(SktF_0));
      WY_THROW( WyRet() );
     }
     catch(const WyRegFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }

   {
     try {
      WyRegFile tmp(static_cast<const WySysFile&>(SktF_1));
      WY_THROW( WyRet() );
     }
     catch(const WyRegFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
 }

 {
   {
     try {
      WyRegFile tmp(static_cast<const WyByteFlow&>(TermF_0));
      WY_THROW( WyRet() );
     }
     catch(const WyRegFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }

   {
     try {
      WyRegFile tmp(static_cast<const WyByteFlow&>(TermF_1));
      WY_THROW( WyRet() );
     }
     catch(const WyRegFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
 }

 {
   WyRegFile tmp;
   if((r=tmp.reset(RegF_0))!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(RegF_1))!=Ok) {
     WY_THROW(r);
   }
   CHK_SAME_SYSF(tmp,RegF_1);

   if((r=tmp.reset(static_cast<const WyByteFlow&>(ChrF_0)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if((r=tmp.reset(static_cast<const WyByteFlow&>(ChrF_1)))!=Wym_EBADF) {
     WY_THROW(r);
   }

   if((r=tmp.reset(static_cast<const WyByteFlow&>(Fifo_0)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if((r=tmp.reset(static_cast<const WyByteFlow&>(Fifo_1)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   
   if((r=tmp.reset(static_cast<const WyByteFlow&>(SktF_0)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if((r=tmp.reset(static_cast<const WyByteFlow&>(SktF_1)))!=Wym_EBADF) {
     WY_THROW(r);
   }

   if((r=tmp.reset(static_cast<const WyByteFlow&>(TermF_0)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if((r=tmp.reset(static_cast<const WyByteFlow&>(TermF_1)))!=Wym_EBADF) {
     WY_THROW(r);
   }
 }
};

void Chk_Poly::ck_sockfile(void)
{
 WyRet r;

 {
   {
     try {
      WySockFile tmp(static_cast<const WyByteFlow&>(RegF_0));
      WY_THROW( WyRet() );
     }
     catch(const WySockFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
  
   {
     try {
      WySockFile tmp(static_cast<const WyByteFlow&>(RegF_1));
      WY_THROW( WyRet() );
     }
     catch(const WySockFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
 }

 {
   {
     try {
      WySockFile tmp(static_cast<const WyByteFlow&>(ChrF_0));
      WY_THROW( WyRet() );
     }
     catch(const WySockFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
  
   {
     try {
      WySockFile tmp(static_cast<const WyByteFlow&>(ChrF_1));
      WY_THROW( WyRet() );
     }
     catch(const WySockFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
 }

 {
   {
     try {
      WySockFile tmp(static_cast<const WyByteFlow&>(Fifo_0));
      WY_THROW( WyRet() );
     }
     catch(const WySockFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }

   {
     try {
      WySockFile tmp(static_cast<const WyByteFlow&>(Fifo_1));
      WY_THROW( WyRet() );
     }
     catch(const WySockFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
 }

 {
   {
      WySockFile tmp(SktF_0);
      CHK_DEFAULT(tmp);
   }

   {
      WySockFile tmp(SktF_1);
      CHK_SAME_SYSF(tmp,SktF_1);
   }
 }

 {
   {
     try {
      WySockFile tmp(static_cast<const WyByteFlow&>(TermF_0));
      WY_THROW( WyRet() );
     }
     catch(const WySockFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }

   {
     try {
      WySockFile tmp(static_cast<const WyByteFlow&>(TermF_1));
      WY_THROW( WyRet() );
     }
     catch(const WySockFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
 }

 {
   WySockFile tmp;
   if((r=tmp.reset(static_cast<const WyByteFlow&>(RegF_0)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if((r=tmp.reset(static_cast<const WyByteFlow&>(RegF_1)))!=Wym_EBADF) {
     WY_THROW(r);
   }

   if((r=tmp.reset(static_cast<const WyByteFlow&>(ChrF_0)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if((r=tmp.reset(static_cast<const WyByteFlow&>(ChrF_1)))!=Wym_EBADF) {
     WY_THROW(r);
   }

   if((r=tmp.reset(static_cast<const WyByteFlow&>(Fifo_0)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if((r=tmp.reset(static_cast<const WyByteFlow&>(Fifo_1)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   
   if((r=tmp.reset(SktF_0))!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(SktF_1))!=Ok) {
     WY_THROW(r);
   }
   CHK_SAME_SYSF(tmp,SktF_1);

   if((r=tmp.reset(static_cast<const WyByteFlow&>(TermF_0)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if((r=tmp.reset(static_cast<const WyByteFlow&>(TermF_1)))!=Wym_EBADF) {
     WY_THROW(r);
   }
 }
};

void Chk_Poly::ck_fifofile(void)
{
 WyRet r;

 {
   {
     try {
      WyFifoFile tmp(static_cast<const WyByteFlow&>(RegF_0));
      WY_THROW( WyRet() );
     }
     catch(const WyFifoFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
  
   {
     try {
      WyFifoFile tmp(static_cast<const WyByteFlow&>(RegF_1));
      WY_THROW( WyRet() );
     }
     catch(const WyFifoFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
 }

 {
   {
     try {
      WyFifoFile tmp(static_cast<const WyByteFlow&>(ChrF_0));
      WY_THROW( WyRet() );
     }
     catch(const WyFifoFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
  
   {
     try {
      WyFifoFile tmp(static_cast<const WyByteFlow&>(ChrF_1));
      WY_THROW( WyRet() );
     }
     catch(const WyFifoFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
 }

 {
   {
      WyFifoFile tmp(Fifo_0);
      CHK_DEFAULT(tmp);
   }

   {
      WyFifoFile tmp(Fifo_1);
      CHK_SAME_SYSF(tmp,Fifo_1);
   }
 }

 {
   {
     try {
      WyFifoFile tmp(static_cast<const WyByteFlow&>(SktF_0));
      WY_THROW( WyRet() );
     }
     catch(const WyFifoFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }

   {
     try {
      WyFifoFile tmp(static_cast<const WyByteFlow&>(SktF_1));
      WY_THROW( WyRet() );
     }
     catch(const WyFifoFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
 }

 {
   {
     try {
      WyFifoFile tmp(static_cast<const WyByteFlow&>(TermF_0));
      WY_THROW( WyRet() );
     }
     catch(const WyFifoFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }

   {
     try {
      WyFifoFile tmp(static_cast<const WyByteFlow&>(TermF_1));
      WY_THROW( WyRet() );
     }
     catch(const WyFifoFile::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
 }

 {
   WyFifoFile tmp;
   if((r=tmp.reset(static_cast<const WyByteFlow&>(RegF_0)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if((r=tmp.reset(static_cast<const WyByteFlow&>(RegF_1)))!=Wym_EBADF) {
     WY_THROW(r);
   }

   if((r=tmp.reset(static_cast<const WyByteFlow&>(ChrF_0)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if((r=tmp.reset(static_cast<const WyByteFlow&>(ChrF_1)))!=Wym_EBADF) {
     WY_THROW(r);
   }

   if((r=tmp.reset(Fifo_0))!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(Fifo_1))!=Ok) {
     WY_THROW(r);
   }
   CHK_SAME_SYSF(tmp,Fifo_1);
   
   if((r=tmp.reset(static_cast<const WyByteFlow&>(SktF_0)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if((r=tmp.reset(static_cast<const WyByteFlow&>(SktF_1)))!=Wym_EBADF) {
     WY_THROW(r);
   }

   if((r=tmp.reset(static_cast<const WyByteFlow&>(TermF_0)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if((r=tmp.reset(static_cast<const WyByteFlow&>(TermF_1)))!=Wym_EBADF) {
     WY_THROW(r);
   }
 }
};

void Chk_Poly::ck_terminal(void)
{
 WyRet r;

 {
   {
     try {
      WyTerminal tmp(static_cast<const WyByteFlow&>(RegF_0));
      WY_THROW( WyRet() );
     }
     catch(const WyTerminal::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
  
   {
     try {
      WyTerminal tmp(static_cast<const WyByteFlow&>(RegF_1));
      WY_THROW( WyRet() );
     }
     catch(const WyTerminal::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
 }

 {
   {
      WyTerminal tmp(ChrF_0);
      CHK_DEFAULT(tmp);
   }
  
   {
     try {
      WyTerminal tmp(ChrF_1);
      WY_THROW( WyRet() );
     }
     catch(const WyTerminal::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
 }

 {
   {
     try {
      WyTerminal tmp(static_cast<const WyByteFlow&>(Fifo_0));
      WY_THROW( WyRet() );
     }
     catch(const WyTerminal::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }

   {
     try {
      WyTerminal tmp(static_cast<const WyByteFlow&>(Fifo_1));
      WY_THROW( WyRet() );
     }
     catch(const WyTerminal::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
 }

 {
   {
     try {
      WyTerminal tmp(static_cast<const WyByteFlow&>(SktF_0));
      WY_THROW( WyRet() );
     }
     catch(const WyTerminal::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }

   {
     try {
      WyTerminal tmp(static_cast<const WyByteFlow&>(SktF_1));
      WY_THROW( WyRet() );
     }
     catch(const WyTerminal::Reply& e) {
       if(e!=Wym_EBADF) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
   }
 }

 {
   {
      WyTerminal tmp(TermF_0);
      CHK_DEFAULT(tmp);
   }

   {
      WyTerminal tmp(TermF_1);
      CHK_SAME_SYSF(tmp,TermF_1);
   }
 }

 {
   WyTerminal tmp;
   if((r=tmp.reset(static_cast<const WyByteFlow&>(RegF_0)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if((r=tmp.reset(static_cast<const WyByteFlow&>(RegF_1)))!=Wym_EBADF) {
     WY_THROW(r);
   }

   if((r=tmp.reset(ChrF_0))!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(ChrF_1))!=Wym_EBADF) {
     WY_THROW(r);
   }

   if((r=tmp.reset(static_cast<const WyByteFlow&>(Fifo_0)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if((r=tmp.reset(static_cast<const WyByteFlow&>(Fifo_1)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   
   if((r=tmp.reset(static_cast<const WyByteFlow&>(SktF_0)))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if((r=tmp.reset(static_cast<const WyByteFlow&>(SktF_1)))!=Wym_EBADF) {
     WY_THROW(r);
   }

   if((r=tmp.reset(TermF_0))!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(TermF_1))!=Ok) {
     WY_THROW(r);
   }
   CHK_SAME_SYSF(tmp,TermF_1);
 }
};

// some WyByteFlow members are checked here
//
void Chk_Poly::ck_byteflow(void)
{
 WyRet r;

 {
   {
      WyByteFlow tmp(RegF_0);
      CHK_DEFAULT(tmp);
   }
  
   {
      WyByteFlow tmp(RegF_1);
      CHK_SAME_SYSF(tmp,RegF_1);
   }
 }

 {
   {
      WyByteFlow tmp(ChrF_0);
      CHK_DEFAULT(tmp);
   }
  
   {
      WyByteFlow tmp(ChrF_1);
      CHK_SAME_SYSF(tmp,ChrF_1);
   }
 }

 {
   {
      WyByteFlow tmp(Fifo_0);
      CHK_DEFAULT(tmp);
   }
  
   {
      WyByteFlow tmp(Fifo_1);
      CHK_SAME_SYSF(tmp,Fifo_1);
   }
 }

 {
   {
      WyByteFlow tmp(SktF_0);
      CHK_DEFAULT(tmp);
   }

   {
      WyByteFlow tmp(SktF_1);
      CHK_SAME_SYSF(tmp,SktF_1);
   }
 }

 {
   {
      WyByteFlow tmp(TermF_0);
      CHK_DEFAULT(tmp);
   }

   {
      WyByteFlow tmp(TermF_1);
      CHK_SAME_SYSF(tmp,TermF_1);
   }
 }

 {
   WyByteFlow tmp;
   if((r=tmp.reset(RegF_0))!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(RegF_1))!=Ok) {
     WY_THROW(r);
   }
   CHK_SAME_SYSF(tmp,RegF_1);

   if((r=tmp.reset(ChrF_0))!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(ChrF_1))!=Ok) {
     WY_THROW(r);
   }
   CHK_SAME_SYSF(tmp,ChrF_1);

   if((r=tmp.reset(Fifo_0))!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(Fifo_1))!=Ok) {
     WY_THROW(r);
   }
   CHK_SAME_SYSF(tmp,Fifo_1);
   
   if((r=tmp.reset(SktF_0))!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(SktF_1))!=Ok) {
     WY_THROW(r);
   }
   CHK_SAME_SYSF(tmp,SktF_1);

   if((r=tmp.reset(TermF_0))!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(TermF_1))!=Ok) {
     WY_THROW(r);
   }
   CHK_SAME_SYSF(tmp,TermF_1);
 }
};

static void ck_poly_1(void)
{
 const char FifoPath[]="tmp.chk_poly.fifo.tmp";
 const bool IsRoot(::getuid()==0);
 WyRet r;

 Wy::unlink(FifoPath);
 if((r=WyFifoFile::create(FifoPath,S_IRUSR|S_IWUSR))!=Ok) {
   WY_THROW(r);
 }

 try {
  const WyRegFile C_RegFile_0;
  const WyChrFile C_ChrFile_0;
  const WyTerminal C_Terminal_0;
  const WyFifoFile C_FifoFile_0;
  const WySockFile C_SockFile_0;

  const WyRegFile C_RegFile(TestFile_1A,O_RDONLY);
  
  WyChrFile C_ChrFile;
  if(IsRoot) {
    if((r=C_ChrFile.reset("/dev/mouse",O_RDONLY))!=Ok) {
      WY_THROW(r);
    }
  } else {
    if((r=C_ChrFile.reset("/dev/tty",O_RDONLY))!=Ok) {
      WY_THROW(r);
    }
  }
  const WyTerminal C_Terminal("/dev/tty",O_RDONLY);
  const WyFifoFile C_FifoFile(FifoPath,O_RDWR);
  const WySockFile C_SockFile(PF_LOCAL,SOCK_STREAM,0);

  // chk WyChrFile(const WyByteFlow&)
  {
   WyChrFile tmp(Wy::cerr);  // cerr assumed a chrfile
   CHK_SAME_SYSF(tmp,Wy::cerr);

   WyChrFile tmp2(static_cast<const WyByteFlow&>(C_Terminal));
   CHK_SAME_SYSF(tmp2,C_Terminal);
  }

  try {
    WyChrFile tmp(static_cast<const WyByteFlow&>(C_RegFile));
    WY_THROW( WyRet() );
  } catch(const WyChrFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };
  try {
    WyChrFile tmp(static_cast<const WyByteFlow&>(C_RegFile_0));
    WY_THROW( WyRet() );
  } catch(const WyChrFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };

  try {
    WyChrFile tmp(static_cast<const WyByteFlow&>(C_FifoFile));
    WY_THROW( WyRet() );
  } catch(const WyChrFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };
  try {
    WyChrFile tmp(static_cast<const WyByteFlow&>(C_FifoFile_0));
    WY_THROW( WyRet() );
  } catch(const WyChrFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };

  try {
    WyChrFile tmp(static_cast<const WyByteFlow&>(C_SockFile));
    WY_THROW( WyRet() );
  } catch(const WyChrFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };
  try { 
    WyChrFile tmp(static_cast<const WyByteFlow&>(C_SockFile_0));
    WY_THROW( WyRet() );
  } catch(const WyChrFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };

  // chk WyRegFile(const WyByteFlow&)
  {
   WyRegFile tmp(C_RegFile);
   CHK_SAME_SYSF(tmp,C_RegFile);

   WyRegFile tmp2(static_cast<const WyByteFlow&>(C_RegFile));
   CHK_SAME_SYSF(tmp2,C_RegFile);
  }

  try {
    WyRegFile tmp(static_cast<const WyByteFlow&>(C_ChrFile));
    WY_THROW( WyRet() );
  } catch(const WyRegFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };
  try {
    WyRegFile tmp(static_cast<const WyByteFlow&>(C_ChrFile_0));
    WY_THROW( WyRet() );
  } catch(const WyRegFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };

  try {
    WyRegFile tmp(static_cast<const WyByteFlow&>(C_FifoFile));
    WY_THROW( WyRet() );
  } catch(const WyRegFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };
  try {
    WyRegFile tmp(static_cast<const WyByteFlow&>(C_FifoFile_0));
    WY_THROW( WyRet() );
  } catch(const WyRegFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };

  try {
    WyRegFile tmp(static_cast<const WyByteFlow&>(C_SockFile));
    WY_THROW( WyRet() );
  } catch(const WyRegFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };
  try {
    WyRegFile tmp(static_cast<const WyByteFlow&>(C_SockFile_0));
    WY_THROW( WyRet() );
  } catch(const WyRegFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };

  // chk WySockFile(const WyByteFlow&)
  {
   WySockFile tmp(C_SockFile);
   CHK_SAME_SYSF(tmp,C_SockFile);

   WySockFile tmp2(static_cast<const WyByteFlow&>(C_SockFile_0));
   CHK_SAME_SYSF(tmp2,C_SockFile_0);
  }

  try {
    WySockFile tmp(static_cast<const WyByteFlow&>(C_RegFile));
    WY_THROW( WyRet() );
  } catch(const WySockFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };
  try {
    WySockFile tmp(static_cast<const WyByteFlow&>(C_RegFile_0));
    WY_THROW( WyRet() );
  } catch(const WySockFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };

  try {
    WySockFile tmp(static_cast<const WyByteFlow&>(C_FifoFile));
    WY_THROW( WyRet() );
  } catch(const WySockFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };
  try {
    WySockFile tmp(static_cast<const WyByteFlow&>(C_FifoFile_0));
    WY_THROW( WyRet() );
  } catch(const WySockFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };

  try {
    WySockFile tmp(static_cast<const WyByteFlow&>(C_ChrFile));
    WY_THROW( WyRet() );
  } catch(const WySockFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };
  try { 
    WySockFile tmp(static_cast<const WyByteFlow&>(C_ChrFile_0));
    WY_THROW( WyRet() );
  } catch(const WySockFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };

  // chk WyFifoFile(const WyByteFlow&)
  {
   WyFifoFile tmp(C_FifoFile);
   CHK_SAME_SYSF(tmp,C_FifoFile);

   WyFifoFile tmp2(static_cast<const WyByteFlow&>(C_FifoFile));
   CHK_SAME_SYSF(tmp2,C_FifoFile);
  }

  try {
    WyFifoFile tmp(static_cast<const WyByteFlow&>(C_RegFile));
    WY_THROW( WyRet() );
  } catch(const WyFifoFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };
  try {
    WyFifoFile tmp(static_cast<const WyByteFlow&>(C_RegFile_0));
    WY_THROW( WyRet() );
  } catch(const WyFifoFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };

  try {
    WyFifoFile tmp(static_cast<const WyByteFlow&>(C_ChrFile));
    WY_THROW( WyRet() );
  } catch(const WyFifoFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };
  try {
    WyFifoFile tmp(static_cast<const WyByteFlow&>(C_ChrFile_0));
    WY_THROW( WyRet() );
  } catch(const WyFifoFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };

  try {
    WyFifoFile tmp(static_cast<const WyByteFlow&>(C_SockFile));
    WY_THROW( WyRet() );
  } catch(const WyFifoFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };
  try { 
    WyFifoFile tmp(static_cast<const WyByteFlow&>(C_SockFile_0));
    WY_THROW( WyRet() );
  } catch(const WyFifoFile::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };

  // chk WyTerminal(const WyByteFlow&)
  {
   WyTerminal tmp(C_Terminal);
   CHK_SAME_SYSF(tmp,C_Terminal);

   WyTerminal tmp2(static_cast<const WyByteFlow&>(C_Terminal));
   CHK_SAME_SYSF(tmp2,C_Terminal);
  }

  try {
    WyTerminal tmp(static_cast<const WyByteFlow&>(C_RegFile));
    WY_THROW( WyRet() );
  } catch(const WyTerminal::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };
  try {
    WyTerminal tmp(static_cast<const WyByteFlow&>(C_RegFile_0));
    WY_THROW( WyRet() );
  } catch(const WyTerminal::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };

  if(IsRoot) {
    try {
      WyTerminal tmp(C_ChrFile);
      WY_THROW( WyRet() );
    } catch(const WyTerminal::Reply& e) {
      if(e!=Wym_EBADF) {
        throw(e);
      }
    };
    try {
      WyTerminal tmp(static_cast<const WyByteFlow&>(C_ChrFile_0));
      WY_THROW( WyRet() );
    } catch(const WyTerminal::Reply& e) {
      if(e!=Wym_EBADF) {
        throw(e);
      }
    };
  }

  try {
    WyTerminal tmp(static_cast<const WyByteFlow&>(C_FifoFile));
    WY_THROW( WyRet() );
  } catch(const WyTerminal::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };
  try {
    WyTerminal tmp(static_cast<const WyByteFlow&>(C_FifoFile_0));
    WY_THROW( WyRet() );
  } catch(const WyTerminal::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };

  try {
    WyTerminal tmp(static_cast<const WyByteFlow&>(C_SockFile));
    WY_THROW( WyRet() );
  } catch(const WyTerminal::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };
  try { 
    WyTerminal tmp(static_cast<const WyByteFlow&>(C_SockFile_0));
    WY_THROW( WyRet() );
  } catch(const WyTerminal::Reply& e) {
    if(e!=Wym_EBADF) {
      throw(e);
    }
  };

 }
 catch(...) {
   Wy::unlink(FifoPath);
   throw;
 };
 Wy::unlink(FifoPath);
};

static void ck_poly_2(void)
{
  pchk_obj.ck_chrfile();
  pchk_obj.ck_regfile();
  pchk_obj.ck_sockfile();
  pchk_obj.ck_fifofile();
  pchk_obj.ck_terminal();
  pchk_obj.ck_byteflow();
};

static const WyStr chdr(
                  "+---------------+\n"
                  "| main() caught:|\n"
                  "+---------------+\n");
int main(void) throw()
try {
 std::cout << "Checking polymorphism of system file classes ... ";
 ck_poly_1();
 ck_poly_2();
 std::cout << "Checked Ok\n";
 return(0);
}
catch(const WyRet& e) {
 std::cerr << chdr.c_str() << Wy::wrd(e).c_str() << std::endl;
 return(-1);
}
catch(const std::exception& e) {
 std::cerr << chdr.c_str() << "std::exception" << std::endl;
 return(-1);
}
catch(...) {
 std::cerr << chdr.c_str() << "unknown unwind" << std::endl;
 return(-1);
};
