/* Copyright is licensed under GNU LGPL.                 by IJ. Wang 2007

  Build: make chk_ptr
*/
#include "WyStr.h"
#include "WyPtr.hh"
#include <exception>
#include <iostream>

template <typename T>
static void ck_WyPtr(const T& NObj)
{
 const WyPtr<T> ObjRef(NObj);
 if(NObj==NULL) {
    WY_THROW( WyRet() );
 }
 {
  WyPtr<T> tmp(NObj);
  if(static_cast<T>(tmp)!=NObj) {
    WY_THROW( WyRet() );
  }
 }
 if(NObj==ObjRef) {} else {
   WY_THROW( WyRet() );
 }
 if(NObj!=ObjRef) {
   WY_THROW( WyRet() );
 }

 // type conversion
 {
  WyPtr<T> p;
  T t(p);
  if(t!=p) {
   WY_THROW( WyRet() );
  }
  t=p;
  if(t!=p) {
   WY_THROW( WyRet() );
  }
 }

 // WyPtr()
 {
  WyPtr<T> tmp;
  const WyPtr<T> tmp1(tmp);
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(static_cast<T>(tmp)!=NULL) {
    WY_THROW( WyRet() );
  }
  if(tmp==tmp1) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp!=tmp1) {
    WY_THROW( WyRet() );
  }
  tmp.reset();
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  tmp=tmp1;
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if( static_cast<T>(tmp)!=NULL) {
    WY_THROW( WyRet() );
  }
 }

 // WyPtr(const WyPtr&)
 {
  WyPtr<T> tmp(ObjRef);
  if(tmp.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp==ObjRef) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp!=ObjRef) {
    WY_THROW( WyRet() );
  }
  tmp.reset();
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
 }

 // WyPtr(T)
 {
  WyPtr<T> tmp(NObj);
  if(tmp.is_default()) {
    WY_THROW( WyRet() );
  }
  if(tmp==NObj) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp!=NObj) {
    WY_THROW( WyRet() );
  }
  tmp.reset();
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
 }

 // WyPtr(WyPtr&, Wy::ByMove_t)
 {
  WyPtr<T> arr[2];
  arr[1]=NObj;
  if(arr[0].is_default()==false) {
    WY_THROW( WyRet() );
  }
  if(arr[1].is_default()) {
    WY_THROW( WyRet() );
  }
  new(&arr[0]) WyPtr<T>(arr[1],Wy::ByMove);
  if(arr[1]!=NObj) {
    WY_THROW( WyRet() );
  }
 }

 // reset(const T&)/ reset(const WyPtr&)
 {
  WyPtr<T> tmp;
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  tmp.reset(NObj);
  if(tmp==NObj) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp!=NObj) {
    WY_THROW( WyRet() );
  }
  tmp.reset();
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  tmp.reset(ObjRef);
  if(tmp!=ObjRef) {
    WY_THROW( WyRet() );
  }
 }

 // operator=(const T&)/ operator=(const WyPtr&)
 {
  WyPtr<T> tmp;
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  tmp=NObj;
  if(tmp==NObj) {} else {
    WY_THROW( WyRet() );
  }
  if(tmp!=NObj) {
    WY_THROW( WyRet() );
  }
  tmp.reset();
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  tmp=ObjRef;
  if(tmp!=ObjRef) {
    WY_THROW( WyRet() );
  }
 }
};

static void tf0(void) {};
static int itf0(void) { return(-2); };
static void tf1(int) {};
static int itf1(int x) { return(x); };

struct S {
 mutable int m_cnt;
 void f0() { ++m_cnt; };
 void f0c() const { ++m_cnt; };
 int if0() { return ++m_cnt; };
 int if0c() const { return ++m_cnt; };
 void f1(int v) { m_cnt=v; };
 void f1c(int v) const { m_cnt=v; };
 int if1(int v) { return m_cnt=v; };
 int if1c(int v) const { return m_cnt=v; };
 void f2(int v1, int v2) { m_cnt=v1+v2; };
 void f2c(int v1, int v2) const { m_cnt=v1+v2; };
 int if2(int v1, int v2) { return m_cnt=v1+v2; };
 int if2c(int v1, int v2) const { return m_cnt=v1+v2; };
};

int main(void)
try {
 std::cout << "Checking WyPtr.hh ...\n";

 int arr[1];

 ck_WyPtr<void*>((void*)"");
 ck_WyPtr<const void*>("");
 ck_WyPtr<char*>((char*)"");
 ck_WyPtr<const char*>("");
 ck_WyPtr<int(*)[1]>(&arr);
 ck_WyPtr<void(*)()>(&tf0);
 ck_WyPtr<int(*)()>(&itf0);
 ck_WyPtr<void(*)(int)>(&tf1);
 ck_WyPtr<int(*)(int)>(&itf1);
 ck_WyPtr<void(S::*)()>(&S::f0);
 ck_WyPtr<int(S::*)(int)>(&S::if1);

 std::cout << "Checked Ok\n";
 return(0);
}
catch(const WyRet& e) {
 std::cerr << Wy::wrd(e).c_str() << "\n";
 return(e->repcode());
}
catch(const std::exception& e) {
 std::cerr << "std::exception\n";
 return(-1);
}
catch(...) {
 std::cerr << "unknown unwind\n";
 return(-1);
};
