/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   Build: make chk_reply
*/

#include "WyRet.h"
#include <stdexcept>
#include <iostream>

#if WYR_REPLY_VERSION!=33
#error Check code is for WYR_REPLY_VERSION 33
#endif

static void t_reply(void)
{
};

static void chk_default(const WyReply& em)
{
 if(em.is_default()==false) {
   throw WyRet();
 }
 {
  WyReply tmp;
  if(em.operator->()!=tmp.operator->()) {
   throw WyRet();
  }
 }
 if(em->repcode()!=-1) {
   throw WyRet();
 }
 static const char* DeiMsg="General error";
 static const size_t DeiMsgLen=std::strlen(DeiMsg);
 if(strcmp(em->repstr(),DeiMsg)!=0) {
   throw WyRet();
 }

 char buf[16];
 const char* mtid=NULL;
 if(em->getmsg(mtid,buf,sizeof(buf))!=DeiMsgLen) {
   throw WyRet();
 }
 if((mtid==NULL)||(mtid[0]!=0)) {
   throw WyRet();
 }
 if(memcmp(buf,DeiMsg,DeiMsgLen)!=0) {
   throw WyRet();
 }
};

#define CHK_DEFAULT(em) try { chk_default(em); } catch(const WyRet& e) {\
                            WY_THROW( WyRet(e) ); };

// Check the public interface of WyReply
//
static void t1(void)
{
 WyReply e1;            // default constructor
 WyReply e2(ENOENT);    // construct by errno
 WyReply e3(e2);        // constructor by copy
 WyReply e4(0);         // construct by errno (Ok object)
 char buf[128];

 if(WyReply().operator->()!=e1.operator->()) {
   WY_THROW( WyRet() );
 }
 // Basic construct and compare
 //
 if(e1.is_default()==false) {
   WY_THROW(WyRet());
 }
 if(e2.is_default()==true) {
   WY_THROW( WyRet() );
 }
 if(e3.is_default()==true) {
   WY_THROW( WyRet() );
 }
 if(e1==e2) {
   WY_THROW( ::WyRet() );
 }
 if(e1!=e2) {
 } else {
   WY_THROW( WyRet() );
 }
 if(e2!=e3) {
   WY_THROW( WyRet() );
 }
 if(e2==e3) {
 } else {
   WY_THROW( WyRet() );
 }
 CHK_DEFAULT(e1);

 // chk Ok==WyReply(0)
 //
 if(WyReply(0)!=Ok) {
   WY_THROW( WyRet() );
 }

 if(e1==Ok) {
   WY_THROW( WyRet() );
 }
 if(e1!=Ok) {
 } else {
   WY_THROW( WyRet() );
 }
 if(e2==Ok) {
   WY_THROW( WyRet() );
 }
 if(e2!=Ok) {
 } else {
   WY_THROW( WyRet() );
 }
 if(e4==Ok) {
 } else {
   WY_THROW( WyRet() );
 }
 if(e4!=Ok) {
   WY_THROW( WyRet() );
 }
 if(e4.is_default()) {
   WY_THROW( WyRet() );
 }

 // chk WyReply(en)
 {
  WyReply tmp(Wy::_max_errno+1);
  if(tmp.is_default()==false) {
    WY_THROW( WyRet() );
  }
  CHK_DEFAULT(tmp);

  WyReply tmp2(ENOENT);
  if(tmp2->repcode()!=ENOENT) {
    WY_THROW( WyRet() );
  };
 }

 // chk WyReply(Wy_RepInfo&) (manual example)
 {
  class AppErrInfo : public Wy_RepInfo {
    public:
      AppErrInfo(int ec, const char* estr) throw()
         : Wy_RepInfo(ec, estr) {};
      void setmsg(const char* mtid,const void*,size_t) throw()
         {};
  };

  static AppErrInfo e1(45,"Bad Input");

  const WyReply BadInput(e1);
  const char* mtid=NULL;

  if(BadInput->repcode()!=45) {
    WY_THROW( WyRet() );
  }
  if(BadInput->getmsg(mtid,buf,sizeof(buf))!=9) {
    WY_THROW( WyRet() );
  }
  if((mtid==NULL)||(mtid[0]!=0)) {
    WY_THROW( WyRet() );
  }
  if(::memcmp(buf,e1.repstr(),9)!=0) {
    WY_THROW( WyRet() );
  }
 }

 // chk reset
 //
 e4.reset();
 CHK_DEFAULT(e4);

 if(e2.is_default()) {
   WY_THROW( WyRet() );
 }
 e4.reset(e2);
 if(e4!=e2) {
   WY_THROW( WyRet() );
 }
 e4.reset(Wym_EBADF);
 if(e4!=Wym_EBADF) {
   WY_THROW( WyRet() );
 }
 if(e4.reset(4)!=4) {
   WY_THROW( WyRet() );
 }
 if(e4->repcode()!=4) {
   WY_THROW( WyRet() );
 }
 if(e4.reset(Wy::_max_errno+1)!=-1) {
   WY_THROW( WyRet() );
 }
 CHK_DEFAULT(e4);
 e4.reset(Wym_EBADF);
 if(e4.reset(-2)!=-1) {
   WY_THROW( WyRet() );
 }
 CHK_DEFAULT(e4);

 // chk operator=
 //
 if(e4==e2) {
   WY_THROW( WyRet() );
 }
 e4=e2;
 if(e4!=e2) {
   WY_THROW( WyRet() );
 }
 e4=Wym_EBADF;
 if(e4!=Wym_EBADF) {
   WY_THROW( WyRet() );
 }

 if(e4.is_default()) {
   WY_THROW( WyRet() );
 }
 e4=200;
 CHK_DEFAULT(e4);

 // chk some repcode()
 //
 if(Wym_EPERM->repcode()!=EPERM) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENOENT->repcode()!=ENOENT) {
   WY_THROW( WyRet() );
 }
 if(Wym_ESRCH->repcode()!=ESRCH) {
   WY_THROW( WyRet() );
 }
 if(Wym_EINTR->repcode()!=EINTR) {
   WY_THROW( WyRet() );
 }
 if(Wym_ENETRESET->repcode()!=ENETRESET) {
   WY_THROW( WyRet() );
 }
 if(Wym_EMEDIUMTYPE->repcode()!=EMEDIUMTYPE) {
   WY_THROW( WyRet() );
 }
 if(Wym_E127->repcode()!=127) {
   WY_THROW( WyRet() );
 }
 if(Wym_E131->repcode()!=131) {
   WY_THROW( WyRet() );
 }

 // chk setmsg(..)/getmsg(..)
 //
 {
   char buf[64];
   const char* mtid=NULL;
   const WyReply& ChkR= Wym_E125;
   const char* ChkR_cstr= ChkR->repstr();
   const size_t ChkR_mlen=std::strlen(ChkR_cstr);

   if(ChkR->getmsg(mtid,buf,sizeof(buf))!=ChkR_mlen) {
     WY_THROW( WyRet() );
   }
   if((mtid==NULL)||(mtid[0]!=0)) {
     WY_THROW( WyRet() );
   }
   if(std::memcmp(buf,ChkR_cstr,ChkR_mlen)!=0) {
     WY_THROW( WyRet() );
   }
   const char Ustr[]="Test Msg for setmsg";
   const char Umid[]="Test Msg ID";
   mtid=NULL;
   ChkR->setmsg(Umid,Ustr,std::strlen(Ustr));
   if(ChkR->getmsg(mtid,buf,sizeof(buf))!=::strlen(Ustr)) {
     WY_THROW( WyRet() );
   }
   if(memcmp(buf,Ustr,::strlen(Ustr))!=0) {
     WY_THROW( WyRet() );
   }
   if(mtid!=Umid) {
     WY_THROW( WyRet() );
   }

   // Recover ChkR
   ChkR->setmsg("",ChkR_cstr,ChkR_mlen);
   if(ChkR->getmsg(mtid,buf,sizeof(buf))!=ChkR_mlen) {
     WY_THROW( WyRet() );
   }
   if(memcmp(buf,ChkR_cstr,ChkR_mlen)!=0) {
     WY_THROW( WyRet() );
   }
   if((mtid==NULL)||(mtid[0]!=0)) {
     WY_THROW( WyRet() );
   }

   if(Ok->repcode()!=0) {
     WY_THROW( WyRet() );
   }
 }

 // chk move ctor
 {
  WyReply raw_buf[2];      // ok WyReply is discardable
  ::memset(raw_buf,0,sizeof(raw_buf));
  WyReply* const p1( reinterpret_cast<WyReply*>(&raw_buf[0]) );
  WyReply* const p2( reinterpret_cast<WyReply*>(&raw_buf[1]) );

  new(p1) WyReply(Wym_ENOENT);
  if(*p1!=Wym_ENOENT) {
    p1->~WyReply();
    WY_THROW( WyRet() );
  }

  // move *p1 to p2
  new(p2) WyReply(*p1,Wy::ByMove);
  if(*p2!=Wym_ENOENT) {
    p2->~WyReply();
    WY_THROW( WyRet() );
  }
  p2->~WyReply();
 }
};

static void ck_errno(const WyReply& reply, int en)
{
 if(reply!=WyReply(en)) {
   WY_THROW( WyRet() );
 }
 if(reply->repcode()!=en) {
   WY_THROW( WyRet() );
 }
};

// Check the mapped errno 0-131
//
static void t2(void)
{
 ck_errno(Ok,0);
 ck_errno(Wym_EPERM,EPERM);
 ck_errno(Wym_ENOENT,ENOENT);
 ck_errno(Wym_ESRCH,ESRCH);
 ck_errno(Wym_EINTR,EINTR);
 ck_errno(Wym_EIO,EIO);
 ck_errno(Wym_ENXIO,ENXIO);
 ck_errno(Wym_E2BIG,E2BIG);
 ck_errno(Wym_ENOEXEC,ENOEXEC);
 ck_errno(Wym_EBADF,EBADF);
 ck_errno(Wym_ECHILD,ECHILD);
 ck_errno(Wym_EAGAIN,EAGAIN);
 ck_errno(Wym_ENOMEM,ENOMEM);
 ck_errno(Wym_EACCES,EACCES);
 ck_errno(Wym_EFAULT,EFAULT);
 ck_errno(Wym_ENOTBLK,ENOTBLK);
 ck_errno(Wym_EBUSY,EBUSY);
 ck_errno(Wym_EEXIST,EEXIST);
 ck_errno(Wym_EXDEV,EXDEV);
 ck_errno(Wym_ENODEV,ENODEV);
 ck_errno(Wym_ENOTDIR,ENOTDIR);
 ck_errno(Wym_EISDIR,EISDIR);
 ck_errno(Wym_EINVAL,EINVAL);
 ck_errno(Wym_ENFILE,ENFILE);
 ck_errno(Wym_EMFILE,EMFILE);
 ck_errno(Wym_ENOTTY,ENOTTY);
 ck_errno(Wym_ETXTBSY,ETXTBSY);
 ck_errno(Wym_EFBIG,EFBIG);
 ck_errno(Wym_ENOSPC,ENOSPC);
 ck_errno(Wym_ESPIPE,ESPIPE);
 ck_errno(Wym_EROFS,EROFS);
 ck_errno(Wym_EMLINK,EMLINK);
 ck_errno(Wym_EPIPE,EPIPE);
 ck_errno(Wym_EDOM,EDOM);
 ck_errno(Wym_ERANGE,ERANGE);
 ck_errno(Wym_EDEADLK,EDEADLK);
 ck_errno(Wym_ENAMETOOLONG,ENAMETOOLONG);
 ck_errno(Wym_ENOLCK,ENOLCK);
 ck_errno(Wym_ENOSYS,ENOSYS);
 ck_errno(Wym_ENOTEMPTY,ENOTEMPTY);
 ck_errno(Wym_ELOOP,ELOOP);
 ck_errno(Wym_EWOULDBLOCK,EWOULDBLOCK);
 ck_errno(Wym_E41,41);
 ck_errno(Wym_ENOMSG,ENOMSG);
 ck_errno(Wym_EIDRM,EIDRM);
 ck_errno(Wym_ECHRNG,ECHRNG);
 ck_errno(Wym_EL2NSYNC,EL2NSYNC);
 ck_errno(Wym_EL3HLT,EL3HLT);
 ck_errno(Wym_EL3RST,EL3RST);
 ck_errno(Wym_ELNRNG,ELNRNG);
 ck_errno(Wym_EUNATCH,EUNATCH);
 ck_errno(Wym_ENOCSI,ENOCSI);
 ck_errno(Wym_EL2HLT,EL2HLT);
 ck_errno(Wym_EBADE,EBADE);
 ck_errno(Wym_EBADR,EBADR);
 ck_errno(Wym_EXFULL,EXFULL);
 ck_errno(Wym_ENOANO,ENOANO);
 ck_errno(Wym_EBADRQC,EBADRQC);
 ck_errno(Wym_EBADSLT,EBADSLT);
 ck_errno(Wym_EDEADLOCK,EDEADLOCK);
 ck_errno(Wym_E58,58);
 ck_errno(Wym_EBFONT,EBFONT);
 ck_errno(Wym_ENOSTR,ENOSTR);
 ck_errno(Wym_ENODATA,ENODATA);
 ck_errno(Wym_ETIME,ETIME);
 ck_errno(Wym_ENOSR,ENOSR);
 ck_errno(Wym_ENONET,ENONET);
 ck_errno(Wym_ENOPKG,ENOPKG);
 ck_errno(Wym_EREMOTE,EREMOTE);
 ck_errno(Wym_ENOLINK,ENOLINK);
 ck_errno(Wym_EADV,EADV);
 ck_errno(Wym_ESRMNT,ESRMNT);
 ck_errno(Wym_ECOMM,ECOMM);
 ck_errno(Wym_EPROTO,EPROTO);
 ck_errno(Wym_EMULTIHOP,EMULTIHOP);
 ck_errno(Wym_EDOTDOT,EDOTDOT);
 ck_errno(Wym_EBADMSG,EBADMSG);
 ck_errno(Wym_EOVERFLOW,EOVERFLOW);
 ck_errno(Wym_ENOTUNIQ,ENOTUNIQ);
 ck_errno(Wym_EBADFD,EBADFD);
 ck_errno(Wym_EREMCHG,EREMCHG);
 ck_errno(Wym_ELIBACC,ELIBACC);
 ck_errno(Wym_ELIBBAD,ELIBBAD);
 ck_errno(Wym_ELIBSCN,ELIBSCN);
 ck_errno(Wym_ELIBMAX,ELIBMAX);
 ck_errno(Wym_ELIBEXEC,ELIBEXEC);
 ck_errno(Wym_EILSEQ,EILSEQ);
 ck_errno(Wym_ERESTART,ERESTART);
 ck_errno(Wym_ESTRPIPE,ESTRPIPE);
 ck_errno(Wym_EUSERS,EUSERS);
 ck_errno(Wym_ENOTSOCK,ENOTSOCK);
 ck_errno(Wym_EDESTADDRREQ,EDESTADDRREQ);
 ck_errno(Wym_EMSGSIZE,EMSGSIZE);
 ck_errno(Wym_EPROTOTYPE,EPROTOTYPE);
 ck_errno(Wym_ENOPROTOOPT,ENOPROTOOPT);
 ck_errno(Wym_EPROTONOSUPPORT,EPROTONOSUPPORT);
 ck_errno(Wym_ESOCKTNOSUPPORT,ESOCKTNOSUPPORT);
 ck_errno(Wym_EOPNOTSUPP,EOPNOTSUPP);
 ck_errno(Wym_EPFNOSUPPORT,EPFNOSUPPORT);
 ck_errno(Wym_EAFNOSUPPORT,EAFNOSUPPORT);
 ck_errno(Wym_EADDRINUSE,EADDRINUSE);
 ck_errno(Wym_EADDRNOTAVAIL,EADDRNOTAVAIL);
 ck_errno(Wym_ENETDOWN,ENETDOWN);
 ck_errno(Wym_ENETUNREACH,ENETUNREACH);
 ck_errno(Wym_ENETRESET,ENETRESET);
 ck_errno(Wym_ECONNABORTED,ECONNABORTED);
 ck_errno(Wym_ECONNRESET,ECONNRESET);
 ck_errno(Wym_ENOBUFS,ENOBUFS);
 ck_errno(Wym_EISCONN,EISCONN);
 ck_errno(Wym_ENOTCONN,ENOTCONN);
 ck_errno(Wym_ESHUTDOWN,ESHUTDOWN);
 ck_errno(Wym_ETOOMANYREFS,ETOOMANYREFS);
 ck_errno(Wym_ETIMEDOUT,ETIMEDOUT);
 ck_errno(Wym_ECONNREFUSED,ECONNREFUSED);
 ck_errno(Wym_EHOSTDOWN,EHOSTDOWN);
 ck_errno(Wym_EHOSTUNREACH,EHOSTUNREACH);
 ck_errno(Wym_EALREADY,EALREADY);
 ck_errno(Wym_EINPROGRESS,EINPROGRESS);
 ck_errno(Wym_ESTALE,ESTALE);
 ck_errno(Wym_EUCLEAN,EUCLEAN);
 ck_errno(Wym_ENOTNAM,ENOTNAM);
 ck_errno(Wym_ENAVAIL,ENAVAIL);
 ck_errno(Wym_EISNAM,EISNAM);
 ck_errno(Wym_EREMOTEIO,EREMOTEIO);
 ck_errno(Wym_EDQUOT,EDQUOT);
 ck_errno(Wym_ENOMEDIUM,ENOMEDIUM);
 ck_errno(Wym_EMEDIUMTYPE,EMEDIUMTYPE);
 ck_errno(Wym_E125,125);
 ck_errno(Wym_E126,126);
 ck_errno(Wym_E127,127);
 ck_errno(Wym_E128,128);
 ck_errno(Wym_E129,129);
 ck_errno(Wym_E130,130);
 ck_errno(Wym_E131,131);

 // chk EWOULDBLOCK EAGAIN
 ck_errno(Wym_EWOULDBLOCK,EWOULDBLOCK);
 ck_errno(Wym_EDEADLK,EDEADLK);
 ck_errno(Wym_EWOULDBLOCK,EWOULDBLOCK);
 ck_errno(Wym_EDEADLK,EDEADLK);

 // chkeck repstr()
 //
 //setlocale(LC_ALL,"");
 for(int i=0; i<=Wy::_max_errno; ++i) {
   WyReply ei(i);
   if(std::strcmp(::strerror(i),ei->repstr())!=0) {
     static void * dummy_ptr=std::cout << "Different string from ::strerror(errno):\n";
     if(dummy_ptr) {} ;   // for supressing compiler warning
     std::cout << "\n  strerror(" << i << ")=" << ::strerror(i)
               << "\n  repstr()=" << ei->repstr() << std::endl;
   }
 }
}

static void t_error(void)
{
 WyReply tmp(-3);
 if(tmp.is_default()==false) {
   WY_THROW(WyRet());
 }
 tmp.reset(-3);    // supress unused warning
 if(tmp.is_default()==false) {
   WY_THROW(WyRet());
 }

 WyReply tmp1;
 if(tmp1.is_default()==false) {
   WY_THROW(WyRet());
 }
 tmp1=-4;
 if(tmp1.is_default()==false) {
   WY_THROW(WyRet());
 }
};

static void t_locale_reply(void)
{
#ifdef LC_ALL
 const char* mtid;  // dummy

 // dump locale specific error string
 const char* loc=setlocale(LC_ALL,"");
 if(loc==NULL) {
   WY_THROW( WyRet() );
 }
 std::cout << "\nLocale used reply string:\n";
 
 const char* cstr;
 cstr=::strerror(Ok->repcode()), Ok->setmsg(0,cstr,::strlen(cstr));

 cstr=::strerror(Wym_EPERM->repcode()), Wym_EPERM->setmsg(0,cstr,::strlen(cstr));

 cstr=::strerror(Wym_ENOENT->repcode()), Wym_ENOENT->setmsg(0,cstr,::strlen(cstr));

 cstr=::strerror(Wym_ESRCH->repcode()), Wym_ESRCH->setmsg(0,cstr,::strlen(cstr));

 cstr=::strerror(Wym_EBADMSG->repcode()), Wym_EBADMSG->setmsg(0,cstr,::strlen(cstr));

 cstr=::strerror(Wym_E127->repcode()), Wym_E127->setmsg(0,cstr,::strlen(cstr));

 char buf[128];
 memset(buf,sizeof(buf),0);

 buf[ Ok->getmsg(mtid,buf,sizeof(buf)-1) ]=0;
 std::cout << "          Ok => " << buf << std::endl;

 buf[ Wym_EPERM->getmsg(mtid,buf,sizeof(buf)-1) ]=0;
 std::cout << "   Wym_EPERM => " << buf << std::endl;

 buf[ Wym_ENOENT->getmsg(mtid,buf,sizeof(buf)-1) ]=0;
 std::cout << "  Wym_ENOENT => " << buf << std::endl;

 buf[ Wym_ESRCH->getmsg(mtid,buf,sizeof(buf)-1) ]=0;
 std::cout << "   Wym_ESRCH => " << buf << std::endl;

 buf[ Wym_EBADMSG->getmsg(mtid,buf,sizeof(buf)-1) ]=0;
 std::cout << " Wym_EBADMSG => " << buf << std::endl;

 buf[ Wym_E127->getmsg(mtid,buf,sizeof(buf)-1) ]=0;
 std::cout << "    Wym_E127 => " << buf << std::endl;

 std::cout << "    ........." << std::endl;
 setlocale(LC_ALL,loc);
#endif
};

static int static_chkf(void)
{
 t_reply();
 t1(); 
 t2();
 t_error();
// t_locale_reply();
 return(0);
};
static int static_chkv=static_chkf();

static const std::string chdr(
                  "+---------------+\n"
                  "| main() caught:|\n"
                  "+---------------+\n");
int main(void) throw()
try {
 std::cout << "Checking WyReply ...\n";
 t_reply();
 t1(); 
 t2();
 t_error();
 t_locale_reply();

 std::cout << "Checked Ok\n";
 return(0);
}
catch(const WyRet& e) {
 std::cerr << chdr << "WyRet thrown" << std::endl;
 return(-1);
}
catch(const std::exception& e) {
 std::cerr << chdr << "std::exception" << std::endl;
 return(-1);
}
catch(...) {
 std::cerr << chdr << "unknown unwind" << std::endl;
 return(-1);
}
