/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   Check WySockAddrINet4 public members

   Build: make chk_sockaddrlocal

   Note: Test data files should exist in the working directory
         Files may be created in the working directory and unlinked
*/

#include "WySockAddrINet4.h"
#include "Wy_AtDestroy.hh"
#include <memory>
#include <iostream>

#if WYSOCKADDRINET_VERSION!=33
#error Test code is for WYSOCKADDRINET_VERSION 33
#endif

static bool equ(const WySockAddrINet4::IPAddr& a1, const ::in_addr& a2)
{
  return ::memcmp(&a1.wy_addr_ref(),&a2,sizeof(::in_addr))==0;
};

// Check WySockAddrINet4::IPAddr
//
static void chk_ipaddr(void)
{
 WyRet r;

 // IPAddr(),IPAddr(const IPAddr&),IPAddr(const ::in_addr&),..
 {
  WySockAddrINet4::IPAddr tmp1;
  ::in_addr ina( tmp1.c_addr() );
  if(ina.s_addr!=INADDR_ANY) {
    WY_THROW( WyRet() );
  }
  if(equ(tmp1,ina)==false) {
    WY_THROW( WyRet() );
  }
  if(tmp1.is_default()==false) {
    WY_THROW( WyRet() );
  }

  WySockAddrINet4::IPAddr tmp2(tmp1);
  if(equ(tmp2,ina)==false) {
    WY_THROW( WyRet() );
  }
  if(tmp1!=tmp2) {
    WY_THROW( WyRet() );
  }
  if(tmp1==tmp2) {
  } else {
    WY_THROW( WyRet() );
  }
  if(tmp2.is_default()==false) {
    WY_THROW( WyRet() );
  }

  WySockAddrINet4::IPAddr tmp3(ina);
  if(equ(tmp3,ina)==false) {
    WY_THROW( WyRet() );
  }
  if(tmp3!=tmp1) {
    WY_THROW( WyRet() );
  }
  if(tmp3.is_default()==false) {
    WY_THROW( WyRet() );
  }

  WySockAddrINet4::IPAddr tmp4("0.0.0.0");
  if(equ(tmp4,ina)==false) {
    WY_THROW( WyRet() );
  }
  if(tmp4!=tmp1) {
    WY_THROW( WyRet() );
  }
  if(tmp4.is_default()==false) {
    WY_THROW( WyRet() );
  }

  WySockAddrINet4::IPAddr tmp5( WyStr("0.0.0.0") );
  if(equ(tmp5,ina)==false) {
    WY_THROW( WyRet() );
  }
  if(tmp5!=tmp1) {
    WY_THROW( WyRet() );
  }
  if(tmp5.is_default()==false) {
    WY_THROW( WyRet() );
  }

  WySockAddrINet4::IPAddr tmp6(uint32_t(0));
  if(equ(tmp6,ina)==false) {
    WY_THROW( WyRet() );
  }
  if(tmp6!=tmp1) {
    WY_THROW( WyRet() );
  }
  if(tmp6.is_default()==false) {
    WY_THROW( WyRet() );
  }
 }

 // notation
 {
   WySockAddrINet4::IPAddr tmp;
   WyStr str,str2;
   if((r=tmp.notation(str))!=Ok) {
     WY_THROW(r);
   }
   if(str!="0.0.0.0") {
     WY_THROW( WyRet() );
   }
   if((r=tmp.notation(str2))!=Ok) {
     WY_THROW(r);
   }
   if(str2!=str) {
     WY_THROW( WyRet() );
   }
   if(Wy::wrd(tmp)!=str) {
     WY_THROW( WyRet() );
   }

   WySockAddrINet4::IPAddr tmp2("1.2.3.4");
   if((r=tmp2.notation(str))!=Ok) {
     WY_THROW(r);
   }
   if(str!="1.2.3.4") {
    WY_THROW( WyRet() );
   }
   if((r=tmp2.notation(str2))!=Ok) {
     WY_THROW(r);
   }
   if(str2!=str) {
     WY_THROW( WyRet() );
   }
   if(Wy::wrd(tmp2)!=str) {
     WY_THROW( WyRet() );
   }

   WySockAddrINet4::IPAddr tmp3(uint32_t(0x1ff));
   if((r=tmp3.notation(str))!=Ok) {
     WY_THROW(r);
   }
   if(str!="0.0.1.255") {
    WY_THROW( WyRet() );
   }
   if((r=tmp3.notation(str2))!=Ok) {
     WY_THROW(r);
   }
   if(str2!=str) {
     WY_THROW( WyRet() );
   }
   if(Wy::wrd(tmp3)!=str) {
     WY_THROW( WyRet() );
   }
 }

 // reset()
 {
   const char* AName="2.3.4.5";
   const WySockAddrINet4::IPAddr Tmp1,Tmp2(AName);
   WySockAddrINet4::IPAddr tmp;
   ::in_addr ina;
   tmp.reset();
   if(tmp.is_default()==false) {
     WY_THROW( WyRet() );
   }
   tmp.reset(Tmp1);
   if(tmp.is_default()==false) {
     WY_THROW( WyRet() );
   }
   tmp.reset(Tmp2);
   if(tmp!=Tmp2) {
     WY_THROW( WyRet() );
   }
   tmp.reset();
   if(tmp.is_default()==false) {
     WY_THROW( WyRet() );
   }

   ina=Tmp2.c_addr();
   tmp.reset(ina);
   if(tmp!=ina) {
     WY_THROW( WyRet() );
   }

   tmp.reset(AName);
   if(tmp.is_default()) {
     WY_THROW( WyRet() );
   }
   if(tmp!=Tmp2) {
     WY_THROW( WyRet() );
   }

   tmp.reset( WyStr(AName) );
   if(tmp.is_default()) {
     WY_THROW( WyRet() );
   }
   if(tmp!=Tmp2) {
     WY_THROW( WyRet() );
   }

   tmp.reset(0x02030405);
   if(tmp.is_default()) {
     WY_THROW( WyRet() );
   }
   if(tmp!=Tmp2) {
     WY_THROW( WyRet() );
   }
 }

 {
   const WySockAddrINet4::IPAddr Tmp1,Tmp2("6.6.126.6");
   WySockAddrINet4::IPAddr tmp1(Tmp1),tmp2(Tmp2);
   tmp1.swap(tmp2);
   if(tmp1!=Tmp2) {
     WY_THROW( WyRet() );
   }
   if(tmp2!=Tmp1) {
     WY_THROW( WyRet() );
   }

   tmp2.swap(tmp2);
   if(tmp2!=Tmp1) {
     WY_THROW( WyRet() );
   }

   WyStr str;
   if((r=Tmp2.notation(str))!=Ok) {
     WY_THROW( WyRet() );
   }
   if(str!="6.6.126.6") {
     WY_THROW( WyRet() );
   }
   if(Wy::wrd(Tmp2)!=str) {
     WY_THROW( WyRet() );
   }
 }

 {
   const WySockAddrINet4::IPAddr Tmp1,Tmp2("0.5.6.7");
   WySockAddrINet4::IPAddr tmp;
   tmp=Tmp2;
   if(tmp!=Tmp2) {
     WY_THROW( WyRet() );
   }
   tmp=Tmp1;
   if(tmp!=Tmp1) {
     WY_THROW( WyRet() );
   }
 }

 // operator ==/!=
 {
   const WyStr IpStr("1.2.6.4");
   const WySockAddrINet4::IPAddr Tmp1,Tmp2(IpStr);
   uint32_t InAddrAny=INADDR_ANY;
   if(Tmp2==Tmp2) {
   } else {
     WY_THROW( WyRet() );
   }
   if(Tmp2!=Tmp2) {
     WY_THROW( WyRet() );
   }
   if(InAddrAny==Tmp1) {
   } else {
     WY_THROW( WyRet() );
   }
   if(InAddrAny!=Tmp1) {
     WY_THROW( WyRet() );
   }

   if(IpStr==Tmp2) {
   } else {
     WY_THROW( WyRet() );
   }
   if(IpStr!=Tmp2) {
     WY_THROW( WyRet() );
   }
   if(IpStr.c_str()==Tmp2) {
   } else {
     WY_THROW( WyRet() );
   }
   if(IpStr.c_str()!=Tmp2) {
     WY_THROW( WyRet() );
   }
 }

 // chk error
 {
   try {
     WySockAddrINet4::IPAddr tmp("xxx");
     tmp.reset();          // supress unused warning
     WY_THROW( WyRet() );
   }
   catch(const WySockAddrINet4::IPAddr::Reply& e) {
     if(e!=Wym_EADDRNOTAVAIL) {
       throw;
     }
     // FALLTHROUGH
   };
 };

 // chk error
 {
   try {
     WySockAddrINet4::IPAddr tmp( WyStr("xxx") );
     tmp.reset();          // supress unused warning
     WY_THROW( WyRet() );
   }
   catch(const WySockAddrINet4::IPAddr::Reply& e) {
     if(e!=Wym_EADDRNOTAVAIL) {
       throw;
     }
     // FALLTHROUGH
   };

   {
     WySockAddrINet4::IPAddr tmp;
     if((r=tmp.reset("xxy"))!=Wym_EADDRNOTAVAIL) {
       WY_THROW(r);
     }
     if((r=tmp.reset(WyStr("xxy")))!=Wym_EADDRNOTAVAIL) {
       WY_THROW(r);
     }
   }
 }

 // chk error ==/!= (const char*)
 {
   WySockAddrINet4::IPAddr tmp;
   if(tmp==(const char*)0) {
     WY_THROW( WyRet() );
   }
   if(tmp!=(const char*)0) {
   } else {
     WY_THROW( WyRet() );
   }
 }

 // chk error ==/!= (const WyStr&)
 {
   WySockAddrINet4::IPAddr tmp;
   if(tmp==WyStr()) {
     WY_THROW( WyRet() );
   }
   if(tmp!=WyStr()) {
   } else {
     WY_THROW( WyRet() );
   }
   if(tmp==(const char*)0) {
     WY_THROW( WyRet() );
   }
   if(tmp!=(const char*)0) {
   } else {
     WY_THROW( WyRet() );
   }
 }

 // chk error ==/!=( const WyStr&,WySockAddrINet4::IPAddr)
 {
   WySockAddrINet4::IPAddr tmp;
   if(WyStr()==tmp) {
     WY_THROW( WyRet() );
   }
   if(WyStr()!=tmp) {
   } else {
     WY_THROW( WyRet() );
   }
   if((const char*)0==tmp) {
     WY_THROW( WyRet() );
   }
   if((const char*)0!=tmp) {
   } else {
     WY_THROW( WyRet() );
   }
 }

 // chk _alloc not override
 {
   class T : public WySockAddrINet4 {
   } tmp;
   Wy_SockAddr* p=tmp._alloc(r);
   if(r!=Wym_ENOSYS) {
     if(r==Ok) {
       delete p;
     }
     WY_THROW(r);
   }
 }

 // chk move ctor
 {
  const uint32_t TstIP=0x1a2b3c4d;
  char raw_buf[2*sizeof(WySockAddrINet4::IPAddr)];
  ::memset(raw_buf,0,sizeof(raw_buf));
  WySockAddrINet4::IPAddr* const p1( reinterpret_cast<WySockAddrINet4::IPAddr*>(raw_buf) );
  WySockAddrINet4::IPAddr* const p2(p1+1);

  new(p1) WySockAddrINet4::IPAddr(TstIP);
  if((p1->is_default()==true)||
     (p1->c_addr().s_addr!=htonl(TstIP)) ) {
    p1->WySockAddrINet4::IPAddr::~IPAddr();
    WY_THROW(r);
  }

  // move *p1 to p2
  try {
    new(p2) WySockAddrINet4::IPAddr(*p1,Wy::ByMove);
  }
  catch(...) {
    p1->WySockAddrINet4::IPAddr::~IPAddr();
    throw;
  };
  if((p2->is_default()==true)||
     (p2->c_addr().s_addr!=htonl(TstIP)) ) {
    p2->WySockAddrINet4::IPAddr::~IPAddr();
    WY_THROW(r);
  }
  p2->WySockAddrINet4::IPAddr::~IPAddr();
 }

 // chk equ of WySockAddrINet4::IPAddr::C_AddrType
 {
  WySockAddrINet4::IPAddr::C_AddrType a1={4}, a2={4}, a3={5};
  if(a1==a2) {
  } else {
    WY_THROW( WyRet() );
  }
  if(a1!=a2) {
    WY_THROW( WyRet() );
  } else {
  }
  if(a3==a1) {
    WY_THROW( WyRet() );
  } else {
  }
  if(a3!=a1) {
  } else {
    WY_THROW( WyRet() );
  }
  if(a1==a3) {
    WY_THROW( WyRet() );
  } else {
  }
  if(a1!=a3) {
  } else {
    WY_THROW( WyRet() );
  }
 }
};

// Assert ff is default
//
static void assert_default(const WySockAddrINet4& ff)
{
 static const WySockAddrINet4 Def_Obj;

 if(ff.is_default()==false) {
   WY_THROW( WyRet() );
 }
 if(ff!=Def_Obj) {
   WY_THROW( WyRet() );
 }
 if(ff==Def_Obj) {
 } else {
   WY_THROW( WyRet() );
 }
 if(ff.port()!=0) {
   WY_THROW( WyRet() );
 }
 if(ff.ip()!=INADDR_ANY) {
   WY_THROW( WyRet() );
 }
};

static void assert_not_default(const WySockAddrINet4& ff)
{
 static const WySockAddrINet4 Def_Obj;

 if(ff.is_default()) {
   WY_THROW( WyRet() );
 }
 if(ff==Def_Obj) {
   WY_THROW( WyRet() );
 }
 if(ff!=Def_Obj) {
 } else {
   WY_THROW( WyRet() );
 }
 if((ff.port()==0)&&(ff.ip()!=INADDR_ANY)) {
   WY_THROW( WyRet() );
 }
 
};

static void assert_equ(const WySockAddrINet4& a1,const WySockAddrINet4& a2)
{
 if(a1!=a2) {
   WY_THROW( WyRet() );
 }
 if(a1.port()!=a2.port()) {
   WY_THROW( WyRet() );
 }
 if(a1.ip()!=a2.ip()) {
   WY_THROW( WyRet() );
 }
};

#define CHK_DEFAULT(p)\
    try { assert_default(p); } catch(...) { WY_THROW( WyRet()); }
#define CHK_NOT_DEFAULT(p)\
    try { assert_not_default(p); } catch(...) { WY_THROW( WyRet()); }
#define CHK_EQU(a1,a2)\
    try { assert_equ((a1),(a2)); } catch(...) { WY_THROW( WyRet()); }

// Check basic operations on default object
//
static void t1(void)
{
 WyRet r;

 // constructor
 {
   WySockAddrINet4 a0;
   CHK_DEFAULT(a0);

   WySockAddrINet4 a1(a0);
   CHK_DEFAULT(a0);
   CHK_DEFAULT(a1);
   CHK_EQU(a0,a1);
   
   WySockAddrINet4 a2(WySockAddrINet4::IPAddr(1),2);
   if(a2.is_default()) {
     WY_THROW( WyRet() );
   }
   if(a2.port()!=2) {
     WY_THROW( WyRet() );
   }
   if(a2.ip()!=1) {
     WY_THROW( WyRet() );
   }

   WySockAddrINet4 a3( WySockAddrINet4::IPAddr("1.2.3.4"),9);
   if(a3.is_default()) {
     WY_THROW( WyRet() );
   }
   if(a3.port()!=9) {
     WY_THROW( WyRet() );
   }
   if(a3.ip()!=WySockAddrINet4::IPAddr(0x01020304)) {
     WY_THROW( WyRet() );
   }

   WySockAddrINet4 a4(a3);
   CHK_EQU(a3,a4);

   const WySockAddrINet4::IPAddr ip5;
   const WySockAddrINet4 a5(ip5,0);
   CHK_DEFAULT(a5);
 }

 // chk ops on parameter const ::sockaddr_in&
 {
   struct ::sockaddr_in saddr;
   const uint16_t PortNum=2467;
   const uint32_t IPNum=0x5678abcd;
   saddr.sin_port=htons(PortNum);
   saddr.sin_addr.s_addr=htonl(IPNum);
   WySockAddrINet4 tmp(saddr);
   if(tmp.ip()!=IPNum) {
     WY_THROW( WyRet() );
   }
   if(tmp.port()!=PortNum) {
     WY_THROW( WyRet() );
   }

   WySockAddrINet4 tmp2;
   CHK_DEFAULT(tmp2);
   tmp2.reset(saddr);
   if(tmp2.ip()!=IPNum) {
     WY_THROW( WyRet() );
   }
   if(tmp2.port()!=PortNum) {
     WY_THROW( WyRet() );
   }
   CHK_EQU(tmp,tmp2);

   WySockAddrINet4 tmp3;
   tmp3=saddr;
   CHK_EQU(tmp,tmp2);

   if(tmp3==saddr) {
   } else {
     WY_THROW( WyRet() );
   }
   if(tmp3!=saddr) {
     WY_THROW( WyRet() );
   }
/*
   if(saddr==tmp3) {
   } else {
     WY_THROW( WyRet() );
   }
   if(saddr!=tmp3) {
     WY_THROW( WyRet() );
   }*/
 }

 // operator==/!=
 {
   WySockAddrINet4 a0,a1;
   if(a0==a1) {
   } else {
     WY_THROW( WyRet() );
   }
   if(a0!=a1) {
     WY_THROW( WyRet() );
   }

   a1.reset(WySockAddrINet4::IPAddr(3),4);
   if(a1==a0) {
     WY_THROW( WyRet() );
   }
   if(a1!=a0) {
   } else {
     WY_THROW( WyRet() );
   }
 }

 // notation/wrd
 {
  const WyCSeg IPStr("1.0.11.222");
  WySockAddrINet4::PortType PortNum=40000;
  WySockAddrINet4 saddr( WySockAddrINet4::IPAddr(IPStr),PortNum);
  WyStr str(WyCSeg("x"));
  if((r=saddr.notation(str))!=Ok) {
    WY_THROW(r);
  }
  WyStr str2=WyStr(IPStr)+':'+Wy::wrd(PortNum);
  if(str!=str2) {
    WY_THROW( WyRet() );
  }
  if(Wy::wrd(saddr)!=str2) {
    WY_THROW( WyRet() );
  }
 }

 // reset()
 {
   WySockAddrINet4 a0;
   a0.reset();
   CHK_DEFAULT(a0);

   WySockAddrINet4 a1(WySockAddrINet4::IPAddr(5),6);
   CHK_NOT_DEFAULT(a1);
   a1.reset();
   CHK_DEFAULT(a1);
 }

 // reset(const WySockAddrINet4&)/reset(ulong,ushort)/reset(const WyStr&,ushort)
 {
   WySockAddrINet4 a0;
   WySockAddrINet4 a1;
   a1.reset(a0);
   CHK_DEFAULT(a0);
   CHK_DEFAULT(a1);
   CHK_EQU(a0,a1);

   a0.reset(WySockAddrINet4::IPAddr(5),6);
   a1.reset(a0);
   if(a0.is_default()) {
     WY_THROW( WyRet() );
   }
   if(a1!=a0) {
     WY_THROW( WyRet() );
   }
   if(a0.port()!=6) {
     WY_THROW( WyRet() );
   }
   if(a1.port()!=6) {
     WY_THROW( WyRet() );
   }
   
   WySockAddrINet4 a2;
   a2.reset( WySockAddrINet4::IPAddr("2.2.3.3"),9);
   a0=a1=a2;
   CHK_EQU(a0,a1);
   CHK_EQU(a0,a2);
   CHK_EQU(a1,a0);
   CHK_EQU(a1,a2);
   CHK_EQU(a2,a0);
   CHK_EQU(a2,a1);
   if(a0.port()!=9) {
     WY_THROW( WyRet() );
   }
   if(a1.port()!=9) {
     WY_THROW( WyRet() );
   }
   if(a2.ip()!=0x02020303) {
     WY_THROW( WyRet() );
   }
 }

 // operator=
 {
   WySockAddrINet4 a0;
   WySockAddrINet4 a1;
   a1=a0;
   CHK_DEFAULT(a0);
   CHK_DEFAULT(a1);

   a0.reset(WySockAddrINet4::IPAddr(4),5);
   a1=a0;
   if(a0.is_default()) {
     WY_THROW( WyRet() );
   }
   if(a1.is_default()) {
     WY_THROW( WyRet() );
   }
   if(a1!=a0) {
     WY_THROW( WyRet() );
   }
   if(a1==a0) {
   } else {
     WY_THROW( WyRet() );
   }
   if(a0.port()!=5) {
     WY_THROW( WyRet() );
   }
   if(a1.port()!=5) {
     WY_THROW( WyRet() );
   }
 }

};

// Check basic operations on non-default object
//
static void t2(void)
{
 WyRet r;
 WySockAddrINet4 A0;
 WySockAddrINet4 N0;

 CHK_DEFAULT(A0);
 CHK_DEFAULT(N0);
 N0.reset( WySockAddrINet4::IPAddr("12.34.56.78"),8);
 CHK_NOT_DEFAULT(N0);

 // constructor
 {
   WySockAddrINet4 a0(N0);
   CHK_NOT_DEFAULT(a0);
   CHK_EQU(a0,N0);
 }

 // operator==/!=
 {
   WySockAddrINet4 a0(N0),a1(a0);
   CHK_NOT_DEFAULT(a0);
   CHK_NOT_DEFAULT(a1);
   if(a0==a1) {
   } else {
     WY_THROW( WyRet() );
   }
   if(a0!=a1) {
     WY_THROW( WyRet() );
   }
 }
 
 // reset
 {
   WySockAddrINet4 a0(N0);
   CHK_NOT_DEFAULT(a0);
   CHK_EQU(a0,N0);
   a0.reset();
   CHK_DEFAULT(a0);
 }

 // _swap
 {
  WySockAddrINet4 tmp0(N0),tmp1,tmp2;
  CHK_NOT_DEFAULT(tmp0);
  CHK_DEFAULT(tmp1);
  CHK_DEFAULT(tmp2);
  CHK_NOT_DEFAULT(N0);

  if((r=tmp1._swap(tmp2))!=Ok) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp1);
  CHK_DEFAULT(tmp2);

  if((r=tmp1._swap(tmp0))!=Ok) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp0);
  CHK_NOT_DEFAULT(tmp1);
  CHK_EQU(tmp1,N0);

  if((r=tmp0._swap(tmp1))!=Ok) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp1);
  CHK_NOT_DEFAULT(tmp0);
  CHK_EQU(tmp0,N0);

  Wy_SockAddr* bp_tmp0(&tmp0);
  if((r=tmp1._swap(*bp_tmp0))!=Ok) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp0);
  CHK_NOT_DEFAULT(tmp1);
  CHK_EQU(tmp1,N0);
 }

 // _alloc
 {
   WySockAddrINet4 tmp;
   WySockAddrINet4* cptr( tmp._alloc(r) );
   if(r!=Ok) {
     WY_THROW(r);
   }
   if(cptr==0) {
     WY_THROW( WyRet() );
   }
   Wy_AtDestroy<void,Wy_SockAddr*> rrid(Wy::_delete_one,cptr);
   CHK_DEFAULT(*cptr);
 }

 // reset
 {
   WySockAddrINet4 a0(N0);
   CHK_EQU(a0,N0);
   a0.reset(N0);
   CHK_EQU(a0,N0);
   a0.reset(A0);
   CHK_EQU(a0,A0);
   CHK_DEFAULT(a0);
   a0.reset( WySockAddrINet4::IPAddr("1.2.3.4"),5);
   WySockAddrINet4 a1;
   a1.reset(a0);
   CHK_EQU(a0,a1);
 }

 // operator=
 {
   WySockAddrINet4 a0(N0);
   a0=A0;
   CHK_DEFAULT(a0);
   CHK_EQU(a0,A0);
   a0=N0;
   CHK_EQU(a0,N0);
 }

 // chk move ctor
 {
  const WySockAddrINet4::IPAddr TstIP("10.20.30.40");
  const uint16_t TstPort=567;
  char raw_buf[2*sizeof(WySockAddrINet4)];
  ::memset(raw_buf,0,sizeof(raw_buf));
  WySockAddrINet4* const p1( reinterpret_cast<WySockAddrINet4*>(raw_buf) );
  WySockAddrINet4* const p2(p1+1);

  new(p1) WySockAddrINet4(TstIP,TstPort);
  if((p1->is_default()==true)||
     (p1->port()!=TstPort)||
     (p1->ip()!=TstIP) ) {
    p1->~WySockAddrINet4();
    WY_THROW(r);
  }

  // move *p1 to p2
  try {
    new(p2) WySockAddrINet4(*p1,Wy::ByMove);
  }
  catch(...) {
    p1->~WySockAddrINet4();
    throw;
  };
  if((p2->is_default()==true)||
     (p2->port()!=TstPort)||
     (p2->ip()!=TstIP) ) {
    p2->~WySockAddrINet4();
    WY_THROW(r);
  }
  p2->~WySockAddrINet4();
 }
};

static void t_exception(void)
{

 // Test copy constructor exception
 //
 {
   try {
     WyStr tbuf("abc.efg.1234zaz");    // should cause error
     WySockAddrINet4 tmp(tbuf,4);
     tmp.reset();          // supress unuse warning
     WY_THROW( WyRet() );  // no expected exception
   }
   catch(const WySockAddrINet4::IPAddr::Reply& e) {
     if(e!=Wym_EADDRNOTAVAIL) {
       WyRet r=e;
       WY_THROW(r);
     }
     // FALL_THROUGH 
   };
 };

 // chk WySockAddrINet4(const char*,unsigned short)
 {
   try {
     WySockAddrINet4 tmp( WySockAddrINet4::IPAddr((const char*)0),1);
     tmp.reset();        // supress unused warning of compiler
     WY_THROW( WyRet() );
   }
   catch(const WySockAddrINet4::IPAddr::Reply& e) {
     if(e!=Wym_EFAULT) {
       throw;
     }
     // FALL_THROUGH 
   };
 };
};

static void t_sockaddr(void)
{
 const WyCSeg IPStr("1.0.11.222");
 WySockAddrINet4::PortType PortNum=40000;
 WySockAddrINet4 saddr( WySockAddrINet4::IPAddr(IPStr),PortNum);
 WyStr str(WyCSeg("x"));
 WyRet r;
 const Wy_SockAddr &sref( saddr );
 if(sref.family()!=AF_INET) {
   WY_THROW( WyRet() );
 }
 if((r=sref.notation(str))!=Ok) {
   WY_THROW(r);
 }
 if(str!=WyStr(IPStr)+':'+Wy::wrd(PortNum)) {
   WY_THROW( WyRet() );
 }
};

#include "Wy_Array.hh"
#include "ck_array.h"

static void t_array(void)
{
 ck_array<WySockAddrINet4>();
/*
 // chk struct in_addr { unsigned long s_addr; }
 // (simple check)
 {
   ck_array< ::in_addr >();
   Wy_Array< ::in_addr > tmp;
   ::in_addr iaddr={ 123 };
   tmp.push_back(iaddr);
   if(tmp.size()!=1) {
     WY_THROW( WyRet() );
   }
   if(tmp[0].s_addr!=iaddr.s_addr) {
     WY_THROW( WyRet() );
   }
   tmp.push_back(::in_addr());
   if(tmp.size()!=2) {
     WY_THROW( WyRet() );
   }
   if(tmp[1].s_addr!=0) {
     WY_THROW( WyRet() );
   }
 }*/
/*
 // chk struct sockaddr_in
 {
   //ck_array< ::sockaddr_in > tmp;       // wont compile
   Wy_Array< ::sockaddr_in > tmp;
   ::sockaddr_in sa;
   ::in_addr iaddr={ 321 };

   sa.sin_family=AF_INET;
   ::memcpy(&sa.sin_addr,&iaddr,sizeof(sa.sin_addr));
   sa.sin_port=5678;

   tmp.push_back(sa);
   if(tmp.size()!=1) {
     WY_THROW( WyRet() );
   }
   if(tmp[0].sin_family!=AF_INET) {
     WY_THROW( WyRet() );
   }
   if(tmp[0].sin_addr.s_addr!=iaddr.s_addr) {
     WY_THROW( WyRet() );
   }
   if(tmp[0].sin_port!=5678) {
     WY_THROW( WyRet() );
   }

   tmp.push_back(::sockaddr_in());
   if(tmp.size()!=2) {
     WY_THROW( WyRet() );
   }
   if(tmp[1].sin_family!=0) {
     WY_THROW( WyRet() );
   }
   if(tmp[1].sin_addr.s_addr!=0) {
     WY_THROW( WyRet() );
   }
   if(tmp[1].sin_port!=0) {
     WY_THROW( WyRet() );
   }

 }*/
};

static WyStr chdr("+---------------+\n"
                  "| main() caught:|\n"
                  "+---------------+\n");
int main(void) throw()
try {
 std::cout << "Checking WySockAddrINet4 ...\n";
 if(WySockAddrINet4::family()!=AF_INET) {
   WY_THROW( WyRet() );
 }
 chk_ipaddr();
 t1();
 t2();
 t_exception();
 t_sockaddr();

 {
   WySockAddrINet4::IPAddr tmp;
   WyStr str;
   WyRet r;
   if((r=tmp.notation(str))!=Ok) {
     WY_THROW(r);
   }
   if(Wy::wrd(tmp)!=str) {
     WY_THROW( WyRet() );
   }
   std::cout << "IPAddr()=\"" << str.c_str() << "\"\n";
 }

 // chk new
 {
   class CkNew {
     public:
      CkNew() : _p(new WySockAddrINet4()) {};
      ~CkNew() { if(_p) { delete _p; _p=0; } };
       WySockAddrINet4* _p;
   } cn;
   CHK_DEFAULT(*cn._p);
 }

 t_array();
 std::cout << "Checked Ok\n";
 return(0);
}
catch(const WyRet& e) {
 std::cerr << chdr.c_str() << Wy::wrd(e).c_str() << std::endl;
 return(-1);
}
catch(const std::exception& e) {
 std::cerr << chdr.c_str() << "std::exception" << std::endl;
 return(-1);
}
catch(...) {
 std::cerr << chdr.c_str() << "unknown unwind" << std::endl;
 return(-1);
};
