/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   Check WySockAddrLocal public members

   Build: make chk_sockaddrlocal

   Note: Test data files should exist in the working directory
         Files may be created in the working directory and unlinked
*/

#include "WySockAddrLocal.h"
#include "Wy_AtDestroy.hh"
#include <memory>
#include <iostream>

#if WYSOCKADDRLOCAL_VERSION!=33
#error Test code is for WYSOCKADDRLOCAL_VERSION 33
#endif

//
// Assert ff is default
//
static void assert_default(const WySockAddrLocal& laddr)
{
 static const WySockAddrLocal Def_Obj;
 WyRet r;

 if(laddr.is_default()==false) {
   WY_THROW( WyRet() );
 }
 if(laddr.family()!=AF_LOCAL) {
   WY_THROW( WyRet() );
 }
 if(*laddr.addrname()!=char(0)) {
   WY_THROW(r);
 }
 if(laddr.addrsize()!=0) {
   WY_THROW(r);
 }
 if(laddr!=Def_Obj) {
   WY_THROW( WyRet() );
 }
 if(laddr==Def_Obj) {
 } else {
   WY_THROW( WyRet() );
 }
};

static void assert_not_default(const WySockAddrLocal& laddr)
{
 static const WySockAddrLocal Def_Obj;
 WyRet r;

 if(laddr.is_default()) {
   WY_THROW( WyRet() );
 }
 if(laddr.family()!=AF_LOCAL) {
   WY_THROW( WyRet() );
 }
 if(laddr.addrsize()==0) {
   WY_THROW(r);
 }
 if(laddr==Def_Obj) {
   WY_THROW( WyRet() );
 }
 if(laddr!=Def_Obj) {
 } else {
   WY_THROW( WyRet() );
 }
};

static void assert_equ(const WySockAddrLocal& a1,const WySockAddrLocal& a2)
{
 if(a1.family()!=AF_LOCAL) {
   WY_THROW( WyRet() );
 }
 if(a2.family()!=AF_LOCAL) {
   WY_THROW( WyRet() );
 }
 if(a1.addrsize()!=a2.addrsize()) {
   WY_THROW( WyRet() );
 }
 if(::memcmp(a1.addrname(),a2.addrname(),a1.addrsize())!=0) {
   WY_THROW( WyRet() );
 }
};

#define CHK_DEFAULT(p)\
    try { assert_default(p); } catch(...) { WY_THROW( WyRet()); }
#define CHK_NOT_DEFAULT(p)\
    try { assert_not_default(p); } catch(...) { WY_THROW( WyRet()); }
#define CHK_EQU(a1,a2)\
    try { assert_equ((a1),(a2)); } catch(...) { WY_THROW( WyRet()); }

// Check basic operations on default object
//
static void t1(void)
{
 WyRet r;

 // WySockAddrLocal()/WySockAddrLocal(copy)
 {
   WySockAddrLocal a0;
   CHK_DEFAULT(a0);

   WySockAddrLocal a1(a0);
   CHK_DEFAULT(a1);
   CHK_EQU(a1,a0);
 }

 // WySockAddrLocal(const WyStr&)
 {
   const WyStr Notation("abcd2");
   WyStr pn0;
   WyStr pn1(Notation);
   WyStr str;

   WySockAddrLocal a0(pn0);
   CHK_DEFAULT(a0);

   WySockAddrLocal a1(pn1);
   CHK_NOT_DEFAULT(a1);
   if(a1.addrname()!=Notation) {
     WY_THROW( WyRet() );
   }

   WySockAddrLocal a2(Notation);
   CHK_EQU(a1,a2);

   if((r=str.reset(WySockAddrLocal::MaxAddrLen,char('A')))!=Ok) {
     WY_THROW( WyRet(r) );
   }
   WySockAddrLocal a3(str);
   if(str!=a3.addrname()) {
     WY_THROW( WyRet() );
   }

   if((r=str.reset(WySockAddrLocal::MaxAddrLen+1,char('B')))!=Ok) {
     WY_THROW( WyRet(r) );
   }
   try {
     WySockAddrLocal a4(str);
     a4.is_default();
     WY_THROW( WyRet() );
   }
   catch(const WySockAddrLocal::Reply& e) {
     if(e!=Wym_ENAMETOOLONG) {
       throw;
     }
     // FALLTHROUGH
   };
 }

 // WySockAddrLocal(const char*)
 {
  WySockAddrLocal tmp1("");
  CHK_DEFAULT(tmp1);

  WySockAddrLocal tmp2("abb");
  CHK_NOT_DEFAULT(tmp2);
  if(Wy::_strcmp(tmp2.addrname(),"abb")!=0) {
    WY_THROW( WyRet() );
  }
 }

 // WySockAddrLocal(WyCSeg)
 {
  WyCSeg path("path");
  WySockAddrLocal tmp1(path);
  CHK_NOT_DEFAULT(tmp1);
  if(Wy::_strcmp(tmp1.addrname(),path)!=0) {
    WY_THROW( WyRet() );
  }
 }

 // notation(WyStr&)
 {
  WyCSeg path("testpath.12");
  WySockAddrLocal laddr(path);
  WyStr str(WyCSeg("xx"));
  if((r=laddr.notation(str))!=Ok) {
    WY_THROW( r );
  }
  if(str!=path) {
    WY_THROW( WyRet() );
  }
  
  laddr.reset();
  if((r=laddr.notation(str))!=Ok) {
    WY_THROW( r );
  }
  if(str.size()!=0) {
    WY_THROW( WyRet() );
  }
 }

 // reset()
 {
   WySockAddrLocal a0;
   a0.reset();
   CHK_DEFAULT(a0);

   WySockAddrLocal a1("x");
   CHK_NOT_DEFAULT(a1);
   a1.reset();
   CHK_DEFAULT(a1);
 }

 // reset(const WySockAddrLocal&)
 {
   WySockAddrLocal a0;
   WySockAddrLocal a1;
   a1.reset(a0);
   CHK_DEFAULT(a0);
   CHK_DEFAULT(a1);
   CHK_EQU(a1,a0);

   if((r=a1.reset(WyStr()))!=Ok) {
     WY_THROW(r);
   }
   if((r=a1.reset(WyStr("helo")))!=Ok) {
     WY_THROW(r);
   }
   CHK_NOT_DEFAULT(a1);
 }

 // reset(const WyStr&)
 {
  WyStr pn("ddd");
  WyRet r;

  WySockAddrLocal tmp;
  CHK_DEFAULT(tmp);
  if((r=tmp.reset(pn))!=Ok) {
     WY_THROW(r);
   }
  CHK_NOT_DEFAULT(tmp);
 }

 // reset(const WyStr&)
 {
  WyStr str(2,char(0));
  WyRet r;

  str+='a';
  WySockAddrLocal tmp;
  CHK_DEFAULT(tmp);
  if((r=tmp.reset(str))!=Ok) {
    WY_THROW(r);
  }
  CHK_NOT_DEFAULT(tmp);

  if(str!=WyCSeg(tmp.addrname(),tmp.addrsize())) {
    WY_THROW( WyRet() );
  }
  if(Wy::wrd(tmp)!=str) {
    WY_THROW( WyRet() );
  }
 }

 // operator=
 {
   WySockAddrLocal a0;
   WySockAddrLocal a1;
   a1=a0;
   CHK_DEFAULT(a0);
   CHK_DEFAULT(a1);

   if((r=a0.reset(WyStr("helo")))!=Ok) {
     WY_THROW(r);
   }
   a1=a0;
   CHK_NOT_DEFAULT(a0);
   CHK_NOT_DEFAULT(a1);
   CHK_EQU(a0,a1);

   a0.reset();
   CHK_DEFAULT(a0);
   a0=WyStr("helo");
   CHK_EQU(a0,a1);

   a0="";
   CHK_DEFAULT(a0);
   a0="1234";
   if(Wy::_strcmp(a0.addrname(),"1234")!=0) {
     WY_THROW( WyRet() );
   }

   WyCSeg seg("asdf");
   a0=seg;
   if(Wy::_strcmp(a0.addrname(),seg)!=0) {
     WY_THROW( WyRet() );
   }
 }

 // addrname()/addrsize()
 {
   WySockAddrLocal a0;
   CHK_DEFAULT(a0);
   WyStr str;
   
   if(a0.addrsize()!=0) {
     WY_THROW(r);
   }
   WyStr LStr(WySockAddrLocal::MaxAddrLen,'x');
                      
   if((r=a0.reset(LStr))!=Ok) {
     WY_THROW(r);
   }
   if(LStr!=a0.addrname()) {
     WY_THROW(r);
   }
   if(LStr.size()!=a0.addrsize()) {
     WY_THROW(r);
   }
   if(LStr!=Wy::wrd(a0)) {
     WY_THROW( WyRet() );
   }

   LStr+='x';    // add one char
   if((r=a0.reset(LStr))!=Wym_ENAMETOOLONG) {
     WY_THROW(r);
   }
 }

 // operator==/!=
 {
   WySockAddrLocal a0,a1;
   const WyStr Str("helo");

   if(a0==a1) {
   } else {
     WY_THROW( WyRet() );
   }
   if(a0!=a1) {
     WY_THROW( WyRet() );
   }
   if(a0==WyStr()) {
   } else {
     WY_THROW( WyRet() );
   }
   if(a0!=WyStr()) {
     WY_THROW( WyRet() );
   } else {
   }
   if(a0==WyCSeg("",size_t(0))) {
   } else {
     WY_THROW( WyRet() );
   }
   if(a0!=WyCSeg("",size_t(0))) {
     WY_THROW( WyRet() );
   }

   if((r=a1.reset(Str))!=Ok) {
     WY_THROW(r);
   }
   if(a1==a0) {
     WY_THROW( WyRet() );
   }
   if(a1!=a0) {
   } else {
     WY_THROW( WyRet() );
   }
   if(a1==a0.addrname()) {
     WY_THROW( WyRet() );
   }
   if(a1!=a0.addrname()) {
   } else {
     WY_THROW( WyRet() );
   }
   if(a1==WyStr(a0.addrname())) {
     WY_THROW( WyRet() );
   }
   if(a1!=WyStr(a0.addrname())) {
   } else {
     WY_THROW( WyRet() );
   }

   if(WyStr("helo")==a1) {
   } else {
     WY_THROW( WyRet() );
   }
   if(WyStr("helo")!=a1) {
     WY_THROW( WyRet() );
   }
   if(Str.cseg()==a1) {
   } else {
     WY_THROW( WyRet() );
   }
   if(Str.cseg()!=a1) {
     WY_THROW( WyRet() );
   }
 }
 
 // operator==/!=
 {
  WyStr str("helo");
  WySockAddrLocal tmp0(str);
  if(tmp0==str) {
  } else {
    WY_THROW( WyRet() );
  }
  if(str==tmp0) {
  } else {
    WY_THROW( WyRet() );
  }
  if(tmp0!=str) {
    WY_THROW( WyRet() );
  }
  if(str!=tmp0) {
    WY_THROW( WyRet() );
  }

  WyCSeg tseg("he\0lo",5);
  if((r=tmp0.reset(tseg))!=Ok) {
    WY_THROW( WyRet(r) );
  }
  if(tmp0==tseg) {
  } else {
    WY_THROW( WyRet() );
  }
  if(tseg==tmp0) {
  } else {
    WY_THROW( WyRet() );
  }
  if(tmp0!=tseg) {
    WY_THROW( WyRet() );
  }
  if(tseg!=tmp0) {
    WY_THROW( WyRet() );
  }
/*
  struct sockaddr_un un_addr;
  std::strcpy(un_addr.sun_path,str.c_str());
  tmp0=str;
  if(tmp0==un_addr) {
  } else {
    WY_THROW( WyRet() );
  }
  if(un_addr==tmp0) {
  } else {
    WY_THROW( WyRet() );
  }
  if(tmp0!=un_addr) {
    WY_THROW( WyRet() );
  }
  if(un_addr!=tmp0) {
    WY_THROW( WyRet() );
  }
 */
 }
};

// Check basic operations on non-default object
//
static void t2(void)
{
 WyRet r;
 WySockAddrLocal A0;
 WySockAddrLocal N0;

 CHK_DEFAULT(A0);
 CHK_DEFAULT(N0);
 if((r=N0.reset(WyStr("helo")))!=Ok) {
   WY_THROW(r);
 }
 CHK_NOT_DEFAULT(N0);

 // constructor
 {
   WySockAddrLocal a0(N0);
   CHK_NOT_DEFAULT(a0);
 }

 // operator==/!=
 {
   WySockAddrLocal a0(N0),a1(a0);
   CHK_NOT_DEFAULT(a0);
   CHK_NOT_DEFAULT(a1);
   if(a0==a1) {
   } else {
     WY_THROW( WyRet() );
   }
   if(a0!=a1) {
     WY_THROW( WyRet() );
   }
   if(a0==a1.addrname()) {
   } else {
     WY_THROW( WyRet() );
   }
   if(a0!=a1.addrname()) {
     WY_THROW( WyRet() );
   }
 }
 
 // reset
 {
   WySockAddrLocal a0(N0);
   CHK_NOT_DEFAULT(a0);
   a0.reset();
   CHK_DEFAULT(a0);
 }

 // _swap
 {
  WySockAddrLocal tmp0(N0),tmp1,tmp2;
  CHK_NOT_DEFAULT(tmp0);
  CHK_DEFAULT(tmp1);
  CHK_DEFAULT(tmp2);
  CHK_NOT_DEFAULT(N0);

  if((r=tmp1._swap(tmp2))!=Ok) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp1);
  CHK_DEFAULT(tmp2);

  if((r=tmp1._swap(tmp0))!=Ok) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp0);
  CHK_NOT_DEFAULT(tmp1);
  //CHK_EQU(tmp1,NObj);

  if((r=tmp0._swap(tmp1))!=Ok) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp1);
  CHK_NOT_DEFAULT(tmp0);
  //CHK_EQU(tmp0,NObj);

  Wy_SockAddr* bptr(&tmp0);
  if((r=tmp1._swap(*bptr))!=Ok) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp0);
  CHK_NOT_DEFAULT(tmp1);

  struct ABC : public WySockAddrLocal {} tmp3;
  if((r=tmp0._swap(tmp3))!=Wym_EBADTYPE) {
    WY_THROW( WyRet() );
  }
 }

 // chk _alloc
 {
   WySockAddrLocal tmp;
   WySockAddrLocal* p( tmp._alloc(r) );
   if(r!=Ok) {
     WY_THROW(r);
   }
   if(p==0) {
     WY_THROW( WyRet() );
   }
   Wy_AtDestroy<void,WySockAddrLocal*> rrid(Wy::_delete_one,p);
   CHK_DEFAULT(*p);
   if((r=tmp._swap(*p))!=Ok) {
     WY_THROW(r);
   }
    CHK_DEFAULT(*p);
    CHK_DEFAULT(tmp);
 }

 // reset
 {
   WySockAddrLocal a0(N0);
   a0.reset(N0);
   a0.reset(A0);
   CHK_DEFAULT(a0);
 }

 // operator=
 {
   WySockAddrLocal a0(N0);
   CHK_NOT_DEFAULT(a0);
   a0=A0;
   CHK_DEFAULT(a0);
 }

 // addrname/addrsize
 {
   WySockAddrLocal a0(N0);
   CHK_NOT_DEFAULT(a0);
   WyStr str(a0.addrname());

   if(str!=N0) {   // N0 addrname is "helo"
     WY_THROW( WyRet() );
   }
   if(str!=Wy::wrd(a0)) {
     WY_THROW( WyRet() );
   }
 }

 // chk move ctor
 {
  const WyStr TPath("MoveCtorTest");
  char raw_buf[2*sizeof(WySockAddrLocal)];
  ::memset(raw_buf,0,sizeof(raw_buf));
  WySockAddrLocal* const p1( reinterpret_cast<WySockAddrLocal*>(raw_buf) );
  WySockAddrLocal* const p2(p1+1);

  new(p1) WySockAddrLocal(TPath);
  if((p1->is_default()==true)||
     (Wy::_strcmp(p1->addrname(),TPath.cseg())!=0) ) {
    p1->~WySockAddrLocal();
    WY_THROW(r);
  }

  // move *p1 to p2
  try {
    new(p2) WySockAddrLocal(*p1,Wy::ByMove);
  }
  catch(...) {
    p1->~WySockAddrLocal();
    throw;
  };
  if((p2->is_default()==true)||
     (Wy::_strcmp(p2->addrname(),TPath.cseg())!=0) ) {
    p2->~WySockAddrLocal();
    WY_THROW(r);
  }
  p2->~WySockAddrLocal();
 }
};

static void t_error(void)
{
 WyRet r;

 // WySockAddrLocal(const char*)
 //
 {
   try {
     WyStr path(WySockAddrLocal::MaxAddrLen+1,char('A'));
     WySockAddrLocal tmp(path.c_str());
     tmp.reset();          // supress warning message
     WY_THROW( WyRet() );  // no expected exception
   }
   catch(const WySockAddrLocal::Reply& e) {
     if(e!=Wym_ENAMETOOLONG) {
       WY_THROW( WyRet(e) );
     }
     // FALL_THROUGH 
   };

   try {
     WySockAddrLocal tmp((const char*)0);
     tmp.reset();          // supress warning message
     WY_THROW( WyRet() );  // no expected exception
   }
   catch(const WySockAddrLocal::Reply& e) {
     if(e!=Wym_EFAULT) {
       WY_THROW( WyRet(e) );
     }
     // FALL_THROUGH 
   };
 }

 // WySockAddrLocal(const WyStr&)
 //
 {
   try {
     WyStr path(WySockAddrLocal::MaxAddrLen+1,char('A'));
     WySockAddrLocal tmp(path);
     tmp.reset();          // supress warning message
     WY_THROW( WyRet() );  // no expected exception
   }
   catch(const WySockAddrLocal::Reply& e) {
     if(e!=Wym_ENAMETOOLONG) {
       WY_THROW( WyRet(e) );
     }
     // FALL_THROUGH 
   };
 }

 // WySockAddrLocal(WyCSeg)
 //
 {
   try {
     WyStr path(WySockAddrLocal::MaxAddrLen+1,char('A'));
     WySockAddrLocal tmp(path.cseg());
     tmp.reset();          // supress warning message
     WY_THROW( WyRet() );  // no expected exception
   }
   catch(const WySockAddrLocal::Reply& e) {
     if(e!=Wym_ENAMETOOLONG) {
       WY_THROW( WyRet(e) );
     }
     // FALL_THROUGH 
   };
 }

 // reset error
 {
   WyRet r;
   WySockAddrLocal tmp;

   WyStr path(WySockAddrLocal::MaxAddrLen+1,char('A'));
   if((r=tmp.reset(path.c_str()))!=Wym_ENAMETOOLONG) {
     WY_THROW(r);
   }
   if((r=tmp.reset(path))!=Wym_ENAMETOOLONG) {
     WY_THROW(r);
   }
   if((r=tmp.reset(path.cseg()))!=Wym_ENAMETOOLONG) {
     WY_THROW(r);
   }
   if((r=tmp.reset(path.c_str()))!=Wym_ENAMETOOLONG) {
     WY_THROW(r);
   }
   if((r=tmp.reset((const char*)0))!=Wym_EFAULT) {
     WY_THROW(r);
   }
 }

 // _swap error
 {
  WyRet r;
  struct T : public WySockAddrLocal {} t;
  WySockAddrLocal tmp;
  if((r=tmp._swap(t))!=Wym_EBADTYPE) {
     WY_THROW(r);
  }
 }

 // operator=(const char*)
 {
   try {
     WySockAddrLocal tmp;
     const char* p=0;
     tmp=p;
     WY_THROW( WyRet() );  // no expected exception
   }
   catch(const WySockAddrLocal::Reply& e) {
     if(e!=Wym_EFAULT) {
       WY_THROW( WyRet(e) );
     }
     // FALL_THROUGH 
   };

   try {
     WySockAddrLocal tmp;
     WyStr p(WySockAddrLocal::MaxAddrLen+1,'d');
     tmp=p.c_str();
     WY_THROW( WyRet() );  // no expected exception
   }
   catch(const WySockAddrLocal::Reply& e) {
     if(e!=Wym_ENAMETOOLONG) {
       WY_THROW( WyRet(e) );
     }
     // FALL_THROUGH 
   };
 }

 // operator=(const WyStr&)
 {
   try {
     WyStr path(WySockAddrLocal::MaxAddrLen+1,char('A'));
     WySockAddrLocal tmp;
     tmp=path;
     WY_THROW( WyRet() );  // no expected exception
   }
   catch(const WySockAddrLocal::Reply& e) {
     if(e!=Wym_ENAMETOOLONG) {
       WY_THROW( WyRet(e) );
     }
     // FALL_THROUGH 
   };
 }

 // operator=(WyCSeg)
 {
   try {
     WyStr path(WySockAddrLocal::MaxAddrLen+1,char('A'));
     WySockAddrLocal tmp;
     tmp=path.cseg();
     WY_THROW( WyRet() );  // no expected exception
   }
   catch(const WySockAddrLocal::Reply& e) {
     if(e!=Wym_ENAMETOOLONG) {
       WY_THROW( WyRet(e) );
     }
     // FALL_THROUGH 
   };
 }

 // operator==(const char*)
 {
   try {
     WySockAddrLocal tmp;
     const char* p=0;
     if(tmp==p) {
       WY_THROW( WyRet() );  // no expected exception
     } else {
       WY_THROW( WyRet() );  // no expected exception
     }
   }
   catch(const WySockAddrLocal::Reply& e) {
     if(e!=Wym_EFAULT) {
       WY_THROW( WyRet(e) );
     }
     // FALL_THROUGH 
   };

   try {
     WySockAddrLocal tmp;
     WyStr p(WySockAddrLocal::MaxAddrLen+1,char('A'));
     if(p.c_str()==tmp) {
       WY_THROW( WyRet() );  // no expected exception
     } else {
       WY_THROW( WyRet() );  // no expected exception
     }
   }
   catch(const WySockAddrLocal::Reply& e) {
     if(e!=Wym_ENAMETOOLONG) {
       WY_THROW( WyRet(e) );
     }
     // FALL_THROUGH 
   };

   try {
     WySockAddrLocal tmp;
     const char* p=0;
     if(p==tmp) {
       WY_THROW( WyRet() );  // no expected exception
     } else {
       WY_THROW( WyRet() );  // no expected exception
     }
   }
   catch(const WySockAddrLocal::Reply& e) {
     if(e!=Wym_EFAULT) {
       WY_THROW( WyRet(e) );
     }
     // FALL_THROUGH 
   };

 }

 // operator==(const WyStr&)
 {
   try {
     WySockAddrLocal tmp;
     WyStr p(WySockAddrLocal::MaxAddrLen+1,char('A'));
     if(p==tmp) {
       WY_THROW( WyRet() );  // no expected exception
     } else {
       WY_THROW( WyRet() );  // no expected exception
     }
   }
   catch(const WySockAddrLocal::Reply& e) {
     if(e!=Wym_ENAMETOOLONG) {
       WY_THROW( WyRet(e) );
     }
     // FALL_THROUGH 
   };

   try {
     WySockAddrLocal tmp;
     WyStr p(WySockAddrLocal::MaxAddrLen+1,char('A'));
     if(tmp==p) {
       WY_THROW( WyRet() );  // no expected exception
     } else {
       WY_THROW( WyRet() );  // no expected exception
     }
   }
   catch(const WySockAddrLocal::Reply& e) {
     if(e!=Wym_ENAMETOOLONG) {
       WY_THROW( WyRet(e) );
     }
     // FALL_THROUGH 
   };
 }

 // operator==(const WyCSeg&)
 {
   try {
     WySockAddrLocal tmp;
     WyStr p(WySockAddrLocal::MaxAddrLen+1,char('A'));
     if(p.cseg()==tmp) {
       WY_THROW( WyRet() );  // no expected exception
     } else {
       WY_THROW( WyRet() );  // no expected exception
     }
   }
   catch(const WySockAddrLocal::Reply& e) {
     if(e!=Wym_ENAMETOOLONG) {
       WY_THROW( WyRet(e) );
     }
     // FALL_THROUGH 
   };

   try {
     WySockAddrLocal tmp;
     WyStr p(WySockAddrLocal::MaxAddrLen+1,char('A'));
     if(tmp==p.cseg()) {
       WY_THROW( WyRet() );  // no expected exception
     } else {
       WY_THROW( WyRet() );  // no expected exception
     }
   }
   catch(const WySockAddrLocal::Reply& e) {
     if(e!=Wym_ENAMETOOLONG) {
       WY_THROW( WyRet(e) );
     }
     // FALL_THROUGH 
   };
 }

 // chk _alloc not override
 {
   class T : public WySockAddrLocal {
   } tmp;
   Wy_SockAddr* p=tmp._alloc(r);
   if(r!=Wym_ENOSYS) {
     if(r==Ok) {
       delete p;
     }
     WY_THROW(r);
   }
 }
};

#include "Wy_Array.hh"

static void t_array(void)
{
/*
 // chk struct struct sockaddr_un
 // (simple chk)
 {
   typedef struct ::sockaddr_un ElemType;
   Wy_Array<ElemType> tmp;
   ElemType ele;
   ele.sun_family=45;
   ele.sun_path[1]=123;

   tmp.push_back(ele);
   if(tmp.size()!=1) {
     WY_THROW( WyRet() );
   }
   if(tmp[0].sun_family!=ele.sun_family) {
     WY_THROW( WyRet() );
   }
   if(tmp[0].sun_path[1]!=ele.sun_path[1]) {
     WY_THROW( WyRet() );
   }

   tmp.push_back(ElemType());
   if(tmp.size()!=2) {
     WY_THROW( WyRet() );
   }
   if(tmp[1].sun_family!=0) {
     WY_THROW( WyRet() );
   }
   if(tmp[1].sun_path[1]!=0) {
     WY_THROW( WyRet() );
   }
 }*/
};

// chk Wy_SockAddr interface
static void t_sockaddr(void)
{
 WySockAddrLocal a0;
 const Wy_SockAddr &saf=a0;
 if(saf.family()!=AF_LOCAL) {
   WY_THROW( WyRet() );
 }

 WyStr str;
 WyRet r;
 str="?";
 if((r=saf.notation(str))!=Ok) {
   WY_THROW(r);
 }
 if(str.size()!=0) {
   WY_THROW( WyRet() );
 }
};

static WyStr chdr("+---------------+\n"
                  "| main() caught:|\n"
                  "+---------------+\n");
int main(void) throw()
try {
 std::cout << "Checking WySockAddrLocal.h ...\n";
 std::cout << "MaxAddrLen=" << WySockAddrLocal::MaxAddrLen << std::endl;
 t1();
 t2();
 t_error();
 t_array();
 t_sockaddr();
 std::cout << "Checked Ok\n";
 return(0);
}
catch(const WyRet& e) {
 std::cerr << chdr.c_str() << Wy::wrd(e).c_str() << std::endl;
 return(-1);
}
catch(const std::exception& e) {
 std::cerr << chdr.c_str() << "std::exception" << std::endl;
 return(-1);
}
catch(...) {
 std::cerr << chdr.c_str() << "unknown unwind" << std::endl;
 return(-1);
};
