/*
  Build: g++ chk_strnum_float.cpp wy_strnum_float.cpp -pthread -lpthread -lwy

 todo:
  ._charnum(char),  +example
  
*/
#include "WyNum.h"  // for _strnum(..)
#include "WyStr.h"   // has to include latter because there is wynum.h included
#include <iostream>
#include <sstream>
#include <string>
#include <assert.h>
#include <stdlib.h>  // for rand()
#include <cmath>     // for std::ldexp(double,int)

struct ChkStrNum_float {
  typedef float NumT;
  const char* const nstr;       // test string
  const size_t nstr_len;        //

  // Expected results
  const WyReply res_reply;
  const NumT res_floor;
  const NumT res_ceil;
  const size_t res_plen;
  const Wy::WyRadix res_radix_i;
  const Wy::WyRadix res_radix_o;

  void verify(void) const
     try {
       WyRet r;
       NumT num;
       Wy::WyRadix radix=res_radix_i;
       WyCSeg cs;

       // _strlen(Num&, WyCSeg&, int&)
       radix=res_radix_i;
       if((r=cs.reset(nstr,nstr_len))!=Ok) {
         WY_THROW(r);
       }
       r=Wy::_strnum(num, cs, radix);
       
       if(r!=res_reply) {
         std::cerr << "'res_reply' inconsistent: " << Wy::wrd(r).c_str() << '\n';
         WY_THROW( WyRet() );
       }
       if(num<res_floor) {
         std::cerr << "num < res_floor: " << num << '\n';
         WY_THROW( WyRet() );
       } else if(num>res_ceil) {
         std::cerr << "num > res_ceil: " << num << '\n';
         WY_THROW( WyRet() );
       } else {};
       if(r==Ok) {
         if(cs.size()!=0) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EINVAL) {
         if(cs.size()!=nstr_len) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ENOENT) {
         if(cs.size()!=nstr_len) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EBADMSG) {
         if(nstr_len-cs.size()!=res_plen) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ERANGE) {
         if(nstr_len-cs.size()!=res_plen) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else {
         WY_THROW( WyRet() );
       }

       // _strlen(Num&, WyCSeg&, const int&)
       radix=res_radix_i;
       if((r=cs.reset(nstr,nstr_len))!=Ok) {
         WY_THROW(r);
       }
       r=Wy::_strnum(num, cs, (const int)radix);
       if(radix!=res_radix_i) {
         std::cerr << "'radix' overwritten :" << radix << '\n';
         WY_THROW( WyRet() );
       }
       
       if(r!=res_reply) {
         std::cerr << "'res_reply' inconsistent: " << Wy::wrd(r).c_str() << '\n';
         WY_THROW( WyRet() );
       }
       if(num<res_floor) {
         std::cerr << "num < res_floor: " << num << '\n';
         WY_THROW( WyRet() );
       } else if(num>res_ceil) {
         std::cerr << "num > res_ceil: " << num << '\n';
         WY_THROW( WyRet() );
       } else {};
       if(r==Ok) {
         if(cs.size()!=0) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EINVAL) {
         if(cs.size()!=nstr_len) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ENOENT) {
         if(cs.size()!=nstr_len) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EBADMSG) {
         if(nstr_len-cs.size()!=res_plen) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ERANGE) {
         if(nstr_len-cs.size()!=res_plen) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else {
         WY_THROW( WyRet() );
       }

       // _strlen(Num&, WyCSeg&)
       if(res_radix_i==10) {
         radix=res_radix_i;
         if((r=cs.reset(nstr,nstr_len))!=Ok) {
           WY_THROW(r);
         }
         r=Wy::_strnum(num, cs);
         
         if(radix!=res_radix_i) {
           std::cerr << "'radix' overwritten :" << radix << '\n';
           WY_THROW( WyRet() );
         }
         if(r!=res_reply) {
           std::cerr << "'res_reply' inconsistent: " << Wy::wrd(r).c_str() << '\n';
           WY_THROW( WyRet() );
         }
         if(num<res_floor) {
           std::cerr << "num < res_floor: " << num << '\n';
           WY_THROW( WyRet() );
         } else if(num>res_ceil) {
           std::cerr << "num > res_ceil: " << num << '\n';
           WY_THROW( WyRet() );
         } else {};
         if(r==Ok) {
           if(cs.size()!=0) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else if(r==Wym_EINVAL) {
           if(cs.size()!=nstr_len) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else if(r==Wym_ENOENT) {
           if(cs.size()!=nstr_len) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else if(r==Wym_EBADMSG) {
           if(nstr_len-cs.size()!=res_plen) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else if(r==Wym_ERANGE) {
           if(nstr_len-cs.size()!=res_plen) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else {
           WY_THROW( WyRet() );
         }
       }

       // _strlen(Num&, const WyCSeg&, int&)
       radix=res_radix_i;
       if((r=cs.reset(nstr,nstr_len))!=Ok) {
         WY_THROW(r);
       }
       r=Wy::_strnum(num, (const WyCSeg&)cs, radix);
       
       if(cs!=WyCSeg(nstr,nstr_len)) {
         std::cerr << "cs overwritten: " << WyStr(cs).c_str() << '\n';
         WY_THROW( WyRet() );
       }
       if(num<res_floor) {
         std::cerr << "num < res_floor: " << num << '\n';
         WY_THROW( WyRet() );
       } else if(num>res_ceil) {
         std::cerr << "num > res_ceil: " << num << '\n';
         WY_THROW( WyRet() );
       } else {};
       if(r==Ok) {
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EINVAL) {
       } else if(r==Wym_ENOENT) {
       } else if(r==Wym_EBADMSG) {
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ERANGE) {
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else {
         WY_THROW( WyRet() );
       }

       // _strlen(Num&, const WyCSeg&, const int&)
       radix=res_radix_i;
       if((r=cs.reset(nstr,nstr_len))!=Ok) {
         WY_THROW(r);
       }
       r=Wy::_strnum(num, (const WyCSeg&)cs, (const int&)radix);
       
       if(cs!=WyCSeg(nstr,nstr_len)) {
         std::cerr << "cs overwritten: " << WyStr(cs).c_str() << '\n';
         WY_THROW( WyRet() );
       }
       if(radix!=res_radix_i) {
         std::cerr << "'radix' overwritten :" << radix << '\n';
         WY_THROW( WyRet() );
       }
       if(num<res_floor) {
         std::cerr << "num < res_floor: " << num << '\n';
         WY_THROW( WyRet() );
       } else if(num>res_ceil) {
         std::cerr << "num > res_ceil: " << num << '\n';
         WY_THROW( WyRet() );
       } else {};
       if(r==Ok) {
       } else if(r==Wym_EINVAL) {
       } else if(r==Wym_ENOENT) {
       } else if(r==Wym_EBADMSG) {
       } else if(r==Wym_ERANGE) {
       } else {
         WY_THROW( WyRet() );
       }

       // _strlen(Num&, const WyCSeg&)
       if(res_radix_i==10) {
         radix=res_radix_i;
         if((r=cs.reset(nstr,nstr_len))!=Ok) {
           WY_THROW(r);
         }
         r=Wy::_strnum(num, (const WyCSeg&)cs);
         
         if(cs!=WyCSeg(nstr,nstr_len)) {
           std::cerr << "cs overwritten: " << WyStr(cs).c_str() << '\n';
           WY_THROW( WyRet() );
         }
         if(num<res_floor) {
           std::cerr << "num < res_floor: " << num << '\n';
           WY_THROW( WyRet() );
         } else if(num>res_ceil) {
           std::cerr << "num > res_ceil: " << num << '\n';
           WY_THROW( WyRet() );
         } else {};
         if(r==Ok) {
         } else if(r==Wym_EINVAL) {
         } else if(r==Wym_ENOENT) {
         } else if(r==Wym_EBADMSG) {
         } else if(r==Wym_ERANGE) {
         } else {
           WY_THROW( WyRet() );
         }
       }

     }
     catch(...) {
       std::cerr << "ChkStrNum\n";
       std::cerr << "Failed on {\"" << ((nstr==NULL)? "NULL" : nstr) << "\"," << nstr_len
             << ',' << Wy::wrd(res_reply).c_str()
             << ',' << res_floor << ',' << res_ceil << ',' << res_plen 
             << ',' << res_radix_i << ',' << res_radix_o << "}\n";
       throw;
     };
};

// NumT= float or double
template <typename NumT> 
inline NumT Floor(NumT n) { return n; };

template <typename NumT> 
inline NumT Ceil(NumT n) { return n; };

static void ck_strnum_float(void)
{
 typedef float NumT;
 const ChkStrNum_float chk_tab[]={
   // nstr slen  reply val plen fdig  rad_i rad_o
   {"0",0 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"0",0 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},

   // one char "?"
   {"0",1 ,        Ok,Floor(0.0),Ceil(0.0),1, 0,10},
   {"1",1 ,        Ok,Floor(1.0),Ceil(1.0),1, 0,10},
   {"7",1 ,        Ok,Floor(7.0),Ceil(7.0),1, 0,10},
   {"8",1 ,        Ok,Floor(8.0),Ceil(8.0),1, 0,10},
   {"9",1 ,        Ok,Floor(9.0),Ceil(9.0),1, 0,10},
   {"a",1 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"e",1 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"g",1 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"X",1 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"+",1 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"-",1 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {".",1 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},

   // two char's "+?"
   {"+0",2 ,        Ok,Floor(0.0),Ceil(0.0),2, 0,10},
   {"+1",2 ,        Ok,Floor(1.0),Ceil(1.0),2, 0,10},
   {"+7",2 ,        Ok,Floor(7.0),Ceil(7.0),2, 0,10},
   {"+8",2 ,        Ok,Floor(8.0),Ceil(8.0),2, 0,10},
   {"+9",2 ,        Ok,Floor(9.0),Ceil(9.0),2, 0,10},
   {"+a",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"+e",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"+g",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"+X",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"++",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"+-",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"+.",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},

   // two char's "-?"
   {"-0",2 ,        Ok,Floor(-0.0),Ceil(-0.0),2, 0,10},
   {"-1",2 ,        Ok,Floor(-1.0),Ceil(-1.0),2, 0,10},
   {"-7",2 ,        Ok,Floor(-7.0),Ceil(-7.0),2, 0,10},
   {"-8",2 ,        Ok,Floor(-8.0),Ceil(-8.0),2, 0,10},
   {"-9",2 ,        Ok,Floor(-9.0),Ceil(-9.0),2, 0,10},
   {"-a",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"-e",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"-g",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"-X",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"-+",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"-.",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},

   // two char's ".?"
   {".0",2 ,        Ok,Floor(0.0),Ceil(0.0),2, 0,10},
   {".1",2 ,        Ok,Floor(0.1),Ceil(0.1),2, 0,10},
   {".7",2 ,        Ok,Floor(0.7),Ceil(0.7),2, 0,10},
   {".8",2 ,        Ok,Floor(0.8),Ceil(0.8),2, 0,10},
   {".9",2 ,        Ok,Floor(0.89999),Ceil(0.90001),2, 0,10},
   {".a",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {".e",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {".g",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {".X",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {".+",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {".-",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"..",2 ,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},

   // two char's "0?"
   {"00",2 ,         Ok,Floor(0),Ceil(0),2, 0,10},
   {"01",2 ,         Ok,Floor(1),Ceil(1),2, 0,10},
   {"07",2 ,         Ok,Floor(7),Ceil(7),2, 0,10},
   {"08",2 ,         Ok,Floor(8),Ceil(8),2, 0,10},
   {"09",2 ,         Ok,Floor(9),Ceil(9),2, 0,10},
   {"0a",2 ,Wym_EBADMSG,Floor(0.0),Ceil(0.0),1, 0,10},
   {"0e",2 ,Wym_EBADMSG,Floor(0.0),Ceil(0.0),1, 0,10},
   {"0g",2 ,Wym_EBADMSG,Floor(0.0),Ceil(0.0),1, 0,10},
   {"0X",2 ,Wym_EBADMSG,Floor(0.0),Ceil(0.0),1, 0,10},
   {"0+",2 ,Wym_EBADMSG,Floor(0.0),Ceil(0.0),1, 0,10},
   {"0-",2 ,Wym_EBADMSG,Floor(0.0),Ceil(0.0),1, 0,10},
   {"0.",2 ,         Ok,Floor(0.0),Ceil(0.0),2, 0,10},

   // two char's "1?"
   {"10",2 ,         Ok,Floor(10.0),Ceil(10.0),2, 0,10},
   {"11",2 ,         Ok,Floor(11.0),Ceil(11.0),2, 0,10},
   {"17",2 ,         Ok,Floor(17.0),Ceil(17.0),2, 0,10},
   {"18",2 ,         Ok,Floor(18.0),Ceil(18.0),2, 0,10},
   {"19",2 ,         Ok,Floor(19.0),Ceil(19.0),2, 0,10},
   {"1a",2 ,Wym_EBADMSG,Floor(1.0),Ceil(1.0),1, 0,10},
   {"1e",2 ,Wym_EBADMSG,Floor(1.0),Ceil(1.0),1, 0,10},
   {"1g",2 ,Wym_EBADMSG,Floor(1.0),Ceil(1.0),1, 0,10},
   {"1X",2 ,Wym_EBADMSG,Floor(1.0),Ceil(1.0),1, 0,10},
   {"1+",2 ,Wym_EBADMSG,Floor(1.0),Ceil(1.0),1, 0,10},
   {"1-",2 ,Wym_EBADMSG,Floor(1.0),Ceil(1.0),1, 0,10},
   {"1.",2 ,         Ok,Floor(1.0),Ceil(1.0),2, 0,10},

   // ".0?"
   {".00",3 ,         Ok,Floor(0.00),Ceil(0.00),3, 0,10},
   {".01",3 ,         Ok,Floor(0.01),Ceil(0.01),3, 0,10},
   {".07",3 ,         Ok,Floor(0.07),Ceil(0.07),3, 0,10},
   {".08",3 ,         Ok,Floor(0.08),Ceil(0.08),3, 0,10},
   {".09",3 ,         Ok,Floor(0.089999),Ceil(0.090001),3, 0,10},
   {".0a",3 ,Wym_EBADMSG,Floor(0.0),Ceil(0.0),2, 0,10},
   {".0e",3 ,Wym_EBADMSG,Floor(0.0),Ceil(0.0),2, 0,10},
   {".0g",3 ,Wym_EBADMSG,Floor(0.0),Ceil(0.0),2, 0,10},
   {".0X",3 ,Wym_EBADMSG,Floor(0.0),Ceil(0.0),2, 0,10},
   {".0+",3 ,Wym_EBADMSG,Floor(0.0),Ceil(0.0),2, 0,10},
   {".0-",3 ,Wym_EBADMSG,Floor(0.0),Ceil(0.0),2, 0,10},
   {".0.",3 ,Wym_EBADMSG,Floor(0.0),Ceil(0.0),2, 0,10},

   // ".7?"
   {".70",3 ,         Ok,Floor(0.70),Ceil(0.70),3, 0,10},
   {".71",3 ,         Ok,Floor(0.71),Ceil(0.71),3, 0,10},
   {".77",3 ,         Ok,Floor(0.77),Ceil(0.77),3, 0,10},
   {".78",3 ,         Ok,Floor(0.78),Ceil(0.78),3, 0,10},
   {".79",3 ,         Ok,Floor(0.78999),Ceil(0.79001),3, 0,10},
   {".7a",3 ,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},
   {".7e",3 ,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},
   {".7g",3 ,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},
   {".7X",3 ,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},
   {".7+",3 ,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},
   {".7-",3 ,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},
   {".7.",3 ,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},

    // ".7?e"
   {".70e",4,Wym_EBADMSG,Floor(0.70),Ceil(0.70),3, 0,10},
   {".71e",4,Wym_EBADMSG,Floor(0.71),Ceil(0.71),3, 0,10},
   {".77e",4 ,Wym_EBADMSG,Floor(0.77),Ceil(0.77),3, 0,10},
   {".78e",4 ,Wym_EBADMSG,Floor(0.78),Ceil(0.78),3, 0,10},
   {".79e",4 ,Wym_EBADMSG,Floor(0.78999),Ceil(0.79001),3, 0,10},
   {".7ae",4 ,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},
   {".7ee",4 ,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},
   {".7ge",4 ,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},
   {".7Xe",4 ,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},
   {".7+e",4,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},
   {".7-e",4 ,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},
   {".7.e",4 ,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},

    // ".7?e5"
   {".70e5",5,Ok,Floor(0.70e5),Ceil(0.70e5),5, 0,10},
   {".71e5",5,Ok,Floor(0.71e5),Ceil(0.71e5),5, 0,10},
   {".77e5",5 ,Ok,Floor(0.77e5),Ceil(0.77e5),5, 0,10},
   {".78e5",5 ,Ok,Floor(0.78e5),Ceil(0.78e5),5, 0,10},
   {".79e5",5 ,Ok,Floor(0.79e5),Ceil(0.79e5),5, 0,10},
   {".7ae5",5 ,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},
   {".7ee5",5 ,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},
   {".7ge5",5 ,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},
   {".7Xe5",5 ,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},
   {".7+e5",5,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},
   {".7-e5",5,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10},
   {".7.e5",5,Wym_EBADMSG,Floor(0.7),Ceil(0.7),2, 0,10}, 

   // "0x?"
   {"0x0",3 ,         Ok,Floor(0.0),Ceil(0.0),3, 0,16},
   {"0x1",3 ,         Ok,Floor(1.0),Ceil(1.0),3,16,16},
   {"0x7",3 ,         Ok,Floor(7.0),Ceil(7.0),3, 0,16},
   {"0x8",3 ,         Ok,Floor(8.0),Ceil(8.0),3,16,16},
   {"0x9",3 ,         Ok,Floor(9.0),Ceil(9.0),3, 0,16},
   {"0xa",3 ,         Ok,Floor(10.0),Ceil(10.0),3, 0,16},
   {"0xe",3 ,Wym_EBADMSG,Floor(0.0),Ceil(0.0),1,10,10},
   {"0xg",3 ,Wym_EBADMSG,Floor(0.0),Ceil(0.0),1, 0,10},
   {"0xX",3 ,Wym_EBADMSG,Floor(0.0),Ceil(0.0),1, 0,10},
   {"0x+",3 ,Wym_EBADMSG,Floor(0.0),Ceil(0.0),1, 0,10},
   {"0x-",3 ,Wym_EBADMSG,Floor(0.0),Ceil(0.0),1, 0,10},
   {"0x.",3 ,Wym_EBADMSG,Floor(0.0),Ceil(0.0),1, 0,10},

   // mis
   {"+0x",3,Wym_EBADMSG,Floor(0.0),Ceil(0.0),2, 0,10}, 
   {"-0x",3,Wym_EBADMSG,Floor(0.0),Ceil(0.0),2, 0,10},
   {"0x.0.2",6,Wym_EBADMSG,Floor(0.0),Ceil(0.0),4, 0,16},
   {"0x.E0",5,         Ok,Floor(14.0/16),Ceil(14.0/16),5, 0,16},
   {"0x.f0",5,         Ok,Floor(15.0/16),Ceil(15.0/16),5, 0,16},
   {"-0x.ea",6,        Ok,Floor(-234.0/256),Ceil(-234.0/256),6, 16,16},
   {"-.E0",6,Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"9999999",7,       Ok,Floor(9999990.0),Ceil(9999999.9),7, 0,10},
   {"-09999999",9,     Ok,Floor(-9999999.9),Ceil(-9999990.0),9, 0,10},
   {"999.9999e4",10,   Ok,Floor(999.9990e4),Ceil(999.99999e4),10, 0,10},
   {"-0999.9999e4",12, Ok,Floor(-999.99999e4),Ceil(-999.99900e4),12, 0,10},

   {"0.0.2", 5,Wym_EBADMSG,Floor(0.0),Ceil(0.0),3, 0,10}, 

   {"0x.2.2",6,Wym_EBADMSG,Floor(2.0/16),Ceil(2.0/16),4, 0,16}, 
   {"0.1.",  4,Wym_EBADMSG,Floor(0.1),Ceil(0.1),3, 0,10},
   {"0.1.2", 5,Wym_EBADMSG,Floor(0.1),Ceil(0.1),3, 0,10},
   {"0.1.",  4,Wym_EBADMSG,Floor(0.1),Ceil(0.1),3, 0,10},
   {"0.1.2", 5,Wym_EBADMSG,Floor(0.1),Ceil(0.1),3, 0,10},

   {"-.", 2,   Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"+.9",3,           Ok,Floor(0.89999),Ceil(0.90001),3, 0,10},
   {"-.9",3,           Ok,Floor(-0.90001),Ceil(-0.89999),3, 0,10},
   {"+0", 2,           Ok,Floor(0.0),Ceil(0.0),2, 0,10},
   {"-0", 2,           Ok,Floor(0.0),Ceil(0.0),2, 0,10}, 
   {"0x.",3,  Wym_EBADMSG,Floor(0.0),Ceil(0.0),1, 0,10},
   {"0x.g",4, Wym_EBADMSG,Floor(0.0),Ceil(0.0),1, 0,10},
   {"0x.a",4, Wym_EBADMSG,Floor(0.0),Ceil(0.0),1, 10,10},
   {"0x.a",4,          Ok,Floor(10.0/16),Ceil(10.0/16),4, 0,16},
   {".0",  2,          Ok,Floor(0.0),Ceil(0.0),2, 0,10},
   {"+9.", 3,          Ok,Floor(9.0),Ceil(9.0),3, 0,10},
   {"-9.", 3,          Ok,Floor(-9.0),Ceil(-9.0),3, 0,10},
   {"+.x", 3,  Wym_ENOENT,Floor(0.0),Ceil(0.0),0, 0,10},
   {"-0.1a",5,Wym_EBADMSG,Floor(-0.1),Ceil(-0.1),4, 0,10},
   {"-.1a", 4,Wym_EBADMSG,Floor(-0.1),Ceil(-0.1),3, 0,10},

   {"31",2,              Ok,Floor(31.0),Ceil(31.0),2, 0,10},
   {"031",3,             Ok,Floor(31.0),Ceil(31.0),3, 0,10},
   {"00.03",5,           Ok,Floor(0.03),Ceil(0.03),5, 0,10},
   {"0x00.03",7,         Ok,Floor(3.0/256),Ceil(3.0/256),7, 0,16},
   {"0x00.03",7,         Ok,Floor(3.0/256),Ceil(3.0/256),7,16,16},
   {"0x00.03",7,Wym_EBADMSG,Floor(0.0),Ceil(0.0),1,10,10},
   {"00x0",4,   Wym_EBADMSG,Floor(0.0),Ceil(0.0),2, 0,10},
   {"1.2p10",6, Ok,Floor(0x1.2p10),Ceil(0x1.2p10),6,16,16}, 
   {"-1.2p1a",7, Wym_EBADMSG,Floor(-0x1.2p1),Ceil(-0x1.2p1),6,16,16},

   {"+.0",    3,Ok,Floor(0.0),    Ceil(0.0),3, 0,10}, 
   {"-.0",    3,Ok,Floor(0.0),    Ceil(0.0),3, 0,10},
   {".00",    3,Ok,Floor(0.0),    Ceil(0.0),3, 0,10},
   {"+.00",   4,Ok,Floor(0.0),    Ceil(0.0),4, 0,10},
   {"-.00",   4,Ok,Floor(0.0),    Ceil(0.0),4, 0,10},
   {"0.0",    3,Ok,Floor(0.0),    Ceil(0.0),3, 0,10},
   {"+0.0",   4,Ok,Floor(0.0),    Ceil(0.0),4, 0,10},
   {"-0.0",   4,Ok,Floor(0.0),    Ceil(0.0),4, 0,10},
   {"0.010",  5,Ok,Floor(0.0099999),   Ceil(0.010001),5, 0,10},
   {"-0.010", 6,Ok,Floor(-0.010001),  Ceil(-0.0099999),6, 0,10},
   {"123",    3,Ok,Floor(123.0),  Ceil(123.0),3, 0,10},
   {"+123",   4,Ok,Floor(123.0),  Ceil(123.0),4, 0,10},
   {"+0123",  5,Ok,Floor(0123.0), Ceil(0123.0),5, 0,10},
   {"-0123",  5,Ok,Floor(-0123.0),Ceil(-0123.0),5, 0,10},
   {".123",   4,Ok,Floor(0.123),  Ceil(0.123),4, 0,10},
   {"+.123",  5,Ok,Floor(0.123),  Ceil(0.123),5, 0,10},
   {"-.123",  5,Ok,Floor(-0.123), Ceil(-0.123),5, 0,10},
   {".120",   4,Ok,Floor(0.12),   Ceil(0.12001),4, 0,10},
   {"+.120",  5,Ok,Floor(0.12),   Ceil(0.12001),5, 0,10},
   {"-.120",  5,Ok,Floor(-0.12001),  Ceil(-0.11999),5, 0,10},
   {"12.0",   4,Ok,Floor(12.0),   Ceil(12.0),4, 0,10},
   {"+12.0",  5,Ok,Floor(12.0),   Ceil(12.0),5, 0,10},
   {"-12.0",  5,Ok,Floor(-12.0),  Ceil(-12.0),5, 0,10},
   {"-012.3", 6,Ok,Floor(-12.3),  Ceil(-12.3),6, 0,10},
   {"+0012.3",7,Ok,Floor(12.3),   Ceil(12.3),7, 0,10}, 
   {"1111111.11111",13,Ok,Floor(1111111.11111),Ceil(1111111.11111),13, 0,10},
   {"-1111111.11111",14,Ok,Floor(-1111111.1111111),Ceil(-1111111.1111111),14, 0,10}, 
   {"177.177",7,Ok,Floor(177.177),Ceil(177.177),7, 0,10}, 
   {"0177.177",8,Ok,Floor(0177.177),Ceil(0177.177),8, 0,10}, 
   {"-177.177",8,Ok,Floor(-177.177),Ceil(-177.177),8, 0,10}, 
   {"-7f.7f",6,Wym_EBADMSG,Floor(-7.0),Ceil(-7.0),2, 0,10}, 
   {"-0x7f.7f",8,         Ok,Floor(-127.0-127.0/256),Ceil(-127.0-127.0/256),8, 0,16}, 
   {"+0x7f.7f",8,         Ok,Floor(127.0+127.0/256),Ceil(127.0+127.0/256),8,16,16}, 
   {"0x7f.7f",7,         Ok,Floor(127.0+127.0/256),Ceil(127.0+127.0/256),7,16,16}, 
   {"-0x7f.7f",8,Wym_EBADMSG,Floor(0.0),Ceil(0.0),2,10,10}, 
   {"+000000000000000000012.100",26,Ok,Floor(12.1),Ceil(12.1),26, 0,10},
   {"123.321E-3",10,Ok,Floor(123.321e-3),Ceil(123.321e-3),10, 0,10},
   {"123.321E+3",10,Ok,Floor(123.321e+3),Ceil(123.321e+3),10, 0,10},
   {"456.789E12",10,Ok,Floor(456.789e12),Ceil(456.789e12),10, 0,10},
   {".0E0",4,Ok,Floor(.0e0),Ceil(.0e0),4, 0,10}, 
   {".3E3",4,Ok,Floor(.3e3),Ceil(.3e3),4, 0,10},
   {".3E 3",5,Wym_EBADMSG,Floor(.3),Ceil(.3),2, 0,10},
   {".3 E3",5,Wym_EBADMSG,Floor(.3),Ceil(.3),2, 0,10},
   {".3E+ 3",6,Wym_EBADMSG,Floor(.3),Ceil(.3),2, 0,10},
   {".3E3.",5,Wym_EBADMSG,Floor(.3e3),Ceil(.3e3),4, 0,10},
   {".3E3.0",6,Wym_EBADMSG,Floor(.3e3),Ceil(.3e3),4, 0,10},
   {".1.2.3", 6,Wym_EBADMSG,Floor(.1),  Ceil(.1),2, 0,10},
   {".0.1.2", 6,Wym_EBADMSG,Floor(0),  Ceil(0),2, 0,10},
   {"0.1.2",5,Wym_EBADMSG,Floor(.1),  Ceil(.1),3, 0,10}, 
   {"1.2.3",5,Wym_EBADMSG,Floor(1.2),  Ceil(1.2),3, 0,10},
   {"0x.1.2",6,Wym_EBADMSG,Floor(0.0),  Ceil(0.0),1,10,10},
   {"0x.1.2",6,Wym_EBADMSG,Floor(1.0/16),  Ceil(1.0/16),4, 0,16},
   {"0x0.1.2",7,Wym_EBADMSG,Floor(1.0/16),  Ceil(1.0/16),5,16,16},

   {"+.3E+3",6,Ok,Floor(+.3e+3),Ceil(+.3e+3),6, 0,10},
   {"-.3e-3",6,Ok,Floor(-.30001e-3),Ceil(-.29999e-3),6, 0,10},
   {"-.3E-3",6,Ok,Floor(-.30001e-3),Ceil(-.29999e-3),6, 0,10}, 
   {"-.3E-03",7,         Ok,Floor(-.3001e-3),Ceil(-.2999e-3),7, 0,10},
   {"-.3E-03",7,Wym_EBADMSG,Floor(-62.0/256),Ceil(-62.0/256),4,16,16},
   {"-.3E-0x3",8,Wym_EBADMSG,Floor(-.3),Ceil(-.3),6, 0,10},
   {"9.9463072e-39",13, Ok,Floor(9.94630e-39),Ceil(9.94631e-39),13, 0,10},
   {".0000000000435678e-36",21, Ok,Floor(0.0),Ceil(4.36e-47),21, 0,10},
   {".0000435678e+41",15, Ok,Floor(4.35677e36),Ceil(4.35679e36),15, 0,10},
   {".0000435678e+49",15, Wym_ERANGE,Floor(0.435677),Ceil(0.435679),14, 0,10},
   {"-.0000000000435678e-36",22, Ok,Floor(-4.35e-47),Ceil(0.0),22,10,10},
   {"-.0000435678e+41",16, Ok,Floor(-4.35679e36),Ceil(-4.35677e36),16, 0,10},
   {"-.0000435678e+49",16, Wym_ERANGE,Floor(-0.435679),Ceil(-0.435677),15, 0,10},

   {"7654321e-90", 11, Ok, Floor(7654320e-90), Ceil(7654329e-90),11,10,10},
   {"7654321e36", 10, Wym_ERANGE, Floor(7654320e3), Ceil(7654329e3),9,10,10},
   {"-7654321e36", 11, Wym_ERANGE, Floor(-7654330e3), Ceil(-7654320e3),10,10,10}, 
   {"001.0000000001234567890", 23,Ok, Floor(.999999), Ceil(1.00001),23,10,10}, 
   {"00000000000000000000000000000000000000000123.4560000000000000000000000000000000000000000e2",
     90,Ok, Floor(12345.59), Ceil(12345.61),90, 0,10},
   {".000000000000000000000000000000123456789000000000000000000000000000000000000000",
     79,Ok, Floor(1.2345599e-31), Ceil(1.23457e-31),79,10,10}, 
   {".000000000000000000000000000000000000000123456789000000000000000000000000000000",
     79,Ok, Floor(0), Ceil(1.235e-40),79,10,10}, 
   {"-.000000000000000000000000000000123456789000000000000000000000000000000000000000e-5",
     83,Ok, Floor(-1.24e-36), Ceil(-1.2e-36),83,10,10}, 
   {"-.000000000000000000000000000000123456789000000000000000000000000000000000000000e31",
     83,Ok, Floor(-1.24), Ceil(-1.20),83,10,10}, 
   {"0x1fp3",6,  Ok,Floor(0x1fp3),Ceil(0x1fp3),6,0,16},
   // implement feature: conversion limited to 38 digits of radix 10
   {"12345678901234567890123456789012345678901234.5678900000000000000000000000000000000000000e-8",
     91, Wym_ERANGE, Floor(1.23456e37),Ceil(1.23457e37),38, 0,10},
   {"123.123123123123123123123123123123123E3",39,Ok,Floor(123.123e3),Ceil(123.124e3),39, 0,10},
   {"-.3E-0x3",8,Wym_EBADMSG,Floor(-.3),Ceil(-.3),6, 0,10},

   {"0x74e6p5", 8, Ok, Floor(0x74e5fp1), Ceil(0x74e61p1),8,0,16},
   {"-0x74e6p5", 9, Ok, Floor(-0x74e61p1), Ceil(-0x74e5fp1),9,16,16},
   {"74e6p5", 6, Ok, Floor(0x74e5fp1), Ceil(0x74e61p1),6,16,16},
 }; 

 for(size_t i=0; i<sizeof(chk_tab)/sizeof(ChkStrNum_float); ++i) {
   chk_tab[i].verify();
 }

 try {
   std::ostringstream os;
   const NumT NumMax=9.9999e37;
   const NumT NumMin=-9.9999e37;
   std::string StrMax,StrMin;

   assert(NumMax==-NumMin);   // assert representable
   os.precision(7);
   os.str("");
   os << std::dec << NumMax;
   StrMax=os.str();

   os.precision(7);
   os.str("");
   os << std::dec << NumMin;
   StrMin=os.str(); 

   try { // chk <max10>
     std::string nstr(StrMax);
     ChkStrNum_float cksmp1=
        {nstr.c_str(),nstr.size(),Ok,NumMax*0.9999,NumMax,nstr.size(),10,10};
     cksmp1.verify();

     ChkStrNum_float cksmp2=
        {nstr.c_str(),nstr.size(),Ok,NumMax*0.9999,NumMax,nstr.size(), 0,10};
     cksmp2.verify();
   }
   catch(...) {
     std::cerr << "verify(): chk <max10>\n";
     throw;
   };

   try { // chk <min10>
     std::string nstr(StrMin);
     ChkStrNum_float cksmp1=
        {nstr.c_str(),nstr.size(),Ok,NumMin,NumMin*.9999,nstr.size(),10,10};
     cksmp1.verify();

     ChkStrNum_float cksmp2=
        {nstr.c_str(),nstr.size(),Ok,NumMin,NumMin*.9999,nstr.size(), 0,10};
     cksmp2.verify();
   }
   catch(...) {
     std::cerr << "verify(): chk <min>";
     throw;
   };
 }
 catch(...) {
   std::cerr << " of radix 10\n";
   throw;
 };

 try {
   std::ostringstream os;
   const NumT NumMax= 0x.1p128;
   const NumT NumMin=-0x.1p128;
   std::string StrMax1=".1p128";
   std::string StrMax2="0x.1p128";
   std::string StrMin1="-.1p128";
   std::string StrMin2="-0x.1p128";

   assert(NumMax==-NumMin);   // assert representable

   try { // chk <max16>
     std::string nstr(StrMax1);
     ChkStrNum_float cksmp1=
        {nstr.c_str(),nstr.size(),Ok,NumMax*0.9999,NumMax,nstr.size(),16,16};
     cksmp1.verify();

     nstr=StrMax2;
     ChkStrNum_float cksmp2=
        {nstr.c_str(),nstr.size(),Ok,NumMax*0.9999,NumMax,nstr.size(), 0,16};
     cksmp2.verify();
   }
   catch(...) {
     std::cerr << "verify(): chk <max>";
     throw;
   };

   try { // chk <min16>
     std::string nstr(StrMin1);
     ChkStrNum_float cksmp1=
        {nstr.c_str(),nstr.size(),Ok,NumMin,NumMin*.9999,nstr.size(),16,16};
     cksmp1.verify();

     nstr=StrMin2;
     ChkStrNum_float cksmp2=
        {nstr.c_str(),nstr.size(),Ok,NumMin,NumMin*.9999,nstr.size(), 0,16};
     cksmp2.verify();
   }
   catch(...) {
     std::cerr << "verify(): chk <min>";
     throw;
   };
 }
 catch(...) {
   std::cerr << " of radix 16\n";
   throw;
 };
};

void ck_strnum_rand(void)
{
 typedef float ResType;
 const int PreDigs=9;          
 const long long RandExp=127;

 std::ostringstream os;

 for(int i=0; i<10000; ++i) {
   //  res= x*2^y  ,x in [-1.0,+1.0], y in [-RandExp, +RandExp]
   //
   const ResType Res= std::ldexp( ::rand()*(2.0/(double)RAND_MAX)-1.0
              , (::rand()*(long long)2*RandExp)/(long long)RAND_MAX-RandExp 
             );

   os.precision(PreDigs);
   os.str("");
   os << Res;
   const std::string NStr( os.str() );

   ResType num1,num2;

   // Set num1 from strtof(..)
   {
    char* eptr;
    num1=::strtof(NStr.c_str(),&eptr);
    if(eptr!=NStr.c_str()+NStr.size()) {
      WY_THROW( WyRet() );
    }
   }

   // Set num2 from Wy::_strnum(..)
   {
    WyRet r;
    int radix=10;
    WyCSeg cs(NStr.c_str(), NStr.size());
    r=Wy::_strnum(num2, cs, radix);
    if(cs.size()!=0) {
      WY_THROW(r);
    }
   }

   // Dump instances of larger erronous ratio
   {
     double ErrRat;
     if(num2!=0) {
       ErrRat=(double)num1/num2;
     } else if(num1!=0) {
       ErrRat=(double)num2/num1;
     } else {
       ErrRat=1.0;
     }
     if((ErrRat>1.000001)||(ErrRat<0.999999)) {  // 1/million precision
       std::cout << NStr;
       std::cout.precision(PreDigs);
       std::cout << "\n" << num1 << '\n' << num2 << '\n';
       std::cout << std::endl;
     };
   }
 }
};

int main(void)
try {
 std::cout << "Checking _strnum(..) for float ";
  ck_strnum_float();
  ck_strnum_rand();
  std::cout << "Ok\n";
}
catch(const WyRet& e) {
 std::cerr << Wy::wrd(e).c_str() << '\n';
 return(-1);
}
catch(...) {
 std::cerr << "unknown throw\n";
 throw;
};

