/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2007

  Check _strnum(..) of built-in integral types

  Build: g++ chk_strnum_integers.cpp wy_strnum_char.cpp wy_strnum_short.cpp wy_strnum_int.cpp
             wy_strnum_long.cpp wy_strnum_llong.cpp -pthread -lpthread -lwy
*/
#include "WyNum.h"
#include "WyStr.h"
#include <iostream>
#include <sstream>
#include <string>
#include <limits>

// Check Item for signed integral types
//  (only the radix modifiable version is checked)
//
template <typename NumT>
struct ChkItem_Signed {
  const char* const nstr;       // test string
  const size_t nstr_len;        //

  // Expected results
  const WyReply res_reply;
  const NumT res_val;
  const size_t res_plen;
  const Wy::WyRadix res_radix_i;
  const Wy::WyRadix res_radix_o;

  void verify(void) const
     try {
       WyRet r;
       NumT num;
       Wy::WyRadix radix=res_radix_i;
       WyCSeg cs;

       // _strlen(Num&, WyCSeg&, int&)
       radix=res_radix_i;
       if((r=cs.reset(nstr,nstr_len))!=Ok) {
         WY_THROW(r);
       }
       r=Wy::_strnum(num, cs, radix);
       
       if(r!=res_reply) {
         std::cerr << "'res_reply' inconsistent: " << Wy::wrd(r).c_str() << '\n';
         WY_THROW( WyRet() );
       }
       if(num!=res_val) {
         std::cerr << "'res_val' inconsistent: " << (long long)num << '\n';
         WY_THROW( WyRet() );
       }
       if(r==Ok) {
         if(cs.size()!=0) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EINVAL) {
         if(cs.size()!=nstr_len) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ENOENT) {
         if(cs.size()!=nstr_len) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EBADMSG) {
         if(nstr_len-cs.size()!=res_plen) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ERANGE) {
         if(nstr_len-cs.size()!=res_plen) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else {
         WY_THROW( WyRet() );
       }

       // _strlen(Num&, WyCSeg&, const int&)
       radix=res_radix_i;
       if((r=cs.reset(nstr,nstr_len))!=Ok) {
         WY_THROW(r);
       }
       r=Wy::_strnum(num, cs, (const int)radix);
       if(radix!=res_radix_i) {
         std::cerr << "'radix' overwritten :" << radix << '\n';
         WY_THROW( WyRet() );
       }
       
       if(r!=res_reply) {
         std::cerr << "'res_reply' inconsistent: " << Wy::wrd(r).c_str() << '\n';
         WY_THROW( WyRet() );
       }
       if(num!=res_val) {
         std::cerr << "'res_val' inconsistent: " << (long long)num << '\n';
         WY_THROW( WyRet() );
       }
       if(r==Ok) {
         if(cs.size()!=0) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EINVAL) {
         if(cs.size()!=nstr_len) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ENOENT) {
         if(cs.size()!=nstr_len) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EBADMSG) {
         if(nstr_len-cs.size()!=res_plen) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ERANGE) {
         if(nstr_len-cs.size()!=res_plen) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else {
         WY_THROW( WyRet() );
       }

       // _strlen(Num&, WyCSeg&)
       if(res_radix_i==10) {
         radix=res_radix_i;
         if((r=cs.reset(nstr,nstr_len))!=Ok) {
           WY_THROW(r);
         }
         r=Wy::_strnum(num, cs);
         
         if(radix!=res_radix_i) {
           std::cerr << "'radix' overwritten :" << radix << '\n';
           WY_THROW( WyRet() );
         }
         if(r!=res_reply) {
           std::cerr << "'res_reply' inconsistent: " << Wy::wrd(r).c_str() << '\n';
           WY_THROW( WyRet() );
         }
         if(num!=res_val) {
           std::cerr << "'res_val' inconsistent: " << (long long)num << '\n';
           WY_THROW( WyRet() );
         }
         if(r==Ok) {
           if(cs.size()!=0) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else if(r==Wym_EINVAL) {
           if(cs.size()!=nstr_len) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else if(r==Wym_ENOENT) {
           if(cs.size()!=nstr_len) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else if(r==Wym_EBADMSG) {
           if(nstr_len-cs.size()!=res_plen) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else if(r==Wym_ERANGE) {
           if(nstr_len-cs.size()!=res_plen) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else {
           WY_THROW( WyRet() );
         }
       }

       // _strlen(Num&, const WyCSeg&, int&)
       radix=res_radix_i;
       if((r=cs.reset(nstr,nstr_len))!=Ok) {
         WY_THROW(r);
       }
       r=Wy::_strnum(num, (const WyCSeg&)cs, radix);
       
       if(cs!=WyCSeg(nstr,nstr_len)) {
         std::cerr << "cs overwritten: " << WyStr(cs).c_str() << '\n';
         WY_THROW( WyRet() );
       }
       if(num!=res_val) {
         std::cerr << "'res_val' inconsistent: " << (long long)num << '\n';
         WY_THROW( WyRet() );
       }
       if(r==Ok) {
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EINVAL) {
       } else if(r==Wym_ENOENT) {
       } else if(r==Wym_EBADMSG) {
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ERANGE) {
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else {
         WY_THROW( WyRet() );
       }

       // _strlen(Num&, const WyCSeg&, const int&)
       radix=res_radix_i;
       if((r=cs.reset(nstr,nstr_len))!=Ok) {
         WY_THROW(r);
       }
       r=Wy::_strnum(num, (const WyCSeg&)cs, (const int&)radix);
       
       if(cs!=WyCSeg(nstr,nstr_len)) {
         std::cerr << "cs overwritten: " << WyStr(cs).c_str() << '\n';
         WY_THROW( WyRet() );
       }
       if(radix!=res_radix_i) {
         std::cerr << "'radix' overwritten :" << radix << '\n';
         WY_THROW( WyRet() );
       }
       if(num!=res_val) {
         std::cerr << "'res_val' inconsistent: " << (long long)num << '\n';
         WY_THROW( WyRet() );
       }
       if(r==Ok) {
       } else if(r==Wym_EINVAL) {
       } else if(r==Wym_ENOENT) {
       } else if(r==Wym_EBADMSG) {
       } else if(r==Wym_ERANGE) {
       } else {
         WY_THROW( WyRet() );
       }

       // _strlen(Num&, const WyCSeg&)
       if(res_radix_i==10) {
         radix=res_radix_i;
         if((r=cs.reset(nstr,nstr_len))!=Ok) {
           WY_THROW(r);
         }
         r=Wy::_strnum(num, (const WyCSeg&)cs);
         
         if(cs!=WyCSeg(nstr,nstr_len)) {
           std::cerr << "cs overwritten: " << WyStr(cs).c_str() << '\n';
           WY_THROW( WyRet() );
         }
         if(num!=res_val) {
           std::cerr << "'res_val' inconsistent: " << (long long)num << '\n';
           WY_THROW( WyRet() );
         }
         if(r==Ok) {
         } else if(r==Wym_EINVAL) {
         } else if(r==Wym_ENOENT) {
         } else if(r==Wym_EBADMSG) {
         } else if(r==Wym_ERANGE) {
         } else {
           WY_THROW( WyRet() );
         }
       }

     }
     catch(...) {
       std::cerr << "ChkItem_Signed\n";
       std::cerr << "Failed on {\"" << ((nstr==NULL)? "NULL" : nstr) << "\"," << nstr_len
             << ',' << Wy::wrd(res_reply).c_str() << ',' 
             << (long long)res_val << ',' << res_plen
              << ',' << (int)res_radix_i << ',' << (int)res_radix_o
             << "}\n";
       throw;
     };
};

template <typename NumT>
static void ck_nstr_signed(void)
{
 const ChkItem_Signed<NumT> chk_tab[]={
   // nstr slen  reply val plen rad_i rad_o

   {"0",0 ,Wym_ENOENT,0,0, 2, 2},
   {"0",0 ,Wym_ENOENT,0,0, 0, 2},

   // one char "?"
   {"0",1 ,        Ok,0,1, 2,2},
   {"1",1 ,        Ok,1,1, 2,2},
   {"7",1 ,Wym_ENOENT,0,0, 2,2},
   {"8",1 ,Wym_ENOENT,0,0, 2,2},
   {"9",1 ,Wym_ENOENT,0,0, 2,2},
   {"a",1 ,Wym_ENOENT,0,0, 2,2},
   {"e",1 ,Wym_ENOENT,0,0, 2,2},
   {"g",1 ,Wym_ENOENT,0,0, 2,2},
   {"X",1 ,Wym_ENOENT,0,0, 2,2},
   {"+",1 ,Wym_ENOENT,0,0, 2,2},
   {"-",1 ,Wym_ENOENT,0,0, 2,2},
   {".",1 ,Wym_ENOENT,0,0, 2,2},

   {"0",1 ,        Ok,0,1, 8,8},
   {"1",1 ,        Ok,1,1, 8,8},
   {"7",1 ,        Ok,7,1, 8,8},
   {"8",1 ,Wym_ENOENT,0,0, 8,8},
   {"9",1 ,Wym_ENOENT,0,0, 8,8},
   {"a",1 ,Wym_ENOENT,0,0, 8,8},
   {"e",1 ,Wym_ENOENT,0,0, 8,8},
   {"g",1 ,Wym_ENOENT,0,0, 8,8},
   {"X",1 ,Wym_ENOENT,0,0, 8,8},
   {"+",1 ,Wym_ENOENT,0,0, 8,8},
   {"-",1 ,Wym_ENOENT,0,0, 8,8},
   {".",1 ,Wym_ENOENT,0,0, 8,8},

   {"0",1 ,        Ok,0,1,10,10},
   {"1",1 ,        Ok,1,1,10,10},
   {"7",1 ,        Ok,7,1,10,10},
   {"8",1 ,        Ok,8,1,10,10},
   {"9",1 ,        Ok,9,1,10,10},
   {"a",1 ,Wym_ENOENT,0,0,10,10},
   {"e",1 ,Wym_ENOENT,0,0,10,10},
   {"g",1 ,Wym_ENOENT,0,0,10,10},
   {"X",1 ,Wym_ENOENT,0,0,10,10},
   {"+",1 ,Wym_ENOENT,0,0,10,10},
   {"-",1 ,Wym_ENOENT,0,0,10,10},
   {".",1 ,Wym_ENOENT,0,0,10,10},

   {"0",1 ,        Ok,0x0,1,16,16},
   {"1",1 ,        Ok,0x1,1,16,16},
   {"7",1 ,        Ok,0x7,1,16,16},
   {"8",1 ,        Ok,0x8,1,16,16},
   {"9",1 ,        Ok,0x9,1,16,16},
   {"a",1 ,        Ok,0xa,1,16,16},
   {"e",1 ,        Ok,0xe,1,16,16},
   {"g",1 ,Wym_ENOENT,0x0,0,16,16},
   {"X",1 ,Wym_ENOENT,0x0,0,16,16},
   {"+",1 ,Wym_ENOENT,0x0,0,16,16},
   {"-",1 ,Wym_ENOENT,0x0,0,16,16},
   {".",1 ,Wym_ENOENT,0x0,0,16,16},

   {"0",1 ,        Ok,0,1, 0,10},
   {"1",1 ,        Ok,1,1, 0,10},
   {"7",1 ,        Ok,7,1, 0,10},
   {"8",1 ,        Ok,8,1, 0,10},
   {"9",1 ,        Ok,9,1, 0,10},
   {"a",1 ,Wym_ENOENT,0,0, 0,10},   // radix not defined but for verify implement
   {"e",1 ,Wym_ENOENT,0,0, 0,10},   // radix not defined but for verify implement
   {"g",1 ,Wym_ENOENT,0,0, 0,10},   // radix not defined but for verify implement
   {"X",1 ,Wym_ENOENT,0,0, 0,10},   // radix not defined but for verify implement
   {"+",1 ,Wym_ENOENT,0,0, 0,0},    // radix not defined but for verify implement
   {"-",1 ,Wym_ENOENT,0,0, 0,0},    // radix not defined but for verify implement
   {".",1 ,Wym_ENOENT,0,0, 0,10},   // radix not defined but for verify implement
 
   // two char's "+?"
   {"+0",2 ,        Ok,0,2, 2,2},
   {"+1",2 ,        Ok,1,2, 2,2},
   {"+7",2 ,Wym_ENOENT,0,0, 2,2},
   {"+8",2 ,Wym_ENOENT,0,0, 2,2},
   {"+9",2 ,Wym_ENOENT,0,0, 2,2},
   {"+a",2 ,Wym_ENOENT,0,0, 2,2},
   {"+e",2 ,Wym_ENOENT,0,0, 2,2},
   {"+g",2 ,Wym_ENOENT,0,0, 2,2},
   {"+X",2 ,Wym_ENOENT,0,0, 2,2},
   {"++",2 ,Wym_ENOENT,0,0, 2,2},
   {"+-",2 ,Wym_ENOENT,0,0, 2,2},
   {"+.",2 ,Wym_ENOENT,0,0, 2,2},

   {"+0",2 ,        Ok,0,2, 8,8},
   {"+1",2 ,        Ok,1,2, 8,8},
   {"+7",2 ,        Ok,7,2, 8,8},
   {"+8",2 ,Wym_ENOENT,0,0, 8,8},
   {"+9",2 ,Wym_ENOENT,0,0, 8,8},
   {"+a",2 ,Wym_ENOENT,0,0, 8,8},
   {"+e",2 ,Wym_ENOENT,0,0, 8,8},
   {"+g",2 ,Wym_ENOENT,0,0, 8,8},
   {"+X",2 ,Wym_ENOENT,0,0, 8,8},
   {"++",2 ,Wym_ENOENT,0,0, 8,8},
   {"+-",2 ,Wym_ENOENT,0,0, 8,8},
   {"+.",2 ,Wym_ENOENT,0,0, 8,8},

   {"+0",2 ,        Ok,0,2,10,10},
   {"+1",2 ,        Ok,1,2,10,10},
   {"+7",2 ,        Ok,7,2,10,10},
   {"+8",2 ,        Ok,8,2,10,10},
   {"+9",2 ,        Ok,9,2,10,10},
   {"+a",2 ,Wym_ENOENT,0,0,10,10},
   {"+e",2 ,Wym_ENOENT,0,0,10,10},
   {"+g",2 ,Wym_ENOENT,0,0,10,10},
   {"+X",2 ,Wym_ENOENT,0,0,10,10},
   {"++",2 ,Wym_ENOENT,0,0,10,10},
   {"+-",2 ,Wym_ENOENT,0,0,10,10},
   {"+.",2 ,Wym_ENOENT,0,0,10,10},

   {"+0",2 ,        Ok,  0,2,16,16},
   {"+1",2 ,        Ok,  1,2,16,16},
   {"+7",2 ,        Ok,  7,2,16,16},
   {"+8",2 ,        Ok,  8,2,16,16},
   {"+9",2 ,        Ok,  9,2,16,16},
   {"+a",2 ,        Ok,0xa,2,16,16},
   {"+e",2 ,        Ok,0xe,2,16,16},
   {"+g",2 ,Wym_ENOENT,  0,0,16,16},
   {"+X",2 ,Wym_ENOENT,  0,0,16,16},
   {"++",2 ,Wym_ENOENT,  0,0,16,16},
   {"+-",2 ,Wym_ENOENT,  0,0,16,16},
   {"+.",2 ,Wym_ENOENT,  0,0,16,16},

   {"+0",2 ,        Ok,0,2, 0,10},
   {"+1",2 ,        Ok,1,2, 0,10},
   {"+7",2 ,        Ok,7,2, 0,10},
   {"+8",2 ,        Ok,8,2, 0,10},
   {"+9",2 ,        Ok,9,2, 0,10},
   {"+a",2 ,Wym_ENOENT,0,0, 0,10},
   {"+e",2 ,Wym_ENOENT,0,0, 0,10},
   {"+g",2 ,Wym_ENOENT,0,0, 0,10},
   {"+X",2 ,Wym_ENOENT,0,0, 0,10},
   {"++",2 ,Wym_ENOENT,0,0, 0,10},
   {"+-",2 ,Wym_ENOENT,0,0, 0,10},
   {"+.",2 ,Wym_ENOENT,0,0, 0,10},

   // two char's "-?"
   {"-0",2 ,        Ok, 0,2, 2,2},
   {"-1",2 ,        Ok,-1,2, 2,2},
   {"-7",2 ,Wym_ENOENT, 0,0, 2,2},
   {"-8",2 ,Wym_ENOENT, 0,0, 2,2},
   {"-9",2 ,Wym_ENOENT, 0,0, 2,2},
   {"-a",2 ,Wym_ENOENT, 0,0, 2,2},
   {"-e",2 ,Wym_ENOENT, 0,0, 2,2},
   {"-g",2 ,Wym_ENOENT, 0,0, 2,2},
   {"-X",2 ,Wym_ENOENT, 0,0, 2,2},
   {"-+",2 ,Wym_ENOENT, 0,0, 2,2},
   {"-.",2 ,Wym_ENOENT, 0,0, 2,2},

   {"-0",2 ,        Ok, 0,2, 8,8},
   {"-1",2 ,        Ok,-1,2, 8,8},
   {"-7",2 ,        Ok,-7,2, 8,8},
   {"-8",2 ,Wym_ENOENT, 0,0, 8,8},
   {"-9",2 ,Wym_ENOENT, 0,0, 8,8},
   {"-a",2 ,Wym_ENOENT, 0,0, 8,8},
   {"-e",2 ,Wym_ENOENT, 0,0, 8,8},
   {"-g",2 ,Wym_ENOENT, 0,0, 8,8},
   {"-X",2 ,Wym_ENOENT, 0,0, 8,8},
   {"-+",2 ,Wym_ENOENT, 0,0, 8,8},
   {"-.",2 ,Wym_ENOENT, 0,0, 8,8},

   {"-0",2 ,        Ok, 0,2,10,10},
   {"-1",2 ,        Ok,-1,2,10,10},
   {"-7",2 ,        Ok,-7,2,10,10},
   {"-8",2 ,        Ok,-8,2,10,10},
   {"-9",2 ,        Ok,-9,2,10,10},
   {"-a",2 ,Wym_ENOENT, 0,0,10,10},
   {"-e",2 ,Wym_ENOENT, 0,0,10,10},
   {"-g",2 ,Wym_ENOENT, 0,0,10,10},
   {"-X",2 ,Wym_ENOENT, 0,0,10,10},
   {"-+",2 ,Wym_ENOENT, 0,0,10,10},
   {"-.",2 ,Wym_ENOENT, 0,0,10,10},

   {"-0",2 ,        Ok,   0,2,16,16},
   {"-1",2 ,        Ok,  -1,2,16,16},
   {"-7",2 ,        Ok,  -7,2,16,16},
   {"-8",2 ,        Ok,  -8,2,16,16},
   {"-9",2 ,        Ok,  -9,2,16,16},
   {"-a",2 ,        Ok,-0xa,2,16,16},
   {"-e",2 ,        Ok,-0xe,2,16,16},
   {"-g",2 ,Wym_ENOENT,   0,0,16,16},
   {"-X",2 ,Wym_ENOENT,   0,0,16,16},
   {"-+",2 ,Wym_ENOENT,   0,0,16,16},
   {"--",2 ,Wym_ENOENT,   0,0,16,16},
   {"-.",2 ,Wym_ENOENT,   0,0,16,16},

   {"-0",2 ,        Ok, 0,2, 0,10},
   {"-1",2 ,        Ok,-1,2, 0,10},
   {"-7",2 ,        Ok,-7,2, 0,10},
   {"-8",2 ,        Ok,-8,2, 0,10},
   {"-9",2 ,        Ok,-9,2, 0,10},
   {"-a",2 ,Wym_ENOENT, 0,0, 0,10},
   {"-e",2 ,Wym_ENOENT, 0,0, 0,10},
   {"-g",2 ,Wym_ENOENT, 0,0, 0,10},
   {"-X",2 ,Wym_ENOENT, 0,0, 0,10},
   {"-+",2 ,Wym_ENOENT, 0,0, 0,10},
   {"--",2 ,Wym_ENOENT, 0,0, 0,10},
   {"-.",2 ,Wym_ENOENT, 0,0, 0,10},

   // two char's ".?"
   {".0",2 ,Wym_ENOENT,0,0, 2,2},
   {".1",2 ,Wym_ENOENT,0,0, 2,2},
   {".7",2 ,Wym_ENOENT,0,0, 2,2},
   {".8",2 ,Wym_ENOENT,0,0, 2,2},
   {".9",2 ,Wym_ENOENT,0,0, 2,2},
   {".a",2 ,Wym_ENOENT,0,0, 2,2},
   {".e",2 ,Wym_ENOENT,0,0, 2,2},
   {".g",2 ,Wym_ENOENT,0,0, 2,2},
   {".X",2 ,Wym_ENOENT,0,0, 2,2},
   {".+",2 ,Wym_ENOENT,0,0, 2,2},
   {".-",2 ,Wym_ENOENT,0,0, 2,2},
   {"..",2 ,Wym_ENOENT,0,0, 2,2},

   {".0",2 ,Wym_ENOENT,0,0, 8,8},
   {".1",2 ,Wym_ENOENT,0,0, 8,8},
   {".7",2 ,Wym_ENOENT,0,0, 8,8},
   {".8",2 ,Wym_ENOENT,0,0, 8,8},
   {".9",2 ,Wym_ENOENT,0,0, 8,8},
   {".a",2 ,Wym_ENOENT,0,0, 8,8},
   {".e",2 ,Wym_ENOENT,0,0, 8,8},
   {".g",2 ,Wym_ENOENT,0,0, 8,8},
   {".X",2 ,Wym_ENOENT,0,0, 8,8},
   {".+",2 ,Wym_ENOENT,0,0, 8,8},
   {".-",2 ,Wym_ENOENT,0,0, 8,8},
   {"..",2 ,Wym_ENOENT,0,0, 8,8},

   {".0",2 ,Wym_ENOENT,0,0,10,10},
   {".1",2 ,Wym_ENOENT,0,0,10,10},
   {".7",2 ,Wym_ENOENT,0,0,10,10},
   {".8",2 ,Wym_ENOENT,0,0,10,10},
   {".9",2 ,Wym_ENOENT,0,0,10,10},
   {".a",2 ,Wym_ENOENT,0,0,10,10},
   {".e",2 ,Wym_ENOENT,0,0,10,10},
   {".g",2 ,Wym_ENOENT,0,0,10,10},
   {".X",2 ,Wym_ENOENT,0,0,10,10},
   {".+",2 ,Wym_ENOENT,0,0,10,10},
   {".-",2 ,Wym_ENOENT,0,0,10,10},
   {"..",2 ,Wym_ENOENT,0,0,10,10},

   {".0",2 ,Wym_ENOENT,  0,0,16,16},
   {".1",2 ,Wym_ENOENT,  0,0,16,16},
   {".7",2 ,Wym_ENOENT,  0,0,16,16},
   {".8",2 ,Wym_ENOENT,  0,0,16,16},
   {".9",2 ,Wym_ENOENT,  0,0,16,16},
   {".a",2 ,Wym_ENOENT,  0,0,16,16},
   {".e",2 ,Wym_ENOENT,  0,0,16,16},
   {".g",2 ,Wym_ENOENT,  0,0,16,16},
   {".X",2 ,Wym_ENOENT,  0,0,16,16},
   {".+",2 ,Wym_ENOENT,  0,0,16,16},
   {".-",2 ,Wym_ENOENT,  0,0,16,16},
   {"..",2 ,Wym_ENOENT,  0,0,16,16},

   {".0",2 ,Wym_ENOENT,0,0, 0,10},
   {".1",2 ,Wym_ENOENT,0,0, 0,10},
   {".7",2 ,Wym_ENOENT,0,0, 0,10},
   {".8",2 ,Wym_ENOENT,0,0, 0,10},
   {".9",2 ,Wym_ENOENT,0,0, 0,10},
   {".a",2 ,Wym_ENOENT,0,0, 0,10},
   {".e",2 ,Wym_ENOENT,0,0, 0,10},
   {".g",2 ,Wym_ENOENT,0,0, 0,10},
   {".X",2 ,Wym_ENOENT,0,0, 0,10},
   {".+",2 ,Wym_ENOENT,0,0, 0,10},
   {".-",2 ,Wym_ENOENT,0,0, 0,10},
   {"..",2 ,Wym_ENOENT,0,0, 0,10},

   // two char's "0?"
   {"00",2 ,         Ok,0,2, 2,2},
   {"01",2 ,         Ok,1,2, 2,2},
   {"07",2 ,Wym_EBADMSG,0,1, 2,2},
   {"08",2 ,Wym_EBADMSG,0,1, 2,2},
   {"09",2 ,Wym_EBADMSG,0,1, 2,2},
   {"0a",2 ,Wym_EBADMSG,0,1, 2,2},
   {"0e",2 ,Wym_EBADMSG,0,1, 2,2},
   {"0g",2 ,Wym_EBADMSG,0,1, 2,2},
   {"0X",2 ,Wym_EBADMSG,0,1, 2,2},
   {"0+",2 ,Wym_EBADMSG,0,1, 2,2},
   {"0-",2 ,Wym_EBADMSG,0,1, 2,2},
   {"0.",2 ,Wym_EBADMSG,0,1, 2,2},

   {"00",2 ,         Ok,0,2, 8,8},
   {"01",2 ,         Ok,1,2, 8,8},
   {"07",2 ,         Ok,7,2, 8,8},
   {"08",2 ,Wym_EBADMSG,0,1, 8,8},
   {"09",2 ,Wym_EBADMSG,0,1, 8,8},
   {"0a",2 ,Wym_EBADMSG,0,1, 8,8},
   {"0e",2 ,Wym_EBADMSG,0,1, 8,8},
   {"0g",2 ,Wym_EBADMSG,0,1, 8,8},
   {"0X",2 ,Wym_EBADMSG,0,1, 8,8},
   {"0+",2 ,Wym_EBADMSG,0,1, 8,8},
   {"0-",2 ,Wym_EBADMSG,0,1, 8,8},
   {"0.",2 ,Wym_EBADMSG,0,1, 8,8},

   {"00",2 ,         Ok,0,2,10,10},
   {"01",2 ,         Ok,1,2,10,10},
   {"07",2 ,         Ok,7,2,10,10},
   {"08",2 ,         Ok,8,2,10,10},
   {"09",2 ,         Ok,9,2,10,10},
   {"0a",2 ,Wym_EBADMSG,0,1,10,10},
   {"0e",2 ,Wym_EBADMSG,0,1,10,10},
   {"0g",2 ,Wym_EBADMSG,0,1,10,10},
   {"0X",2 ,Wym_EBADMSG,0,1,10,10},
   {"0+",2 ,Wym_EBADMSG,0,1,10,10},
   {"0-",2 ,Wym_EBADMSG,0,1,10,10},
   {"0.",2 ,Wym_EBADMSG,0,1,10,10},

   {"00",2 ,         Ok,  0,2,16,16},
   {"01",2 ,         Ok,  1,2,16,16},
   {"07",2 ,         Ok,  7,2,16,16},
   {"08",2 ,         Ok,  8,2,16,16},
   {"09",2 ,         Ok,  9,2,16,16},
   {"0a",2 ,         Ok,0xa,2,16,16},
   {"0e",2 ,         Ok,0xe,2,16,16},
   {"0g",2 ,Wym_EBADMSG,  0,1,16,16},
   {"0X",2 ,Wym_EBADMSG,  0,1,16,16},
   {"0+",2 ,Wym_EBADMSG,  0,1,16,16},
   {"0-",2 ,Wym_EBADMSG,  0,1,16,16},
   {"0.",2 ,Wym_EBADMSG,  0,1,16,16},

   {"00",2 ,         Ok,0,2, 0,8},
   {"01",2 ,         Ok,1,2, 0,8},
   {"07",2 ,         Ok,7,2, 0,8},
   {"08",2 ,Wym_EBADMSG,0,1, 0,10},
   {"09",2 ,Wym_EBADMSG,0,1, 0,10},
   {"0a",2 ,Wym_EBADMSG,0,1, 0,10},
   {"0e",2 ,Wym_EBADMSG,0,1, 0,10},
   {"0g",2 ,Wym_EBADMSG,0,1, 0,10},
   {"0X",2 ,Wym_EBADMSG,0,1, 0,10},
   {"0+",2 ,Wym_EBADMSG,0,1, 0,10},
   {"0-",2 ,Wym_EBADMSG,0,1, 0,10},
   {"0.",2 ,Wym_EBADMSG,0,1, 0,10},

   // two char's "1?"
   {"10",2 ,         Ok,2,2, 2,2},
   {"11",2 ,         Ok,3,2, 2,2},
   {"17",2 ,Wym_EBADMSG,1,1, 2,2},
   {"18",2 ,Wym_EBADMSG,1,1, 2,2},
   {"19",2 ,Wym_EBADMSG,1,1, 2,2},
   {"1a",2 ,Wym_EBADMSG,1,1, 2,2},
   {"1e",2 ,Wym_EBADMSG,1,1, 2,2},
   {"1g",2 ,Wym_EBADMSG,1,1, 2,2},
   {"1X",2 ,Wym_EBADMSG,1,1, 2,2},
   {"1+",2 ,Wym_EBADMSG,1,1, 2,2},
   {"1-",2 ,Wym_EBADMSG,1,1, 2,2},
   {"1.",2 ,Wym_EBADMSG,1,1, 2,2},

   {"10",2 ,         Ok, 8,2, 8,8},
   {"11",2 ,         Ok, 9,2, 8,8},
   {"17",2 ,         Ok,15,2, 8,8},
   {"18",2 ,Wym_EBADMSG, 1,1, 8,8},
   {"19",2 ,Wym_EBADMSG, 1,1, 8,8},
   {"1a",2 ,Wym_EBADMSG, 1,1, 8,8},
   {"1e",2 ,Wym_EBADMSG, 1,1, 8,8},
   {"1g",2 ,Wym_EBADMSG, 1,1, 8,8},
   {"1X",2 ,Wym_EBADMSG, 1,1, 8,8},
   {"1+",2 ,Wym_EBADMSG, 1,1, 8,8},
   {"1-",2 ,Wym_EBADMSG, 1,1, 8,8},
   {"1.",2 ,Wym_EBADMSG, 1,1, 8,8},

   {"10",2 ,         Ok,10,2,10,10},
   {"11",2 ,         Ok,11,2,10,10},
   {"17",2 ,         Ok,17,2,10,10},
   {"18",2 ,         Ok,18,2,10,10},
   {"19",2 ,         Ok,19,2,10,10},
   {"1a",2 ,Wym_EBADMSG, 1,1,10,10},
   {"1e",2 ,Wym_EBADMSG, 1,1,10,10},
   {"1g",2 ,Wym_EBADMSG, 1,1,10,10},
   {"1X",2 ,Wym_EBADMSG, 1,1,10,10},
   {"1+",2 ,Wym_EBADMSG, 1,1,10,10},
   {"1-",2 ,Wym_EBADMSG, 1,1,10,10},
   {"1.",2 ,Wym_EBADMSG, 1,1,10,10},

   {"10",2 ,         Ok,0x10,2,16,16},
   {"11",2 ,         Ok,0x11,2,16,16},
   {"17",2 ,         Ok,0x17,2,16,16},
   {"18",2 ,         Ok,0x18,2,16,16},
   {"19",2 ,         Ok,0x19,2,16,16},
   {"1a",2 ,         Ok,0x1a,2,16,16},
   {"1e",2 ,         Ok,0x1e,2,16,16},
   {"1g",2 ,Wym_EBADMSG,   1,1,16,16},
   {"1X",2 ,Wym_EBADMSG,   1,1,16,16},
   {"1+",2 ,Wym_EBADMSG,   1,1,16,16},
   {"1-",2 ,Wym_EBADMSG,   1,1,16,16},
   {"1.",2 ,Wym_EBADMSG,   1,1,16,16},

   {"10",2 ,         Ok,10,2, 0,10},
   {"11",2 ,         Ok,11,2, 0,10},
   {"17",2 ,         Ok,17,2, 0,10},
   {"18",2 ,         Ok,18,2, 0,10},
   {"19",2 ,         Ok,19,2, 0,10},
   {"1a",2 ,Wym_EBADMSG, 1,1, 0,10},
   {"1e",2 ,Wym_EBADMSG, 1,1, 0,10},
   {"1g",2 ,Wym_EBADMSG, 1,1, 0,10},
   {"1X",2 ,Wym_EBADMSG, 1,1, 0,10},
   {"1+",2 ,Wym_EBADMSG, 1,1, 0,10},
   {"1-",2 ,Wym_EBADMSG, 1,1, 0,10},
   {"1.",2 ,Wym_EBADMSG, 1,1, 0,10},

   // ".0?"
   {".00",3 ,Wym_ENOENT,0,0, 2,2},
   {".01",3 ,Wym_ENOENT,0,0, 2,2},
   {".07",3 ,Wym_ENOENT,0,0, 2,2},
   {".08",3 ,Wym_ENOENT,0,0, 2,2},
   {".09",3 ,Wym_ENOENT,0,0, 2,2},
   {".0a",3 ,Wym_ENOENT,0,0, 2,2},
   {".0e",3 ,Wym_ENOENT,0,0, 2,2},
   {".0g",3 ,Wym_ENOENT,0,0, 2,2},
   {".0X",3 ,Wym_ENOENT,0,0, 2,2},
   {".0+",3 ,Wym_ENOENT,0,0, 2,2},
   {".0-",3 ,Wym_ENOENT,0,0, 2,2},
   {".0.",3 ,Wym_ENOENT,0,0, 2,2},

   {".00",3 ,Wym_ENOENT, 0,0, 8,8},
   {".01",3 ,Wym_ENOENT, 0,0, 8,8},
   {".07",3 ,Wym_ENOENT, 0,0, 8,8},
   {".08",3 ,Wym_ENOENT, 0,0, 8,8},
   {".09",3 ,Wym_ENOENT, 0,0, 8,8},
   {".0a",3 ,Wym_ENOENT, 0,0, 8,8},
   {".0e",3 ,Wym_ENOENT, 0,0, 8,8},
   {".0g",3 ,Wym_ENOENT, 0,0, 8,8},
   {".0X",3 ,Wym_ENOENT, 0,0, 8,8},
   {".0+",3 ,Wym_ENOENT, 0,0, 8,8},
   {".0-",3 ,Wym_ENOENT, 0,0, 8,8},
   {".0.",3 ,Wym_ENOENT, 0,0, 8,8},

   {".00",3 ,Wym_ENOENT, 0,0,10,10},
   {".01",3 ,Wym_ENOENT, 0,0,10,10},
   {".07",3 ,Wym_ENOENT, 0,0,10,10},
   {".08",3 ,Wym_ENOENT, 0,0,10,10},
   {".09",3 ,Wym_ENOENT, 0,0,10,10},
   {".0a",3 ,Wym_ENOENT, 0,0,10,10},
   {".0e",3 ,Wym_ENOENT, 0,0,10,10},
   {".0g",3 ,Wym_ENOENT, 0,0,10,10},
   {".0X",3 ,Wym_ENOENT, 0,0,10,10},
   {".0+",3 ,Wym_ENOENT, 0,0,10,10},
   {".0-",3 ,Wym_ENOENT, 0,0,10,10},
   {".0.",3 ,Wym_ENOENT, 0,0,10,10},

   {".00",3 ,Wym_ENOENT,  0,0,16,16},
   {".01",3 ,Wym_ENOENT,  0,0,16,16},
   {".07",3 ,Wym_ENOENT,  0,0,16,16},
   {".08",3 ,Wym_ENOENT,  0,0,16,16},
   {".09",3 ,Wym_ENOENT,  0,0,16,16},
   {".0a",3 ,Wym_ENOENT,  0,0,16,16},
   {".0e",3 ,Wym_ENOENT,  0,0,16,16},
   {".0g",3 ,Wym_ENOENT,  0,0,16,16},
   {".0X",3 ,Wym_ENOENT,  0,0,16,16},
   {".0+",3 ,Wym_ENOENT,  0,0,16,16},
   {".0-",3 ,Wym_ENOENT,  0,0,16,16},
   {".0.",3 ,Wym_ENOENT,  0,0,16,16},

   {".00",3 ,Wym_ENOENT,0x0,0, 0,10},
   {".01",3 ,Wym_ENOENT,0x0,0, 0,10},
   {".07",3 ,Wym_ENOENT,0x0,0, 0,10},
   {".08",3 ,Wym_ENOENT,0x0,0, 0,10},
   {".09",3 ,Wym_ENOENT,0x0,0, 0,10},
   {".0a",3 ,Wym_ENOENT,0x0,0, 0,10},
   {".0e",3 ,Wym_ENOENT,0x0,0, 0,10},
   {".0g",3 ,Wym_ENOENT,  0,0, 0,10},
   {".0X",3 ,Wym_ENOENT,  0,0, 0,10},
   {".0+",3 ,Wym_ENOENT,  0,0, 0,10},
   {".0-",3 ,Wym_ENOENT,  0,0, 0,10},
   {".0.",3 ,Wym_ENOENT,  0,0, 0,10},

   // "0x?"
   {"0x0",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0x1",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0x7",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0x8",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0x9",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0xa",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0xe",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0xg",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0xX",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0x+",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0x-",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0x.",3 ,Wym_EBADMSG,0,1, 2,2},

   {"0x0",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0x1",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0x7",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0x8",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0x9",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0xa",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0xe",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0xg",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0xX",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0x+",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0x-",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0x.",3 ,Wym_EBADMSG, 0,1, 8,8},

   {"0x0",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0x1",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0x7",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0x8",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0x9",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0xa",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0xe",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0xg",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0xX",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0x+",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0x-",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0x.",3 ,Wym_EBADMSG, 0,1,10,10},

   {"0x0",3 ,         Ok,0x0,3,16,16},
   {"0x1",3 ,         Ok,0x1,3,16,16},
   {"0x7",3 ,         Ok,0x7,3,16,16},
   {"0x8",3 ,         Ok,0x8,3,16,16},
   {"0x9",3 ,         Ok,0x9,3,16,16},
   {"0xa",3 ,         Ok,0xa,3,16,16},
   {"0xe",3 ,         Ok,0xe,3,16,16},
   {"0xg",3 ,Wym_EBADMSG,  0,1,16,16},
   {"0xX",3 ,Wym_EBADMSG,  0,1,16,16},
   {"0x+",3 ,Wym_EBADMSG,  0,1,16,16},
   {"0x-",3 ,Wym_EBADMSG,  0,1,16,16},
   {"0x.",3 ,Wym_EBADMSG,  0,1,16,16},

   {"0x0",3 ,         Ok,0x0,3, 0,16},
   {"0x1",3 ,         Ok,0x1,3, 0,16},
   {"0x7",3 ,         Ok,0x7,3, 0,16},
   {"0x8",3 ,         Ok,0x8,3, 0,16},
   {"0x9",3 ,         Ok,0x9,3, 0,16},
   {"0xa",3 ,         Ok,0xa,3, 0,16},
   {"0xe",3 ,         Ok,0xe,3, 0,16},
   {"0xg",3 ,Wym_EBADMSG,  0,1, 0,10},
   {"0xX",3 ,Wym_EBADMSG,  0,1, 0,10},
   {"0x+",3 ,Wym_EBADMSG,  0,1, 0,10},
   {"0x-",3 ,Wym_EBADMSG,  0,1, 0,10},
   {"0x.",3 ,Wym_EBADMSG,  0,1, 0,10},

   // chk radix
  {"00",2 ,         Ok,0,2, 0,8},
  {"110",3 ,        Ok,    1*  2*2+  2    ,3, 2, 2},
  {"201",3 ,        Ok,   2*   3*3      +1,3, 3, 3},
  {"313",3 ,        Ok,   3*   4*4+  4+3,3, 4,4}, 
  {"413",3 ,        Ok,   4*   5*5+  5+3,3, 5, 5},

   // mis
   {"0x.0.2",6,Wym_EBADMSG,0,1,16,16},
   {"0x.0.2",6,Wym_EBADMSG,0,1,0,10},
   {"0.0.2",5,Wym_EBADMSG,0,1,10,10},
   {"0.0.2",5,Wym_EBADMSG,0,1,0,10},

   {"0x.2",4,Wym_EBADMSG,0,1,16,16},
   {"0x.2.",5,Wym_EBADMSG,0,1,16,16},
   {"0x.2.2",6,Wym_EBADMSG,0,1,16,16},
   {"0x.2",4,Wym_EBADMSG,0,1,0,10},
   {"0x.2.",5,Wym_EBADMSG,0,1,0,10},
   {"0x.2.2",6,Wym_EBADMSG,0,1,0,10},
   {"0.1.",4,Wym_EBADMSG,0,1,10,10},
   {"0.1.2",5,Wym_EBADMSG,0,1,10,10},
   {"0.1.",4,Wym_EBADMSG,0,1,0,10},
   {"0.1.2",5,Wym_EBADMSG,0,1,0,10},

   {"-.",2 ,Wym_ENOENT,0,0,36,36},
   {"+.9",3,Wym_ENOENT,0,0,0,10},
   {"-.9",3,Wym_ENOENT,0,0,36,36},
   {"1",1 ,Wym_EINVAL,0,0,37,37},
   {"1",1 ,Wym_EINVAL,0,0, 1, 1},
   {"0",1 ,        Ok,0,1, 0,10},
   {"x",1,Wym_ENOENT,0,0, 2,2},
   {"0x",2,Wym_EBADMSG,0,1, 0,10},
   {"0xx",3,Wym_EBADMSG,0,1, 0,10},
   {"0xg",3,Wym_EBADMSG,0,1, 0,10},
   {"0",1 ,        Ok,0,1, 16,16},
   {"x",1,Wym_ENOENT,0,0, 2,2},
   {"0x",2,Wym_EBADMSG,0,1, 16,16},
   {"0xx",3,Wym_EBADMSG,0,1,16,16},
   {"0xg",3,Wym_EBADMSG,0,1,16,16},
   {"+0", 2,         Ok,0,2,16,16},
   {"-0", 2,         Ok,0,2,16,16},
   {"+0", 2,         Ok,0,2, 0,10},
   {"-0", 2,         Ok,0,2, 0,10}, 
   {"0x.",3,Wym_EBADMSG,0,1, 0,10},
   {"0x.x",4,Wym_EBADMSG,0,1, 0,10},
   {"0x.g",4,Wym_EBADMSG,0,1, 0,10}, 
   {"0x.a",4,Wym_EBADMSG,0,1, 0,10},
   {".0",2 ,Wym_ENOENT,0,0, 16,16},
   {"0x.",3,Wym_EBADMSG,0,1, 16,16},
   {"0x.x",4,Wym_EBADMSG,0,1,16,16},
   {"0x.g",4,Wym_EBADMSG,0,1,16,16},
   {"0.",2,Wym_EBADMSG,0,1, 2,2},
   {"+9.",3,Wym_EBADMSG, 9,2, 0,10},
   {"-9.",3,Wym_EBADMSG,-9,2,10,10},
   {"+.x",3,Wym_ENOENT,0,0,10,10},
   {"-0.1a",5,Wym_EBADMSG,0,2,10,10},
   {"-.1a", 4,Wym_ENOENT,0,0,10,10},

   {"0",1,Ok,0,1,10,10},
   {"31",2,Ok,31,2,10,10},
   {"031",3,Ok,031,3,0,8},
   {"00.03",5,Wym_EBADMSG,0,2,0,8},
   {"00.03",5,Wym_EBADMSG,0,2,10,10},
   {"0x00.03",7,Wym_EBADMSG,0,4,0,16},
   {"00x0",4,Wym_EBADMSG,0,2,0,8},    // 00= zero of radix 8

   {"+.0",3,Wym_ENOENT,0,0,10,10},
   {"-.0",3,Wym_ENOENT,0,0,0,10},
   {".00",3,Wym_ENOENT,0,0,10,10},
   {"+.00",4,Wym_ENOENT,0,0,0,10},
   {"-.00",4,Wym_ENOENT,0,0,10,10},
   {"0.0",3,Wym_EBADMSG,0,1,10,10},
   {"0.0",3,Wym_EBADMSG,0,1,0,10},
   {"+0.0",4,Wym_EBADMSG,0,2,10,10},
   {"-0.0",4,Wym_EBADMSG,0,2,0,10},
   {"0.010",5,Wym_EBADMSG,0,1,10,10},
   {"-0.010",6,Wym_EBADMSG,0,2,0,10},
   {"123",3,Ok,123,3,0,10},
   {"+123",4,Ok,123,4,0,10},
   {"+0123",5,Ok,0123,5,0,8},
   {"-0123",5,Ok,-123,5,10,10},
   {".123",4,Wym_ENOENT,0,0,10,10},
   {"+.123",5,Wym_ENOENT,0,0,10,10},
   {"-.123",5,Wym_ENOENT,0,0, 0,10},
   {".120",4,Wym_ENOENT,0,0,10,10},
   {"+.120",5,Wym_ENOENT,0,0,10,10},
   {"-.120",5,Wym_ENOENT,0,0, 0,10},
   {"12.0",4,Wym_EBADMSG,12,2, 0,10},
   {"+12.0",5,Wym_EBADMSG,12,3,10,10},
   {"-12.0",5,Wym_EBADMSG,-12,3, 0,10},
   {"-012.3",6,Wym_EBADMSG, -12,4,10,10},
   {"-012.3",6,Wym_EBADMSG,-012,4, 0, 8},
   {"+0012.3",7,Wym_EBADMSG, 12,5,10,10}, 
   {"1111111.1111111",15,Wym_EBADMSG,127,7,2,2}, 
   {"-1111111.1111111",16,Wym_EBADMSG,-127,8,2,2}, 
   {"177.177",7,Wym_EBADMSG,0177,3,8,8}, 
   {"-177.177",8,Wym_EBADMSG,-0177,4,8,8}, 
   {"-7f.7f",6,Wym_EBADMSG,-0x7f,3,16,16}, 
   {"-0x7f.7f",8,Wym_EBADMSG,-0x7f,5,0,16}, 
   {"+000000000000000000012.100",26,Wym_EBADMSG,12,22,10,10},
   {"+000000000000000000012.100",26,Wym_EBADMSG,5,22,3,3},
 };

 for(size_t i=0; i<sizeof(chk_tab)/sizeof(ChkItem_Signed<NumT>); ++i) {
   chk_tab[i].verify();
 }

 {
   std::ostringstream os;
   const NumT NumMax=std::numeric_limits<NumT>::max();
   const NumT NumMin=std::numeric_limits<NumT>::min();
   std::string StrMax10,StrMin10;
   std::string StrMax8;
   std::string StrMax16;

   os.str("");
   os << std::hex << static_cast<long long>(NumMax);
   StrMax16=os.str();

   os.str("");
   os << std::dec << static_cast<long long>(NumMax);
   StrMax10=os.str();

   os.str("");
   os << std::dec << static_cast<long long>(NumMin);
   StrMin10=os.str();

   os.str("");
   os << std::oct << static_cast<long long>(NumMax);
   StrMax8=os.str();

   { // chk <max> radix=8
     std::string nstr(StrMax8);
     ChkItem_Signed<NumT> cksmp=
        {nstr.c_str(),nstr.size(),Ok,NumMax,nstr.size(),8,8};
     cksmp.verify();
   }

   { // chk <max> radix=10
     std::string nstr(StrMax10);
     ChkItem_Signed<NumT> cksmp=
        {nstr.c_str(),nstr.size(),Ok,NumMax,nstr.size(),10,10};
     cksmp.verify();
   }

   { // chk <max> radix=16
     std::string nstr(StrMax16);
     ChkItem_Signed<NumT> cksmp=
        {nstr.c_str(),nstr.size(),Ok,NumMax,nstr.size(),16,16};
     cksmp.verify();
   }

   { // chk <-min> radix=10
     std::string nstr(StrMin10);
     ChkItem_Signed<NumT> cksmp=
        {nstr.c_str(),nstr.size(),Ok,NumMin,nstr.size(),10,10};
     cksmp.verify();
   }

   { // chk <ovl> radix=10
     std::string nstr(StrMax10+'1');
     ChkItem_Signed<NumT> cksmp=
        {nstr.c_str(),nstr.size(),Wym_ERANGE,NumMax,StrMax10.size(),10,10};
     cksmp.verify();
   }

   { // chk <-ovl> radix=10
     std::string nstr(StrMin10+'1');
     ChkItem_Signed<NumT> cksmp=
        {nstr.c_str(),nstr.size(),Wym_ERANGE,NumMin,StrMin10.size(),10,10};
     cksmp.verify();
   }

   { // ck <-max> radix=8
     const std::string nstr('-'+StrMax8);
     ChkItem_Signed<NumT> cksmp=
        {nstr.c_str(),nstr.size(),Ok,-NumMax,nstr.size(),8,8};
     cksmp.verify();
   }
   { // ck <-max> radix=10
     const std::string nstr('-'+StrMax10);
     ChkItem_Signed<NumT> cksmp=
        {nstr.c_str(),nstr.size(),Ok,-NumMax,nstr.size(),10,10};
     cksmp.verify();
   }
   { // ck <-max> radix=16
     const std::string nstr('-'+StrMax16);
     ChkItem_Signed<NumT> cksmp=
        {nstr.c_str(),nstr.size(),Ok,-NumMax,nstr.size(),16,16};
     cksmp.verify();
   }
 }
};

// Check Item for unsigned integral types
//  (only the radix modifiable version is checked)
//
template <typename NumT>
struct ChkItem_UnSigned {
  const char* const nstr;       // test string
  const size_t nstr_len;        //

  // Expected results
  const WyReply res_reply;
  const NumT res_val;
  const size_t res_plen;
  const Wy::WyRadix res_radix_i;
  const Wy::WyRadix res_radix_o;

  void verify(void) const
     try {
       WyRet r;
       NumT num;
       Wy::WyRadix radix=res_radix_i;
       WyCSeg cs;

       // NOTE: check codes are exact copy of the signed version
       //
       // _strlen(Num&, WyCSeg&, int&)
       radix=res_radix_i;
       if((r=cs.reset(nstr,nstr_len))!=Ok) {
         WY_THROW(r);
       }
       r=Wy::_strnum(num, cs, radix);
       
       if(r!=res_reply) {
         std::cerr << "'res_reply' inconsistent: " << Wy::wrd(r).c_str() << '\n';
         WY_THROW( WyRet() );
       }
       if(num!=res_val) {
         std::cerr << "'res_val' inconsistent: " << (long long)num << '\n';
         WY_THROW( WyRet() );
       }
       if(r==Ok) {
         if(cs.size()!=0) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EINVAL) {
         if(cs.size()!=nstr_len) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ENOENT) {
         if(cs.size()!=nstr_len) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EBADMSG) {
         if(nstr_len-cs.size()!=res_plen) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ERANGE) {
         if(nstr_len-cs.size()!=res_plen) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else {
         WY_THROW( WyRet() );
       }

       // _strlen(Num&, WyCSeg&, const int&)
       radix=res_radix_i;
       if((r=cs.reset(nstr,nstr_len))!=Ok) {
         WY_THROW(r);
       }
       r=Wy::_strnum(num, cs, (const int)radix);
       
       if(radix!=res_radix_i) {
         std::cerr << "'radix' overwritten :" << radix << '\n';
         WY_THROW( WyRet() );
       }
       if(r!=res_reply) {
         std::cerr << "'res_reply' inconsistent: " << Wy::wrd(r).c_str() << '\n';
         WY_THROW( WyRet() );
       }
       if(num!=res_val) {
         std::cerr << "'res_val' inconsistent: " << (long long)num << '\n';
         WY_THROW( WyRet() );
       }
       if(r==Ok) {
         if(cs.size()!=0) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EINVAL) {
         if(cs.size()!=nstr_len) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ENOENT) {
         if(cs.size()!=nstr_len) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EBADMSG) {
         if(nstr_len-cs.size()!=res_plen) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ERANGE) {
         if(nstr_len-cs.size()!=res_plen) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else {
         WY_THROW( WyRet() );
       }

       // _strlen(Num&, WyCSeg&)
       if(res_radix_i==10) {
         radix=res_radix_i;
         if((r=cs.reset(nstr,nstr_len))!=Ok) {
           WY_THROW(r);
         }
         r=Wy::_strnum(num, cs);
         
         if(r!=res_reply) {
           std::cerr << "'res_reply' inconsistent: " << Wy::wrd(r).c_str() << '\n';
           WY_THROW( WyRet() );
         }
         if(num!=res_val) {
           std::cerr << "'res_val' inconsistent: " << (long long)num << '\n';
           WY_THROW( WyRet() );
         }
         if(r==Ok) {
           if(cs.size()!=0) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else if(r==Wym_EINVAL) {
           if(cs.size()!=nstr_len) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else if(r==Wym_ENOENT) {
           if(cs.size()!=nstr_len) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else if(r==Wym_EBADMSG) {
           if(nstr_len-cs.size()!=res_plen) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else if(r==Wym_ERANGE) {
           if(nstr_len-cs.size()!=res_plen) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else {
           WY_THROW( WyRet() );
         }
       }

       // _strlen(Num&, const WyCSeg&, int&)
       radix=res_radix_i;
       if((r=cs.reset(nstr,nstr_len))!=Ok) {
         WY_THROW(r);
       }
       r=Wy::_strnum(num, (const WyCSeg&)cs, radix);
       
       if(cs!=WyCSeg(nstr,nstr_len)) {
         std::cerr << "cs overwritten: " << WyStr(cs).c_str() << '\n';
         WY_THROW( WyRet() );
       }
       if(num!=res_val) {
         std::cerr << "'res_val' inconsistent: " << (long long)num << '\n';
         WY_THROW( WyRet() );
       }
       if(r==Ok) {
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EINVAL) {
       } else if(r==Wym_ENOENT) {
       } else if(r==Wym_EBADMSG) {
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ERANGE) {
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else {
         WY_THROW( WyRet() );
       }

       // _strlen(Num&, const WyCSeg&, const int&)
       radix=res_radix_i;
       if((r=cs.reset(nstr,nstr_len))!=Ok) {
         WY_THROW(r);
       }
       r=Wy::_strnum(num, (const WyCSeg&)cs, (const int&)radix);
       
       if(cs!=WyCSeg(nstr,nstr_len)) {
         std::cerr << "cs overwritten: " << WyStr(cs).c_str() << '\n';
         WY_THROW( WyRet() );
       }
       if(radix!=res_radix_i) {
         std::cerr << "'radix' overwritten :" << radix << '\n';
         WY_THROW( WyRet() );
       }
       if(num!=res_val) {
         std::cerr << "'res_val' inconsistent: " << (long long)num << '\n';
         WY_THROW( WyRet() );
       }
       if(r==Ok) {
       } else if(r==Wym_EINVAL) {
       } else if(r==Wym_ENOENT) {
       } else if(r==Wym_EBADMSG) {
       } else if(r==Wym_ERANGE) {
       } else {
         WY_THROW( WyRet() );
       }

       // _strlen(Num&, const WyCSeg&)
       if(res_radix_i==10) {
         radix=res_radix_i;
         if((r=cs.reset(nstr,nstr_len))!=Ok) {
           WY_THROW(r);
         }
         r=Wy::_strnum(num, (const WyCSeg&)cs);
         
         if(cs!=WyCSeg(nstr,nstr_len)) {
           std::cerr << "cs overwritten: " << WyStr(cs).c_str() << '\n';
           WY_THROW( WyRet() );
         }
         if(num!=res_val) {
           std::cerr << "'res_val' inconsistent: " << (long long)num << '\n';
           WY_THROW( WyRet() );
         }
         if(r==Ok) {
         } else if(r==Wym_EINVAL) {
         } else if(r==Wym_ENOENT) {
         } else if(r==Wym_EBADMSG) {
         } else if(r==Wym_ERANGE) {
         } else {
           WY_THROW( WyRet() );
         }
       }

     }
     catch(...) {
       std::cerr << "ChkItem_UnSigned\n";
       std::cerr << "Failed on {\"" << ((nstr==NULL)? "NULL" : nstr) << "\"," << nstr_len
             << ',' << Wy::wrd(res_reply).c_str() << ',' 
             << (long long)res_val << ',' << res_plen
              << ',' << res_radix_i << ',' << res_radix_o
             << "}\n";
       throw;
     };
};

template <typename NumT>
static void ck_nstr_unsigned(void)
{
 const ChkItem_UnSigned<NumT> chk_tab[]={
   // nstr slen  reply val plen rad_i rad_o
   {"0",0 ,Wym_ENOENT,0,0, 2, 2},
   {"0",0 ,Wym_ENOENT,0,0, 0, 2},

   // one char "?"
   {"0",1 ,        Ok,0,1, 2,2},
   {"1",1 ,        Ok,1,1, 2,2},
   {"7",1 ,Wym_ENOENT,0,0, 2,2},
   {"8",1 ,Wym_ENOENT,0,0, 2,2},
   {"9",1 ,Wym_ENOENT,0,0, 2,2},
   {"a",1 ,Wym_ENOENT,0,0, 2,2},
   {"e",1 ,Wym_ENOENT,0,0, 2,2},
   {"g",1 ,Wym_ENOENT,0,0, 2,2},
   {"X",1 ,Wym_ENOENT,0,0, 2,2},
   {"+",1 ,Wym_ENOENT,0,0, 2,2},
   {"-",1 ,Wym_ENOENT,0,0, 2,2},
   {".",1 ,Wym_ENOENT,0,0, 2,2},

   {"0",1 ,        Ok,0,1, 8,8},
   {"1",1 ,        Ok,1,1, 8,8},
   {"7",1 ,        Ok,7,1, 8,8},
   {"8",1 ,Wym_ENOENT,0,0, 8,8},
   {"9",1 ,Wym_ENOENT,0,0, 8,8},
   {"a",1 ,Wym_ENOENT,0,0, 8,8},
   {"e",1 ,Wym_ENOENT,0,0, 8,8},
   {"g",1 ,Wym_ENOENT,0,0, 8,8},
   {"X",1 ,Wym_ENOENT,0,0, 8,8},
   {"+",1 ,Wym_ENOENT,0,0, 8,8},
   {"-",1 ,Wym_ENOENT,0,0, 8,8},
   {".",1 ,Wym_ENOENT,0,0, 8,8},

   {"0",1 ,        Ok,0,1,10,10},
   {"1",1 ,        Ok,1,1,10,10},
   {"7",1 ,        Ok,7,1,10,10},
   {"8",1 ,        Ok,8,1,10,10},
   {"9",1 ,        Ok,9,1,10,10},
   {"a",1 ,Wym_ENOENT,0,0,10,10},
   {"e",1 ,Wym_ENOENT,0,0,10,10},
   {"g",1 ,Wym_ENOENT,0,0,10,10},
   {"X",1 ,Wym_ENOENT,0,0,10,10},
   {"+",1 ,Wym_ENOENT,0,0,10,10},
   {"-",1 ,Wym_ENOENT,0,0,10,10},
   {".",1 ,Wym_ENOENT,0,0,10,10},

   {"0",1 ,        Ok,0x0,1,16,16},
   {"1",1 ,        Ok,0x1,1,16,16},
   {"7",1 ,        Ok,0x7,1,16,16},
   {"8",1 ,        Ok,0x8,1,16,16},
   {"9",1 ,        Ok,0x9,1,16,16},
   {"a",1 ,        Ok,0xa,1,16,16},
   {"e",1 ,        Ok,0xe,1,16,16},
   {"g",1 ,Wym_ENOENT,0x0,0,16,16},
   {"X",1 ,Wym_ENOENT,0x0,0,16,16},
   {"+",1 ,Wym_ENOENT,0x0,0,16,16},
   {"-",1 ,Wym_ENOENT,0x0,0,16,16},
   {".",1 ,Wym_ENOENT,0x0,0,16,16},

   {"0",1 ,        Ok,0,1, 0,10},
   {"1",1 ,        Ok,1,1, 0,10},
   {"7",1 ,        Ok,7,1, 0,10},
   {"8",1 ,        Ok,8,1, 0,10},
   {"9",1 ,        Ok,9,1, 0,10},
   {"a",1 ,Wym_ENOENT,0,0, 0,10},   // radix not defined but for verify implement
   {"e",1 ,Wym_ENOENT,0,0, 0,10},   // radix not defined but for verify implement
   {"g",1 ,Wym_ENOENT,0,0, 0,10},   // radix not defined but for verify implement
   {"X",1 ,Wym_ENOENT,0,0, 0,10},   // radix not defined but for verify implement
   {"+",1 ,Wym_ENOENT,0,0, 0,0},    // radix not defined but for verify implement
   {"-",1 ,Wym_ENOENT,0,0, 0,0},    // radix not defined but for verify implement
   {".",1 ,Wym_ENOENT,0,0, 0,10},   // radix not defined but for verify implement
 
   // two char's "+?"
   {"+0",2 ,        Ok,0,2, 2,2},
   {"+1",2 ,        Ok,1,2, 2,2},
   {"+7",2 ,Wym_ENOENT,0,0, 2,2},
   {"+8",2 ,Wym_ENOENT,0,0, 2,2},
   {"+9",2 ,Wym_ENOENT,0,0, 2,2},
   {"+a",2 ,Wym_ENOENT,0,0, 2,2},
   {"+e",2 ,Wym_ENOENT,0,0, 2,2},
   {"+g",2 ,Wym_ENOENT,0,0, 2,2},
   {"+X",2 ,Wym_ENOENT,0,0, 2,2},
   {"++",2 ,Wym_ENOENT,0,0, 2,2},
   {"+-",2 ,Wym_ENOENT,0,0, 2,2},
   {"+.",2 ,Wym_ENOENT,0,0, 2,2},

   {"+0",2 ,        Ok,0,2, 8,8},
   {"+1",2 ,        Ok,1,2, 8,8},
   {"+7",2 ,        Ok,7,2, 8,8},
   {"+8",2 ,Wym_ENOENT,0,0, 8,8},
   {"+9",2 ,Wym_ENOENT,0,0, 8,8},
   {"+a",2 ,Wym_ENOENT,0,0, 8,8},
   {"+e",2 ,Wym_ENOENT,0,0, 8,8},
   {"+g",2 ,Wym_ENOENT,0,0, 8,8},
   {"+X",2 ,Wym_ENOENT,0,0, 8,8},
   {"++",2 ,Wym_ENOENT,0,0, 8,8},
   {"+-",2 ,Wym_ENOENT,0,0, 8,8},
   {"+.",2 ,Wym_ENOENT,0,0, 8,8},

   {"+0",2 ,        Ok,0,2,10,10},
   {"+1",2 ,        Ok,1,2,10,10},
   {"+7",2 ,        Ok,7,2,10,10},
   {"+8",2 ,        Ok,8,2,10,10},
   {"+9",2 ,        Ok,9,2,10,10},
   {"+a",2 ,Wym_ENOENT,0,0,10,10},
   {"+e",2 ,Wym_ENOENT,0,0,10,10},
   {"+g",2 ,Wym_ENOENT,0,0,10,10},
   {"+X",2 ,Wym_ENOENT,0,0,10,10},
   {"++",2 ,Wym_ENOENT,0,0,10,10},
   {"+-",2 ,Wym_ENOENT,0,0,10,10},
   {"+.",2 ,Wym_ENOENT,0,0,10,10},

   {"+0",2 ,        Ok,  0,2,16,16},
   {"+1",2 ,        Ok,  1,2,16,16},
   {"+7",2 ,        Ok,  7,2,16,16},
   {"+8",2 ,        Ok,  8,2,16,16},
   {"+9",2 ,        Ok,  9,2,16,16},
   {"+a",2 ,        Ok,0xa,2,16,16},
   {"+e",2 ,        Ok,0xe,2,16,16},
   {"+g",2 ,Wym_ENOENT,  0,0,16,16},
   {"+X",2 ,Wym_ENOENT,  0,0,16,16},
   {"++",2 ,Wym_ENOENT,  0,0,16,16},
   {"+-",2 ,Wym_ENOENT,  0,0,16,16},
   {"+.",2 ,Wym_ENOENT,  0,0,16,16},

   {"+0",2 ,        Ok,0,2, 0,10},
   {"+1",2 ,        Ok,1,2, 0,10},
   {"+7",2 ,        Ok,7,2, 0,10},
   {"+8",2 ,        Ok,8,2, 0,10},
   {"+9",2 ,        Ok,9,2, 0,10},
   {"+a",2 ,Wym_ENOENT,0,0, 0,10},
   {"+e",2 ,Wym_ENOENT,0,0, 0,10},
   {"+g",2 ,Wym_ENOENT,0,0, 0,10},
   {"+X",2 ,Wym_ENOENT,0,0, 0,10},
   {"++",2 ,Wym_ENOENT,0,0, 0,10},
   {"+-",2 ,Wym_ENOENT,0,0, 0,10},
   {"+.",2 ,Wym_ENOENT,0,0, 0,10},

   // two char's "-?"
   {"-0",2 ,Wym_ENOENT, 0,0, 2,2},
   {"-1",2 ,Wym_ENOENT, 0,0, 2,2},
   {"-7",2 ,Wym_ENOENT, 0,0, 2,2},
   {"-8",2 ,Wym_ENOENT, 0,0, 2,2},
   {"-9",2 ,Wym_ENOENT, 0,0, 2,2},
   {"-a",2 ,Wym_ENOENT, 0,0, 2,2},
   {"-e",2 ,Wym_ENOENT, 0,0, 2,2},
   {"-g",2 ,Wym_ENOENT, 0,0, 2,2},
   {"-X",2 ,Wym_ENOENT, 0,0, 2,2},
   {"-+",2 ,Wym_ENOENT, 0,0, 2,2},
   {"-.",2 ,Wym_ENOENT, 0,0, 2,2},

   {"-0",2 ,Wym_ENOENT, 0,0, 8,8},
   {"-1",2 ,Wym_ENOENT, 0,0, 8,8},
   {"-7",2 ,Wym_ENOENT, 0,0, 8,8},
   {"-8",2 ,Wym_ENOENT, 0,0, 8,8},
   {"-9",2 ,Wym_ENOENT, 0,0, 8,8},
   {"-a",2 ,Wym_ENOENT, 0,0, 8,8},
   {"-e",2 ,Wym_ENOENT, 0,0, 8,8},
   {"-g",2 ,Wym_ENOENT, 0,0, 8,8},
   {"-X",2 ,Wym_ENOENT, 0,0, 8,8},
   {"-+",2 ,Wym_ENOENT, 0,0, 8,8},
   {"-.",2 ,Wym_ENOENT, 0,0, 8,8},

   {"-0",2 ,Wym_ENOENT, 0,0,10,10},
   {"-1",2 ,Wym_ENOENT, 0,0,10,10},
   {"-7",2 ,Wym_ENOENT, 0,0,10,10},
   {"-8",2 ,Wym_ENOENT, 0,0,10,10},
   {"-9",2 ,Wym_ENOENT, 0,0,10,10},
   {"-a",2 ,Wym_ENOENT, 0,0,10,10},
   {"-e",2 ,Wym_ENOENT, 0,0,10,10},
   {"-g",2 ,Wym_ENOENT, 0,0,10,10},
   {"-X",2 ,Wym_ENOENT, 0,0,10,10},
   {"-+",2 ,Wym_ENOENT, 0,0,10,10},
   {"-.",2 ,Wym_ENOENT, 0,0,10,10},

   {"-0",2 ,Wym_ENOENT,   0,0,16,16},
   {"-1",2 ,Wym_ENOENT,   0,0,16,16},
   {"-7",2 ,Wym_ENOENT,   0,0,16,16},
   {"-8",2 ,Wym_ENOENT,   0,0,16,16},
   {"-9",2 ,Wym_ENOENT,   0,0,16,16},
   {"-a",2 ,Wym_ENOENT,   0,0,16,16},
   {"-e",2 ,Wym_ENOENT,   0,0,16,16},
   {"-g",2 ,Wym_ENOENT,   0,0,16,16},
   {"-X",2 ,Wym_ENOENT,   0,0,16,16},
   {"-+",2 ,Wym_ENOENT,   0,0,16,16},
   {"--",2 ,Wym_ENOENT,   0,0,16,16},
   {"-.",2 ,Wym_ENOENT,   0,0,16,16},

   {"-0",2 ,Wym_ENOENT, 0,0, 0,10},
   {"-1",2 ,Wym_ENOENT, 0,0, 0,10},
   {"-7",2 ,Wym_ENOENT, 0,0, 0,10},
   {"-8",2 ,Wym_ENOENT, 0,0, 0,10},
   {"-9",2 ,Wym_ENOENT, 0,0, 0,10},
   {"-a",2 ,Wym_ENOENT, 0,0, 0,10},
   {"-e",2 ,Wym_ENOENT, 0,0, 0,10},
   {"-g",2 ,Wym_ENOENT, 0,0, 0,10},
   {"-X",2 ,Wym_ENOENT, 0,0, 0,10},
   {"-+",2 ,Wym_ENOENT, 0,0, 0,10},
   {"--",2 ,Wym_ENOENT, 0,0, 0,10},
   {"-.",2 ,Wym_ENOENT, 0,0, 0,10},

   // two char's ".?"
   {".0",2 ,Wym_ENOENT,0,0, 2,2},
   {".1",2 ,Wym_ENOENT,0,0, 2,2},
   {".7",2 ,Wym_ENOENT,0,0, 2,2},
   {".8",2 ,Wym_ENOENT,0,0, 2,2},
   {".9",2 ,Wym_ENOENT,0,0, 2,2},
   {".a",2 ,Wym_ENOENT,0,0, 2,2},
   {".e",2 ,Wym_ENOENT,0,0, 2,2},
   {".g",2 ,Wym_ENOENT,0,0, 2,2},
   {".X",2 ,Wym_ENOENT,0,0, 2,2},
   {".+",2 ,Wym_ENOENT,0,0, 2,2},
   {".-",2 ,Wym_ENOENT,0,0, 2,2},
   {"..",2 ,Wym_ENOENT,0,0, 2,2},

   {".0",2 ,Wym_ENOENT,0,0, 8,8},
   {".1",2 ,Wym_ENOENT,0,0, 8,8},
   {".7",2 ,Wym_ENOENT,0,0, 8,8},
   {".8",2 ,Wym_ENOENT,0,0, 8,8},
   {".9",2 ,Wym_ENOENT,0,0, 8,8},
   {".a",2 ,Wym_ENOENT,0,0, 8,8},
   {".e",2 ,Wym_ENOENT,0,0, 8,8},
   {".g",2 ,Wym_ENOENT,0,0, 8,8},
   {".X",2 ,Wym_ENOENT,0,0, 8,8},
   {".+",2 ,Wym_ENOENT,0,0, 8,8},
   {".-",2 ,Wym_ENOENT,0,0, 8,8},
   {"..",2 ,Wym_ENOENT,0,0, 8,8},

   {".0",2 ,Wym_ENOENT,0,0,10,10},
   {".1",2 ,Wym_ENOENT,0,0,10,10},
   {".7",2 ,Wym_ENOENT,0,0,10,10},
   {".8",2 ,Wym_ENOENT,0,0,10,10},
   {".9",2 ,Wym_ENOENT,0,0,10,10},
   {".a",2 ,Wym_ENOENT,0,0,10,10},
   {".e",2 ,Wym_ENOENT,0,0,10,10},
   {".g",2 ,Wym_ENOENT,0,0,10,10},
   {".X",2 ,Wym_ENOENT,0,0,10,10},
   {".+",2 ,Wym_ENOENT,0,0,10,10},
   {".-",2 ,Wym_ENOENT,0,0,10,10},
   {"..",2 ,Wym_ENOENT,0,0,10,10},

   {".0",2 ,Wym_ENOENT,  0,0,16,16},
   {".1",2 ,Wym_ENOENT,  0,0,16,16},
   {".7",2 ,Wym_ENOENT,  0,0,16,16},
   {".8",2 ,Wym_ENOENT,  0,0,16,16},
   {".9",2 ,Wym_ENOENT,  0,0,16,16},
   {".a",2 ,Wym_ENOENT,  0,0,16,16},
   {".e",2 ,Wym_ENOENT,  0,0,16,16},
   {".g",2 ,Wym_ENOENT,  0,0,16,16},
   {".X",2 ,Wym_ENOENT,  0,0,16,16},
   {".+",2 ,Wym_ENOENT,  0,0,16,16},
   {".-",2 ,Wym_ENOENT,  0,0,16,16},
   {"..",2 ,Wym_ENOENT,  0,0,16,16},

   {".0",2 ,Wym_ENOENT,0,0, 0,10},
   {".1",2 ,Wym_ENOENT,0,0, 0,10},
   {".7",2 ,Wym_ENOENT,0,0, 0,10},
   {".8",2 ,Wym_ENOENT,0,0, 0,10},
   {".9",2 ,Wym_ENOENT,0,0, 0,10},
   {".a",2 ,Wym_ENOENT,0,0, 0,10},
   {".e",2 ,Wym_ENOENT,0,0, 0,10},
   {".g",2 ,Wym_ENOENT,0,0, 0,10},
   {".X",2 ,Wym_ENOENT,0,0, 0,10},
   {".+",2 ,Wym_ENOENT,0,0, 0,10},
   {".-",2 ,Wym_ENOENT,0,0, 0,10},
   {"..",2 ,Wym_ENOENT,0,0, 0,10},

   // two char's "0?"
   {"00",2 ,         Ok,0,2, 2,2},
   {"01",2 ,         Ok,1,2, 2,2},
   {"07",2 ,Wym_EBADMSG,0,1, 2,2},
   {"08",2 ,Wym_EBADMSG,0,1, 2,2},
   {"09",2 ,Wym_EBADMSG,0,1, 2,2},
   {"0a",2 ,Wym_EBADMSG,0,1, 2,2},
   {"0e",2 ,Wym_EBADMSG,0,1, 2,2},
   {"0g",2 ,Wym_EBADMSG,0,1, 2,2},
   {"0X",2 ,Wym_EBADMSG,0,1, 2,2},
   {"0+",2 ,Wym_EBADMSG,0,1, 2,2},
   {"0-",2 ,Wym_EBADMSG,0,1, 2,2},
   {"0.",2 ,Wym_EBADMSG,0,1, 2,2},

   {"00",2 ,         Ok,0,2, 8,8},
   {"01",2 ,         Ok,1,2, 8,8},
   {"07",2 ,         Ok,7,2, 8,8},
   {"08",2 ,Wym_EBADMSG,0,1, 8,8},
   {"09",2 ,Wym_EBADMSG,0,1, 8,8},
   {"0a",2 ,Wym_EBADMSG,0,1, 8,8},
   {"0e",2 ,Wym_EBADMSG,0,1, 8,8},
   {"0g",2 ,Wym_EBADMSG,0,1, 8,8},
   {"0X",2 ,Wym_EBADMSG,0,1, 8,8},
   {"0+",2 ,Wym_EBADMSG,0,1, 8,8},
   {"0-",2 ,Wym_EBADMSG,0,1, 8,8},
   {"0.",2 ,Wym_EBADMSG,0,1, 8,8},

   {"00",2 ,         Ok,0,2,10,10},
   {"01",2 ,         Ok,1,2,10,10},
   {"07",2 ,         Ok,7,2,10,10},
   {"08",2 ,         Ok,8,2,10,10},
   {"09",2 ,         Ok,9,2,10,10},
   {"0a",2 ,Wym_EBADMSG,0,1,10,10},
   {"0e",2 ,Wym_EBADMSG,0,1,10,10},
   {"0g",2 ,Wym_EBADMSG,0,1,10,10},
   {"0X",2 ,Wym_EBADMSG,0,1,10,10},
   {"0+",2 ,Wym_EBADMSG,0,1,10,10},
   {"0-",2 ,Wym_EBADMSG,0,1,10,10},
   {"0.",2 ,Wym_EBADMSG,0,1,10,10},

   {"00",2 ,         Ok,  0,2,16,16},
   {"01",2 ,         Ok,  1,2,16,16},
   {"07",2 ,         Ok,  7,2,16,16},
   {"08",2 ,         Ok,  8,2,16,16},
   {"09",2 ,         Ok,  9,2,16,16},
   {"0a",2 ,         Ok,0xa,2,16,16},
   {"0e",2 ,         Ok,0xe,2,16,16},
   {"0g",2 ,Wym_EBADMSG,  0,1,16,16},
   {"0X",2 ,Wym_EBADMSG,  0,1,16,16},
   {"0+",2 ,Wym_EBADMSG,  0,1,16,16},
   {"0-",2 ,Wym_EBADMSG,  0,1,16,16},
   {"0.",2 ,Wym_EBADMSG,  0,1,16,16},

   {"00",2 ,         Ok,0,2, 0,8},
   {"01",2 ,         Ok,1,2, 0,8},
   {"07",2 ,         Ok,7,2, 0,8},
   {"08",2 ,Wym_EBADMSG,0,1, 0,10},
   {"09",2 ,Wym_EBADMSG,0,1, 0,10},
   {"0a",2 ,Wym_EBADMSG,0,1, 0,10},
   {"0e",2 ,Wym_EBADMSG,0,1, 0,10},
   {"0g",2 ,Wym_EBADMSG,0,1, 0,10},
   {"0X",2 ,Wym_EBADMSG,0,1, 0,10},
   {"0+",2 ,Wym_EBADMSG,0,1, 0,10},
   {"0-",2 ,Wym_EBADMSG,0,1, 0,10},
   {"0.",2 ,Wym_EBADMSG,0,1, 0,10},

   // two char's "1?"
   {"10",2 ,         Ok,2,2, 2,2},
   {"11",2 ,         Ok,3,2, 2,2},
   {"17",2 ,Wym_EBADMSG,1,1, 2,2},
   {"18",2 ,Wym_EBADMSG,1,1, 2,2},
   {"19",2 ,Wym_EBADMSG,1,1, 2,2},
   {"1a",2 ,Wym_EBADMSG,1,1, 2,2},
   {"1e",2 ,Wym_EBADMSG,1,1, 2,2},
   {"1g",2 ,Wym_EBADMSG,1,1, 2,2},
   {"1X",2 ,Wym_EBADMSG,1,1, 2,2},
   {"1+",2 ,Wym_EBADMSG,1,1, 2,2},
   {"1-",2 ,Wym_EBADMSG,1,1, 2,2},
   {"1.",2 ,Wym_EBADMSG,1,1, 2,2},

   {"10",2 ,         Ok, 8,2, 8,8},
   {"11",2 ,         Ok, 9,2, 8,8},
   {"17",2 ,         Ok,15,2, 8,8},
   {"18",2 ,Wym_EBADMSG, 1,1, 8,8},
   {"19",2 ,Wym_EBADMSG, 1,1, 8,8},
   {"1a",2 ,Wym_EBADMSG, 1,1, 8,8},
   {"1e",2 ,Wym_EBADMSG, 1,1, 8,8},
   {"1g",2 ,Wym_EBADMSG, 1,1, 8,8},
   {"1X",2 ,Wym_EBADMSG, 1,1, 8,8},
   {"1+",2 ,Wym_EBADMSG, 1,1, 8,8},
   {"1-",2 ,Wym_EBADMSG, 1,1, 8,8},
   {"1.",2 ,Wym_EBADMSG, 1,1, 8,8},

   {"10",2 ,         Ok,10,2,10,10},
   {"11",2 ,         Ok,11,2,10,10},
   {"17",2 ,         Ok,17,2,10,10},
   {"18",2 ,         Ok,18,2,10,10},
   {"19",2 ,         Ok,19,2,10,10},
   {"1a",2 ,Wym_EBADMSG, 1,1,10,10},
   {"1e",2 ,Wym_EBADMSG, 1,1,10,10},
   {"1g",2 ,Wym_EBADMSG, 1,1,10,10},
   {"1X",2 ,Wym_EBADMSG, 1,1,10,10},
   {"1+",2 ,Wym_EBADMSG, 1,1,10,10},
   {"1-",2 ,Wym_EBADMSG, 1,1,10,10},
   {"1.",2 ,Wym_EBADMSG, 1,1,10,10},

   {"10",2 ,         Ok,0x10,2,16,16},
   {"11",2 ,         Ok,0x11,2,16,16},
   {"17",2 ,         Ok,0x17,2,16,16},
   {"18",2 ,         Ok,0x18,2,16,16},
   {"19",2 ,         Ok,0x19,2,16,16},
   {"1a",2 ,         Ok,0x1a,2,16,16},
   {"1e",2 ,         Ok,0x1e,2,16,16},
   {"1g",2 ,Wym_EBADMSG,   1,1,16,16},
   {"1X",2 ,Wym_EBADMSG,   1,1,16,16},
   {"1+",2 ,Wym_EBADMSG,   1,1,16,16},
   {"1-",2 ,Wym_EBADMSG,   1,1,16,16},
   {"1.",2 ,Wym_EBADMSG,   1,1,16,16},

   {"10",2 ,         Ok,10,2, 0,10},
   {"11",2 ,         Ok,11,2, 0,10},
   {"17",2 ,         Ok,17,2, 0,10},
   {"18",2 ,         Ok,18,2, 0,10},
   {"19",2 ,         Ok,19,2, 0,10},
   {"1a",2 ,Wym_EBADMSG, 1,1, 0,10},
   {"1e",2 ,Wym_EBADMSG, 1,1, 0,10},
   {"1g",2 ,Wym_EBADMSG, 1,1, 0,10},
   {"1X",2 ,Wym_EBADMSG, 1,1, 0,10},
   {"1+",2 ,Wym_EBADMSG, 1,1, 0,10},
   {"1-",2 ,Wym_EBADMSG, 1,1, 0,10},
   {"1.",2 ,Wym_EBADMSG, 1,1, 0,10},

   // ".0?"
   {".00",3 ,Wym_ENOENT,0,0, 2,2},
   {".01",3 ,Wym_ENOENT,0,0, 2,2},
   {".07",3 ,Wym_ENOENT,0,0, 2,2},
   {".08",3 ,Wym_ENOENT,0,0, 2,2},
   {".09",3 ,Wym_ENOENT,0,0, 2,2},
   {".0a",3 ,Wym_ENOENT,0,0, 2,2},
   {".0e",3 ,Wym_ENOENT,0,0, 2,2},
   {".0g",3 ,Wym_ENOENT,0,0, 2,2},
   {".0X",3 ,Wym_ENOENT,0,0, 2,2},
   {".0+",3 ,Wym_ENOENT,0,0, 2,2},
   {".0-",3 ,Wym_ENOENT,0,0, 2,2},
   {".0.",3 ,Wym_ENOENT,0,0, 2,2},

   {".00",3 ,Wym_ENOENT, 0,0, 8,8},
   {".01",3 ,Wym_ENOENT, 0,0, 8,8},
   {".07",3 ,Wym_ENOENT, 0,0, 8,8},
   {".08",3 ,Wym_ENOENT, 0,0, 8,8},
   {".09",3 ,Wym_ENOENT, 0,0, 8,8},
   {".0a",3 ,Wym_ENOENT, 0,0, 8,8},
   {".0e",3 ,Wym_ENOENT, 0,0, 8,8},
   {".0g",3 ,Wym_ENOENT, 0,0, 8,8},
   {".0X",3 ,Wym_ENOENT, 0,0, 8,8},
   {".0+",3 ,Wym_ENOENT, 0,0, 8,8},
   {".0-",3 ,Wym_ENOENT, 0,0, 8,8},
   {".0.",3 ,Wym_ENOENT, 0,0, 8,8},

   {".00",3 ,Wym_ENOENT, 0,0,10,10},
   {".01",3 ,Wym_ENOENT, 0,0,10,10},
   {".07",3 ,Wym_ENOENT, 0,0,10,10},
   {".08",3 ,Wym_ENOENT, 0,0,10,10},
   {".09",3 ,Wym_ENOENT, 0,0,10,10},
   {".0a",3 ,Wym_ENOENT, 0,0,10,10},
   {".0e",3 ,Wym_ENOENT, 0,0,10,10},
   {".0g",3 ,Wym_ENOENT, 0,0,10,10},
   {".0X",3 ,Wym_ENOENT, 0,0,10,10},
   {".0+",3 ,Wym_ENOENT, 0,0,10,10},
   {".0-",3 ,Wym_ENOENT, 0,0,10,10},
   {".0.",3 ,Wym_ENOENT, 0,0,10,10},

   {".00",3 ,Wym_ENOENT,  0,0,16,16},
   {".01",3 ,Wym_ENOENT,  0,0,16,16},
   {".07",3 ,Wym_ENOENT,  0,0,16,16},
   {".08",3 ,Wym_ENOENT,  0,0,16,16},
   {".09",3 ,Wym_ENOENT,  0,0,16,16},
   {".0a",3 ,Wym_ENOENT,  0,0,16,16},
   {".0e",3 ,Wym_ENOENT,  0,0,16,16},
   {".0g",3 ,Wym_ENOENT,  0,0,16,16},
   {".0X",3 ,Wym_ENOENT,  0,0,16,16},
   {".0+",3 ,Wym_ENOENT,  0,0,16,16},
   {".0-",3 ,Wym_ENOENT,  0,0,16,16},
   {".0.",3 ,Wym_ENOENT,  0,0,16,16},

   {".00",3 ,Wym_ENOENT,0x0,0, 0,10},
   {".01",3 ,Wym_ENOENT,0x0,0, 0,10},
   {".07",3 ,Wym_ENOENT,0x0,0, 0,10},
   {".08",3 ,Wym_ENOENT,0x0,0, 0,10},
   {".09",3 ,Wym_ENOENT,0x0,0, 0,10},
   {".0a",3 ,Wym_ENOENT,0x0,0, 0,10},
   {".0e",3 ,Wym_ENOENT,0x0,0, 0,10},
   {".0g",3 ,Wym_ENOENT,  0,0, 0,10},
   {".0X",3 ,Wym_ENOENT,  0,0, 0,10},
   {".0+",3 ,Wym_ENOENT,  0,0, 0,10},
   {".0-",3 ,Wym_ENOENT,  0,0, 0,10},
   {".0.",3 ,Wym_ENOENT,  0,0, 0,10},

   // "0x?"
   {"0x0",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0x1",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0x7",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0x8",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0x9",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0xa",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0xe",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0xg",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0xX",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0x+",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0x-",3 ,Wym_EBADMSG,0,1, 2,2},
   {"0x.",3 ,Wym_EBADMSG,0,1, 2,2},

   {"0x0",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0x1",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0x7",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0x8",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0x9",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0xa",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0xe",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0xg",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0xX",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0x+",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0x-",3 ,Wym_EBADMSG, 0,1, 8,8},
   {"0x.",3 ,Wym_EBADMSG, 0,1, 8,8},

   {"0x0",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0x1",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0x7",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0x8",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0x9",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0xa",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0xe",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0xg",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0xX",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0x+",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0x-",3 ,Wym_EBADMSG, 0,1,10,10},
   {"0x.",3 ,Wym_EBADMSG, 0,1,10,10},

   {"0x0",3 ,         Ok,0x0,3,16,16},
   {"0x1",3 ,         Ok,0x1,3,16,16},
   {"0x7",3 ,         Ok,0x7,3,16,16},
   {"0x8",3 ,         Ok,0x8,3,16,16},
   {"0x9",3 ,         Ok,0x9,3,16,16},
   {"0xa",3 ,         Ok,0xa,3,16,16},
   {"0xe",3 ,         Ok,0xe,3,16,16},
   {"0xg",3 ,Wym_EBADMSG,  0,1,16,16},
   {"0xX",3 ,Wym_EBADMSG,  0,1,16,16},
   {"0x+",3 ,Wym_EBADMSG,  0,1,16,16},
   {"0x-",3 ,Wym_EBADMSG,  0,1,16,16},
   {"0x.",3 ,Wym_EBADMSG,  0,1,16,16},

   {"0x0",3 ,         Ok,0x0,3, 0,16},
   {"0x1",3 ,         Ok,0x1,3, 0,16},
   {"0x7",3 ,         Ok,0x7,3, 0,16},
   {"0x8",3 ,         Ok,0x8,3, 0,16},
   {"0x9",3 ,         Ok,0x9,3, 0,16},
   {"0xa",3 ,         Ok,0xa,3, 0,16},
   {"0xe",3 ,         Ok,0xe,3, 0,16},
   {"0xg",3 ,Wym_EBADMSG,  0,1, 0,10},
   {"0xX",3 ,Wym_EBADMSG,  0,1, 0,10},
   {"0x+",3 ,Wym_EBADMSG,  0,1, 0,10},
   {"0x-",3 ,Wym_EBADMSG,  0,1, 0,10},
   {"0x.",3 ,Wym_EBADMSG,  0,1, 0,10},

   // mis
   {"0x.0.2",6,Wym_EBADMSG,0,1,16,16},
   {"0x.0.2",6,Wym_EBADMSG,0,1,0,10},
   {"0.0.2",5,Wym_EBADMSG,0,1,10,10},
   {"0.0.2",5,Wym_EBADMSG,0,1,0,10},

   {"0x.2",4,Wym_EBADMSG,0,1,16,16},
   {"0x.2.",5,Wym_EBADMSG,0,1,16,16},
   {"0x.2.2",6,Wym_EBADMSG,0,1,16,16},
   {"0x.2",4,Wym_EBADMSG,0,1,0,10},
   {"0x.2.",5,Wym_EBADMSG,0,1,0,10},
   {"0x.2.2",6,Wym_EBADMSG,0,1,0,10},
   {"0.1.",4,Wym_EBADMSG,0,1,10,10},
   {"0.1.2",5,Wym_EBADMSG,0,1,10,10},
   {"0.1.",4,Wym_EBADMSG,0,1,0,10},
   {"0.1.2",5,Wym_EBADMSG,0,1,0,10},

   {"-.",2 ,Wym_ENOENT,0,0,36,36},
   {"+.9",3,Wym_ENOENT,0,0,0,10},
   {"-.9",3,Wym_ENOENT,0,0,36,36},
   {"1",1 ,Wym_EINVAL,0,0,37,37},
   {"1",1 ,Wym_EINVAL,0,0, 1, 1},
   {"0",1 ,        Ok,0,1, 0,10},
   {"x",1,Wym_ENOENT,0,0, 2,2},
   {"0x",2,Wym_EBADMSG,0,1, 0,10},
   {"0xx",3,Wym_EBADMSG,0,1, 0,10},
   {"0xg",3,Wym_EBADMSG,0,1, 0,10},
   {"0",1 ,        Ok,0,1, 16,16},
   {"x",1,Wym_ENOENT,0,0, 2,2},
   {"0x",2,Wym_EBADMSG,0,1, 16,16},
   {"0xx",3,Wym_EBADMSG,0,1,16,16},
   {"0xg",3,Wym_EBADMSG,0,1,16,16},
   {"+0", 2,         Ok,0,2,16,16},
   {"-0", 2,Wym_ENOENT,0,0,16,16},
   {"+0", 2,         Ok,0,2, 0,10},
   {"-0", 2,Wym_ENOENT,0,0, 0,10}, 
   {"0x.",3,Wym_EBADMSG,0,1, 0,10},
   {"0x.x",4,Wym_EBADMSG,0,1, 0,10},
   {"0x.g",4,Wym_EBADMSG,0,1, 0,10}, 
   {"0x.a",4,Wym_EBADMSG,0,1, 0,10},
   {".0",2 ,Wym_ENOENT,0,0, 16,16},
   {"0x.",3,Wym_EBADMSG,0,1, 16,16},
   {"0x.x",4,Wym_EBADMSG,0,1,16,16},
   {"0x.g",4,Wym_EBADMSG,0,1,16,16},
   {"0.",2,Wym_EBADMSG,0,1, 2,2},
   {"+9.",3,Wym_EBADMSG, 9,2, 0,10},
   {"-9.",3,Wym_ENOENT,0,0,10,10},
   {"+.x",3,Wym_ENOENT,0,0,10,10},
   {"-0.1a",5,Wym_ENOENT,0,0,10,10},
   {"-.1a", 4,Wym_ENOENT,0,0,10,10},

   {"0",1,Ok,0,1,10,10},
   {"31",2,Ok,31,2,10,10},
   {"031",3,Ok,031,3,0,8},
   {"00.03",5,Wym_EBADMSG,0,2,0,8},
   {"00.03",5,Wym_EBADMSG,0,2,10,10},
   {"0x00.03",7,Wym_EBADMSG,0,4,0,16},
   {"00x0",4,Wym_EBADMSG,0,2,0,8},    // 00= zero of radix 8

   {"+.0",3,Wym_ENOENT,0,0,10,10}, 
   {"-.0",3,Wym_ENOENT,0,0,0,10},
   {".00",3,Wym_ENOENT,0,0,10,10},
   {"+.00",4,Wym_ENOENT,0,0,0,10},
   {"-.00",4,Wym_ENOENT,0,0,10,10},
   {"0.0",3,Wym_EBADMSG,0,1,10,10},
   {"0.0",3,Wym_EBADMSG,0,1,0,10},
   {"+0.0",4,Wym_EBADMSG,0,2,10,10},
   {"-0.0",4,Wym_ENOENT,0,0,0,10},
   {"0.010",5,Wym_EBADMSG,0,1,10,10},
   {"-0.010",6,Wym_ENOENT,0,0,0,10},
   {"123",3,Ok,123,3,0,10},
   {"+123",4,Ok,123,4,0,10},
   {"+0123",5,Ok,0123,5,0,8},
   {"-0123",5,Wym_ENOENT,0,0,10,10},
   {".123",4,Wym_ENOENT,0,0,10,10},
   {"+.123",5,Wym_ENOENT,0,0,10,10},
   {"-.123",5,Wym_ENOENT,0,0, 0,10},
   {".120",4,Wym_ENOENT,0,0,10,10},
   {"+.120",5,Wym_ENOENT,0,0,10,10},
   {"-.120",5,Wym_ENOENT,0,0, 0,10},
   {"12.0",4,Wym_EBADMSG,12,2, 0,10},
   {"+12.0",5,Wym_EBADMSG,12,3,10,10},
   {"-12.0",5,Wym_ENOENT,0,0, 0,10},
   {"-012.3",6,Wym_ENOENT,0,0,10,10},
   {"-012.3",6,Wym_ENOENT,0,0, 0, 8},
   {"+0012.3",7,Wym_EBADMSG, 12,5,10,10}, 
   {"1111111.1111111",15,Wym_EBADMSG,127,7,2,2}, 
   {"-1111111.1111111",16,Wym_ENOENT,0,0,2,2}, 
   {"177.177",7,Wym_EBADMSG,0177,3,8,8}, 
   {"-177.177",8,Wym_ENOENT,0,0,8,8}, 
   {"-7f.7f",6,Wym_ENOENT,0,0,16,16}, 
   {"-0x7f.7f",8,Wym_ENOENT,0,0,0,16}, 
   {"+000000000000000000012.100",26,Wym_EBADMSG,12,22,10,10}, 
   {"+000000000000000000012.100",26,Wym_EBADMSG,5,22,3,3}, 
 };

 for(size_t i=0; i<sizeof(chk_tab)/sizeof(ChkItem_UnSigned<NumT>); ++i) {
   chk_tab[i].verify();
 }

 {
   std::ostringstream os;
   const NumT NumMax=std::numeric_limits<NumT>::max();
   const NumT NumMin=std::numeric_limits<NumT>::min();
   std::string StrMax10,StrMin10;
   std::string StrMax8;
   std::string StrMax16;

   os.str("");
   os << std::hex << static_cast<unsigned long long>(NumMax);
   StrMax16=os.str();

   os.str("");
   os << std::dec << static_cast<unsigned long long>(NumMax);
   StrMax10=os.str();

   os.str("");
   os << std::dec << static_cast<unsigned long long>(NumMin);
   StrMin10=os.str();

   os.str("");
   os << std::oct << static_cast<unsigned long long>(NumMax);
   StrMax8=os.str();

   { // chk <max> radix=8
     std::string nstr(StrMax8);
     ChkItem_UnSigned<NumT> cksmp=
        {nstr.c_str(),nstr.size(),Ok,NumMax,nstr.size(),8,8};
     cksmp.verify();
   }

   { // chk <max> radix=10
     std::string nstr(StrMax10);
     ChkItem_UnSigned<NumT> cksmp=
        {nstr.c_str(),nstr.size(),Ok,NumMax,nstr.size(),10,10};
     cksmp.verify();
   }

   { // chk <max> radix=16
     std::string nstr(StrMax16);
     ChkItem_UnSigned<NumT> cksmp=
        {nstr.c_str(),nstr.size(),Ok,NumMax,nstr.size(),16,16};
     cksmp.verify();
   }

   { // chk <-min> radix=10
     std::string nstr(StrMin10);
     ChkItem_UnSigned<NumT> cksmp=
        {nstr.c_str(),nstr.size(),Ok,NumMin,nstr.size(),10,10};
     cksmp.verify();
   }

   { // chk <ovl> radix=10
     std::string nstr(StrMax10+'1');
     ChkItem_Signed<NumT> cksmp=
        {nstr.c_str(),nstr.size(),Wym_ERANGE,NumMax,StrMax10.size(),10,10};
     cksmp.verify();
   }
 }
};

int main(void)
try {
 std::cout << "Checking _strnum(..) for integers ";
 ck_nstr_signed<char>();
 ck_nstr_signed<signed char>();
 ck_nstr_signed<signed short>();
 ck_nstr_signed<signed int>();
 ck_nstr_signed<signed long>();
 ck_nstr_signed<signed long long>();

 ck_nstr_unsigned<unsigned char>(); 
 ck_nstr_unsigned<unsigned short>();
 ck_nstr_unsigned<unsigned int>();
 ck_nstr_unsigned<unsigned long>();
 ck_nstr_unsigned<unsigned long long>();

 std::cout << "Ok\n";
}
catch(const WyRet& e) {
 std::cerr << Wy::wrd(e).c_str() << '\n';
 return(-1);
}
catch(...) {
 std::cerr << "unknown throw\n";
 throw;
};
