/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2007

  Build: make chk_strnum_timespec
*/

#include "WyNum.h"
#include "WyStr.h"
#include "WyTimeSpec.h"
#include "WyByteFlow.h"
#include <iostream>
#include <sstream>
#include <string>
#include <assert.h>
#include <stdlib.h>  // for rand()
#include <cmath>     // for std::ldexp(double,int)

struct ChkStrNum_timespec {
  typedef WyTimeSpec NumT;
  const char* const nstr;       // test string
  const size_t nstr_len;        //

  // Expected results
  const WyReply res_reply;
  const NumT res_floor;
  const NumT res_ceil;
  const size_t res_plen;
  const Wy::WyRadix res_radix_i;
  const Wy::WyRadix res_radix_o;

  void verify(void) const
     try {
       WyRet r;
       NumT num;
       Wy::WyRadix radix=res_radix_i;
       WyCSeg cs;

       // _strlen(Num&, WyCSeg&, int&)
       radix=res_radix_i;
       if((r=cs.reset(nstr,nstr_len))!=Ok) {
         WY_THROW(r);
       }
       r=Wy::_strnum(num, cs, radix);
       
       if(r!=res_reply) {
         std::cerr << "'res_reply' inconsistent: " << Wy::wrd(r).c_str() << '\n';
         WY_THROW( WyRet() );
       }
       if(num<res_floor) {
         std::cerr << "num < res_floor: " << Wy::wrd(num).c_str() << '\n';
         WY_THROW( WyRet() );
       } else if(num>res_ceil) {
         std::cerr << "num > res_ceil: " << Wy::wrd(num).c_str() << '\n';
         WY_THROW( WyRet() );
       } else {};
       if(r==Ok) {
         if(cs.size()!=0) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EINVAL) {
         if(cs.size()!=nstr_len) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ENOENT) {
         if(cs.size()!=nstr_len) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EBADMSG) {
         if(nstr_len-cs.size()!=res_plen) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ERANGE) {
         if(nstr_len-cs.size()!=res_plen) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else {
         WY_THROW( WyRet() );
       }

       // _strlen(Num&, WyCSeg&, const int&)
       radix=res_radix_i;
       if((r=cs.reset(nstr,nstr_len))!=Ok) {
         WY_THROW(r);
       }
       r=Wy::_strnum(num, cs, (const int)radix);
       if(radix!=res_radix_i) {
         std::cerr << "'radix' overwritten :" << radix << '\n';
         WY_THROW( WyRet() );
       }
       
       if(r!=res_reply) {
         std::cerr << "'res_reply' inconsistent: " << Wy::wrd(r).c_str() << '\n';
         WY_THROW( WyRet() );
       }
       if(num<res_floor) {
         std::cerr << "num < res_floor: " << Wy::wrd(num).c_str() << '\n';
         WY_THROW( WyRet() );
       } else if(num>res_ceil) {
         std::cerr << "num > res_ceil: " << Wy::wrd(num).c_str() << '\n';
         WY_THROW( WyRet() );
       } else {};
       if(r==Ok) {
         if(cs.size()!=0) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EINVAL) {
         if(cs.size()!=nstr_len) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ENOENT) {
         if(cs.size()!=nstr_len) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EBADMSG) {
         if(nstr_len-cs.size()!=res_plen) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ERANGE) {
         if(nstr_len-cs.size()!=res_plen) {
           std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
           WY_THROW( WyRet() );
         }
       } else {
         WY_THROW( WyRet() );
       }

       // _strlen(Num&, WyCSeg&)
       if(res_radix_i==10) {
         radix=res_radix_i;
         if((r=cs.reset(nstr,nstr_len))!=Ok) {
           WY_THROW(r);
         }
         r=Wy::_strnum(num, cs);
         
         if(radix!=res_radix_i) {
           std::cerr << "'radix' overwritten :" << radix << '\n';
           WY_THROW( WyRet() );
         }
         if(r!=res_reply) {
           std::cerr << "'res_reply' inconsistent: " << Wy::wrd(r).c_str() << '\n';
           WY_THROW( WyRet() );
         }
         if(num<res_floor) {
           std::cerr << "num < res_floor: " << Wy::wrd(num).c_str() << '\n';
           WY_THROW( WyRet() );
         } else if(num>res_ceil) {
           std::cerr << "num > res_ceil: " << Wy::wrd(num).c_str() << '\n';
           WY_THROW( WyRet() );
         } else {};
         if(r==Ok) {
           if(cs.size()!=0) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else if(r==Wym_EINVAL) {
           if(cs.size()!=nstr_len) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else if(r==Wym_ENOENT) {
           if(cs.size()!=nstr_len) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else if(r==Wym_EBADMSG) {
           if(nstr_len-cs.size()!=res_plen) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else if(r==Wym_ERANGE) {
           if(nstr_len-cs.size()!=res_plen) {
             std::cerr << "'res_plen' inconsistent: " << (nstr_len-cs.size()) << '\n';
             WY_THROW( WyRet() );
           }
         } else {
           WY_THROW( WyRet() );
         }
       }

       // _strlen(Num&, const WyCSeg&, int&)
       radix=res_radix_i;
       if((r=cs.reset(nstr,nstr_len))!=Ok) {
         WY_THROW(r);
       }
       r=Wy::_strnum(num, (const WyCSeg&)cs, radix);
       
       if(cs!=WyCSeg(nstr,nstr_len)) {
         std::cerr << "cs overwritten: " << WyStr(cs).c_str() << '\n';
         WY_THROW( WyRet() );
       }
       if(num<res_floor) {
         std::cerr << "num < res_floor: " << Wy::wrd(num).c_str() << '\n';
         WY_THROW( WyRet() );
       } else if(num>res_ceil) {
         std::cerr << "num > res_ceil: " << Wy::wrd(num).c_str() << '\n';
         WY_THROW( WyRet() );
       } else {};
       if(r==Ok) {
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_EINVAL) {
       } else if(r==Wym_ENOENT) {
       } else if(r==Wym_EBADMSG) {
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else if(r==Wym_ERANGE) {
         if(radix!=res_radix_o) {
           std::cerr << "'res_radix_o' inconsistent: " << (int)radix << '\n';
           WY_THROW( WyRet() );
         }
       } else {
         WY_THROW( WyRet() );
       }

       // _strlen(Num&, const WyCSeg&, const int&)
       radix=res_radix_i;
       if((r=cs.reset(nstr,nstr_len))!=Ok) {
         WY_THROW(r);
       }
       r=Wy::_strnum(num, (const WyCSeg&)cs, (const int&)radix);
       
       if(cs!=WyCSeg(nstr,nstr_len)) {
         std::cerr << "cs overwritten: " << WyStr(cs).c_str() << '\n';
         WY_THROW( WyRet() );
       }
       if(radix!=res_radix_i) {
         std::cerr << "'radix' overwritten :" << radix << '\n';
         WY_THROW( WyRet() );
       }
       if(num<res_floor) {
         std::cerr << "num < res_floor: " << Wy::wrd(num).c_str() << '\n';
         WY_THROW( WyRet() );
       } else if(num>res_ceil) {
         std::cerr << "num > res_ceil: " << Wy::wrd(num).c_str() << '\n';
         WY_THROW( WyRet() );
       } else {};
       if(r==Ok) {
       } else if(r==Wym_EINVAL) {
       } else if(r==Wym_ENOENT) {
       } else if(r==Wym_EBADMSG) {
       } else if(r==Wym_ERANGE) {
       } else {
         WY_THROW( WyRet() );
       }

       // _strlen(Num&, const WyCSeg&)
       if(res_radix_i==10) {
         radix=res_radix_i;
         if((r=cs.reset(nstr,nstr_len))!=Ok) {
           WY_THROW(r);
         }
         r=Wy::_strnum(num, (const WyCSeg&)cs);
         
         if(cs!=WyCSeg(nstr,nstr_len)) {
           std::cerr << "cs overwritten: " << WyStr(cs).c_str() << '\n';
           WY_THROW( WyRet() );
         }
         if(num<res_floor) {
           std::cerr << "num < res_floor: " << Wy::wrd(num).c_str() << '\n';
           WY_THROW( WyRet() );
         } else if(num>res_ceil) {
           std::cerr << "num > res_ceil: " << Wy::wrd(num).c_str() << '\n';
           WY_THROW( WyRet() );
         } else {};
         if(r==Ok) {
         } else if(r==Wym_EINVAL) {
         } else if(r==Wym_ENOENT) {
         } else if(r==Wym_EBADMSG) {
         } else if(r==Wym_ERANGE) {
         } else {
           WY_THROW( WyRet() );
         }
       }

     }
     catch(...) {
       std::cerr << "ChkStrNum\n";
       std::cerr << "Failed on {\"" << ((nstr==NULL)? "NULL" : nstr) << "\"," << nstr_len
             << ',' << Wy::wrd(res_reply).c_str()
             << ',' << Wy::wrd(res_floor).c_str() << ',' << Wy::wrd(res_ceil).c_str()
             << ',' << res_plen << ',' << res_radix_i << ',' << res_radix_o << "}\n";
       throw;
     };
};

static void ck_strnum_timespec(void)
{
 typedef WyTimeSpec NumT;
 const ChkStrNum_timespec chk_tab[]={
   // nstr slen  reply floor ceil plen rad_i rad_o
   {"0",0 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"0",0 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},

   // one char "?"
   {"0",1 ,        Ok,WyTimeSpec(0,0),WyTimeSpec(0,0),1, 0,10},
   {"1",1 ,        Ok,WyTimeSpec(1,0),WyTimeSpec(1,0),1, 0,10},
   {"7",1 ,        Ok,WyTimeSpec(7,0),WyTimeSpec(7,0),1, 0,10},
   {"8",1 ,        Ok,WyTimeSpec(8,0),WyTimeSpec(8,0),1, 0,10},
   {"9",1 ,        Ok,WyTimeSpec(9,0),WyTimeSpec(9,0),1, 0,10},
   {"a",1 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"e",1 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"g",1 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"X",1 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"+",1 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"-",1 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {".",1 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},

   // two char's "+?"
   {"+0",2 ,        Ok,WyTimeSpec(0,0),WyTimeSpec(0,0),2, 0,10},
   {"+1",2 ,        Ok,WyTimeSpec(1,0),WyTimeSpec(1,0),2, 0,10},
   {"+7",2 ,        Ok,WyTimeSpec(7,0),WyTimeSpec(7,0),2, 0,10},
   {"+8",2 ,        Ok,WyTimeSpec(8,0),WyTimeSpec(8,0),2, 0,10},
   {"+9",2 ,        Ok,WyTimeSpec(9,0),WyTimeSpec(9,0),2, 0,10},
   {"+a",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"+e",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"+g",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"+X",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"++",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"+-",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"+.",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},

   // two char's "-?"
   {"-0",2 ,        Ok,WyTimeSpec(-0,0),WyTimeSpec(-0,0),2, 0,10},
   {"-1",2 ,        Ok,WyTimeSpec(-1,0),WyTimeSpec(-1,0),2, 0,10},
   {"-7",2 ,        Ok,WyTimeSpec(-7,0),WyTimeSpec(-7,0),2, 0,10},
   {"-8",2 ,        Ok,WyTimeSpec(-8,0),WyTimeSpec(-8,0),2, 0,10},
   {"-9",2 ,        Ok,WyTimeSpec(-9,0),WyTimeSpec(-9,0),2, 0,10},
   {"-a",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"-e",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"-g",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"-X",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"-+",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"-.",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},

   // two char's ".?"
   {".0",2 ,        Ok,WyTimeSpec(0,0),WyTimeSpec(0,0),2, 0,10},
   {".1",2 ,        Ok,WyTimeSpec(0,100000000),WyTimeSpec(0,100000000),2, 0,10},
   {".7",2 ,        Ok,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10},
   {".8",2 ,        Ok,WyTimeSpec(0,800000000),WyTimeSpec(0,800000000),2, 0,10},
   {".9",2 ,        Ok,WyTimeSpec(0,900000000),WyTimeSpec(0,900000000),2, 0,10},
   {".a",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {".e",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {".g",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {".X",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {".+",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {".-",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"..",2 ,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},

   // two char's "0?"
   {"00",2 ,         Ok,WyTimeSpec(0,0),WyTimeSpec(0,0),2, 0,10},
   {"01",2 ,         Ok,WyTimeSpec(1,0),WyTimeSpec(1,0),2, 0,10},
   {"07",2 ,         Ok,WyTimeSpec(7,0),WyTimeSpec(7,0),2, 0,10},
   {"08",2 ,         Ok,WyTimeSpec(8,0),WyTimeSpec(8,8),2, 0,10},
   {"09",2 ,         Ok,WyTimeSpec(9,0),WyTimeSpec(9,0),2, 0,10},
   {"0a",2 ,         Ok,WyTimeSpec(10,0),WyTimeSpec(10,0),2,16,16},
   {"0e",2 ,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),1, 0,10},
   {"0g",2 ,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),1, 0,10},
   {"0X",2 ,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),1, 0,10},
   {"0+",2 ,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),1, 0,10},
   {"0-",2 ,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),1, 0,10},
   {"0.",2 ,         Ok,WyTimeSpec(0,0),WyTimeSpec(0,0),2, 0,10},

   // two char's "1?"
   {"10",2 ,         Ok,WyTimeSpec(10,0),WyTimeSpec(10,0),2, 0,10},
   {"11",2 ,         Ok,WyTimeSpec(11,0),WyTimeSpec(11,0),2, 0,10},
   {"17",2 ,         Ok,WyTimeSpec(17,0),WyTimeSpec(17,0),2, 0,10},
   {"18",2 ,         Ok,WyTimeSpec(18,0),WyTimeSpec(18,0),2, 0,10},
   {"19",2 ,         Ok,WyTimeSpec(19,0),WyTimeSpec(19,0),2, 0,10},
   {"1a",2 ,Wym_EBADMSG,WyTimeSpec(1,0),WyTimeSpec(1,0),1, 0,10},
   {"1e",2 ,         Ok,WyTimeSpec(0x1e,0),WyTimeSpec(0x1e,0),2,16,16},
   {"1g",2 ,Wym_EBADMSG,WyTimeSpec(1,0),WyTimeSpec(1,0),1, 0,10},
   {"1X",2 ,Wym_EBADMSG,WyTimeSpec(1,0),WyTimeSpec(1,0),1, 0,10},
   {"1+",2 ,Wym_EBADMSG,WyTimeSpec(1,0),WyTimeSpec(1,0),1, 0,10},
   {"1-",2 ,Wym_EBADMSG,WyTimeSpec(1,0),WyTimeSpec(1,0),1, 0,10},
   {"1.",2 ,         Ok,WyTimeSpec(1,0),WyTimeSpec(1,0),2, 0,10},

   // ".0?"
   {".00",3 ,         Ok,WyTimeSpec(0,0),WyTimeSpec(0,0),3, 0,10},
   {".01",3 ,         Ok,WyTimeSpec(0,10000000),WyTimeSpec(0,10000000),3, 0,10},
   {".07",3 ,         Ok,WyTimeSpec(0,70000000),WyTimeSpec(0,70000000),3, 0,10},
   {".08",3 ,         Ok,WyTimeSpec(0,80000000),WyTimeSpec(0,80000000),3, 0,10},
   {".09",3 ,         Ok,WyTimeSpec(0,90000000),WyTimeSpec(0,90000000),3, 0,10},
   {".0a",3 ,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),2, 0,10},
   {".0e",3 ,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),2, 0,10},
   {".0g",3 ,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),2, 0,10},
   {".0X",3 ,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),2, 0,10},
   {".0+",3 ,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),2, 0,10},
   {".0-",3 ,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),2, 0,10},
   {".0.",3 ,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),2, 0,10},

   // ".7?"
   {".70",3 ,         Ok,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),3, 0,10},
   {".71",3 ,         Ok,WyTimeSpec(0,710000000),WyTimeSpec(0,710000000),3, 0,10},
   {".77",3 ,         Ok,WyTimeSpec(0,770000000),WyTimeSpec(0,770000000),3, 0,10},
   {".78",3 ,         Ok,WyTimeSpec(0,780000000),WyTimeSpec(0,780000000),3, 0,10},
   {".79",3 ,         Ok,WyTimeSpec(0,790000000),WyTimeSpec(0,790000000),3, 0,10},
   {".7a",3 ,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10},
   {".7e",3 ,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10},
   {".7g",3 ,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10},
   {".7X",3 ,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10},
   {".7+",3 ,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10},
   {".7-",3 ,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10},
   {".7.",3 ,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10},

    // ".7?e"
   {".70e",4,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),3, 0,10},
   {".71e",4,Wym_EBADMSG,WyTimeSpec(0,710000000),WyTimeSpec(0,710000000),3, 0,10},
   {".77e",4,Wym_EBADMSG,WyTimeSpec(0,770000000),WyTimeSpec(0,770000000),3, 0,10},
   {".78e",4,Wym_EBADMSG,WyTimeSpec(0,780000000),WyTimeSpec(0,780000000),3, 0,10},
   {".79e",4,Wym_EBADMSG,WyTimeSpec(0,790000000),WyTimeSpec(0,790000000),3, 0,10}, 
   {".7ae",4,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10},
   {".7ee",4,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10},
   {".7ge",4,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10},
   {".7Xe",4,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10},
   {".7+e",4,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10},
   {".7-e",4,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10},
   {".7.e",4,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10},

    // ".7?e5"
   {".70e5",5,Ok,WyTimeSpec(70000,0),WyTimeSpec(70000,0),5, 0,10},
   {".71e5",5,Ok,WyTimeSpec(71000,0),WyTimeSpec(71000,0),5, 0,10},
   {".77e5",5 ,Ok,WyTimeSpec(77000,0),WyTimeSpec(77000,0),5, 0,10},
   {".78e5",5 ,Ok,WyTimeSpec(78000,0),WyTimeSpec(78000,0),5, 0,10},
   {".79e5",5 ,Ok,WyTimeSpec(79000,0),WyTimeSpec(79000,0),5, 0,10},
   {".7ae5",5 ,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10}, 
   {".7ee5",5 ,Ok,WyTimeSpec(0,(0x7ee5*WY__CONST_GIGA)/65536),
                  WyTimeSpec(0,(0x7ee5*WY__CONST_GIGA)/65536),5, 16,16},
   {".7ge5",5 ,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10},
   {".7Xe5",5 ,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10},
   {".7+e5",5,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10},
   {".7-e5",5,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10},
   {".7.e5",5,Wym_EBADMSG,WyTimeSpec(0,700000000),WyTimeSpec(0,700000000),2, 0,10}, 

   // "0x?"
   {"0x0",3 ,         Ok,WyTimeSpec(0,0),WyTimeSpec(0,0),3, 0,16},
   {"0x1",3 ,         Ok,WyTimeSpec(1,0),WyTimeSpec(1,0),3,16,16},
   {"0x7",3 ,         Ok,WyTimeSpec(7,0),WyTimeSpec(7,0),3, 0,16},
   {"0x8",3 ,         Ok,WyTimeSpec(8,0),WyTimeSpec(8,0),3,16,16},
   {"0x9",3 ,         Ok,WyTimeSpec(9,0),WyTimeSpec(9,0),3, 0,16},
   {"0xa",3 ,         Ok,WyTimeSpec(10,0),WyTimeSpec(10,0),3, 0,16},
   {"0xe",3 ,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),1,10,10},
   {"0xg",3 ,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),1, 0,10},
   {"0xX",3 ,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),1, 0,10},
   {"0x+",3 ,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),1, 0,10},
   {"0x-",3 ,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),1, 0,10},
   {"0x.",3 ,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),1, 0,10},

   // mis
   {"+0x",3,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),2, 0,10}, 
   {"-0x",3,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),2, 0,10},
   {"0x.0.2",6,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),4, 0,16},
   {"0x.E0",5,         Ok,WyTimeSpec(0,(0xe*WY__CONST_GIGA)/16),
                          WyTimeSpec(0,(0xe*WY__CONST_GIGA)/16),5, 0,16},
   {"-0x.f0",6,         Ok,WyTimeSpec(0,(0xf*WY__CONST_GIGA)/-16),
                          WyTimeSpec(0,(0xf*WY__CONST_GIGA)/-16),6, 0,16},
   {"0x.ea",5,        Ok,WyTimeSpec(0,(0xea*WY__CONST_GIGA)/256),
                         WyTimeSpec(0,(0xea*WY__CONST_GIGA)/256),5, 16,16},
   {"-.E0",6,Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"9999999",7,       Ok,WyTimeSpec(9999999,0),WyTimeSpec(9999999,0),7, 0,10}, 
   {"-07777777",9,     Ok,WyTimeSpec(-7777777,0),WyTimeSpec(-7777777,0),9, 0,10},
   {"-9999999",8,     Ok,WyTimeSpec(-9999999,0),WyTimeSpec(-9999999,0),8,10,10},
   {"999.9999e3",10,   Ok,WyTimeSpec(999999,900000000),WyTimeSpec(999999,900000000),10, 0,10},
   {"-0999.9999e4",12, Ok,WyTimeSpec(-9999999,0),WyTimeSpec(-9999999,0),12, 0,10},
   {"-0x1f7.ee5p-4",13 ,Ok,
                  WyTimeSpec(-0x1f,(0x7ee5*WY__CONST_GIGA)/-65536LL),
                  WyTimeSpec(-0x1f,(0x7ee5*WY__CONST_GIGA)/-65536LL),13, 16,16},

   {"0.0.2", 5,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),3, 0,10}, 

   {"0x.2.2",6,Wym_EBADMSG,WyTimeSpec(0,(2*WY__CONST_GIGA)/16),
                           WyTimeSpec(0,(2*WY__CONST_GIGA)/16),4, 0,16}, 
   {"0.1.",  4,Wym_EBADMSG,WyTimeSpec(0,100000000),WyTimeSpec(0,100000000),3, 0,10},
   {"0.1.2", 5,Wym_EBADMSG,WyTimeSpec(0,100000000),WyTimeSpec(0,100000000),3, 0,10},
   {"0.1.",  4,Wym_EBADMSG,WyTimeSpec(0,100000000),WyTimeSpec(0,100000000),3,10,10},
   {"0.1.2", 5,Wym_EBADMSG,WyTimeSpec(0,100000000),WyTimeSpec(0,100000000),3, 0,10},

   {"-.", 2,   Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10}, 
   {"+.9",3,           Ok,WyTimeSpec(0,900000000),WyTimeSpec(0,900000000),3, 0,10},
   {"-.9",3,           Ok,WyTimeSpec(-0,-900000000),WyTimeSpec(-0,-900000000),3, 0,10},
   {"+0", 2,           Ok,WyTimeSpec(0,0),WyTimeSpec(0,0),2, 0,10},
   {"-0", 2,           Ok,WyTimeSpec(0,0),WyTimeSpec(0,0),2, 0,10}, 
   {"0x.",3,  Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),1, 0,10},
   {"0x.g",4, Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),1, 0,10},
   {"0x.a",4, Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),1, 10,10},
   {"0x.ap4",6,        Ok,WyTimeSpec(10,0),WyTimeSpec(10,0),6, 0,16},
   {".0",  2,          Ok,WyTimeSpec(0,0),WyTimeSpec(0,0),2, 0,10},
   {"+9.", 3,          Ok,WyTimeSpec(9,0),WyTimeSpec(9,0),3, 0,10},
   {"-9.", 3,          Ok,WyTimeSpec(-9,0),WyTimeSpec(-9,0),3, 0,10},
   {"+.x", 3,  Wym_ENOENT,WyTimeSpec(0,0),WyTimeSpec(0,0),0, 0,10},
   {"-0.1a",5,Wym_EBADMSG,WyTimeSpec(-0,-100000000),WyTimeSpec(-0,-100000000),4, 0,10},
   {"-.1a", 4,Wym_EBADMSG,WyTimeSpec(-0,-100000000),WyTimeSpec(-0,-100000000),3, 0,10},

   {"31",2,              Ok,WyTimeSpec(31,0),WyTimeSpec(31,0),2, 0,10},
   {"031",3,             Ok,WyTimeSpec(31,0),WyTimeSpec(31,0),3, 0,10}, 
   {"00.03",5,           Ok,WyTimeSpec(0,30000000),WyTimeSpec(0,30000000),5, 0,10},
   {"0x00.03",7,         Ok,WyTimeSpec(0,(3*WY__CONST_GIGA)/256),
                            WyTimeSpec(0,(3*WY__CONST_GIGA)/256),7, 0,16},
   {"0x00.0f",7,         Ok,WyTimeSpec(0,(15*WY__CONST_GIGA)/256),
                            WyTimeSpec(0,(15*WY__CONST_GIGA)/256),7, 0,16},
   {"0x00.03",7,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),1,10,10},
   {"00x0",4,   Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),2, 0,10},
   {"1.2P10",6, Ok,WyTimeSpec((long)0x1.2p10,0),
                   WyTimeSpec((long)0x1.2p10,0),6,16,16}, 
   {"-1.2p1a",7, Wym_EBADMSG,WyTimeSpec(-2,(-4*WY__CONST_GIGA)/16),
                             WyTimeSpec(-2,(-4*WY__CONST_GIGA)/16),6,16,16},

   {"+.0",    3,Ok,WyTimeSpec(0,0),    WyTimeSpec(0,0),3, 0,10}, 
   {"-.0",    3,Ok,WyTimeSpec(0,0),    WyTimeSpec(0,0),3,16,16},
   {".00",    3,Ok,WyTimeSpec(0,0),    WyTimeSpec(0,0),3,10,10},
   {"+.00",   4,Ok,WyTimeSpec(0,0),    WyTimeSpec(0,0),4, 0,10},
   {"-.00",   4,Ok,WyTimeSpec(0,0),    WyTimeSpec(0,0),4,16,16},
   {"0.0",    3,Ok,WyTimeSpec(0,0),    WyTimeSpec(0,0),3, 0,10},
   {"+0.0",   4,Ok,WyTimeSpec(0,0),    WyTimeSpec(0,0),4, 0,10},
   {"-0.0",   4,Ok,WyTimeSpec(0,0),    WyTimeSpec(0,0),4, 0,10}, 
   {"0.010",  5,Ok,WyTimeSpec(0,10000000),  WyTimeSpec(0,10000000),5, 0,10},
   {"-0.010", 6,Ok,WyTimeSpec(-0,-10000000),  WyTimeSpec(-0,10000000),6, 0,10},
   {"123",    3,Ok,WyTimeSpec(123,0),  WyTimeSpec(123,0),3, 0,10},
   {"+123",   4,Ok,WyTimeSpec(123,0),  WyTimeSpec(123,0),4, 0,10},
   {"+0123",  5,Ok,WyTimeSpec(123,0), WyTimeSpec(123,0),5, 0,10},
   {"-0123",  5,Ok,WyTimeSpec(-123,0),WyTimeSpec(-123,0),5, 0,10},
   {".123",   4,Ok,WyTimeSpec(0,123000000),  WyTimeSpec(0,123000000),4, 0,10},
   {"+.123",  5,Ok,WyTimeSpec(0,123000000),  WyTimeSpec(0,123000000),5, 0,10},
   {"-.123",  5,Ok,WyTimeSpec(-0,-123000000), WyTimeSpec(-0,-123000000),5, 0,10},
   {".120",   4,Ok,WyTimeSpec(0,120000000),  WyTimeSpec(0,120000000),4, 0,10},
   {"+.120",  5,Ok,WyTimeSpec(0,120000000),  WyTimeSpec(0,120000000),5, 0,10},
   {"-.120",  5,Ok,WyTimeSpec(-0,-120000000),  WyTimeSpec(-0,-120000000),5, 0,10},
   {"12.0",   4,Ok,WyTimeSpec(12,0),   WyTimeSpec(12,0),4, 0,10},
   {"+12.0",  5,Ok,WyTimeSpec(12,0),   WyTimeSpec(12,0),5, 0,10},
   {"-12.0",  5,Ok,WyTimeSpec(-12,0),  WyTimeSpec(-12,0),5, 0,10},
   {"-012.3", 6,Ok,WyTimeSpec(-12,-300000000),  WyTimeSpec(-12,-300000000),6, 0,10},
   {"+0012.000000003",15,Ok,WyTimeSpec(12,3),   WyTimeSpec(12,3),15, 0,10}, 
   {"1111111.11111",13,Ok,WyTimeSpec(1111111,111110000),WyTimeSpec(1111111,111110000),13, 0,10},
   {"-1111111.000011111",18,Ok,WyTimeSpec(-1111111,-11111),WyTimeSpec(-1111111,-11111),18, 0,10}, 
   {"0177.177",8,Ok,WyTimeSpec(177,177000000),WyTimeSpec(177,177000000),8, 0,10}, 
   {"-177.177",8,Ok,WyTimeSpec(-177,-177000000),WyTimeSpec(-177,-177000000),8, 0,10}, 
   {"-7f.7f",6,Wym_EBADMSG,WyTimeSpec(-7,0),WyTimeSpec(-7,0),2, 0,10}, 
   {"-0x7f.7f",8,         Ok,WyTimeSpec(-127,(-0x7f*WY__CONST_GIGA)/256),
                             WyTimeSpec(-127,(-0x7f*WY__CONST_GIGA)/256),8, 0,16}, 
   {"+0x7f.7f",8,         Ok,WyTimeSpec(+127,(+0x7f*WY__CONST_GIGA)/256),
                             WyTimeSpec(+127,(+0x7f*WY__CONST_GIGA)/256),8, 0,16}, 
   {"0x7f.7f",7,         Ok,WyTimeSpec(+127,(+0x7f*WY__CONST_GIGA)/256),
                             WyTimeSpec(+127,(+0x7f*WY__CONST_GIGA)/256),7,16,16}, 
   {"-0x7f.7f",8,Wym_EBADMSG,WyTimeSpec(0,0),WyTimeSpec(0,0),2,10,10}, 
   {"+000000000000000000012.00100",26,Ok,WyTimeSpec(12,1000000),WyTimeSpec(12,1000000),26, 0,10},
   {"123.321E-3",10,Ok,WyTimeSpec(0,123321000),WyTimeSpec(0,123321000),10, 0,10},
   {"123.321E+3",10,Ok,WyTimeSpec(123321,0),WyTimeSpec(123321,0),10, 0,10},
   {"456.789E12",10,Wym_ERANGE,WyTimeSpec(4567,890000000),WyTimeSpec(4567,890000000),9, 0,10},
   {".0E0",4,Ok,WyTimeSpec(0,0),WyTimeSpec(0,0),4, 0,10}, 
   {".3E3",4,Ok,WyTimeSpec(300,0),WyTimeSpec(300,0),4, 0,10},
   {".3E 3",5,Wym_EBADMSG,WyTimeSpec(0,300000000),WyTimeSpec(0,300000000),2, 0,10},
   {".3 E3",5,Wym_EBADMSG,WyTimeSpec(0,300000000),WyTimeSpec(0,300000000),2, 0,10},
   {".3E+ 3",6,Wym_EBADMSG,WyTimeSpec(0,300000000),WyTimeSpec(0,300000000),2, 0,10},
   {".3E3.",5,Wym_EBADMSG,WyTimeSpec(300,0),WyTimeSpec(300,0),4, 0,10},
   {".3E3.0",6,Wym_EBADMSG,WyTimeSpec(300,0),WyTimeSpec(300,0),4, 0,10},
   {".1.2.3", 6,Wym_EBADMSG,WyTimeSpec(0,100000000), WyTimeSpec(0,100000000),2, 0,10},
   {".0.1.2", 6,Wym_EBADMSG,WyTimeSpec(0,0),  WyTimeSpec(0,0),2, 0,10},
   {"0.1.2",5,Wym_EBADMSG,WyTimeSpec(0,100000000), WyTimeSpec(0,100000000),3, 0,10}, 
   {"1.2.3",5,Wym_EBADMSG,WyTimeSpec(1,200000000), WyTimeSpec(1,200000000),3, 0,10},
   {"0x.1.2",6,Wym_EBADMSG,WyTimeSpec(0,0),  WyTimeSpec(0,0),1,10,10},
   {"0x.1.2",6,Wym_EBADMSG,WyTimeSpec(0,62500000),  WyTimeSpec(0,62500000),4, 0,16},
   {"0x0.1.2",7,Wym_EBADMSG,WyTimeSpec(0,62500000),  WyTimeSpec(0,62500000),5,16,16},

   {"+.5E+3",6,Ok,WyTimeSpec(500,0),WyTimeSpec(500,0),6, 0,10},
   {"-.3e-3",6,Ok,WyTimeSpec(0,-300000),WyTimeSpec(0,-300000),6, 0,10},
   {"+.3E-3",6, Ok,WyTimeSpec(0,300000),WyTimeSpec(0,300000),6, 0,10}, 
   {"-.3E-03",7,Ok,WyTimeSpec(0,-300000),WyTimeSpec(0,-300000),7, 0,10},
   {"-.3E-03",7,Wym_EBADMSG,WyTimeSpec(0,(-0x3E*WY__CONST_GIGA)/256),
                            WyTimeSpec(0,(-0x3E*WY__CONST_GIGA)/256),4,16,16},
   {"-.3E-0x3",8,Wym_EBADMSG,WyTimeSpec(0,-300000000),WyTimeSpec(0,-300000000),6, 0,10},
   {"9.946307221e0",13, Ok,WyTimeSpec(9,946307221),WyTimeSpec(9,946307221),13, 0,10},
   {".0000000000435678e-36",21, Ok,WyTimeSpec(0,0),WyTimeSpec(0,0),21, 0,10},
   {".0000435678e+10",15, Ok,WyTimeSpec(435678,0),WyTimeSpec(435678,0),15, 0,10},
   {".0000435678e+399",16, Wym_ERANGE,WyTimeSpec(0,43567800),WyTimeSpec(0,43567800),14, 0,10},
   {"-.0000000000435678e-306",23, Ok,WyTimeSpec(0,0),WyTimeSpec(0,0),23,10,10},
   {"-.0000435678e+41",16, Wym_ERANGE,WyTimeSpec(0,-435678000),WyTimeSpec(0,-435678000),15, 0,10},
   {"-.0000435678e+13",16, Ok,WyTimeSpec(-435678000,0),WyTimeSpec(-435678000,0),16, 0,10},
   {"+.0000435678e+499",17, Wym_ERANGE,WyTimeSpec(0,435678000),WyTimeSpec(0,435678000),15, 0,10},

   {"7654321e-90", 11, Ok, WyTimeSpec(0,0), WyTimeSpec(0,0),11,10,10},
   {"7654321e260", 11, Wym_ERANGE, WyTimeSpec(765432100,0), WyTimeSpec(765432100,0),9,10,10},
   {"-7654321e260", 12, Wym_ERANGE, WyTimeSpec(-765432100,0), WyTimeSpec(-765432100,0),10,10,10}, 
   {"001.00000000012345678900", 24,Ok, WyTimeSpec(1,0), WyTimeSpec(1,0),24,10,10}, 
   {"-001.0000000012345678900", 23,Ok, WyTimeSpec(-1,-1), WyTimeSpec(-1,-1),23,10,10}, 
   {"00000000000000000000000000000000000000000123.4560000000000000000000000000000000000000000e2",
     90,Ok, WyTimeSpec(12345,600000000), WyTimeSpec(12345,600000000),90, 0,10},
   {".000000000000000000000000000000123456789000000000000000000000000000000000000000",
     79,Ok, WyTimeSpec(0,0), WyTimeSpec(0,0),79,10,10}, 
   {".000000000000000000000000000000000000000123456789000000000000000000000000000000",
     79,Ok, WyTimeSpec(0,0), WyTimeSpec(0,0),79,10,10}, 
   {"-.000000000000000000000000000000123456789000000000000000000000000000000000000000e-5",
     83,Ok, WyTimeSpec(0,0), WyTimeSpec(0,0),83,10,10}, 
   {"-.000000000000000000000000000000123456789000000000000000000000000000000000000000e31",
     83,Ok, WyTimeSpec(-1,-234567890), WyTimeSpec(-1,-234567890),83,10,10},
   {"0x1fp3",6,  Ok,WyTimeSpec((long)0x1fp3,0),WyTimeSpec((long)0x1fp3,0),6,0,16},
   // implement feature: conversion limited to 38 digits of radix 10
   {"12345678901234567890123456789012345678901234.5678900000000000000000000000000000000000000e-34",
     93, Wym_ERANGE, WyTimeSpec(1234567890,0),WyTimeSpec(1234567890,0),10, 0,10},
   {"123.123123123123123123123123123123123E3",39,Ok,
                     WyTimeSpec(123123,123123123),WyTimeSpec(123123,123123123),39, 0,10},
   {"-.3E-0x3",8,Wym_EBADMSG,WyTimeSpec(0,-300000000),WyTimeSpec(0,-300000000),6, 0,10},
   {"2147483647e-1",13,Ok,WyTimeSpec(214748364,700000000),WyTimeSpec(214748364,700000000),13, 0,10},

   {"0x74e2p5", 8, Ok, WyTimeSpec((time_t)0x74e2p5,0), WyTimeSpec((time_t)0x74e2p5,0),8,0,16},
   {"74e2p5", 6, Ok, WyTimeSpec((time_t)0x74e2p5,0), WyTimeSpec((time_t)0x74e2p5,0),6,16,16},

   // The following instances should be revised. Implementation will have endptr stoped
   // pointing at 'e'
   //
   {"123456789.7e11",14,Wym_ERANGE,WyTimeSpec(123456789,700000000),
                                   WyTimeSpec(123456789,700000000),11, 0,10},
   {"-123456789.7e11",14,Wym_ERANGE,WyTimeSpec(-123456789,-700000000),
                                    WyTimeSpec(-123456789,-700000000),12, 0,10},
   {"214748364.7e11",14,Wym_ERANGE,WyTimeSpec(214748364,700000000),
                                   WyTimeSpec(214748364,700000000),11, 0,10},
   {"214748364.7e20",14,Wym_ERANGE,WyTimeSpec(214748364,700000000),
                                   WyTimeSpec(214748364,700000000),11, 0,10},
   {"214748364.8e11",14,Wym_ERANGE,WyTimeSpec(214748364,800000000),
                                   WyTimeSpec(214748364,800000000),11, 0,10},
   {"-214748364.9e11",15,Wym_ERANGE,WyTimeSpec(-214748364,-900000000),
                                    WyTimeSpec(-214748364,-900000000),12, 0,10},

   //{"-0x8000000.0p4",14,Ok,WyTimeSpec(-(time_t)0x80000000p0,0),
   //                        WyTimeSpec(-(time_t)0x80000000p0,0),14, 0,10},
   //{"-0x8000000.1p4",14,Wym_ERANGE,WyTimeSpec(-(time_t)0x8000000p0,0),
   //                        WyTimeSpec(-(time_t)0x8000000p0,0),10, 0,10},
 }; 

 for(size_t i=0; i<sizeof(chk_tab)/sizeof(ChkStrNum_timespec); ++i) {
   chk_tab[i].verify();
 }

 try { // chk <max10>
   std::ostringstream os;
   os.str("");
   os << std::numeric_limits<WyTimeSpec::SecondType>::max();
   std::string nstr(os.str()+".999999999");
   ChkStrNum_timespec cksmp1=
      {nstr.c_str(),nstr.size(),Ok,WyTimeSpec::max(),WyTimeSpec::max(),nstr.size(),10,10};
   cksmp1.verify();
 }
 catch(...) {
   std::cerr << "verify(): chk <max10>\n";
   throw;
 };

 try { // chk <min10>
   std::ostringstream os;
   os.str("");
   os << std::numeric_limits<WyTimeSpec::SecondType>::min();
   std::string nstr(os.str()+".999999999");
   ChkStrNum_timespec cksmp1=
      {nstr.c_str(),nstr.size(),Ok,WyTimeSpec::min(),WyTimeSpec::min(),nstr.size(),10,10};
   cksmp1.verify();
 }
 catch(...) {
   std::cerr << "verify(): chk <min/max>\n";
   throw;
 };

};

void ck_strnum_rand(void)
{
 typedef double DType;
 const int PreDigs=12;
 const long long RandExp=33;

 std::ostringstream os;

 for(int i=0; i<10000; ++i) {
   //  res= x*2^y  ,x in [-1.0,+1.0], y in [-RandExp, +RandExp]
   //
   DType Res= std::ldexp( ::rand()*(2.0L/RAND_MAX)-1.0
                         ,(::rand()*(long long)2*RandExp)/(long long)RAND_MAX-RandExp 
                        );
   {
     // Select Res that fit
     WyRet r;
     WyTimeSpec tmp;
     if((r=tmp.reset(Res))!=Ok) {
       if(r==Wym_ERANGE) {
         if((Res>Wy::to_double(WyTimeSpec::min()))&&(Res<Wy::to_double(WyTimeSpec::max()))) {
           std::cerr << "Check Res is really not representable by WyTimeSpec. "
                     << Wy::wrd(r).c_str() << " :" << Res << '\n';
         }
       } else {
         std::cerr << "set Res failed, " << Wy::wrd(r).c_str() << " :" << Res << '\n';
       }
       continue;
     }
     Res=Wy::to_double(tmp);
   }

   DType num1;
   WyTimeSpec num2;

   os.precision(PreDigs);
   os.str("");
   os << Res;
   const std::string NStr( os.str() );

   // Set num1 by strtod(..)
   {
    char* eptr;
    num1=::strtod(NStr.c_str(),&eptr);
    if(eptr!=NStr.c_str()+NStr.size()) {
      WY_THROW( WyRet() );
    }
   }

   // Set num2 by Wy::_strnum(..)
   {
    WyRet r;
    int radix=10;
    WyCSeg cs(NStr.c_str(), NStr.size());
    r=Wy::_strnum(num2, cs, radix);
    if(cs.size()!=0) {
      WY_THROW(r);
    }
   }

   // Dump instances of larger erronous ratio
   {
     long double ErrRat;
     if(num2!=0.0) {
       ErrRat=num1/Wy::to_double(num2);
     } else if(num1!=0.0) {
       ErrRat=Wy::to_double(num2)/num1;
     } else {
       ErrRat=1.0;
     }
     if((ErrRat>1.000000000000001L)||(ErrRat<0.999999999999999L)) {  // 10^-15 precision
       std::cout << NStr;
       std::cout.precision(PreDigs);
       std::cout << "\n" << num1 << '\n' << Wy::to_double(num2) << '\n';
       std::cout << std::endl;
     };
   }
 }
};

int main(void)
try {
 std::cout << "Checking _strnum(..) for WyTimeSpec ";
  ck_strnum_timespec();
  ck_strnum_rand();
  std::cout << "Ok\n";
}
catch(const WyRet& e) {
 std::cerr << Wy::wrd(e).c_str() << '\n';
 return(-1);
}
catch(...) {
 std::cerr << "unknown throw\n";
 throw;
};

