/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2005

   Check WySysFile

   Build: make chk_sysfile

   Note: Test data files should exist in the working directory
         Files may be created in the working directory and unlinked
*/
#include "WySysFile.h"
#include "WyStr.h"
#include "wy__cfunc.h"
#include <iostream>

#if WYSYSFILE_VERSION!=33
#error Test code is for WYSYSFILE_VERSION 33
#endif

extern void ck_sysfile(void);

static const WyStr chdr(
                  "+----------------+\n"
                  "| main(): caught |\n"
                  "+----------------+\n");
int main(void) throw()
try {
 std::cout << "Checking WySysFile.h ...\n";
 if(std::strcmp(WySysFile::class_name,"WySysFile")!=0) {
   WY_THROW( WyRet() );
 }
 #ifdef WY_DEBUG
 if(wydbg_get_fdcnt()!=0) {
   WY_THROW( WyRet() );           // wydbg_get_fdcnt() should be 0 at startup
 }
 if(WySysFile::wydbg_get_objcnt()!=0) {
   WY_THROW( WyRet() );
 }
 #endif
 ck_sysfile();

 std::cout << "Checked Ok ";
 #if defined(WY_DEBUG)||defined(NDEBUG)||(WYLIB_CONFIG_THROWSPC)
   std::cout << ".. Symbol defined: "
   #ifdef WY_DEBUG
     "WY_DEBUG "
   #endif

   #ifdef NDEBUG
     "NDEBUG "
   #endif

   #ifdef WYLIB_CONFIG_THROWSPC
     "WYLIB_CONFIG_THROWSPC "
   #endif
   ;
 #endif
 std::cout<< "\n";

 return(0);
}
catch(const WyRet& e) {
 std::cerr << chdr.c_str() << Wy::wrd(e).c_str() << std::endl;
 return(-1);
}
catch(const std::exception& e) {
 std::cerr << chdr.c_str() << "std::exception" << std::endl;
 return(-1);
}
catch(...) {
 std::cerr << chdr.c_str() << "unknown unwind" << std::endl;
 return(-1);
};
