/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   Test Wy_Thread Part-I

   Basic thread check

   Build: make chk_thread1
*/
#include "tsthread.h"
#include "Wy_AtDestroy.hh"
#include "WyByteFlow.h"
#include <cstdlib>

#include <iostream>
#if WY_THREAD_VERSION!=33
#error Test code is for WY_THREAD_VERSION 33
#endif

class T_ErrInfo : public Wy_RepInfo {
    T_ErrInfo(const T_ErrInfo&);
  public:
    T_ErrInfo(const char* cstr) throw() : Wy_RepInfo(-1,cstr) {};
    void setmsg(const char*,const void*, size_t) throw()
       {};
};
static T_ErrInfo ei("Test Failed");
static const WyReply TEST_FAIL(ei);

// Check default TsThread object properties
//
static WyRet chk_default_attr(TsThread& thrd)
{
 Wy_Thread::ThreadState st;
 WyRet tret;
 WyRet r;

 if(thrd.is_default()==false) {
   WY_RETURN(TEST_FAIL);
 }

 st=thrd.thread_state();
 if(st!=Wy_Thread::Null) {
   WY_RETURN(TEST_FAIL);
 }
 if(thrd.thread_id()==Wy_Thread::Wy__Default_ThreadID==false) {
   WY_RETURN(TEST_FAIL);
 }
 if((r=thrd.cancel())!=Wym_ESRCH) {
   WY_RETURN(TEST_FAIL);
 }
 if((r=thrd.wait_not(Wy_Thread::Stopped))!=Ok) {
   WY_RETURN(TEST_FAIL);
 }
 if((r=thrd.wait_not(Wy_Thread::Active))!=Ok) {
   WY_RETURN(TEST_FAIL);
 }

 tret=thrd.exitcode();
 if(!tret.is_default()) {
   WY_RETURN(TEST_FAIL);
 }
 {
  Wy_Thread::CancelType ct( thrd.set_canceltype(Wy_Thread::Deferred) );
  if(ct!=Wy_Thread::Deferred) {
    thrd.set_canceltype(ct);
    WY_RETURN(TEST_FAIL);
  }
 }
 return(Ok);
};

// begin() the default TsThread thrd configured as the arguments
// and chk for expected behavior
//
static void ck_thread_0(TsThread& thrd,TsThread::ExitType etype,WyReply ec,WyTimeSpec delay)
try {
 WyRet r;

 if((r=chk_default_attr(thrd))!=Ok) {
   WY_THROW(r);
 }
 if((r=thrd.begin())!=Ok) {
   WY_THROW(r);
 }
 if((r=thrd.begin())!=Wym_EBADF) {
   WY_THROW(r);
 }
 const WyTimeSpec TM0( Wy::now() );
 WyTimeSpec tm;

 if((r=thrd.wait_not(Wy_Thread::Null))!=Ok) {
   WY_THROW(r);
 }
 if(thrd.thread_state()!=Wy_Thread::Active) {
   WY_THROW( WyRet() );
 }
 if(thrd.thread_id()==Wy_Thread::Wy__Default_ThreadID) {
   WY_THROW( WyRet() );
 }
 switch(etype) {
   case TsThread::Return: {
                      if((r=thrd.wait_not(Wy_Thread::Active))!=Ok) {
                        WY_THROW(r);
                      }
                      tm= Wy::now()-TM0;
                      if(thrd.thread_state()!=Wy_Thread::Stopped) {
                        WY_THROW( WyRet() );
                      }
                      if(thrd.exitcode()!=ec) {
                        WY_THROW( thrd.exitcode() );
                      }
                      if((Wy::abs(tm-delay))>WyTimeSpec(0,50000000)) {
                        Wy::cerr << "Inaccurate delay: > 0.05 second\n";
                        //WY_THROW( WyRet() );
                      }
                      if((r=thrd.reset())!=Ok) {
                        WY_THROW(r);
                      }
                      if((r=chk_default_attr(thrd))!=Ok) {
                        WY_THROW(r);
                      }
                   }; break;
   case TsThread::Exit: {
                      if((r=thrd.wait_not(Wy_Thread::Active))!=Ok) {
                        WY_THROW(r);
                      }
                      tm= Wy::now()-TM0;
                      if(thrd.thread_state()!=Wy_Thread::Stopped) {
                        WY_THROW( WyRet() );
                      }
                      if(thrd.exitcode()!=ec) {
                        WY_THROW( thrd.exitcode() );
                      }
                      if((Wy::abs(tm-delay))>WyTimeSpec(0,50000000)) {
                        Wy::cerr << "Inaccurate delay: > 0.05 second\n";
                        //WY_THROW( WyRet() );
                      }
                      if((r=thrd.reset())!=Ok) {
                        WY_THROW(r);
                      }
                      if((r=chk_default_attr(thrd))!=Ok) {
                        WY_THROW(r);
                      }
                   }; break;
   case TsThread::Cancel: {
                      Wy::sleep(delay);
                      if((r=thrd.cancel())!=Ok) {
                        WY_THROW(r);
                      }
                      if((r=thrd.wait_not(Wy_Thread::Active))!=Ok) {
                        WY_THROW(r);
                      }
                      tm= Wy::now()-TM0;
                      if(thrd.thread_state()!=Wy_Thread::Stopped) {
                        WY_THROW( WyRet() );
                      }
                      if(thrd.exitcode()!=ec) {
                        WY_THROW( thrd.exitcode() );
                      }
                      if((Wy::abs(tm-delay))>WyTimeSpec(0,50000000)) {
                        Wy::cerr << "Inaccurate delay: > 0.05 second\n";
                        //WY_THROW( WyRet() );
                      }
                      if((r=thrd.reset())!=Ok) {
                        WY_THROW(r);
                      }
                      if((r=chk_default_attr(thrd))!=Ok) {
                        WY_THROW(r);
                      }
                   }; break;
   case TsThread::Reset: {
                      Wy::sleep(delay);
                      if((r=thrd.reset())!=Ok) {
                        WY_THROW(r);
                      }
                      tm= Wy::now()-TM0;
                      if((Wy::abs(tm-delay))>WyTimeSpec(0,50000000)) {
                        Wy::cerr << "Inaccurate delay: > 0.05 second\n";
                        //WY_THROW( WyRet() );
                      }
                      if((r=chk_default_attr(thrd))!=Ok) {
                        WY_THROW(r);
                      }
                   }; break;
   default:
        WY_TERMINATE("");
 };

 // chk reset() again (diff code path)
 if((r=thrd.reset())!=Ok) {
   WY_THROW(r);
 }
 if((r=chk_default_attr(thrd))!=Ok) {
   WY_THROW(r);
 }
}
catch(const WyRet& e) {
 Wy::cerr << Wy::wrd(e) << '\n';
 throw;
}
catch(...) {
 throw;
};

static void ck_thread_a(void)
{
 const WyTimeSpec TDelay(0,500000000);   // 0.5 sec
 WyRet r;

 if(Wy_Thread::max_threads()!=512) {
   WY_THROW( WyRet() );
 }

 // chk stopped by return
 {
   TsThread thrd1(TsThread::Return,Wym_EIO,TDelay);
   ck_thread_0(thrd1,TsThread::Return,Wym_EIO,TDelay);
   if(thrd1.thread_id()!=thrd1.thread_id()) {
     WY_THROW( WyRet() );
   }
   if(thrd1.thread_id()==Wy_Thread::thread_self()) {
     WY_THROW( WyRet() );
   }
 }

 // chk stopped by exit
 {
   TsThread thrd1(TsThread::Exit,Wym_EINVAL,TDelay);
   ck_thread_0(thrd1,TsThread::Exit,Wym_EINVAL,TDelay);
 }

 // chk stopped by cancel
 {
   TsThread thrd1(TsThread::Cancel,Wym_ETHRDCAN,TDelay);
   ck_thread_0(thrd1,TsThread::Cancel,Wym_ETHRDCAN,TDelay);
 }

 // chk stopped by reset
 {
   TsThread thrd1(TsThread::Reset,WyRet(),TDelay);
   ck_thread_0(thrd1,TsThread::Reset,WyRet(),TDelay);
 }

 // chk stopped by destroy TsThread object
 {
   {
     TsThread thrd1(TsThread::Reset,WyRet(),TDelay);
     if((r=thrd1.begin())!=Ok) {
       WY_THROW( WyRet() );
     }
   }
 }
};

// non Wy_Thread created thread function
//
static void* nw_thread_routine(void* arg)
{
 // chk tbase_ptr()
 if(Wy_Thread::tbase_ptr()!=0) {
   WY_TERMINATE("");
 }

 // chk thread_self()
 const ::pthread_t SelfID=::pthread_self();
 if(Wy::_thread_equal(Wy_Thread::thread_self(),SelfID)==false) {
   WY_THROW( WyRet() );
 }
 if(Wy::_thread_equal(Wy_Thread::thread_self(),SelfID)==0) {
   WY_THROW( WyRet() );
 }

 // chk cancel_point(), just verify available
 Wy_Thread::cancel_point();

 // chk exit(WyRet)
 Wy_Thread::exit( WyRet(Wym_E41) );
};

// chk some static member in none Wy_Thread created thread
//
static void ck_thread_nwy(void)
{
 pthread_t tid;
 void* tres;
 int v;
 if((v=::pthread_create(&tid,NULL,nw_thread_routine,NULL))!=0) {
   WY_THROW( WyRet(WyReply(v)) );
 }
 if((v=::pthread_join(tid, &tres))!=0) {
   WY_THROW( WyRet(WyReply(v)) );
 }
 if((int)tres!=Wym_E41->repcode()) {
   WY_THROW( WyRet() );
 }
};

//-------------------

// HThread runs ck_thread_a in tmain()
//
class HThread : public Wy_Thread {
  public:
    HThread() : Wy_Thread() {};
    ~HThread() throw() { Wy_Thread::tmain_close(); };
  protected:
    WyRet tmain(void) {
         ck_thread_a();
         return(Ok);
       };
};

static WyRet exaust_threads(void)
{
 const WyTimeSpec TDelay(0,10000000);   // 0.01 sec
 WyRet r;
 TsThread thrd1(TsThread::Cancel,Wym_ETHRDCAN,WyTimeSpec(1,0));
 if((r=thrd1.begin())!=Ok) {
   WY_RETURN(r);
 }
 TsThread thrd2(TsThread::Return,Wym_EIO,TDelay);
 if((r=thrd2.begin())!=Ok) {
   WY_RETURN(r);
 }
 TsThread thrd3(TsThread::Exit,Wym_EINVAL,TDelay);
 if((r=thrd3.begin())!=Ok) {
   WY_RETURN(r);
 }
 return exaust_threads();
};

static const WyStr chdr(
                  "+---------------+\n"
                  "| main() caught:|\n"
                  "+---------------+\n");
int main(void) throw()
try {

 WyRet r;
 Wy::cout << "Checking Wy_Thread.cpp, basic ...\n";
 if(WyStr(Wy_Thread::class_name)!="Wy_Thread") {
   WY_THROW(WyRet() );
 }
 Wy::cout << "sizeof(Wy_Thread)=" << sizeof(Wy_Thread) << '\n';

 if(Wy_Thread::thread_self()!=Wy_Thread::thread_self()) {
   WY_THROW( WyRet() );
 }
 #ifdef WY_DEBUG
   if(wy_created_cnt!=wy_joined_cnt) {
     WY_THROW( WyRet() );
   }
 #endif
 
 // chk NoCancel in main thread
 {
  {
   Wy_Thread::NoCancel noc1;
   if(noc1.was_cancelable()==false) {
     WY_THROW( WyRet() );
   }
   Wy_Thread::NoCancel noc2;
   if(noc2.was_cancelable()==true) {
     WY_THROW( WyRet() );
   }
  }
  bool canstt;
  canstt=Wy_Thread::set_cancelable(true);
  if(canstt!=true) {
    WY_THROW( WyRet() );
  }
 }

 // chk main thread id
 {
  const ::pthread_t SelfID=::pthread_self();
  if(Wy::_thread_equal(Wy_Thread::wy_main_thrd_id,SelfID)==false) {
    WY_THROW( WyRet() );
  }
  if(Wy::_thread_equal(Wy_Thread::wy_main_thrd_id,SelfID)==0) {
    WY_THROW( WyRet() );
  }
  Wy::cout << "Wy::wrd(Wy_Thread::Wy__Default_ThreadID)=\"" 
           << Wy::wrd(Wy_Thread::Wy__Default_ThreadID) << "\"\n";
 }

 // chk ck_thread_a()
 {
   if(Wy_Thread::active_threads()!=0) {
     WY_THROW( WyRet() );
   }
   ck_thread_a();
   if(Wy_Thread::active_threads()!=0) {
     Wy::cerr << "active_threads()=" << Wy_Thread::active_threads() << '\n';
     WY_THROW( WyRet() );
   }
   #ifdef WY_DEBUG
     if(wy_created_cnt!=wy_joined_cnt) {
       WY_THROW( WyRet() );
     }
   #endif
 }

 // chk ck_thread_a() concurrently in different threads
 {
   const size_t Num_HThreads=20;
   HThread ht[Num_HThreads];

   for(size_t i=0; i< Num_HThreads; ++i) {
     if((r=ht[i].begin())!=Ok) {
       WY_THROW(r);
     }
     if((r=ht[i].begin())!=Wym_EBADF) {
       WY_THROW(r);
     }
   }
   for(size_t i=0; i< Num_HThreads; ++i) {
     if((r=ht[i].wait_not(Wy_Thread::Active))!=Ok) {
       WY_THROW(r);
     }
   }
   if(Wy_Thread::active_threads()!=0) {
     WY_THROW( WyRet() );
   }
 }

 // chk some static functions in non Wy_Thread created thread
 ck_thread_nwy();

 // chk thread exausted
 r=exaust_threads();
 if((r!=Wym_EAGAIN)&&(r!=Wym_ENOMEM)
    &&(r!=Wym_ENFILE)       // on SUSE10 this can occur (g++ v4.03)
   ) {
   WY_THROW(r);
 }

 #ifdef WY_DEBUG
   if(wy_created_cnt!=wy_joined_cnt) {
     WY_THROW( WyRet() );
   }
   Wy::cout << wy_created_cnt << " threads have been created\n";
   // for now 125 is correct
   Wy::cout << wy_act_max << " maximal active threads\n";
 #endif

 Wy::cout << "Checked Ok\n";

 return(0);
}
catch(const WyRet& e) {
 Wy::cerr << chdr << Wy::wrd(e) << '\n';
 return(-1);
}
catch(const std::exception& e) {
 Wy::cerr << chdr << "std::exception\n";
 return(-1);
}
catch(...) {
 Wy::cerr << chdr << "unidentified stack unwound\n";
 return(-1);
};
