/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2003

   Check WyChrFile public members

   Note: Test data files should exist in the working directory
         Files may be created in the working directory and unlinked
*/

#include "WyStr.h"
#include "WyChrFile.h"
#include "WyFileStat.h"
#include "Wy_AtDestroy.hh"
#include "wy__cfunc.h"   // for Wy::_delete..
#include <new>

#if WYCHRFILE_VERSION!=33
#error Test code is for WYCHRFILE_VERSION 33
#endif

void ck_chrfile(bool);

#ifdef WY_DEBUG
static int initial_fdcnt=0;
#endif

static bool exaustfd_flag=true;
static const int NoSuchFd=-2;
static const char NoSuchPathName[]="No.Such.Pathname.tmp";
static const WyStr RefFileName("/dev/tty");

// Assert ff is default
//
static void assert_default(const WyChrFile& ff)
{
 WyRet r;
 if(ff.is_default()==false) {
   WY_THROW( WyRet() );
 }
 if(ff.fh().is_default()==false) {
   WY_THROW( WyRet() );
 }

 // chk fh()
 if(ff.fh()!=WyFileHandle()) {
   WY_THROW( WyRet() );
 }

 // stat
 {
   WyFileStat stt;
   if((r=ff.stat(stt))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if(stt.is_default()==false) {
     WY_THROW( WyRet() );
   }
 }

 // The following are non-const, so construct ff from tmpf for checks
 //
 WyChrFile tmpf(ff);

 // read(void*,..)
 {
  char buf[2];
  size_t n_rd=sizeof(buf);
  if((r=tmpf.read(buf,sizeof(buf),n_rd))!=Wym_EBADF) {
    WY_THROW(r);
  }
  if(n_rd!=0) {
    WY_THROW( WyRet() );
  }
 }

 // read(WyStr&)
 {
  WyStr buf;
  size_t n_rd=5;
  if((r=tmpf.read(buf,size_t(5),n_rd))!=Wym_EBADF) {
    WY_THROW(r);
  }
  if(n_rd!=0) {
    WY_THROW( WyRet() );
  }
  if(buf.is_default()==false) {
    WY_THROW( WyRet() );
  }
 }

 // write
 {
  char buf[2]={5,6};
  size_t n_wr=sizeof(buf);
  if((r=tmpf.write(buf,sizeof(buf),n_wr))!=Wym_EBADF) {
    WY_THROW(r);
  }
  if(n_wr!=0) {
    WY_THROW( WyRet() );
  }
  if((buf[0]!=5)||(buf[1]!=6)) {
    WY_THROW( WyRet() );
  }
  WyStr str("ab");
  if((r=tmpf.write(str,n_wr))!=Wym_EBADF) {
    WY_THROW(r);
  }
  if(n_wr!=0) {
    WY_THROW( WyRet() );
  }
  if((r=tmpf.write( WyCSeg(str.cseg()),n_wr))!=Wym_EBADF) {
    WY_THROW(r);
  }
  if(n_wr!=0) {
    WY_THROW( WyRet() );
  }
 }

 //  _tcdrain,_isatty
 if((r=tmpf._tcdrain())!=Wym_EBADF) {
   WY_THROW(r);
 }

 // reset(const WyStr&,int)
 if((r=tmpf.reset(WyStr(NoSuchPathName),O_RDONLY))!=Wym_ENOENT) {
   WY_THROW(r);
 }
 if(tmpf.is_default()==false) {
   WY_THROW( WyRet() );
 }

 // reset(const char*,int)
 if((r=tmpf.reset(NoSuchPathName,O_RDONLY))!=Wym_ENOENT) {
   WY_THROW(r);
 }
 if(tmpf.is_default()==false) {
   WY_THROW( WyRet() );
 }
 if((r=tmpf.reset("/dev/zero",O_RDONLY))!=Ok) {
   WY_THROW(r);
 }
 if((r=tmpf.reset())!=Ok) {
   WY_THROW(r);
 }

 // reset(WyFileHandle)
 {
  if((r=tmpf.reset(WyFileHandle()))!=Ok) {
    WY_THROW(r);
  }
  if(tmpf.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if((r=tmpf.reset(tmpf))!=Ok) {
    WY_THROW(r);
  }
  if(tmpf.is_default()==false) {
    WY_THROW( WyRet() );
  }
  if((r=tmpf.reset(ff))!=Ok) {
    WY_THROW(r);
  }
  if(tmpf.is_default()==false) {
    WY_THROW( WyRet() );
  }
 }

 // reset()
 if((r=tmpf.reset())!=Ok) {
   WY_THROW(r);
 }

 if((r=tmpf.reset(ff))!=Ok) {
   WY_THROW(r);
 }
 if(tmpf.is_default()==false) {
   WY_THROW( WyRet() );
 }

 if(static_cast<const WyByteFlow&>(ff).ftype()!=0) {
   WY_THROW( WyRet() );
 }
};

// Assert ff is not default
//
static void assert_not_default(const WyChrFile& ff)
{
 WyRet r;

 WyChrFile NObj;     // non-default object for the check
 if(NObj.is_default()==false) {
   WY_THROW( WyRet() );
 }
 if((r=NObj.reset(RefFileName,O_RDONLY))!=Ok) {
   WY_THROW(r);
 }
 if(NObj.is_default()) {
   WY_THROW( WyRet() );
 }

 if(ff.is_default()) {
   WY_THROW( WyRet() );
 }
 if(ff.fh().is_default()) {
   WY_THROW( WyRet() );
 }

 // stat
 {
   WyFileStat stt;
   if((r=ff.stat(stt))!=Ok) {
     WY_THROW(r);
   }
   if(stt.is_chr()==false) {
     WY_THROW( WyRet() );
   }
 }

 // The following are non-const, so construct ff to tmpf for checks
 //
 {
  WyChrFile tmpf(NObj);

  if(tmpf.is_default()) {
    WY_THROW( WyRet() );
  }
  if((r=tmpf.reset())!=Ok) {
    WY_THROW(r);
  }
  if(tmpf.is_default()==false) {
    WY_THROW( WyRet() );
  }
 }

 // read (char)
 {
  WyChrFile tmpf(NObj);
  if(tmpf.is_default()) {
    WY_THROW( WyRet() );
  }
  char buf[2];
  size_t n_rd=sizeof(buf);
  if((r=tmpf.read(buf,size_t(0),n_rd))!=Ok) {
    WY_THROW(r);
  }
  if(n_rd!=0) {
    WY_THROW( WyRet() );
  }
 }

 // read (WyStr)
 {
  WyChrFile tmpf(NObj);
  if(tmpf.is_default()) {
    WY_THROW( WyRet() );
  }
  WyStr buf;
  size_t n_rd=5;
  if((r=tmpf.read(buf,size_t(0),n_rd))!=Ok) {
    WY_THROW(r);
  }
  if(n_rd!=0) {
    WY_THROW( WyRet() );
  }
 }

 // reset(WyFileHandle)
 {
  WyChrFile tmpf(NObj);
  if(tmpf.is_default()) {
    WY_THROW( WyRet() );
  }
  if((r=tmpf.reset(ff))!=Ok) {
    WY_THROW(r);
  }
  if(tmpf.is_default()) {
    WY_THROW( WyRet() );
  }
  if((r=tmpf.reset(WyFileHandle()))!=Ok) {
    WY_THROW(r);
  }
  if(tmpf.is_default()==false) {
    WY_THROW( WyRet() );
  }
 }

 if(static_cast<const WyByteFlow&>(ff).ftype()!=S_IFCHR) {
   WY_THROW( WyRet() );
 }
};

// Assert f1,f2 refers to the same systerm file
//
template<typename Dev1, typename Dev2>
static void assert_same_sysf(const Dev1& f1,const Dev2& f2)
{
 if(f1.is_default()) {
   assert_default(f1);
 } else {
   assert_not_default(f1);
 }

 if(f2.is_default()) {
   assert_default(f2);
 } else {
   assert_not_default(f2);
 }

 if(f1.is_default()&&f2.is_default()) {
   return;
 }
 WyRet r;
 WyFileStat stt1,stt2;
 if((r=f1.stat(stt1))!=Ok) {
   WY_THROW( WyRet() );
 }
 if((r=f2.stat(stt2))!=Ok) {
   WY_THROW( WyRet() );
 }
 if(stt1.sf_ino()!=stt2.sf_ino()) {
   WY_THROW( WyRet() );
 }
 if(stt1.sf_dev()!=stt2.sf_dev()) {
   WY_THROW( WyRet() );
 }
};

#define CHK_DEFAULT(p)\
    try { assert_default(p); } catch(...) { WY_THROW( WyRet()); }
#define CHK_NOT_DEFAULT(p)\
    try { assert_not_default(p); } catch(...) { WY_THROW( WyRet()); }
#define CHK_SAME_SYSF(p1,p2)\
    try { assert_same_sysf(p1,p2); } catch(...) { WY_THROW( WyRet()); }

static void exaust_cpy_ctor(const WySysFile& chrf)
{
 WyChrFile tmp(chrf);
 exaust_cpy_ctor(chrf);   // recursive until failure
};

static void exaust_cpy_ctor(const WyChrFile& chrf)
{
 WyChrFile tmp(chrf);
 exaust_cpy_ctor(chrf);   // recursive until failure
};

//
// Setup the test 
//
static void setup_test(void)
{
};

//
// Undo setup_test
//
static void close_test(void)
{
};

//
// Check basic operations on default object
//
static void t1(void)
{
 WyRet r;

 WyChrFile NObj;     // non-default object for test as argument
 CHK_DEFAULT(NObj);
 if((r=NObj.reset(RefFileName,O_RDONLY))!=Ok) {
   WY_THROW(r);
 }
 CHK_NOT_DEFAULT(NObj);
 
 // stat
 {
   WyFileStat stt;
   if((r=NObj.stat(stt))!=Ok) {
     WY_THROW(r);
   }
   if(stt.is_chr()==false) {
     WY_THROW( WyRet() );
   }
 }

 // WyChrFile(),WyChrFile(const WyChrFile&),WyChrFile(const WyByteFlow&)
 {
   WyChrFile tmp0;
   WyChrFile tmp1(tmp0);
   CHK_DEFAULT(tmp0);
   CHK_DEFAULT(tmp1);

   WyChrFile tmp2(NObj);
   CHK_NOT_DEFAULT(tmp2);
   if(tmp2.fh()==NObj.fh()) {
     WY_THROW( WyRet() );
   }
   CHK_SAME_SYSF(tmp2,NObj);

   WyChrFile tmp3(static_cast<WyByteFlow&>(tmp0));
   CHK_SAME_SYSF(tmp3,tmp0);
   WyChrFile tmp4(static_cast<WyByteFlow&>(NObj));
   CHK_SAME_SYSF(tmp4,NObj);
   WyChrFile tmp5(NObj.fh());
   CHK_SAME_SYSF(tmp5,NObj);
 }

 // WyChrFile(const WyStr&,int) WyChrFile(const char*,int)
 {
   WyChrFile tmp1(RefFileName,O_RDONLY);  // same as NObj
   CHK_SAME_SYSF(tmp1,NObj);

   WyChrFile tmp2(RefFileName.c_str(),O_RDONLY);  // same as NObj
   CHK_SAME_SYSF(tmp2,tmp1);
 }

 // WyChrFile(WyFileHandle)
 {
   WyChrFile tmp( WyFileHandle(STDERR_FILENO) );
   CHK_NOT_DEFAULT(tmp);
   WyFileStat stt;
   if((r=tmp.stat(stt))!=Ok) {
     WY_THROW(r);
   }
   if(stt.is_chr()==false) {
     WY_THROW( WyRet() );
   }
   if(tmp.fh().fd()==STDERR_FILENO) {
     WY_THROW( WyRet() );
   }
 }

 // chk _alloc)
 {
   WyChrFile tmp;
   WyChrFile* p=tmp._alloc(r);
   if(r!=Ok) {
     WY_THROW(r);
   }
   if(p==0) {
     WY_THROW( WyRet() );
   }
   Wy_AtDestroy<void,WyChrFile*> rrid(Wy::_delete_one,p);
   CHK_DEFAULT(*p);
   if((r=tmp._swap(*p))!=Ok) {
     WY_THROW(r);
   }
    CHK_DEFAULT(*p);
    CHK_DEFAULT(tmp);
 }

 // reset(void)
 {
   WyChrFile tmp;
   CHK_DEFAULT(tmp);
   if((r=tmp.reset())!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);

   WyChrFile tmp2(NObj);
   CHK_NOT_DEFAULT(tmp2);
   if((r=tmp2.reset())!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp2);
 }

 // reset(const WyByteFlow&)
 {
   WyChrFile tmp;
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(static_cast<WyByteFlow&>(NObj)))!=Ok) {
     WY_THROW(r);
   }
   CHK_SAME_SYSF(tmp,NObj);
   if((r=tmp.reset(NObj))!=Ok) {
     WY_THROW(r);
   }
   CHK_SAME_SYSF(tmp,NObj);
   const WyFileHandle TmpFh( tmp.fh() );
   if((r=tmp.reset(tmp))!=Ok) {
     WY_THROW(r);
   }
   if(tmp.fh()!=TmpFh) {
     // Ok either way
   }

   tmp.reset();
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(WyByteFlow()))!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(Wy::cin))!=Ok) {
     WY_THROW(r);
   }
   if((r=tmp.reset(WyChrFile()))!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);

 }

 // reset(const char*,int)
 {
   WyChrFile tmp;
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(RefFileName.c_str(),O_RDONLY))!=Ok) {  // same as NObj
     WY_THROW(r);
   }
   CHK_SAME_SYSF(tmp,NObj);
 }

 // reset(const WyStr&,int)
 {
   WyChrFile tmp;
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(RefFileName,O_RDONLY))!=Ok) {  // same as NObj
     WY_THROW(r);
   }
   CHK_SAME_SYSF(tmp,NObj);
 }

 // reset(WyFileHandle)
 {
   WyChrFile tmp;
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(WyFileHandle(STDERR_FILENO)))!=Ok) {  // same as NObj
     WY_THROW(r);
   }
   CHK_NOT_DEFAULT(tmp);
   WyFileStat stt;
   if((r=tmp.stat(stt))!=Ok) {
     WY_THROW(r);
   }
   if(stt.is_chr()==false) {
     WY_THROW( WyRet() );
   }

   tmp.reset();
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(WyChrFile()))!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);

   if((r=tmp.reset(tmp))!=Ok) {
     WY_THROW(r);
   }
   if((r=tmp.reset(NObj.fh()))!=Ok) {
     WY_THROW(r);
   }
   CHK_SAME_SYSF(tmp,NObj);
   const WyFileHandle TmpFh( tmp.fh() );
   if((r=tmp.reset(tmp))!=Ok) {
     WY_THROW(r);
   }
   if(tmp.fh()!=TmpFh) {
     // Ok either way
   }
 }

 // _swap
 {
  WyChrFile tmp0(NObj),tmp1,tmp2;
  CHK_NOT_DEFAULT(tmp0);
  CHK_DEFAULT(tmp1);
  CHK_DEFAULT(tmp2);
  CHK_SAME_SYSF(tmp0,NObj);

  if((r=tmp1._swap(tmp2))!=Ok) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp1);
  CHK_DEFAULT(tmp2);

  if((r=tmp1._swap(tmp0))!=Ok) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp0);
  CHK_NOT_DEFAULT(tmp1);
  CHK_SAME_SYSF(tmp1,NObj);

  if((r=tmp0._swap(tmp1))!=Ok) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp1);
  CHK_NOT_DEFAULT(tmp0);
  CHK_SAME_SYSF(tmp0,NObj);

  WyByteFlow bf1;
  if((r=tmp1._swap(bf1))!=Wym_EBADTYPE) {
    WY_THROW( WyRet() );
  }
 }

 // read,write ...(const char*)
 {
   WyChrFile tmp;
   CHK_DEFAULT(tmp);
   char buf[2]={6,7};
   size_t n_rd(99);
   if((r=tmp.read(buf,sizeof(buf),n_rd))!=Wym_EBADF) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
   if(n_rd!=0) {
     WY_THROW( WyRet() );
   }
   if((buf[0]!=6)||(buf[1]!=7)) {
     WY_THROW( WyRet() );
   }
   n_rd=5;
   if((r=tmp.read(buf,0,n_rd))!=Wym_EBADF) {
     WY_THROW(r);
   }
   if(n_rd!=0) {
     WY_THROW( WyRet() );
   }
   n_rd=5;
   if((r=NObj.read(buf,0,n_rd))!=Ok) {
     WY_THROW(r);
   }
   if(n_rd!=0) {
     WY_THROW( WyRet() );
   }
   size_t n_wr(7);
   if((r=tmp.write(buf,sizeof(buf),n_wr))!=Wym_EBADF) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
   if(n_wr!=0) {
     WY_THROW( WyRet() );
   }
   if((buf[0]!=6)||(buf[1]!=7)) {
     WY_THROW( WyRet() );
   }
   n_wr=100;
   if((r=tmp.write(WyStr(),n_wr))!=Wym_EBADF) {
     WY_THROW( WyRet() );
   }
   CHK_DEFAULT(tmp);
   if(n_wr!=0) {
     WY_THROW( WyRet() );
   }
   n_wr=100;
   if((r=tmp.write(WyCSeg("",size_t(0)),n_wr))!=Wym_EBADF) {
     WY_THROW( WyRet() );
   }
   CHK_DEFAULT(tmp);
   if(n_wr!=0) {
     WY_THROW( WyRet() );
   }
   if((r=tmp._tcdrain())!=Wym_EBADF) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
   if((buf[0]!=6)||(buf[1]!=7)) {
     WY_THROW( WyRet() );
   }
 }

 // read(WyStr)
 {
   WyChrFile tmp;
   CHK_DEFAULT(tmp);
   WyStr buf;
   size_t n_rd=5;
   if((r=tmp.read(buf,size_t(1),n_rd))!=Wym_EBADF) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
   if(n_rd!=0) {
     WY_THROW( WyRet() );
   }
   if(buf.is_default()==false) {
     WY_THROW( WyRet() );
   }
 }

 // reset(WyFileHandle)
 //
 {
   WyChrFile tmp;
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(WyFileHandle()))!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);

   if((r=tmp.reset(NObj.fh()))!=Ok) {
     WY_THROW(r);
   }
   CHK_NOT_DEFAULT(tmp);

   if(tmp.fh()==NObj.fh()) {
     WY_THROW( WyRet() );
   }

   if((r=tmp.reset(WyFileHandle()))!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
 }

 // reset(WyChrFile)
 //
 {
   WyChrFile tmp;
   CHK_DEFAULT(tmp);
   if((r=tmp.reset(WyChrFile()))!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);

   if((r=tmp.reset(NObj))!=Ok) {
     WY_THROW(r);
   }
   CHK_NOT_DEFAULT(tmp);

   if(tmp.fh()==NObj.fh()) {
     WY_THROW( WyRet() );
   }

   if((r=tmp.reset(WyChrFile()))!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
 }

 // chk move ctor
 {
  char raw_buf[2*sizeof(WyChrFile)];
  ::memset(raw_buf,0,sizeof(raw_buf));
  WyChrFile* const p1( reinterpret_cast<WyChrFile*>(raw_buf) );
  WyChrFile* const p2(p1+1);

  new(p1) WyChrFile(RefFileName,O_RDONLY);
  if(p1->_isatty()==false) {
    p1->~WyChrFile();
    WY_THROW( WyRet() );
  }

  // move *p1 to p2
  WyFileHandle ofh( p1->fh() );
  try {
    new(p2) WyChrFile(*p1,Wy::ByMove);
  }
  catch(...) {
    p1->~WyChrFile();
    throw;
  };
  WyFileHandle nfh( p2->fh() );
  p2->~WyChrFile();

  if(nfh!=ofh) {
    WY_THROW( WyRet() );
  }
 }

};

// Check basic operations on non-default object
//
static void t2(void)
{
 WyRet r;
 
 // stat NObj
 WyChrFile NObj;     // non-default object for the test
 CHK_DEFAULT(NObj);
 if((r=NObj.reset(RefFileName,O_RDWR))!=Ok) {
   WY_THROW(r);
 }
 CHK_NOT_DEFAULT(NObj);
 
 // stat
 {
   WyFileStat stt;
   if((r=NObj.stat(stt))!=Ok) {
     WY_THROW(r);
   }
   if(stt.is_chr()==false) {
     WY_THROW( WyRet() );
   }
 }

 // constructor
 {
   WyChrFile tmp(NObj);
   CHK_NOT_DEFAULT(tmp);
   if(tmp.fh()==NObj.fh()) {
     WY_THROW( WyRet() );
   }
   CHK_SAME_SYSF(tmp,NObj);
 }

 // reset()
 {
   WyChrFile tmp(NObj);
   CHK_NOT_DEFAULT(tmp);
   if((r=tmp.reset())!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
 }

 // reset(const WyByteFlow&)
 {
   WyChrFile tmp(NObj);
   CHK_SAME_SYSF(tmp,NObj);
   if((r=tmp.reset(static_cast<WyByteFlow&>(NObj)))!=Ok) {
     WY_THROW(r);
   }
   CHK_SAME_SYSF(tmp,NObj);
 }

 // chk t.reset(t)
 {
   WyChrFile tmp(NObj);
   WyFileHandle TmpFh( tmp.fh() );
   if((r=tmp.reset(tmp))!=Ok) {
     WY_THROW(r);
   }
   if(TmpFh!=tmp.fh()) {
     // Ok either way
   }
   CHK_SAME_SYSF(tmp,NObj);
   if((r=tmp.reset())!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
   if((r=tmp.reset())!=Ok) {
     WY_THROW(r);
   }
   CHK_DEFAULT(tmp);
 }

 // reset(const char*,int)
 {
   WyChrFile tmp(NObj);
   CHK_SAME_SYSF(tmp,NObj);
   if((r=tmp.reset(RefFileName.c_str(),O_RDONLY))!=Ok) {
     WY_THROW(r);
   }
   //CHK_SAME_SYSF(tmp,NObj);
 }

 // reset(const WyStr&,int)
 {
   WyChrFile tmp(NObj);
   CHK_SAME_SYSF(tmp,NObj);
   if((r=tmp.reset(RefFileName,O_RDONLY))!=Ok) {
     WY_THROW(r);
   }
   //CHK_SAME_SYSF(tmp,NObj);
 }

 // reset(WyFileHandle)
 {
   WyChrFile tmp(NObj);
   CHK_SAME_SYSF(tmp,NObj);
   if((r=tmp.reset(WyFileHandle(STDERR_FILENO)))!=Ok) {
     WY_THROW(r);
   }
   //CHK_SAME_SYSF(tmp,NObj);
 }

 // (zero byte) read,write
 {
   WyChrFile tmp(NObj);
   char buf[2]={-1,-1};
   size_t n_rd=5;   // anything >1
   CHK_NOT_DEFAULT(tmp);
   if((r=tmp.read(buf,0,n_rd))!=Ok) {
     WY_THROW(r);
   }
   CHK_NOT_DEFAULT(tmp);
   if(n_rd!=0) {
     WY_THROW( WyRet() );
   }
   n_rd=200;

   if((r=tmp.write(buf,0,n_rd))!=Ok) {
     WY_THROW(r);
   }
   CHK_NOT_DEFAULT(tmp);
   if(n_rd!=0) {
     WY_THROW( WyRet() );
   }
   if((r=tmp.write(WyStr(),n_rd))!=Ok) {
     WY_THROW(r);
   }
   CHK_NOT_DEFAULT(tmp);
   if(n_rd!=0) {
     WY_THROW( WyRet() );
   }
   if(tmp._isatty()==false) {
     WY_THROW( WyRet() );
   }
   if((r=tmp._tcdrain())!=Ok) {  // tmp is tty
     WY_THROW(r);
   }
   CHK_NOT_DEFAULT(tmp);
 }

};

static void t_throw(void)
{
 // WyChrFile(const WyChrFile&)
 //
 if(exaustfd_flag) {
   try {
     const WyChrFile tmp(RefFileName,O_RDONLY);
     exaust_cpy_ctor(tmp);
     WY_THROW( WyRet() );  // not expected
   }
   catch(const WyChrFile::Reply& e) {
     if((e!=Wym_EMFILE)&&(e!=Wym_ENOMEM)) {
       WY_THROW( WyRet(e) );
     }
     // FALL_THROUGH 
   }
   catch(...) {
     throw;
   };
 }

 // WyChrFile(const char*,int)
 //
 try {
   WyChrFile tt2("no.such.name",0);
   WY_THROW( WyRet() );  // not expected
 }
 catch(const WyChrFile::Reply& e) {
   if(e!=Wym_ENOENT) {
     WY_THROW( WyRet(e) );
   }
   // FALL_THROUGH 
 }
 catch(...) {
   throw;
 };

 // WyChrFile(const WyStr&,int)
 //
 try {

   // WyChrFile tt2(WyStr(NoSuchPathName),0);  // wont compile????
   //
   WyChrFile tt2(WyStr("no.such.name"),0);
   WY_THROW( WyRet() );  // not expected
 }
 catch(const WyChrFile::Reply& e) {
   if(e!=Wym_ENOENT) {
     WY_THROW( WyRet(e) );
   }
   // FALL_THROUGH 
 }
 catch(...) {
   throw;
 };

 // WyChrFile(WyFileHandle)
 //
 {
   int v= wyc_open(__FILE__,O_RDONLY);
   if(v==-1) {
     WY_THROW( WyRet(WyReply(errno)) );
   }
   Wy_AtDestroy<int,const Wy__FdType&> rrid(WySysFile::wy_close,v);
   try {
     WyChrFile tmp(( WyFileHandle(v) ));    /* See the following check */
     WY_THROW( WyRet() );
   }
   catch(const WyChrFile::Reply& e) {
     if(e!=Wym_EBADF) {
       WY_THROW( WyRet(e) );
     }
     // FALL_THROUGH 
   };

   try {
     WyChrFile tmp( (WyFileHandle(v)) );
     WY_THROW( WyRet() );
   }
   catch(const WyChrFile::Reply& e) {
     if(e!=Wym_EBADF) {
       WY_THROW( WyRet(e) );
     }
     // FALL_THROUGH 
   };
 }

 try {
   //WyChrFile tmp( WyFileHandle(NoSuchFd) );  // wont compile????
   WyChrFile tmp( WyFileHandle(-2) );  // assert no fd(-2)
   WY_THROW( WyRet() );  // not expected
 }
 catch(const WyChrFile::Reply& e) {
   if(e!=Wym_EBADF) {
     WY_THROW( WyRet(e) );
   }
   // FALL_THROUGH 
 }
 catch(const WyRet& e) {
     WY_THROW( WyRet(e) );
 }
 catch(...) {
   throw;
 };

 // Test destructor throw
 //
 {
   WyChrFile tmp;
   //std::memset(&tmp,NoSuchFd,sizeof(tmp));  
   // destructor of tmp is now not to throw for this setting
 }
};

static void t_error_ret(void)
{
 WyRet r;

 // stat
 {
  WyChrFile tmp;
  WyFileStat stt;
  if((r=tmp.stat(stt))!=Wym_EBADF) {
    WY_THROW(r);
  }
 }

 // reset(const char*,int)
 {
  WyChrFile tmp;
  if((r=tmp.reset("no.such.name",0))!=Wym_ENOENT) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);
 }

 // reset(const WyStr&,int)
 {
  WyChrFile tmp;
  if((r=tmp.reset(WyStr("no.such.name"),0))!=Wym_ENOENT) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);
 }

 // reset(WyFileHanle)
 {
  WyChrFile tmp;
  if((r=tmp.reset( WyFileHandle(-1) ))!=Ok) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);
 }

 // reset() base
 {
  WyChrFile tmp;
  CHK_DEFAULT(tmp);
  WyByteFlow *bp=&tmp;
  if((r=bp->reset())!=Ok) {
    WY_THROW(r);
  }
  CHK_DEFAULT(tmp);
 }

 // EFBIG
 {
  WyChrFile tmp(RefFileName,O_RDONLY);
  WyStr buf;
  size_t n_rd(8);
  CHK_NOT_DEFAULT(tmp);
  if((r=tmp.read(buf,WyStr::max_capacity()+1,n_rd))!=Wym_EFBIG) {
    WY_THROW(r);
  }
  if(n_rd!=0) {
    WY_THROW( WyRet() );
  }
 }

 // chk WyByteFlow::reset(void)
 {
  WyChrFile tmp(RefFileName,O_RDONLY);
  if((r=static_cast<WyByteFlow&>(tmp).reset())!=Ok) {
    WY_THROW(r);
  }
  if(tmp.fh()!=WyFileHandle()) {
    WY_THROW(r);
  }
 }

 // chk _alloc not override
 {
   class T : public WyChrFile {
   } tmp;
   WyChrFile* p=tmp._alloc(r);
   if(r!=Wym_ENOSYS) {
     if(r==Ok) {
       delete p;
     }
     WY_THROW(r);
   }
 }
};

static void t_read_line(void)
{
 //WyRet r;
 WyChrFile f0;
 WyChrFile f1(RefFileName,O_RDONLY);

 CHK_DEFAULT(f0);
 CHK_NOT_DEFAULT(f1);
};

void ck_chrfile(bool exaustfd)
try {
  exaustfd_flag=exaustfd;
#ifdef WY_DEBUG
 initial_fdcnt=wydbg_get_fdcnt();
#endif
   setup_test();
   t1();
   t2();
   t_throw();
   t_error_ret();
   t_read_line();
   close_test();
#ifdef WY_DEBUG
 // fd may be lost. report is only valid in single thread test
 if(wydbg_get_fdcnt()!=initial_fdcnt) {
   WY_THROW( WyRet() );
 }
 Wy::cout << " fd allocated= " << wydbg_get_fdcnt_max() << '\n';
#endif
}
catch(...) {
 close_test();
#ifdef WY_DEBUG
 // fd may be lost. report is only valid in single thread test
 if(wydbg_get_fdcnt()!=initial_fdcnt) {
   WY_THROW( WyRet() );
 }
#endif
 throw;
};
