/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2004

   Build: ck_num.o
*/

#include "WyNum.h"
#include <limits>
#include <cstdlib>
#include <math.h>    // for fabs
#include <cstdio>    // for sscanf

#include <iostream>
#if WYLIB_VERSION!=33
#error Test code is for WYLIB_VERSION 33
#endif

extern void ck_strnum(void);    // defined in ckn_strnum.cpp

static void ck_charnum()
{
 for(char ch=CHAR_MIN; ch<'0'; ++ch) {
   int v=Wy::_charnum(ch);
   if((v>=0)&&(v<=35)) {
     WY_THROW( WyRet() );
   }
 }
 for(char ch='0'; ch<='9'; ++ch) {
   if(Wy::_charnum(ch)!=ch-'0') {
     WY_THROW( WyRet() );
   }
 }
 for(char ch='9'+1; ch<'A'; ++ch) {
   int v=Wy::_charnum(ch);
   if((v>=0)&&(v<=35)) {
     WY_THROW( WyRet() );
   }
 }
 for(char ch='A'; ch<='Z'; ++ch) {
   if(Wy::_charnum(ch)!=ch-'A'+10) {
     WY_THROW( WyRet() );
   }
 }
 for(char ch='Z'+1; ch<'a'; ++ch) {
   int v=Wy::_charnum(ch);
   if((v>=0)&&(v<=35)) {
     WY_THROW( WyRet() );
   }
 }
 for(char ch='a'; ch<='z'; ++ch) {
   if(Wy::_charnum(ch)!=ch-'a'+10) {
     WY_THROW( WyRet() );
   }
 }
 for(char ch=CHAR_MAX; ch>'z'; --ch) {
   int v=Wy::_charnum(ch);
   if((v>=0)&&(v<=35)) {
     WY_THROW( WyRet() );
   }
 }
};

template <typename NumType>
static void ck_uty(void)
{
 WyRet r;

 // chk Wy::_strnum(...)
 {
   NumType tnn;
   const char s1[]="52";
   WyCSeg cs;
   tnn=7;
   if((r=Wy::_strnum(tnn,"0",10))!=Ok) {
     WY_THROW(r);
   }
   if(tnn!=NumType(0)) {
     WY_THROW(r);
   }
   tnn=7;
   if((r=Wy::_strnum(tnn,WyCSeg("0",1),10))!=Ok) {
     WY_THROW(r);
   }
   if(tnn!=NumType(0)) {
     WY_THROW(r);
   }

   tnn=7;
   cs.reset(s1,sizeof(s1)-1);
   if((r=Wy::_strnum(tnn,cs,16))!=Ok) {
     WY_THROW(r);
   }
   if(cs.size()!=0) {
     WY_THROW( WyRet() );
   }
   if(tnn!=NumType(0x52)) {
     WY_THROW( WyRet() );
   }

   tnn=7;
   cs.reset(s1,sizeof(s1));
   if((r=Wy::_strnum(tnn,cs,0))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(cs.size()!=1) {
     WY_THROW( WyRet() );
   }
   if(tnn!=NumType(52)) {
     WY_THROW( WyRet() );
   }

   const char s2[]="0x041";
   int radix;
   radix=0;
   tnn=7;
   if((r=Wy::_strnum(tnn,WyCSeg(s2,sizeof(s2)-1),radix))!=Ok) {
     WY_THROW(r);
   }
   if(radix!=16) {
     WY_THROW( WyRet() );
   }
   if(tnn!=NumType(0x41)) {
     WY_THROW( WyRet() );
   }

   radix=0;
   tnn=7;
   if((r=Wy::_strnum(tnn,WyCSeg(s2+2,sizeof(s2)-3),radix))!=Ok) {
     WY_THROW(r);
   }
   if(radix!=8) {
     WY_THROW( WyRet() );
   }
   if(tnn!=NumType(041)) {
     WY_THROW( WyRet() );
   }

   radix=0;
   tnn=99;
   if((r=Wy::_strnum(tnn,WyCSeg(s2,1),radix))!=Ok) {
     WY_THROW(r);
   }
   if(radix!=10) {
     WY_THROW( WyRet() );
   }
   if(tnn!=NumType(0)) {
     WY_THROW( WyRet() );
   }

   radix=0;
   tnn=99;
   if((r=Wy::_strnum(tnn,WyCSeg("00"),radix))!=Ok) {
     WY_THROW(r);
   }
   if(radix!=8) {
     WY_THROW( WyRet() );
   }
   if(tnn!=NumType(0)) {
     WY_THROW( WyRet() );
   }

   radix=5;
   if((r=Wy::_strnum(tnn,WyCSeg(s2+2,sizeof(s2)-3),radix))!=Ok) {
     WY_THROW(r);
   }
   if(radix!=5) {
     WY_THROW( WyRet() );
   }
   if(tnn!=NumType(21)) {
     WY_THROW( WyRet() );
   }
 
   if(std::numeric_limits<NumType>::is_signed) {
     const char ns1[]="-52";
     if((r=Wy::_strnum(tnn,WyCSeg("-0",2),10))!=Ok) {
       WY_THROW(r);
     }
     if(tnn!=NumType(0)) {
       WY_THROW(r);
     }

     if((r=Wy::_strnum(tnn,WyCSeg(ns1,sizeof(ns1)-1),10))!=Ok) {
       WY_THROW(r);
     }
     if(tnn!=NumType(-52)) {
       WY_THROW( WyRet() );
     }

     const char ns2[]="-0x041";
     radix=0;
     if((r=Wy::_strnum(tnn,WyCSeg(ns2,sizeof(ns2)-1),radix))!=Ok) {
       WY_THROW(r);
     }
     if(radix!=16) {
       WY_THROW( WyRet() );
     }
     if(tnn!=NumType(-0x41)) {
       WY_THROW( WyRet() );
     }
  
     const char ns3[]="-041";
     radix=0;
     if((r=Wy::_strnum(tnn,WyCSeg(ns3,sizeof(ns3)-1),radix))!=Ok) {
       WY_THROW(r);
     }
     if(radix!=8) {
       WY_THROW( WyRet() );
     }
     if(tnn!=NumType(-041)) {
       WY_THROW( WyRet() );
     }
  
     const char ns4[]="-41";
     radix=5;
     if((r=Wy::_strnum(tnn,WyCSeg(ns4,sizeof(ns4)-1),radix))!=Ok) {
       WY_THROW(r);
     }
     if(radix!=5) {
       WY_THROW( WyRet() );
     }
     if(tnn!=NumType(-21)) {
       WY_THROW( WyRet() );
     }
   }
 }

 // chk error Wy::_strnum(...)
 {
   NumType tnn=3;
   const char* buf=0;
   int radix;
   WyCSeg cs;
   try {
     r=Wy::_strnum(tnn,WyCSeg(buf,1),3);
     WY_THROW(r);
   }
   catch(const WyCSeg::Reply& e) {
     if(e!=Wym_EFAULT) {
       WY_THROW( WyRet(e) );
     }
     // FALLTHROUGH
   };

   buf="73!";
   tnn=99;
   if((r=Wy::_strnum(tnn,WyCSeg(buf,2),1))!=Wym_EINVAL) {
     WY_THROW(r);
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }

   tnn=99;
   if((r=Wy::_strnum(tnn,WyCSeg(buf,2),37))!=Wym_EINVAL) {
     WY_THROW(r);
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }

   tnn=99;
   cs.reset(buf,3);
   if((r=Wy::_strnum(tnn,cs,10))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(cs.size()!=1) {
     WY_THROW( WyRet() );
   }
   if(cs[0]!='!') {
     WY_THROW( WyRet() );
   }
   if(tnn!=73) {
     WY_THROW( WyRet() );
   }

   buf="+";
   tnn=99;
   if((r=Wy::_strnum(tnn,WyCSeg(buf,1),2))!=Wym_ENOENT) {
     WY_THROW(r);
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }

   buf="0xx";
   tnn=99;
   if((r=Wy::_strnum(tnn,WyCSeg(buf,3),0))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }
   tnn=99;
   if((r=Wy::_strnum(tnn,WyCSeg(buf,3),16))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }

   buf="0xg";
   tnn=99;
   if((r=Wy::_strnum(tnn,WyCSeg(buf,3),0))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }
   tnn=99;
   if((r=Wy::_strnum(tnn,WyCSeg(buf,3),16))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }

   buf="0xx";
   radix=0;
   tnn=99;
   if((r=Wy::_strnum(tnn,WyCSeg(buf,3),radix))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }
   radix=0;
   tnn=99;
   if((r=Wy::_strnum(tnn,WyCSeg(buf,3),radix))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }
   radix=16;
   if((r=Wy::_strnum(tnn,WyCSeg(buf,3),radix))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }
   if(radix!=16) {
     WY_THROW( WyRet() );
   }

   buf="0xg";
   radix=0;
   tnn=99;
   if((r=Wy::_strnum(tnn,WyCSeg(buf,3),radix))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }
   tnn=99;
   radix=0;
   if((r=Wy::_strnum(tnn,WyCSeg(buf,3),radix))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }
   radix=0;
   if((r=Wy::_strnum(tnn,WyCSeg(buf,3),radix))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }
   radix=16;
   if((r=Wy::_strnum(tnn,WyCSeg(buf,3),radix))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(tnn!=0) {
     WY_THROW( WyRet() );
   }
   if(radix!=16) {
     WY_THROW( WyRet() );
   }

   buf="0xfg";
   tnn=99;
   if((r=Wy::_strnum(tnn,WyCSeg(buf,4),16))!=Wym_EBADMSG) {
     WY_THROW(r);
   }
   if(tnn!=0xf) {
     WY_THROW( WyRet() );
   }
   if(radix!=16) {
     WY_THROW( WyRet() );
   }

   if((r=Wy::_strnum(tnn,WyCSeg("fffffffffffffffffffffff"),16))!=Wym_ERANGE) {
     WY_THROW(r);
   }
   if(tnn<15) {
     WY_THROW( WyRet() );
   }

   if(std::numeric_limits<NumType>::is_signed) {
     buf=(const char*)0;
     tnn=4;
     try {
       r=Wy::_strnum(tnn,WyCSeg(buf,1),3);
       WY_THROW(r);
     }
     catch(const WyCSeg::Reply& e) {
       if(e!=Wym_EFAULT) {
         WY_THROW( WyRet(e) );
       }
       // FALLTHROUGH
     };
  
     buf="-7";
     tnn=99;
     if((r=Wy::_strnum(tnn,WyCSeg(buf,2),37))!=Wym_EINVAL) {
       WY_THROW(r);
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }

     tnn=99;
     if((r=Wy::_strnum(tnn,WyCSeg(buf,3),8))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(tnn!=NumType(-7)) {
       WY_THROW( WyRet() );
     }

     tnn=99;
     if((r=Wy::_strnum(tnn,WyCSeg(buf,2),1))!=Wym_EINVAL) {
       WY_THROW(r);
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }

     buf="-0xx";
     tnn=99;
     if((r=Wy::_strnum(tnn,WyCSeg(buf,4),0))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
     tnn=99;
     if((r=Wy::_strnum(tnn,WyCSeg(buf,4),16))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
  
     buf="-0xg";
     tnn=99;
     if((r=Wy::_strnum(tnn,WyCSeg(buf,4),0))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
     tnn=99;
     if((r=Wy::_strnum(tnn,WyCSeg(buf,4),16))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
  
     buf="-0xx";
     radix=0;
     tnn=99;
     if((r=Wy::_strnum(tnn,WyCSeg(buf,4),radix))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
     radix=0;
     tnn=99;
     if((r=Wy::_strnum(tnn,WyCSeg(buf,4),radix))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
     radix=16;
     if((r=Wy::_strnum(tnn,WyCSeg(buf,4),radix))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
     if(radix!=16) {
       WY_THROW( WyRet() );
     }
  
     buf="-0xg";
     radix=0;
     tnn=99;
     if((r=Wy::_strnum(tnn,WyCSeg(buf,4),radix))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
     tnn=99;
     radix=0;
     if((r=Wy::_strnum(tnn,WyCSeg(buf,4),radix))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
     radix=0;
     if((r=Wy::_strnum(tnn,WyCSeg(buf,4),radix))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
     radix=16;
     if((r=Wy::_strnum(tnn,WyCSeg(buf,4),radix))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(tnn!=0) {
       WY_THROW( WyRet() );
     }
     if(radix!=16) {
       WY_THROW( WyRet() );
     }
  
     buf="-0xfg";
     tnn=99;
     if((r=Wy::_strnum(tnn,WyCSeg(buf,5),16))!=Wym_EBADMSG) {
       WY_THROW(r);
     }
     if(tnn+0xf!=0) {          // compiler complains "tnn!=-0xf"
       WY_THROW( WyRet() );
     }
     if(radix!=16) {
       WY_THROW( WyRet() );
     }

     if((r=Wy::_strnum(tnn,WyCSeg("-fffffffffffffffffffffff"),16))!=Wym_ERANGE) {
       WY_THROW(r);
     }
     if(tnn>0) {
       WY_THROW( WyRet() );
     }
   }
 }
};

template <typename NumT>
static void ck_sqrt(void)
{
 if(std::numeric_limits<NumT>::is_signed) {
   return;
 }
 if(Wy::_sqrt(NumT(0))!=0) {
   WY_THROW( WyRet() );
 }
 if(Wy::_sqrt(NumT(1))!=1) {
   WY_THROW( WyRet() );
 }
 if(Wy::_sqrt(NumT(99))!=9) {
   WY_THROW( WyRet() );
 }
 if(Wy::_sqrt(NumT(100))!=10) {
   WY_THROW( WyRet() );
 }
 if(Wy::_sqrt(NumT(120))!=10) {
   WY_THROW( WyRet() );
 }
 if(Wy::_sqrt(NumT(121))!=11) {
   WY_THROW( WyRet() );
 }
 for(int i=0; i<=255; ++i) {  // chk i in char range
   NumT res=Wy::_sqrt((NumT)i);
   int q(res),t2(q*q);
   if(t2>i) {
     WY_THROW( WyRet() );
   }
   if(t2+2*q+1<=i) {
     WY_THROW( WyRet() );
   }
 }

 for(NumT i=std::numeric_limits<NumT>::max(); 
          i>=std::numeric_limits<NumT>::max()-254; --i) {
   NumT res=Wy::_sqrt(i);
   NumT q(res),t2(q*q);
   if(t2>i) {
     WY_THROW( WyRet() );
   }
   if(t2+2*q<=i-1) {
     std::cout << t2 << ", " << q << ", " << i << std::endl;
     std::cout << t2+2*q+1 << std::endl;
     WY_THROW( WyRet() );
   }
 }
};

template <typename T>
static void ck_num_frm_str(void)
{
 struct ChkSmp {
   T num;
   const char* nstr;
   size_t nstr_size;
   int    radix;
   void verify(void) const 
     {
      try {
        WyRet r;
        T a1;
        WyCSeg cs(nstr,nstr_size);
        if((r=Wy::_strnum(a1,cs,radix))!=Ok) {
          WY_THROW(r);
        }
        if(a1!=num) {
          WY_THROW( WyRet() );
        }
        if(cs.size()!=0) {
          WY_THROW( WyRet() );
        }
      }
      catch(...) {
        if((std::numeric_limits<T>::max()==std::numeric_limits<char>::max())||
           (std::numeric_limits<T>::max()==std::numeric_limits<unsigned char>::max())) {
          std::cerr << "ChkSmp{ T(" << (int)num << "),\""
                    << nstr << "\"," << nstr_size << ","
                    << radix << "}" << std::endl;
        } else {
          std::cerr << "ChkSmp{ T(" << num << "),\""
                    << nstr << "\"," << nstr_size << ","
                    << radix << "}" << std::endl;
        }
        throw;
      };
     }
 } const chk_tab[]={
  {T(0),"0",1,10}, {T(0),"+0",2,10}, {T(0),"-0",2,10},
  {T(0),"0.0",3,10},{T(0),"+0.0",4,10},{T(0),"-0.0",4,10},
  {T(0),"00.4",3,10},{T(0),"+00.0",5,10},{T(0),"-00.0",5,10},
  {T(0),"0.00",4,10},{T(0),"+0.00",5,10},{T(0),"-0.00",5,10},
  {T(0),"0.00e0",6,10},{T(0),"+0.00e0",7,10},{T(0),"-0.00e0",7,10},

  {T(1.0),"1",1,10},{T(1.0),"+1",2,10},{T(-1.0),"-1",2,10},
  {T(1.0),"1.0",3,10},{T(1.0),"+1.0",4,10},{T(-1.0),"-1.0",4,10},
  {T(1.0),"01.0",4,10},{T(1.0),"+01.0",5,10},{T(-1.0),"-01.0",5,10},
  {T(1.0),"1.00",4,10},{T(1.0),"+1.00",5,10},{T(-1.0),"-1.00",5,10},
  {T(1.0),"1.00e0",6,10},{T(1.0),"+1.00e0",7,10},{T(-1.0),"-1.00e0",7,10},
  {T(1.0),"1.00e+0",7,10},{T(1.0),"+1.00e+0",8,10},{T(-1.0),"-1.00e+0",8,10},
  {T(1.0),"1.00e-0",7,10},{T(1.0),"+1.00e-0",8,10},{T(-1.0),"-1.00e-0",8,10},

  {T(10.0),"1.00e1",6,10},{T(10.0),"+1.00e1",7,10},{T(-10.0),"-1.00e1",7,10},
  {T(10.0),"1.00e+1",7,10},{T(10.0),"+1.00e+1",8,10},{T(-10.0),"-1.00e+1",8,10},
  {T(0.1),"1.00e-1",7,10},{T(0.1),"+1.00e-1",8,10},{T(-0.1),"-1.00e-1",8,10},
 };

 for(size_t i=0; i<sizeof(chk_tab)/sizeof(ChkSmp); ++i) {
   chk_tab[i].verify();
 }

};

template <typename NumType>
static void ck_num_float(void)
{
 ck_num_frm_str<NumType>();
};

template <typename NumType>
static void ck_num_integer(void)
{
 ck_uty<NumType>();
};

static void ck_examp1(void)
{
 WyRet r;

 // chk size_t overflow (debug case)
 {
   const WyCSeg txt="9876543210";
   size_t val;
   if((r=Wy::_strnum(val,txt,0))!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 }

 // chk signed long overflow (debug case)
 {
   const WyCSeg txt="-5000000000";
   signed long val;
   if((r=Wy::_strnum(val,txt,0))!=Wym_ERANGE) {
     WY_THROW( WyRet() );
   }
 }

 // Evaluate the code to parse number " 12/23/44.4  56,456.21 4k22"
 //
 const char nstr[]=" 12/23/44.4  56,456.21 4k22";
 int v_int;
 double v_double;
 unsigned int v_uint;
 WyCSeg cs(nstr+1,sizeof(nstr)-2);

 if((r=Wy::_strnum(v_int,cs))!=Wym_EBADMSG) {
   WY_THROW(r);
 }
 if(v_int!=12) {
   WY_THROW( WyRet() );
 }
 if(cs[0]!='/') {
   WY_THROW( WyRet() );
 }
 
 cs.move_begin(1);
 if((r=Wy::_strnum(v_uint,cs))!=Wym_EBADMSG) {
   WY_THROW(r);
 }
 if(v_uint!=23) {
   WY_THROW( WyRet() );
 }
 if(cs[0]!='/') {
   WY_THROW( WyRet() );
 }

 cs.move_begin(1);
 if((r=Wy::_strnum(v_double,cs))!=Wym_EBADMSG) {
   WY_THROW(r);
 }
 if((v_double<44.39)||(v_double>44.41)) {
   WY_THROW( WyRet() );
 }
 if(cs[0]!=' ') {
   WY_THROW( WyRet() );
 }

 cs.move_begin(2);
 if((r=Wy::_strnum(v_int,cs))!=Wym_EBADMSG) {
   WY_THROW(r);
 }
 if(v_int!=56) {
   WY_THROW( WyRet() );
 }
 if(cs[0]!=',') {
   WY_THROW( WyRet() );
 }

 cs.move_begin(1);
 if((r=Wy::_strnum(v_double,cs))!=Wym_EBADMSG) {
   WY_THROW(r);
 }
 if((v_double<456.209)||(v_double>456.211)) {
   WY_THROW( WyRet() );
 }
 if(cs[0]!=' ') {
   WY_THROW( WyRet() );
 }
 v_double+= 1000.0*v_int;   // i.e. 56456.21

 cs.move_begin(1);
 if((r=Wy::_strnum(v_uint,cs))!=Wym_EBADMSG) {
   WY_THROW(r);
 }
 if(v_uint!=4) {
   WY_THROW( WyRet() );
 }
 if(cs[0]!='k') {
   WY_THROW( WyRet() );
 }

 cs.move_begin(1);
 if((r=Wy::_strnum(v_uint,cs))!=Ok) {
   WY_THROW(r);
 }
 if(v_uint!=22) {
   WY_THROW( WyRet() );
 }
};

static void t_scanum(void)
{
 WyRet r;

  // chk _scanum(ValueType&,WyCSeg&)
  {
    const char Input1[]=" -12.34	 345.67E-1	 78.901e1 555";
    WyCSeg txt=Input1;
    float value_f1,value_f2;
    double value_lf;
    int   value_d;

    // This loop is to use break to exit the block
    //
    do {
      if((r=Wy::_scanum(value_f1,txt))!=Ok) {
        WY_HERE(r);
        break;
      }

      if((r=Wy::_scanum(value_f2,txt))!=Ok) {
        WY_HERE(r);
        break;
      }

      if((r=Wy::_scanum(value_lf,txt))!=Ok) {
        WY_HERE(r);
        break;
      }

      if((r=Wy::_scanum(value_d,txt))!=Ok) {
        WY_HERE(r);
        break;
      }

      r=Ok;
    } while(false);
 
    if(r!=Ok) {
      WY_THROW(r);
    }
    
    float v_f1,v_f2;
    double v_lf;
    int v_d;
    if(std::sscanf(Input1,"%f %f %lf %d",
                   &v_f1,&v_f2,&v_lf,&v_d)!=4) {
      WY_THROW( WyRet() );
    }
    if(::fabs(v_f1-value_f1)>0.001) {
      WY_THROW( WyRet() );
    }
    if(::fabs(v_f2-value_f2)>0.001) {
      WY_THROW( WyRet() );
    }
    if(::fabsl(v_lf-value_lf)>0.00001) {
      WY_THROW( WyRet() );
    }
    if(v_d!=value_d) {
      WY_THROW( WyRet() );
    }
  }

  // chk _scanum(ValueType&,WyCSeg&,const int&)
  {
    const char Input1[]="ffff 	222";
    WyCSeg txt=Input1;
    unsigned int  value_x1, value_x2;

    if((r=Wy::_scanum(value_x1,txt,16))!=Ok) {
      WY_THROW(r);
    }
    if((r=Wy::_scanum(value_x2,txt,16))!=Ok) {
      WY_THROW(r);
    }

    unsigned int v_x1,v_x2;
    if(std::sscanf(Input1,"%x %x",
                   &v_x1,&v_x2)!=2) {
      WY_THROW( WyRet() );
    }
    if(value_x1!=v_x1) {
      WY_THROW( WyRet() );
    }
    if(value_x2!=v_x2) {
      WY_THROW( WyRet() );
    }
  }

  // chk _scanum(ValueType&,WyCSeg&,int&)
  {
    const char Input1[]="0xffff 	222 77";
    WyCSeg txt=Input1;
    Wy::WyRadix radix;
    unsigned int  value_x1, value_x2, value_x3;

    radix=0;
    if((r=Wy::_scanum(value_x1,txt,radix))!=Ok) {
      WY_THROW(r);
    }
    if(radix!=16) {
      WY_THROW(r);
    }

    radix=0;
    if((r=Wy::_scanum(value_x2,txt,radix))!=Ok) {
      WY_THROW(r);
    }
    if(radix!=10) {
      WY_THROW(r);
    }
    if((r=Wy::_scanum(value_x3,txt,radix))!=Ok) {
      WY_THROW(r);
    }
    if(radix!=10) {
      WY_THROW(r);
    }

    unsigned int v_x1,v_x2,v_x3;
    if(std::sscanf(Input1,"%x %u %u",
                   &v_x1,&v_x2,&v_x3)!=3) {
      WY_THROW( WyRet() );
    }
    if(value_x1!=v_x1) {
      WY_THROW( WyRet() );
    }
    if(value_x2!=v_x2) {
      WY_THROW( WyRet() );
    }
    if(value_x3!=v_x3) {
      WY_THROW( WyRet() );
    }
  }

  // chk reply
  {
    WyCSeg txt;
    size_t value;
    if((r=Wy::_scanum(value,txt))!=Wym_ENOENT) {
      WY_THROW( WyRet() );
    }
    if(txt.is_default()==false) {
      WY_THROW( WyRet() );
    }

    const char* Tstr="89489734734782";
    txt=Tstr;
    if((r=Wy::_scanum(value,txt))!=Wym_ERANGE) {
      WY_THROW( WyRet() );
    }
    if(txt!=Tstr) {
      WY_THROW( WyRet() );
    }

    if((r=Wy::_scanum(value,txt,37))!=Wym_EINVAL) {
      WY_THROW( WyRet() );
    }
    if(txt!=Tstr) {
      WY_THROW( WyRet() );
    }
    if((r=Wy::_scanum(value,txt,1))!=Wym_EINVAL) {
      WY_THROW( WyRet() );
    }
    if(txt!=Tstr) {
      WY_THROW( WyRet() );
    }
  }
};

template <typename NumType>
static void ck_scanum_integer(void)
{
 WyRet r;

  // chk _scanum(ValueType&,const WyCSeg&)
  {
    const char Input1[]=" 43	 123	 ";
    WyCSeg txt=Input1;
    NumType value_f1,value_f2;

    if((r=Wy::_scanum(value_f1,txt))!=Ok) {
      WY_THROW(r);
    }
    if(value_f1!=(NumType)43) {
      WY_THROW( WyRet() );
    }

    if((r=Wy::_scanum(value_f2,txt))!=Ok) {
      WY_THROW(r);
    }
    if(value_f2!=(NumType)123) {
      WY_THROW( WyRet() );
    }
  }

  if(std::numeric_limits<NumType>::is_signed) {
    const char Input1[]="-43	 -123	 ";
    WyCSeg txt=Input1;
    NumType value_f1,value_f2;

    if((r=Wy::_scanum(value_f1,txt))!=Ok) {
      WY_THROW(r);
    }
    if(value_f1!=(NumType)-43) {
      WY_THROW( WyRet() );
    }

    if((r=Wy::_scanum(value_f2,txt))!=Ok) {
      WY_THROW(r);
    }
    if(value_f2!=(NumType)-123) {
      WY_THROW( WyRet() );
    }
  }

  // chk _scanum(ValueType&,WyCSeg&,const int&)
  {
    const char Input1[]=" 6a	 2f	 ";
    WyCSeg txt=Input1;
    NumType value_f1,value_f2;

    if((r=Wy::_scanum(value_f1,txt,16))!=Ok) {
      WY_THROW(r);
    }
    if(value_f1!=(NumType)0x6a) {
      WY_THROW( WyRet() );
    }

    if((r=Wy::_scanum(value_f2,txt,16))!=Ok) {
      WY_THROW(r);
    }
    if(value_f2!=(NumType)0x2f) {
      WY_THROW( WyRet() );
    }
  }
  if(std::numeric_limits<NumType>::is_signed) {
    const char Input1[]=" -6a	 -2f	 ";
    WyCSeg txt=Input1;
    NumType value_f1,value_f2;

    if((r=Wy::_scanum(value_f1,txt,16))!=Ok) {
      WY_THROW(r);
    }
    if(value_f1!=(NumType)-0x6a) {
      WY_THROW( WyRet() );
    }

    if((r=Wy::_scanum(value_f2,txt,16))!=Ok) {
      WY_THROW(r);
    }
    if(value_f2!=(NumType)-0x2f) {
      WY_THROW( WyRet() );
    }
  }

  // chk _scanum(ValueType&,WyCSeg&,int&)
  {
    const char Input1[]=" 0x6a	 027	 22";
    WyCSeg txt=Input1;
    NumType value_f1,value_f2,value_f3;
    Wy::WyRadix radix;

    radix=0;
    if((r=Wy::_scanum(value_f1,txt,radix))!=Ok) {
      WY_THROW(r);
    }
    if(radix!=16) {
      WY_THROW( WyRet() );
    }
    if(value_f1!=(NumType)0x6a) {
      WY_THROW( WyRet() );
    }

    radix=0;
    if((r=Wy::_scanum(value_f2,txt,radix))!=Ok) {
      WY_THROW(r);
    }
    if(radix!=8) {
      WY_THROW( WyRet() );
    }
    if(value_f2!=(NumType)027) {
      WY_THROW( WyRet() );
    }

    radix=0;
    if((r=Wy::_scanum(value_f3,txt,radix))!=Ok) {
      WY_THROW(r);
    }
    if(radix!=10) {
      WY_THROW( WyRet() );
    }
    if(value_f3!=(NumType)22) {
      WY_THROW( WyRet() );
    }
  }
  if(std::numeric_limits<NumType>::is_signed) {
    const char Input1[]=" -0x6a	 -027	 -22";
    WyCSeg txt=Input1;
    NumType value_f1,value_f2,value_f3;
    Wy::WyRadix radix;

    radix=0;
    if((r=Wy::_scanum(value_f1,txt,radix))!=Ok) {
      WY_THROW(r);
    }
    if(radix!=16) {
      WY_THROW( WyRet() );
    }
    if(value_f1!=(NumType)-0x6a) {
      WY_THROW( WyRet() );
    }

    radix=0;
    if((r=Wy::_scanum(value_f2,txt,radix))!=Ok) {
      WY_THROW(r);
    }
    if(radix!=8) {
      WY_THROW( WyRet() );
    }
    if(value_f2!=(NumType)-027) {
      WY_THROW( WyRet() );
    }

    radix=0;
    if((r=Wy::_scanum(value_f3,txt,radix))!=Ok) {
      WY_THROW(r);
    }
    if(radix!=10) {
      WY_THROW( WyRet() );
    }
    if(value_f3!=(NumType)-22) {
      WY_THROW( WyRet() );
    }
  }

  // chk reply
  {
    WyCSeg txt;
    NumType value;
    if((r=Wy::_scanum(value,txt))!=Wym_ENOENT) {
      WY_THROW( WyRet() );
    }
    if(txt.is_default()==false) {
      WY_THROW( WyRet() );
    }
    if((r=Wy::_scanum(value,txt,16))!=Wym_ENOENT) {
      WY_THROW( WyRet() );
    }
    if(txt.is_default()==false) {
      WY_THROW( WyRet() );
    }

    const char* Tstr="98765432100000000000";
    txt=Tstr;
    if((r=Wy::_scanum(value,txt))!=Wym_ERANGE) {
      WY_THROW(r);
    }
    if(txt!=Tstr) {
      WY_THROW( WyRet() );
    }

    if((r=Wy::_scanum(value,txt,37))!=Wym_EINVAL) {
      WY_THROW(r);
    }
    if(txt!=Tstr) {
      WY_THROW( WyRet() );
    }
    if((r=Wy::_scanum(value,txt,1))!=Wym_EINVAL) {
      WY_THROW(r);
    }
    if(txt!=Tstr) {
      WY_THROW( WyRet() );
    }
  }
};

template <typename NumType>
static void ck_scanum_float(void)
{
 WyRet r;

  // chk _scanum(ValueType&,const WyCSeg&)
  {
    const char Input1[]=" 43.67	 -123e-2 ";
    WyCSeg txt=Input1;
    NumType value_f1,value_f2;

    if((r=Wy::_scanum(value_f1,txt))!=Ok) {
      WY_THROW(r);
    }
    if( ::fabsl(value_f1-(double)43.67)>1e-5 ) {
      WY_THROW( WyRet() );
    }

    if((r=Wy::_scanum(value_f2,txt))!=Ok) {
      WY_THROW(r);
    }
    if( ::fabsl(value_f2- (double)-123e-2)>1e-5 ) {
      WY_THROW( WyRet() );
    }
  }

  // chk _scanum(ValueType&,const WyCSeg&,const int&)
  {
    const char Input1[]="1.234	 5.78e+2	 ";
    WyCSeg txt=Input1;
    NumType value_f1,value_f2;

    if((r=Wy::_scanum(value_f1,txt,0))!=Ok) {
      WY_THROW(r);
    }
    if( ::fabsl(value_f1- (double)1.234)>1e-5 ) {
      WY_THROW( WyRet() );
    }

    if((r=Wy::_scanum(value_f2,txt,0))!=Ok) {
      WY_THROW(r);
    }
    if( ::fabsl(value_f2- (double)5.78e+2)>1e-5 ) {
      WY_THROW( WyRet() );
    }
  }

  // chk _scanum(ValueType&,const WyCSeg&,int&)
  {
    const char Input1[]="1.234	 5.78e+2	 ";
    WyCSeg txt=Input1;
    NumType value_f1,value_f2;
    Wy::WyRadix radix;
    
    radix=0;
    if((r=Wy::_scanum(value_f1,txt,radix))!=Ok) {
      WY_THROW(r);
    }
    if(radix!=10) {
      WY_THROW( WyRet() );
    }
    if( ::fabsl(value_f1- (double)1.234)>1e-5 ) {
      WY_THROW( WyRet() );
    }

    radix=0;
    if((r=Wy::_scanum(value_f2,txt,radix))!=Ok) {
      WY_THROW(r);
    }
    if(radix!=10) {
      WY_THROW( WyRet() );
    }
    if( ::fabsl(value_f2- (double)5.78e+2)>1e-5 ) {
      WY_THROW( WyRet() );
    }
  }

  // chk reply
  {
    WyCSeg txt;
    NumType value;
    if((r=Wy::_scanum(value,txt))!=Wym_ENOENT) {
      WY_THROW( WyRet() );
    }
    if(txt.is_default()==false) {
      WY_THROW( WyRet() );
    }
    if((r=Wy::_scanum(value,txt,0))!=Wym_ENOENT) {
      WY_THROW(r);
    }
    if(txt.is_default()==false) {
      WY_THROW( WyRet() );
    }

    const char* Tstr="98765432109876543210e99999p999999";
    txt=Tstr;
    if((r=Wy::_scanum(value,txt))!=Wym_ERANGE) {
      WY_THROW(r);
    }
    if(txt!=Tstr) {
      WY_THROW( WyRet() );
    }

    if((r=Wy::_scanum(value,txt,37))!=Wym_EINVAL) {
      WY_THROW(r);
    }
    if(txt!=Tstr) {
      WY_THROW( WyRet() );
    }
    if((r=Wy::_scanum(value,txt,1))!=Wym_EINVAL) {
      WY_THROW(r);
    }
    if(txt!=Tstr) {
      WY_THROW( WyRet() );
    }
    if((r=Wy::_scanum(value,txt,16))!=Wym_ERANGE) {
      WY_THROW(r);
    }
    if(txt!=Tstr) {
      WY_THROW( WyRet() );
    }
  }
};

void ck_num(void)
{
 ck_charnum();
 ck_strnum();

 ck_num_integer<char>(); 
 ck_num_integer<short int>();
 ck_num_integer<int>();
 ck_num_integer<long int>();
 ck_num_integer<long long int>();

 ck_num_integer<signed char>();
 ck_num_integer<signed short int>();
 ck_num_integer<signed int>();
 ck_num_integer<signed long int>();
 ck_num_integer<signed long long int>();

 ck_num_integer<unsigned char>();
 ck_num_integer<unsigned short int>();
 ck_num_integer<unsigned int>();
 ck_num_integer<unsigned long int>();
 ck_num_integer<unsigned long long int>();

 ck_num_float<float>();
 ck_num_float<double>();

 ck_sqrt<unsigned char>();
 ck_sqrt<unsigned short>();
 ck_sqrt<unsigned int>();
 ck_sqrt<unsigned long int>();
 ck_sqrt<unsigned long long int>();

 ck_examp1();
 t_scanum();
 ck_scanum_integer<char>();
 ck_scanum_integer<signed char>();
 ck_scanum_integer<signed short>();
 ck_scanum_integer<signed int>();
 ck_scanum_integer<signed long>();
 ck_scanum_integer<signed long long>();
 ck_scanum_integer<unsigned char>();
 ck_scanum_integer<unsigned short>();
 ck_scanum_integer<unsigned int>();
 ck_scanum_integer<unsigned long>();
 ck_scanum_integer<unsigned long long>();
 ck_scanum_float<float>();
 ck_scanum_float<double>();
};
