/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2007

   Build: g++ g_ntoa_timespec.cpp libwy.a -pthread
*/
#include <iostream>
#include <sstream>
#include <cerrno>
#include <limits>
#include <cmath>    // for pow,log
#include <float.h>
#include <cassert>
#include "WyNum.h"
#include "WyStr.h"

long long iexp(size_t b, size_t exp)
{
 long long rv=1;
 for(size_t i=0; i<exp; ++i) {
   rv*=b;
 }
 return rv;
};

long long read_dbuf(const char* buf, size_t dlen, int radix)
{
 long long rv=0;
 for(size_t i=0; i<dlen; ++i) {
   int d0=Wy::_charnum(buf[i]);
   if((d0<0)||(d0>=radix)) {
     WY_THROW( WyRet() );
   }
   rv*=radix;
   rv+=d0;
 }
 return(rv);
};

int main(void)
try {
 typedef double DType;
 char dbuf[256];
 const DType EB=1e9;
 for(int r=2; r<=36; ++r) {
   // d= digits required for a number of radix r to hold EB-1
   const int d=static_cast<int>(std::log(EB-1)/std::log(r))+1;

   // m= multiplier
   const DType m= (DType)iexp(r,d)/EB;
   long long scale=670483147L;  // tested maximum to have result the maximal precision
   long long mi= static_cast<long long>(m*scale);
   long nano=999999999L;       // the value of t.nano()
   //long nano=123456789L;     // the value of t.nano()

   std::cerr << r << "  " 
             << d << "  ";
   std::cerr << mi << "  ";

   // The real value is nano/Giga which is always less than 1.0
   // compute the value n so that n/iexp(r,d)==nano/Giga
   long long n=static_cast<long long>( (unsigned long long)nano*mi/scale );

   size_t stmp=Wy::_ntoa(dbuf,sizeof(dbuf),n,r);
   dbuf[stmp]=0;

   // dump dbuf and the value it represents
   std::cerr.precision(10);
   std::cerr << '\"' << dbuf << "\"   ";
   std::cerr << (double)read_dbuf(dbuf,stmp,r)/iexp(r,d);

   // dump n, verify max. value
   std::cerr << "   " << n;
   std::cerr << '\n';
 }
 return 0;
}
catch(const WyRet& e) {
 std::cerr << Wy::wrd(e).c_str() << '\n';
 throw;
}
catch(...) {
 std::cerr << "main caught ...\n";
 throw;
};
