/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2007

   upd-31-32 is an auxilary utility searching argument given file(s), generating
   hint report (to standard output) for C++ codes using libwy.a v0.31 to update
   to v0.32

   upd-31-32 is a simple 'grep' like utility, just does it can. upd-31-32
   serves generating diagnostic report before compilation.
   Do see manpages and file UPDATE.

   Build: make upd-31-32
*/
#include "WyRegFile.h"
#include "WyStr.h"

static const WyCSeg Blanks=" \t\n\r\v\f";

// kword
static WyStr g_txt(const WyCSeg& line_data, const WyCSeg& kword, const WyCSeg& hint)
{
 size_t fidx;
 return (line_data.find(fidx,0,kword))? hint : WyStr();
};

// kword()
static WyStr g_f0(const WyCSeg& line_data, const WyCSeg& kword, const WyCSeg& hint)
{
 size_t fidx;
 if(line_data.find(fidx,0,kword)) {
   fidx+=kword.size();
   if((line_data.findc_not(fidx,fidx,Blanks))&&(line_data[fidx]=='(')) {
     if((line_data.findc_not(fidx,fidx+1,Blanks))&&(line_data[fidx]==')')) {
       return hint;
     }
   }
 }
 return WyStr();
};

// kword(..)
static WyStr g_fn(const WyCSeg& line_data, const WyCSeg& kword, const WyCSeg& hint)
{
 size_t fidx;
 if(line_data.find(fidx,0,kword)) {
   fidx+=kword.size();
   if((line_data.findc_not(fidx,fidx,Blanks))&&(line_data[fidx]=='(')) {
     if((line_data.findc_not(fidx,fidx+1,Blanks))&&(line_data[fidx]!=')')) {
       return hint;
     }
   }
 }
 return WyStr();
};

// .kword( or ->kword(
static WyStr g_mf(const WyCSeg& line_data, const WyCSeg& kword, const WyCSeg& hint)
try {
 size_t fidx;
 if(line_data.find(fidx,0,kword)) {
   if((line_data[fidx-1]=='.')||((line_data[fidx-1]=='>')&&(line_data[fidx-2]=='-'))) {
     fidx+=kword.size();
     if((line_data.findc_not(fidx,fidx,Blanks))&&(line_data[fidx]=='(')) {
       return hint;
     }
   }
 }
 return WyStr();
}
catch(const WyCSeg::Reply&) {
 return WyStr();
};

static WyStr hints_for_line(const WyCSeg& hdr, const WyCSeg& lseg)
{
 WyStr ostr;
 ostr+=g_txt(lseg,"wystr.h", "wystr.h-->WyStr.h,");
 ostr+=g_txt(lseg,"wyregfile.h", "wyregfile.h-->WyRegFile.h,");
 ostr+=g_txt(lseg,"wy_uty.h", "wy_uty.h is removed. Include WyByteFlow.h.");
 ostr+=g_txt(lseg,"wy_atdestroy.h", "wy_atdestroy.h-->Wy_AtDestroy.hh");
 ostr+=g_txt(lseg,"wybyteflow.h", "wybyteflow.h-->WyByteFlow.h, ");
 ostr+=g_txt(lseg,"wychrfile.h", "wychrfile.h-->WyChrFile.h, ");
 ostr+=g_txt(lseg,"wycond.h", "wycond.h-->WyCond.h, ");
 ostr+=g_txt(lseg,"wycseg.h", "wycseg.h-->WyCSeg.h, ");
 ostr+=g_txt(lseg,"wydate.h", "wydate.h-->WyDate.h, ");
 ostr+=g_txt(lseg,"wydirent.h", "wydirent.h-->WyDirEnt.h, ");
 ostr+=g_txt(lseg,"wy_dirfile.h", "wy_dirfile.h-->Wy_DirFile.h, ");
 ostr+=g_txt(lseg,"wyfifofile.h", "wyfifofile.h-->WyFifoFile.h, ");
 ostr+=g_txt(lseg,"wyfilehandle.h", "wyfilehandle.h-->WyFileHandle.h, ");
 ostr+=g_txt(lseg,"wyfilestat.h", "wyfilestat.h-->WyFileStat.h, ");
 ostr+=g_txt(lseg,"wyiconv.h", "wyiconv.h-->WyIConv.h, ");
 ostr+=g_txt(lseg,"wy_lnkfile.h", "wy_lnkfile.h-->Wy_LnkFile.h, ");
 ostr+=g_txt(lseg,"wymath.h", "wymath.h-->WyMath.h, ");
 ostr+=g_txt(lseg,"wymutex.h", "wymutex.h-->WyMutex.h, ");
 ostr+=g_txt(lseg,"wynum.h", "wynum.h-->WyNum.h, ");
 ostr+=g_txt(lseg,"wyonce.h", "wyonce.h-->WyOnce.h, ");
 ostr+=g_txt(lseg,"wyret.h", "wyret.h-->WyRet.h, ");
 ostr+=g_txt(lseg,"wyseg.h", "wyseg.h-->WySeg.hh, ");
 ostr+=g_txt(lseg,"wyselect.h", "wyselect.h-->Wy.select.h, ");
 ostr+=g_txt(lseg,"wysockaddrinet4.h", "wysockaddrinet4.h-->WySockAddrINet4.h, ");
 ostr+=g_txt(lseg,"wysockaddrinet6.h", "wysockaddrinet6.h-->WySockAddrINet6.h, ");
 ostr+=g_txt(lseg,"wysockaddrlocal.h", "wysockaddrlocal.h-->WySockAddrLocal.h, ");
 ostr+=g_txt(lseg,"wysockfile.h", "wysockfile.h-->WySockFile.h, ");
 ostr+=g_txt(lseg,"wyterminal.h", "wyterminal.h-->WyTerminal.h, ");
 ostr+=g_txt(lseg,"wytermios.h", "wytermios.h-->WyTermios.h, ");
 ostr+=g_txt(lseg,"wy_thread.h", "wy_thread.h-->Wy_Thread.h, ");
 ostr+=g_txt(lseg,"wytimespec.h", "wytimespec.h-->WyTimeSpec.h, ");
 ostr+=g_txt(lseg,"wy_array.h", "wy_array.h-->Wy_Array.hh, ");
 ostr+=g_mf(lseg,"read", "Wy_DirFile::read-->Wy_DirFile::_readdir, ");
 ostr+=g_mf(lseg,"read", "Wy_LnkFile::read-->Wy_LnkFile::_readlink, ");
 ostr+=g_mf(lseg,"rewind", "Wy_DirFile::rewind-->Wy_DirFile::_rewinddir, ");
 ostr+=g_mf(lseg,"find", "WyStr::find.. WyCSeg::find.. changed, see WyStr.3wy WyCSeg.3wy, ");
 ostr+=g_mf(lseg,"rfind", "WyStr::rfind.. WyCSeg::rfind.. changed, see WyStr.3wy WyCSeg.3wy, ");
 ostr+=g_mf(lseg,"find_not", "WyStr::rfind.. WyCSeg::rfind.. changed, see WyStr.3wy WyCSeg.3wy, ");
 ostr+=g_mf(lseg,"rfind_not", "WyStr::rfind.. WyCSeg::rfind.. changed, see WyStr.3wy WyCSeg.3wy, ");
 ostr+=g_mf(lseg,"_strcmp", "WyStr::_strcmp and WyCSeg::_strcmp are removed, see Wy.3wy for Wy::_strcmp, ");
 ostr+=g_fn(lseg,"_strnum", "Wy::_strnum changed protocol, see Wy.3wy, ");
 ostr+=g_fn(lseg,"_scanum", "Wy::_scanum changed protocol, see Wy.3wy, ");
 ostr+=g_mf(lseg,"copyto", "WyStr::copyto are removed, see WyStr.3wy, ");
 ostr+=g_fn(lseg,"_catstr", "Wy::_catstr are removed, see Wy.3wy, ");
 ostr+=g_txt(lseg,"time_zone", "WyDate::time_zone changed to return WyCSeg, ");
 ostr+=g_txt(lseg,"get_reply", "Wy_RepInfo::get_reply changed to Wy_RepInfo::getmsg(void*,,,), ");
 ostr+=g_txt(lseg,"set_reply", "Wy_RepInfo::set_reply changed to Wy_RepInfo::setmsg(const void*,,,), ");
 ostr+=g_fn(lseg,"iflag", "WyTermios::iflag(tcflag_t) changed to iflag_set(tcflag_t), ");
 ostr+=g_fn(lseg,"oflag", "WyTermios::oflag(tcflag_t) changed to oflag_set(tcflag_t), ");
 ostr+=g_fn(lseg,"cflag", "WyTermios::cflag(tcflag_t) changed to cflag_set(tcflag_t), ");
 ostr+=g_fn(lseg,"lflag", "WyTermios::lflag(tcflag_t) changed to lflag_set(tcflag_t), ");
 ostr+=g_mf(lseg,"devname", "WyTerminal::devname-->WyTerminal::_ttyname, ");
 ostr+=g_txt(lseg,"WyMutex", "WyMutex changed attribute, see WyMutex.3wy");
 ostr+=g_fn(lseg,"_charnum", "Wy::_charnum changed protocol, see Wy.3wy");
 ostr+=g_fn(lseg,"_move_begin", "WyCSeg::_move_begin-->move_begin, and may throw Reply, ");
 ostr+=g_fn(lseg,"_move_end", "WyCSeg::_move_end-->move_end, and may throw Reply, ");
 ostr+=g_txt(lseg,"PathMaxLen", "WySockAddrLocal::PathMaxLen-->MaxAddrLen, ");
 ostr+=g_txt(lseg,"WySockAddrLocal", "WySockAddrLocal(sockaddr_un,size_t) and reset of the same form are removed. see file UPDATE\n");
 ostr+=g_mf(lseg,"_path", "WySockAddrLocal::_path removed, see WySockAddrLocal.3wy");
 ostr+=g_txt(lseg,"InPortType", "WySockAddrINet4::InportType-->PortType");
 ostr+=g_txt(lseg,"dt_off", "WyDirEnt removed member dt_off, ");
 ostr+=g_txt(lseg,"dt_reclen", "WyDirEnt removed member dt_reclen, ");
 ostr+=g_txt(lseg,"Wy_ThreadID", "Wy_ThreadID removed, see Wy_Thread.3wy and Wy::_thread_equal, ");
 ostr+=g_f0(lseg,"cancel", "Wy_Thread::cancel removed reply Wym_EPERM, ");
 ostr+=g_f0(lseg,"wait_not", "Wy_Thread::wait_not removed reply Wym_EPERM, ");
 ostr+=g_txt(lseg,"Wy_Second", "Wy_Second->WyTimeSpec::SecondType, ");
 ostr+=g_txt(lseg,"Wy_Nano", "Wy_Nano->WyTimeSpec::NanoSecType, ");
 ostr+=g_mf(lseg,"abs", "WyTimeSpec::abs removed, use Wy::abs, ");
 ostr+=g_fn(lseg,"_mkstr", "Wy::_mkstr of integer version removed 4-argument overload, "
                              "Wy::_mkstr of float version removed 3-argument overload, "
                              "Wy::_mkstr has no 2-argument overload, see Wy.3wy");
 ostr+=g_txt(lseg,"Wym_EMATHNEG", "Wym_EMATHNEG->Wym_ERANGE, ");
 ostr+=g_mf(lseg,"_setsize", "WyStr::_setsize changed return type to void, ");
 ostr+=g_txt(lseg,"WySockAddrINet4::IPAddr", "WySockAddrINet4::IPAddr(uint32_t) and WySockAddrINet4::IPAddr(const char*) constructors changed to explicit, ");
 ostr+=g_mf(lseg,"setispeed", "WyTermios::_setispeed changed to return speed_t, ");
 ostr+=g_mf(lseg,"setospeed", "WyTermios::_setospeed changed to return speed_t, ");
 ostr+=g_txt(lseg,"Wy_FdSet", "Wy_FdSet::operator= is not available, use reset(..), ");

 return (ostr.size()==0)? WyStr(): hdr+ostr;
};

static void hints_for_file(const char* fname)
{
 WyRet r;
 WyRegFile regf(fname, O_RDONLY);
 WyStr lnbuf,report_str;
 size_t line_num;

 Wy::cout << "\nFile: " << fname << '\n';
 line_num=0;
 for(;;) {
   const size_t MaxLineLength=4096;
   size_t n_rd;
   ++line_num;
   lnbuf.reset();   // the following is an append read
   if((r=regf._read_till(lnbuf, MaxLineLength, n_rd, '\n'))!=Ok) {
     WY_THROW(r);
   }
   if(n_rd==0) {
     break;   // eof
   }
   WyStr lnhdr("Line("+Wy::wrd(line_num)+"):");
   report_str=hints_for_line(lnhdr.cseg(), lnbuf.cseg());
   if(report_str.size()!=0) {
     Wy::cout << report_str << '\n';
   }
 }
};

static const char *cmd_syntax=
  "$upd-31-32 {file}\n";

int main(int argc, char *argv[])
try {
 if(argc<2) {
   Wy::cerr << cmd_syntax;
   return(-1);
 }
 for(int i=1; i<argc; ++i) {
   hints_for_file(argv[i]);
 }
 return(0);
}
catch(const WyRet& e) {
 Wy::cerr << "main caught: " << Wy::wrd(e) << '\n';
 return e->repcode();
}
catch(...) {
 Wy::cerr << "main caught: ...\n";
 throw;
};
