/* Copyright is licensed by GNU LGPL.                         by I.J.Wang 2007

   This file contains function template wy__mkstr_integers
   for inclusion to instantiate integral type of _mkstr
*/
#ifndef WY_MKSTR_INTEGERS_HH__
#define WY_MKSTR_INTEGERS_HH__

#define WYLIB_SOURCE
#include "WyNum.h"
#include "WyStr.h"
#include "wy__itoa.hh"

// Note: For WyStr versions: If str has enough capacity, no reallocation, so no ENOMEM
//       (for now, min_capacity is enough for all arithmetic types)
//
template<typename NumType>
WyRet wy__mkstr_integers(WyStr& str, NumType num, Wy::WyRadix radix) WY__NOTHROW__
{
 if((radix<Wy::Wy_MinRadix)||(radix>Wy::Wy_MaxRadix)) {
   WY_RETURN( Wym_EINVAL );
 }
 size_t v=Wy::wy_itoa_l(str.begin(), str._capacity(), num, radix);
 if(v==0) {
   const WyRet r( str._reserve(std::numeric_limits<NumType>::digits+2) );
   if(r!=Ok) {
     WY_RETURN(r);
   }
   v=Wy::wy_itoa_l(str.begin(), str._capacity(), num, radix);
   if(v==0) {
     WY_ASSERT(v>0);
     WY_RETURN( Wym_ENOMEM );  // should not occur
   }
 }
 str._setsize(v);
 return(Ok);
};

template<typename NumType>
WyStr wy__wrd_integers(NumType num, Wy::WyRadix radix)
{
 WyStr str;
 if((radix<Wy::Wy_MinRadix)||(radix>Wy::Wy_MaxRadix)) {
   WY_THROW( WyStr::Reply(Wym_EINVAL) );
 }
 size_t v=Wy::wy_itoa_l(str.begin(), str._capacity(), num, radix);
 if(v==0) {
   const WyRet r( str._reserve(std::numeric_limits<NumType>::digits+2) );
   if(r!=Ok) {
     WY_THROW( WyStr::Reply(r) );
   }
   v=Wy::wy_itoa_l(str.begin(), str._capacity(), num, radix);
   if(v==0) {
     WY_ASSERT(v>0);
     WY_THROW( WyStr::Reply(Wym_ENOMEM) );  // should not occur
   }
 }
 str._setsize(v);
 return(str);
};

#endif  // end WY_MKSTR_INTEGERS_HH__
