/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2005

   operators <<,>> for WyByteFlow
   Simulate std::cin/cout/cerr usage syntax
*/

#define WYLIB_SOURCE
#include "wy_iouty.h"
#include "wy__put_value.hh"   // intended for inclusion after "wy_iouty.h"
#include <unistd.h>

WyByteFlow& operator <<(WyByteFlow& os, const WyStr& str)
{
 size_t n_wr;
 WyRet r( Wy::full_write(os,str.data(),str.size(),n_wr) );
 if(r!=Ok) {
   WY_THROW( WyByteFlow::Reply(r) );
 }
 if(n_wr!=str.size()) {
   WY_THROW( WyByteFlow::Reply(Wym_EIO) );
 }
 return(os);
};

WyByteFlow& operator <<(WyByteFlow& os, const WyCSeg& str)
{
 size_t n_wr;
 WyRet r( Wy::full_write(os,str.begin(),str.size(),n_wr) );
 if(r!=Ok) {
   WY_THROW( WyByteFlow::Reply(r) );
 }
 if(n_wr!=str.size()) {
   WY_THROW( WyByteFlow::Reply(Wym_EIO) );
 }
 return(os);
};

WyByteFlow& operator <<(WyByteFlow& os, const char* cstr)
{
 const size_t slen( std::strlen(cstr) );
 size_t n_wr;
 WyRet r( Wy::full_write(os,cstr,slen,n_wr) );
 if(r!=Ok) {
   WY_THROW( WyByteFlow::Reply(r) );
 }
 if(n_wr!=slen) {
   WY_THROW( WyByteFlow::Reply(Wym_EIO) );
 }
 return(os);
};

WyByteFlow& operator <<(WyByteFlow& os, char* cstr)
{
 const size_t slen( std::strlen(cstr) );
 size_t n_wr;
 WyRet r( Wy::full_write(os,cstr,slen,n_wr) );
 if(r!=Ok) {
   WY_THROW( WyByteFlow::Reply(r) );
 }
 if(n_wr!=slen) {
   WY_THROW( WyByteFlow::Reply(Wym_EIO) );
 }
 return(os);
};

WyByteFlow& operator <<(WyByteFlow& os, char ch)
{
 size_t n_wr;
 WyRet r( Wy::full_write(os,&ch,sizeof(ch),n_wr) );
 if(r!=Ok) {
   WY_THROW( WyByteFlow::Reply(r) );
 }
 if(n_wr!=1) {
   WY_THROW( WyByteFlow::Reply(Wym_EIO) );
 }
 return(os);
};

WyByteFlow& operator <<(WyByteFlow& os, const void* p)
{
 typedef unsigned long PtrNumType;
 WyRet r;
 size_t n_wr;
 size_t slen;
 char sbuf[sizeof(PtrNumType)*2+2];  // one byte two digits, plus "0x"
 sbuf[0]='0';
 sbuf[1]='x';
 slen=Wy::_ntoa(sbuf+2,sizeof(sbuf)-2,(PtrNumType)p,Wy::WyRadix(16));
 if(slen==0) {
   WY_THROW( WyByteFlow::Reply(Wym_ENOMEM) );  // should not occur
 }
 slen+=2;
 if((r=Wy::full_write(os,sbuf,slen,n_wr))!=Ok) {
   WY_THROW( WyByteFlow::Reply(r) );
 }
 if(n_wr!=slen) {
   WY_THROW( WyByteFlow::Reply(Wym_EIO) );
 }
 return(os);
};

//-------------------------
WyByteFlow& operator <<(WyByteFlow& bf, signed char num)
{ return wy__put_value(bf,num); };
WyByteFlow& operator <<(WyByteFlow& bf, signed short num)
{ return wy__put_value(bf,num); };
WyByteFlow& operator <<(WyByteFlow& bf, signed int num)
{ return wy__put_value(bf,num); };
WyByteFlow& operator <<(WyByteFlow& bf, signed long num)
{ return wy__put_value(bf,num); };
WyByteFlow& operator <<(WyByteFlow& bf, signed long long num)
{ return wy__put_value(bf,num); };

WyByteFlow& operator <<(WyByteFlow& bf, unsigned char num)
{ return wy__put_value(bf,num); };
WyByteFlow& operator <<(WyByteFlow& bf, unsigned short num)
{ return wy__put_value(bf,num); };
WyByteFlow& operator <<(WyByteFlow& bf, unsigned int num)
{ return wy__put_value(bf,num); };
WyByteFlow& operator <<(WyByteFlow& bf, unsigned long num)
{ return wy__put_value(bf,num); };
WyByteFlow& operator <<(WyByteFlow& bf, unsigned long long num)
{ return wy__put_value(bf,num); };

//-------------------------
WyByteFlow& operator >>(WyByteFlow& bf, WyStr& str)
{
 str.reset();
 size_t n_rd;
 WyRet r( bf.read(str,str._capacity(),n_rd) );
 if(r!=Ok) {
   WY_THROW( WyByteFlow::Reply(r) );
 }
 if(n_rd!=str.size()) {
   WY_THROW( WyByteFlow::Reply(Wym_EIO) );
 }
 return(bf);
};

WyByteFlow& operator >>(WyByteFlow& bf, char& ch)
{
 size_t n_rd;
 WyRet r( bf.read(&ch,sizeof(ch),n_rd) );
 if(r!=Ok) {
   WY_THROW( WyByteFlow::Reply(r) );
 }
 if(n_rd!=sizeof(ch)) {
   if(n_rd==0) {
     WY_THROW( WyByteFlow::Reply(Wym_ENOENT) );
   }
   WY_THROW( WyByteFlow::Reply(Wym_EIO) );
 }
 return(bf);
};

//#include <wyiovec.h>
//WyByteFlow& operator >>(WyByteFlow& bf, WyIOVec iov)
//{
//};
