/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2005

   Utilities for library/program development

   This file is part of WyByteFlow.h

   Note: For arithmetic type t, f<< t, and f<< Wy::wrd(t) are both allowed
*/

#ifndef WY_IOUTY_H__
#define WY_IOUTY_H__
#define WY_IOUTY_VERSION 33

#include "WyRet.h"

class WyStr;
class WyCSeg;
class WyByteFlow;

namespace Wy {

//SRC: wy_iouty_fread.cpp
WyRet full_read(WyByteFlow& bf, void* buf, size_t blen, size_t& n_read);
WyRet full_read(WyByteFlow& bf, WyStr& buf, size_t count);

//SRC: wy_iouty_fwrite.cpp
WyRet full_write(WyByteFlow& bf, const void* buf, size_t blen, size_t& n_wr);
WyRet full_write(WyByteFlow& bf, const WyStr& str, size_t& n_wr);
WyRet full_write(WyByteFlow& bf, const WyCSeg& cs, size_t& n_wr);

};  // end Wy

// Operator<< overload
//   Write bytes indicated by the right-hand side argument to the
//   left-hand side WyByteFlow object.
// 
WyByteFlow& operator <<(WyByteFlow& bf, const WyStr& str);
WyByteFlow& operator <<(WyByteFlow& bf, const WyCSeg& str);
WyByteFlow& operator <<(WyByteFlow& bf, const char* cstr);
WyByteFlow& operator <<(WyByteFlow& bf, char* cstr);
WyByteFlow& operator <<(WyByteFlow& bf, char ch);
WyByteFlow& operator <<(WyByteFlow& bf, const void* p);

// Operator<< overload
//   Convert the right-hand side arithmetic type argument to radix 10 
//   string representation and write to the left-hand side WyByteFlow
//   object.
// 
WyByteFlow& operator <<(WyByteFlow&, signed char num);
WyByteFlow& operator <<(WyByteFlow&, signed short num);
WyByteFlow& operator <<(WyByteFlow&, signed int num);
WyByteFlow& operator <<(WyByteFlow&, signed long num);
WyByteFlow& operator <<(WyByteFlow&, signed long long num);
WyByteFlow& operator <<(WyByteFlow&, unsigned char num);
WyByteFlow& operator <<(WyByteFlow&, unsigned short num);
WyByteFlow& operator <<(WyByteFlow&, unsigned int num);
WyByteFlow& operator <<(WyByteFlow&, unsigned long num);
WyByteFlow& operator <<(WyByteFlow&, unsigned long long num);
WyByteFlow& operator <<(WyByteFlow&, float);     // SRC: wy_iouty_float.cpp
WyByteFlow& operator <<(WyByteFlow&, double);    // SRC: wy_iouty_double.cpp

// Operator>> overload
//   Read data from the left-hand side WyByteFlow object and store into
//   the right-hand side object
//
WyByteFlow& operator >>(WyByteFlow& bf, WyStr& str);
WyByteFlow& operator >>(WyByteFlow& bf, char& ch);
inline WyByteFlow& operator >>(WyByteFlow& bf, unsigned char& ch)
     { return operator>>(bf,(char&)ch); };
inline WyByteFlow& operator >>(WyByteFlow& bf, signed char& ch)
     { return operator>>(bf,(char&)ch); };

#endif  // end of WY_IOUTY_H__
