/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2005
*/

#define WYLIB_SOURCE
#include "wy_iouty.h"
#include "wy__put_value.hh"   // intended for inclusion after "wy_iouty.h"
#include <unistd.h>

WyRet Wy::full_read(WyByteFlow& bf, void* buf, size_t blen, size_t& n_read)
{
 WyRet r;
 size_t btr=blen;
 char *bptr=static_cast<char*>(buf);

 n_read=0;    // n_read always reports the bytes read
 for(;;) {
   size_t n;
   try {
     r= bf.read(bptr,btr,n);
     n_read+=n;
   }
   catch(...) {
     n_read+=n;
     throw;
   };
   if(r!=Ok) {
     WY_RETURN(r);
   }
   if((n==btr)||(n==0)) {
     return(Ok);
   }
   #ifdef WY_DEBUG
   if(n>btr) {
     WY_TERMINATE("");
   }
   #endif

   bptr+=n;
   btr-=n;
 }
 WY_ASSERT("UNREACHABLE");
};

WyRet Wy::full_read(WyByteFlow& bf, WyStr& buf, size_t count)
{
 WyRet r;

 if((r=buf._reserve(count))!=Ok) {
   WY_RETURN(r);
 }

 size_t n_rd=0;
 size_t btr=count;
 char *bptr=buf.begin();
 for(;;) {
   size_t n;
   try {
     r= bf.read(bptr,btr,n);
     n_rd+=n;
     buf._setsize(n_rd);   // no failure
   }
   catch(...) {
     n_rd+=n;
     buf._setsize(n_rd);   // no failure
     throw;
   };
   if(r!=Ok) {
     WY_RETURN(r);
   }
   if((n==btr)||(n==0)) {
     return(Ok);
   }
   #ifdef WY_DEBUG
   if(n>btr) {
     WY_TERMINATE("");
   }
   #endif

   bptr+=n;
   btr-=n;
 }
 WY_ASSERT("UNREACHABLE");
};
