/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2005
*/

#define WYLIB_SOURCE
#include "wy_iouty.h"
#include "wy__put_value.hh"   // intended for inclusion after "wy_iouty.h"
#include <unistd.h>

WyRet Wy::full_write(WyByteFlow& bf, const void* buf, size_t blen, size_t& n_wr)
{
 WyRet r;
 const char* wr_ptr= static_cast<const char*>(buf);
 size_t btw( blen );  // byte to write

 n_wr=0;
 for(;;) {
   size_t n;
   try {
     r=bf.write(wr_ptr,btw,n);
     n_wr+=n;
   }
   catch(...) {
     n_wr+=n;
     throw;
   };
   if(r!=Ok) {
     WY_RETURN(r);
   }
   if(n==btw) {
     return(Ok);
   }

   #ifdef WY_DEBUG
   if(n>btw) {
     WY_TERMINATE("");
   }
   #endif

   wr_ptr+=n;
   btw-=n;
 }
 WY_ASSERT("UNREACHABLE");
};

WyRet Wy::full_write(WyByteFlow& bf, const WyStr& str, size_t& n_wr)
{ return full_write(bf,str.begin(),str.size(),n_wr); };

WyRet Wy::full_write(WyByteFlow& bf, const WyCSeg& cs, size_t& n_wr)
{ return full_write(bf,cs.begin(),cs.size(),n_wr); };

