/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2007
*/

#define WYLIB_SOURCE
#include "WyNum.h"
#include "wy__itoa.hh"
#include "wy__mkstr_integers.hh"
#include "WyStr.h"

namespace Wy {

typedef signed short S_Type;   // local number signed type
typedef unsigned short U_Type; // local number unsigned type

size_t wy_itoa_l(char* buf, size_t blen, S_Type num, Wy::WyRadix radix) WY__TSPC()
{
 return wy__itoa_l(buf, blen, num, radix);
};

size_t wy_itoa_l(char* buf, size_t blen, U_Type num, Wy::WyRadix radix) WY__TSPC()
{
 return wy__itoa_l(buf, blen, num, radix);
};

size_t wy_itoa_r(char* buf, size_t blen, S_Type num, Wy::WyRadix radix) WY__TSPC()
{
 return wy__itoa_r(buf, blen, num, radix);
};

size_t wy_itoa_r(char* buf, size_t blen, U_Type num, Wy::WyRadix radix) WY__TSPC()
{
 return wy__itoa_r(buf, blen, num, radix);
};

// [Syn] Convert num(integer type) to string of format [-]<ddd> and store to
//       buf pointed array according to left or right alignment suffix _l/_r
//
//       For _ntoa(..), the result is stored from buf
//       For _ntoa_r(..), the result is stored from buf+blen-n(the return value)
//
// [Ret] Number of characters converted and stored into buf (no trailing zero is written)
//       If 0 is returned, it is one of the following three reasons:
//            1. length blen of buf space is not enough to store the result
//            2. buf is NULL
//            3. radix is not in the range [2,36]
//
size_t _ntoa(char* buf, size_t blen, S_Type num, Wy::WyRadix radix) WY__TSPC()
{
 if((buf==NULL)||(radix<Wy::Wy_MinRadix)||(radix>Wy::Wy_MaxRadix)) {
   return(0);
 }
 return Wy::wy_itoa_l(buf, blen, num, radix);
};

size_t _ntoa(char* buf, size_t blen, U_Type num, Wy::WyRadix radix) WY__TSPC()
{
 if((buf==NULL)||(radix<Wy::Wy_MinRadix)||(radix>Wy::Wy_MaxRadix)) {
   return(0);
 }
 return Wy::wy_itoa_l(buf, blen, num, radix);
};

size_t _ntoa_r(char* buf, size_t blen, S_Type num, Wy::WyRadix radix) WY__TSPC()
{
 if((buf==NULL)||(radix<Wy::Wy_MinRadix)||(radix>Wy::Wy_MaxRadix)) {
   return(0);
 }
 return Wy::wy_itoa_r(buf, blen, num, radix);
};

size_t _ntoa_r(char* buf, size_t blen, U_Type num, Wy::WyRadix radix) WY__TSPC()
{
 if((buf==NULL)||(radix<Wy::Wy_MinRadix)||(radix>Wy::Wy_MaxRadix)) {
   return(0);
 }
 return Wy::wy_itoa_r(buf, blen, num, radix);
};

WyRet _mkstr(WyStr& str, S_Type num, Wy::WyRadix radix) WY__TSPC()
{
 return wy__mkstr_integers(str,num,radix);
};

WyRet _mkstr(WyStr& str, U_Type num, Wy::WyRadix radix) WY__TSPC()
{
 return wy__mkstr_integers(str,num,radix);
};

WyStr wrd(S_Type num, Wy::WyRadix radix)
{
 return wy__wrd_integers(num,radix);
};

WyStr wrd(U_Type num, Wy::WyRadix radix)
{
 return wy__wrd_integers(num,radix);
};

};  // end namespace Wy
