/* Copyright is licensed under GNU LGPL.                 by I.J.Wang 2006

   Header only for inclusion
   Template for instantiate _scanum(..)
*/

#ifndef WY_SCANUM_HH__
#define WY_SCANUM_HH__

namespace Wy {
//
// [Benefit] Make simulating sscanf(text,"%f %ld %d %x",..) easier (less code)
//
template <typename ValueType>
WyRet wy_scanum_c(ValueType& value, WyCSeg& text, const Wy::WyRadix& radix)
{
 static const WyCSeg Blanks=" \t\n\r\v\f";
 size_t fidx;
 if(text.findc_not(fidx,0,Blanks)==false) {
   WY_RETURN( Wym_ENOENT ); // all blanks
 }
 WyCSeg cs(text.cseg(fidx));
 const WyRet r=Wy::_strnum(value, cs, radix);
 if(r==Wym_EBADMSG) {
   text=cs;
   return(Ok);
 }
 if(r==Ok) {
   text=cs;
 }
 return(r);
};

template <typename ValueType>
WyRet wy_scanum_n(ValueType& value, WyCSeg& text, Wy::WyRadix& radix)
{
 static const WyCSeg Blanks=" \f\n\r\t\v";
 size_t fidx;
 if(text.findc_not(fidx,0,Blanks)==false) {
   WY_RETURN( Wym_ENOENT ); // all blanks
 }
 WyCSeg cs(text.cseg(fidx));
 const WyRet r=Wy::_strnum(value, cs, radix);
 if(r==Wym_EBADMSG) {
   text=cs;
   return(Ok);
 }
 if(r==Ok) {
   text=cs;
 }
 return(r);
};

}; // end namespace Wy

#endif  // end WY_SCANUM_HH__

