/* Copyright is licensed under GNU LGPL                       by I.J.Wang 2003

 wydefs.h defines symbols/types/macros for libwy classes.

 Compiler option summary (if defined):
   WYLIB_CONFIG_THROWSPC ... Use throw specification. Default throw specification
                             is empty.
   WYLIB_CONFIG_NORSLI   ... Disable use of SLI

   WY_NOEXITMSG  .... WY_TERMINATE sends msg
   WY_DEBUG      .... generate debug version
   WY_SHORT_SLI  .... Short SLI message
*/
#ifndef WYDEFS_H__
#define WYDEFS_H__

#include <cerrno>      // library-wise
#include <unistd.h>

// libwy requirements
//
#ifdef WYLIB_SOURCE
 #if _POSIX_VERSION<199506L
 #error No Support For POSIX1003.1c     // thread support is required
 #endif

 #ifndef _POSIX_VERSION
 #error No POSIX Support
 #endif

 #ifndef _POSIX_SOURCE
 #define _POSIX_SOURCE
 #endif

 #ifndef _POSIX_C_SOURCE
 #define _POSIX_C_SOURCE 199309
 #endif
#endif

// Throw specification by default is empty, unless
// WYLIB_CONFIG_THROWSPC is defined
//
// Note: gcc has an option -fno-enforce-eh-specs (don't check for
//       violation of throw specification)
//
#ifdef WYLIB_CONFIG_THROWSPC
 #define WY__TSPC(x) throw(x)
#else
 #define WY__TSPC(x)
#endif

// This symbol must be empty and internal use only
//
#define WY__NOTHROW__
//#define WY__NOTHROW__ throw()       // for test

// libwy version number
//
#define WYLIB_VERSION 33

// Object signature of libwy is defined here.
// Object that concerns being accessed after destroyed would have an signature.
//
// The format of ID=0x0RSTRST0, where 'R' 'S' 'T' is a hexidecimal digit 0-f.
//
#include <signal.h>   // for sig_atomic_t
const sig_atomic_t WY_NONE_ID=0xfaad321f;     //  Not an libwy obj ID
const sig_atomic_t WYMUTEX_ID=0x08208200;     //  for WyMutex object
const sig_atomic_t WYCOND_ID =0x08308300;     //  for WyCond object

#include "wy__base.h"

// WY_NOEXITMSG is a compiler option to change behavior of WY_TERMINATE
//
#ifdef WY_NOEXITMSG
#define WY_TERMINATE(cstr) Wy__Base::terminate()
#else
#define WY_TERMINATE(cstr) Wy__Base::put_abnmsg(cstr,__FILE__,__LINE__),\
                      Wy__Base::terminate()
#endif

#ifdef WY_DEBUG 
 #define WY_ASSERT(expr)  if(expr) {} else {\
                          Wy__Base::put_abnmsg(#expr,__FILE__,__LINE__);\
                      Wy__Base::terminate(); }
#else
 #define WY_ASSERT(expr) ((void)0)
#endif

typedef int Wy__FdType;             // file descriptor type
const Wy__FdType WY__DEFAULT_FD=-1; // default file descriptor

#define WY__CONST_GIGA 1000000000LL // 1x10^9

// Source document system cancel point
//
#define WY__CANCEL_POINT  (void)0

namespace Wy {
 enum ByMove_t { ByMove };         // For the move constructor signature

 // Problem: typedef signed char WyRadix;
 //   Wy::WyRadix radix=10;
 //   Wy::_strnum(.., radix);   // executed may be the const version, but using
 //                             // int type is ok !!!
 // 
 typedef int WyRadix;
};

#ifdef NDEBUG
 #define WY_NDEBUG_MSG(r)
#else
 #define WY_NDEBUG_MSG(r) Wy__Base::put_abnmsg((r)->repstr(),__FILE__,__LINE__)
#endif

#include <cstddef>       // for NULL

#endif // End WYDEFS_H__
