/* Copyright is licensed under GNU LGPL.                (I.J. Wang, 2003-2004)

 This file is part of WyRet.h and is included there. 
 Definition code is in wyret.cpp
 */
#ifndef WYR_SRCLOC_H__
#define WYR_SRCLOC_H__
#define WYR_SRCLOC_VERSION 33

// [Thread-Safe]
//
// [Internal] Class storing source location information
//
// Instances of this class should mostly be static const declared to live
// throughout program life time.
//
// Note: All member functions do not throw
// 
class Wy_SrcLoc {
  public:

    // [Syn] Construct the object to hold the source location information.
    //
    // Note: The c-string pointed to by file_name is supposed to exist constantly 
    //       throughout the program life time.
    //
    inline Wy_SrcLoc(const char* file_name, int line) WY__NOTHROW__
	      : _fn(file_name),_ln(line) {};

    // [Syn] Get the file name pointer this object holds
    //
    // [Ret] pointer of the file name this object holds
    //       (may be NULL)
    //
    inline const char *src_file() const WY__NOTHROW__
              { return _fn; };

    // [Syn] Get the line number
    //
    // [Ret] The line number this object holds
    //
    inline int src_line() const WY__NOTHROW__
              { return _ln; };

  private:
    const char* _fn;
    int _ln;

    // Hidden
    Wy_SrcLoc(const Wy_SrcLoc& c) WY__NOTHROW__;
    const Wy_SrcLoc& operator=(const Wy_SrcLoc& rhs) WY__NOTHROW__;
    bool operator==(const Wy_SrcLoc& rhs) const WY__NOTHROW__;
};

#endif
